using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLEDsDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;

using GBMSAPI_CS_Example.UTILITY;

namespace GBMSAPI_CS_Example.CALIBRATION
{
    public partial class CalibrationForm : Form
    {
        protected IntPtr CalibrationImage;
		protected int CalibrationImageSX;
		protected int CalibrationImageSY;
		protected uint Diagnostic;
		Boolean FlatScanArea;
		Image OriginalImage;

        public void ReleaseResources()
        {
           // in order to free allocated resources 
        }

        public CalibrationForm()
        {
            InitializeComponent();

            this.CalibrationImage = (GBMSAPI_Example_Globals.CalibrationBuffer);
            if (CalibrationImage == IntPtr.Zero)
            {
                MessageBox.Show("MEMORY ALLOCATION ERROR", "HEAVY ERROR!!!!");
                return;
            }
            FlatAreaRadioButton.Checked = true;
            this.OriginalImage = this.CalibrationImagePictureBox.Image;
            this.FlatAreaRadioButton_CheckedChanged(null,null);
        }

        private void FlatAreaRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (FlatAreaRadioButton.Checked == true)
			{
                this.FlatScanArea = true;
			}
			else
			{
				this.FlatScanArea = false;
			}

			uint ImSX,ImSY;
			uint ObjTypeToScan = (this.FlatScanArea == true)?
                GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SINGLE_FINGER:
                GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER;

            int RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetCalibrationImageSize(
                    this.FlatScanArea, out ImSX, out ImSY
                );

            if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(RetVal,
                    "FlatAreaRadioButton_CheckedChanged,GBMSAPI_NET_GetCalibrationImageSize");
                return;
            }

			CalibrationImageSX = (int)ImSX;
            CalibrationImageSY = (int)ImSY;

			this.ImSXTextBox.Text = "" + CalibrationImageSX;
			this.ImSYTextBox.Text = "" + CalibrationImageSY;

			this.DiagnosticListBox.Items.Clear();
			this.CalibrationImagePictureBox.Image = this.OriginalImage;

			this.SaveButton.Enabled = false;
        }

        private void GetNewCalibrationImageButton_Click(object sender, EventArgs e)
        {
            try
			{
				uint CalibrationDiagnostic;
                Byte [] CalImageArray;

                int RetVal = GBMSAPI_NET_AuxiliaryRoutines.GBMSAPI_NET_GetCalibrationImage(
                    FlatScanArea,out CalImageArray,out CalibrationDiagnostic);

				if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(
                        RetVal, "GetNewCalibrationImageButton_Click,GBMSAPI_GetImageSize");
                    return;
                }

                Marshal.Copy(CalImageArray, 0, this.CalibrationImage, CalImageArray.Length);

				// show diagnostic
				this.Diagnostic = CalibrationDiagnostic;
				this.DiagnosticListBox.Items.Clear();
				if (CalibrationDiagnostic != 0)
				{
                    List<String> DiagList = GBMSAPI_Example_Util.GetDiagStringsFromDiagMask(CalibrationDiagnostic);
                    if ((DiagList != null) && (DiagList.Count > 0))
                    {
                        for (int i = 0; i < DiagList.Count; i++)
                        {
                            this.DiagnosticListBox.Items.Add(DiagList[i]);
                        }
                    }
				}

                // be aware of stride
                int stride;
				IntPtr BmpBuffer;
				if ((CalibrationImageSX %4) != 0)
				{
					stride = CalibrationImageSX - (CalibrationImageSX %4);

                    Byte [] Dest; Byte [] Src;
                    Dest = new Byte[stride * CalibrationImageSY];
                    Src = new Byte[CalibrationImageSX * CalibrationImageSY];

                    if (Dest == null || Src == null)
                    {
                        MessageBox.Show("Error in memory allocation in GetNewCalibrationImageButton_Click");
                    }

                    Marshal.Copy(GBMSAPI_Example_Globals.CalibrationBuffer, Src, 0, Src.Length);
                    Marshal.Copy(GBMSAPI_Example_Globals.DummyCalibrationBuffer, Dest, 0, Dest.Length);

                    int DestOffset = 0, SrcOffset = 0;
                    for (int i = 0; i < CalibrationImageSY;
                        i ++, DestOffset += stride, SrcOffset += CalibrationImageSX)
                    {
                        Buffer.BlockCopy(Src,SrcOffset,Dest,DestOffset,stride);
                    }

                    Marshal.Copy(Dest, 0, GBMSAPI_Example_Globals.DummyCalibrationBuffer, Dest.Length);

                    BmpBuffer = GBMSAPI_Example_Globals.DummyCalibrationBuffer;
				}
				else
				{
					stride = CalibrationImageSX;
                    BmpBuffer = GBMSAPI_Example_Globals.CalibrationBuffer;
				}
				
				// draw image
				this.CalibrationImagePictureBox.Image = new Bitmap(
					CalibrationImageSX, CalibrationImageSY,
					stride,
                    System.Drawing.Imaging.PixelFormat.Format8bppIndexed,
					BmpBuffer
					);

                System.Drawing.Imaging.ColorPalette pal = this.CalibrationImagePictureBox.Image.Palette;
				for (int i = 0; i < pal.Entries.Length; i++)
				{
					pal.Entries[i] = Color.FromArgb(255, i, i, i);
				}
				this.CalibrationImagePictureBox.Image.Palette = pal;
				this.CalibrationImagePictureBox.SizeMode = PictureBoxSizeMode.StretchImage;

				this.SaveButton.Enabled = true;

				this.Update();
			}
			catch(Exception ex)
			{
				MessageBox.Show("Exception in GetNewCalibrationImageButton_Click: " + ex.Message);
			}
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            /////////////////////
			// SAVE FILE
			/////////////////////
			Byte DeviceID;
            String DeviceSerialNumber;

			// Get Serial Number and ID
            
			int RetVal = GBMSAPI_NET_DeviceSettingRoutines.GBMSAPI_NET_GetCurrentDevice(
                out DeviceID,out DeviceSerialNumber);

            MessageBox.Show("SaveButton_Click: DeviceSerialNumber = " + DeviceSerialNumber, "HEAVY ERROR");

			if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(
                    RetVal, "SaveButton_Click,GBMSAPI_GetImageSize");
                return;
            }

			// build file name
			String FName = GBMSAPI_Example_Util.GBMSAPI_Example_GetDevNameFromDevID(DeviceID) + "_" +
				(DeviceSerialNumber) + "_" +
				((this.FlatScanArea == true)?("FLAT"):("ROLL")) + ".raw";

			// build image array for saving
			Byte [] CalImageArray = new Byte[this.CalibrationImageSX * this.CalibrationImageSY];
			if (CalImageArray == null)
			{
                MessageBox.Show("MEMORY ALLOCATION ERROR in SaveButton_Click(): close application", "HEAVY ERROR");
				return;
			}

			Marshal.Copy(this.CalibrationImage,CalImageArray,0,CalImageArray.Length);

			// save image on file
			System.IO.FileStream FS = System.IO.File.Create(FName);
			if (FS != null)
			{
				FS.Write(CalImageArray,0,CalImageArray.Length);
				FS.Close();
				MessageBox.Show("Image saved in " + FName,"Calibration results");
			}
			else
			{
				MessageBox.Show("Cannot save file " + FName,"WARNING");
			}
        }

        private void SetFactoryCalibrationButton_Click(object sender, EventArgs e)
        {
            ///////////////////////////////
			// CALIBRATE DEVICE
			///////////////////////////////
			int RetVal = GBMSAPI_NET_AuxiliaryRoutines.GBMSAPI_NET_SetCalibrationImage(this.FlatScanArea,null);
            if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(
                    RetVal, "SetFactoryCalibrationButton_Click,GBMSAPI_GetImageSize");
                return;
            }

			MessageBox.Show("Device Calibrated","Calibration results");
        }

        private void CalibrateDeviceButton_Click(object sender, EventArgs e)
        {
            ////////////////////////////
			// OPEN IMAGE FILE
			////////////////////////////
			OpenFileDialog  OFDlg = new OpenFileDialog();
			OFDlg.Title = "Open Calibration Image";
			if (OFDlg.ShowDialog() != DialogResult.OK )
			{
				MessageBox.Show("Error in Opening File");
				return;
			}
			String fname = OFDlg.FileName;
			Byte []  CalImageBytes = System.IO.File.ReadAllBytes(fname);

			if (CalImageBytes.Length != (CalibrationImageSX) * (CalibrationImageSY))
			{
				MessageBox.Show("Error: file size not correct");
				return;
			}

			///////////////////////////////
			// CALIBRATE DEVICE
			///////////////////////////////
			int RetVal = GBMSAPI_NET_AuxiliaryRoutines.GBMSAPI_NET_SetCalibrationImage(
                this.FlatScanArea, CalImageBytes);
            if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(
                    RetVal, "CalibrateDeviceButton_Click,GBMSAPI_GetImageSize");
            }
			else
			{
				MessageBox.Show("Device Calibrated","Calibration results");
			}

			return;
        }
    }
}