using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLEDsDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;

using GBMSAPI_CS_Example.UTILITY;
using GBMSAPI_CS_Example.CALIBRATION;
using GBMSAPI_CS_Example.MAIN_FORM;
using GBMSAPI_CS_Example.ACQUISITION_FORM;

namespace GBMSAPI_CS_Example
{
    public partial class GBMASPI_CsExampleMainForm : Form
    {
        public Boolean GetAcquisitionOptions(
            out uint ObjToScan,
            out uint OptionMask,
            out uint DisplayOptionMask,
            out Byte ContrastLimitToDisplay,
            out Byte CompletenessLimitToDisplay,
            out uint ClipRegionW,
            out uint ClipRegionH,
            out uint RollPreviewTimeout,
            out Byte ArtefactCleaningDepth,
            out Byte ContrastIntermediateLimitToDisplay,
            out Byte CompletenessIntermediateLimitToDisplay
        )
        {
            ObjToScan = 0;
            OptionMask = 0;
            DisplayOptionMask = 0;
            ContrastLimitToDisplay = 0;
            CompletenessLimitToDisplay = 0;
            ClipRegionW = 0;
            ClipRegionH = 0;
            RollPreviewTimeout = 0;
            ArtefactCleaningDepth = 0;
            ContrastIntermediateLimitToDisplay = 0;
            CompletenessIntermediateLimitToDisplay = 0;

            try
            {
                /////////////////////////////
                // GET OBJECT TO SCAN
                /////////////////////////////
                String ObjToScanName = (String)(this.ObjectToScanComboBox.SelectedItem);
                ObjToScan = GBMSAPI_Example_Util.GetObjectToScanIDFromName(ObjToScanName);

                /////////////////////////////
                // GET OPTION MASK
                /////////////////////////////
                (OptionMask) = 0;

                // Auto-Capture
                if (this.AutoCaptureCheckBox.Enabled == true &&
                    this.AutoCaptureCheckBox.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE;
                }

                // No roll preview stop
                if (this.NoRollPreviewCheckBox.Enabled == true &&
                    this.NoRollPreviewCheckBox.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_NO_ROLL_PREVIEW;
                }

                // Roll Preview stop mode
                if (this.PedalRollPreviewStopModeRadioButton.Enabled == true &&
                    this.PedalRollPreviewStopModeRadioButton.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_MANUAL_ROLL_PREVIEW_STOP;
                }

                // Roll automatic preview type: only if automatic roll preview mode
                if (this.CenterRollPreviewStopModeRadioButton.Enabled == true &&
                    this.CenterRollPreviewStopModeRadioButton.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_ROLL_PREVIEW_TYPE;
                }

                // Full res preview image resolution
                if (this.FullRresolutionInPreviewCheckBox.Enabled == true &&
                    this.FullRresolutionInPreviewCheckBox.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FULL_RES_PREVIEW;
                }

                // VER 2600:
                // check HighSpeed Preview
                if (this.HighSpeedCheckBox.Enabled == true &&
                    this.HighSpeedCheckBox.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_HIGH_SPEED_PREVIEW;
                }

                // Flat object on roll scan area
                if (this.FlatSingleFingerOnRollAreaCheckBox.Enabled == true &&
                    this.FlatSingleFingerOnRollAreaCheckBox.Checked == true)
                {
                    (OptionMask) |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA;
                }

                /////////////////////////////////////
                // GET DISPLAY OPTION MASK
                /////////////////////////////////////
                DisplayOptionMask = 0;

                uint ExternalDevicesMask;
                int RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetOptionalExternalEquipment(out ExternalDevicesMask);

                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    GBMSAPI_Example_Util.GBMSAPI_Example_ManageErrors(
                        RetVal, "GetAcquisitionOptions,GBMSAPI_NET_GetOptionalExternalEquipment");
                    return false;
                }

                if ((ExternalDevicesMask & GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_VUI_LCD) != 0)
                {
                    // Stop screen after acquisition
                    if (this.StopScreenAfterAcquisitionCheckBox.Enabled == true &&
                        this.StopScreenAfterAcquisitionCheckBox.Checked == true)
                    {
                        (DisplayOptionMask) |= GBMSAPI_NET_DisplayOptions.GBMSAPI_NET_DO_FINAL_SCREEN;
                    }

                    // customized contrast
                    if (this.ShowCustomizedContrastCheckBox.Enabled == true &&
                        this.ShowCustomizedContrastCheckBox.Checked == true)
                    {
                        // read limit value
                        try
                        {
                            Byte ContrastLimit = Byte.Parse(this.ContrastLimitTextBox.Text);
                            ContrastLimitToDisplay = ContrastLimit;
                        }
                        catch (Exception ReadEx)
                        {
                            if ((ReadEx.GetType()).ToString() == ((new ArgumentNullException()).GetType()).ToString() ||
                                (ReadEx.GetType()).ToString() == ((new FormatException()).GetType()).ToString() ||
                                (ReadEx.GetType()).ToString() == ((new OverflowException()).GetType()).ToString()
                                )
                            {
                                // default value
                                ContrastLimitToDisplay = 120;
                            }
                            else
                            {
                                MessageBox.Show("Exception in GetAcquisitionOptions: " + ReadEx.Message);
                                ObjToScan = 0;
                                OptionMask = 0;
                                DisplayOptionMask = 0;
                                ContrastLimitToDisplay = 0;
                                CompletenessLimitToDisplay = 0;
                                ClipRegionW = 0;
                                ClipRegionH = 0;
                                RollPreviewTimeout = 0;
                                ArtefactCleaningDepth = 0;
                                return false;
                            }
                        }
                        (DisplayOptionMask) |= GBMSAPI_NET_DisplayOptions.GBMSAPI_NET_DO_CUSTOMIZED_CONTRAST;
                    }

                    // customized completeness
                    if (this.ShowCustomizedCompletenessCheckBox.Enabled == true &&
                        this.ShowCustomizedCompletenessCheckBox.Checked == true)
                    {
                        // read limit value
                        try
                        {
                            Byte CompletenessLimit = Byte.Parse(this.CompletenessLimitTextBox.Text);
                            CompletenessLimitToDisplay = CompletenessLimit;
                        }
                        catch (Exception ReadEx)
                        {
                            if ((ReadEx.GetType()).ToString() == ((new ArgumentNullException()).GetType()).ToString() ||
                                (ReadEx.GetType()).ToString() == ((new FormatException()).GetType()).ToString() ||
                                (ReadEx.GetType()).ToString() == ((new OverflowException()).GetType()).ToString()
                                )
                            {
                                // default value
                                CompletenessLimitToDisplay = 90;
                            }
                            else
                            {
                                MessageBox.Show("Exception in GetAcquisitionOptions: " + ReadEx.Message);
                                ObjToScan = 0;
                                OptionMask = 0;
                                DisplayOptionMask = 0;
                                ContrastLimitToDisplay = 0;
                                CompletenessLimitToDisplay = 0;
                                ClipRegionW = 0;
                                ClipRegionH = 0;
                                RollPreviewTimeout = 0;
                                ArtefactCleaningDepth = 0;
                                return false;
                            }
                        }
                        (DisplayOptionMask) |= GBMSAPI_NET_DisplayOptions.GBMSAPI_NET_DO_CUSTOMIZED_COMPLETENESS;
                    }
                }

                ///////////////////////////////////////
                // GET CLIP DATA
                ///////////////////////////////////////
                ClipRegionW = 0;
                ClipRegionH = 0;
                if (this.ClipEnableCheckBox.Enabled == true &&
                    this.ClipEnableCheckBox.Checked == true)
                {
                    // read clip values
                    try
                    {
                        UInt32 ClipSizeX = UInt32.Parse(this.ClipRegionSizeXTextBox.Text);
                        UInt32 ClipSizeY = UInt32.Parse(this.ClipRegionSizeYTextBox.Text);
                        ClipRegionW = ClipSizeX;
                        ClipRegionH = ClipSizeY;
                    }
                    catch (Exception ReadEx)
                    {
                        if ((ReadEx.GetType()).ToString() == ((new ArgumentNullException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new FormatException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new OverflowException()).GetType()).ToString()
                            )
                        {
                            // default value
                            ClipRegionW = 0;
                            ClipRegionH = 0;
                        }
                        else
                        {
                            MessageBox.Show("Exception in GetAcquisitionOptions: " + ReadEx.Message);
                            ObjToScan = 0;
                            OptionMask = 0;
                            DisplayOptionMask = 0;
                            ContrastLimitToDisplay = 0;
                            CompletenessLimitToDisplay = 0;
                            ClipRegionW = 0;
                            ClipRegionH = 0;
                            RollPreviewTimeout = 0;
                            ArtefactCleaningDepth = 0;
                            return false;
                        }
                    }
                }

                ///////////////////////////////////////////
                // GET ROLL OPTIONS
                ///////////////////////////////////////////
                // Roll Preview Timeout
                RollPreviewTimeout = 0;
                if (this.AutomaticRollPreviewTimeoutTextBox.Enabled == true)
                {
                    // read timeout
                    try
                    {
                        UInt32 Timeout = UInt32.Parse(this.AutomaticRollPreviewTimeoutTextBox.Text);
                        RollPreviewTimeout = Timeout;
                    }
                    catch (Exception ReadEx)
                    {
                        if ((ReadEx.GetType()).ToString() == ((new ArgumentNullException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new FormatException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new OverflowException()).GetType()).ToString()
                            )
                        {
                            // default value
                            RollPreviewTimeout = 0;
                        }
                        else
                        {
                            MessageBox.Show("Exception in GetAcquisitionOptions: " + ReadEx.Message);
                            ObjToScan = 0;
                            OptionMask = 0;
                            DisplayOptionMask = 0;
                            ContrastLimitToDisplay = 0;
                            CompletenessLimitToDisplay = 0;
                            ClipRegionW = 0;
                            ClipRegionH = 0;
                            RollPreviewTimeout = 0;
                            ArtefactCleaningDepth = 0;
                            return false;
                        }
                    }
                }
                // Artefact Cleaning Depth
                ArtefactCleaningDepth = 0;
                if (this.ArtefactCleaningDepthTextBox.Enabled == true)
                {
                    // read Clean Depth
                    try
                    {
                        Byte CleanDepth = Byte.Parse(this.ArtefactCleaningDepthTextBox.Text);
                        if (CleanDepth > GBMSAPI_NET_CleaningDepth.GBMSAPI_NET_ARTEFACT_CLEANING_DEPTH_MAX &&
                            CleanDepth != GBMSAPI_NET_CleaningDepth.GBMSAPI_NET_ARTEFACT_CLEANING_DEPTH_UNLIMITED)
                        {
                            MessageBox.Show("Clean depth greater than maximum allowed value; will be automatically set to max", "Warning");
                            CleanDepth = GBMSAPI_NET_CleaningDepth.GBMSAPI_NET_ARTEFACT_CLEANING_DEPTH_MAX;
                        }
                        ArtefactCleaningDepth = CleanDepth;
                    }
                    catch (Exception ReadEx)
                    {
                        if ((ReadEx.GetType()).ToString() == ((new ArgumentNullException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new FormatException()).GetType()).ToString() ||
                            (ReadEx.GetType()).ToString() == ((new OverflowException()).GetType()).ToString()
                            )
                        {
                            // default value
                            ArtefactCleaningDepth = 0;
                        }
                        else
                        {
                            MessageBox.Show("Exception in GetAcquisitionOptions: " + ReadEx.Message);
                            ObjToScan = 0;
                            OptionMask = 0;
                            DisplayOptionMask = 0;
                            ContrastLimitToDisplay = 0;
                            CompletenessLimitToDisplay = 0;
                            ClipRegionW = 0;
                            ClipRegionH = 0;
                            RollPreviewTimeout = 0;
                            ArtefactCleaningDepth = 0;
                            return false;
                        }
                    }
                }

                ///////////////////////////
                //// RETURN OK
                ///////////////////////////
                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error in GetAcquisitionOptions function: " + ex.Message);
                ObjToScan = 0;
                OptionMask = 0;
                DisplayOptionMask = 0;
                ContrastLimitToDisplay = 0;
                CompletenessLimitToDisplay = 0;
                ClipRegionW = 0;
                ClipRegionH = 0;
                RollPreviewTimeout = 0;
                ArtefactCleaningDepth = 0;

                return false;
            }
        }
    }
}