/*
 * testmain.cpp
 *
 *  Created on: Sep 16, 2009
 *      Author: root
 */

#include "GBMSAPI_SystemConfiguration.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include <ctype.h>
#include "sys/unistd.h"
#include <dlfcn.h>

typedef unsigned char GB_BOOL;
#define GB_FALSE	(GB_BOOL)0
#define GB_TRUE		(GB_BOOL)1

GB_BOOL	AcqStopped = GB_FALSE;
int		FrameCounter = 0;
int acq = 0;



int MyAcquisitionEventsManagerCallback
(
	unsigned int OccurredEventCode,
	int GetFrameErrorCode,
	unsigned int EventInfo,
	unsigned char *FramePtr,
	int FrameSizeX,
	int FrameSizeY,
	double CurrentFrameRate,
	double NominalFrameRate,
	unsigned int GB_Diagnostic,
	void *UserDefinedParameters
)
{
	printf("Callback called\n");

	// check occurred event code
	switch (OccurredEventCode)
	{
	case GBMSAPI_AE_SCANNER_STARTED:
		if(acq==0)GBMSAPI_SetClippingRegionSize(300,300);
		acq++;
		printf("Callback called: event code = GBMSAPI_AE_SCANNER_STARTED\n");
		break;
	case GBMSAPI_AE_PREVIEW_PHASE_END:
		printf("Callback called: event code = GBMSAPI_AE_PREVIEW_PHASE_END\n");
		break;
	case GBMSAPI_AE_VALID_FRAME_ACQUIRED:
		if (GB_Diagnostic != 0)
		{
			GBMSAPI_VUI_LED_BlinkDuringAcquisition(TRUE);
		}
		else
		{
			GBMSAPI_VUI_LED_BlinkDuringAcquisition(FALSE);
		}
		int AcquisitionPhase;
		(void)GBMSAPI_GetAutoCapturePhase(&AcquisitionPhase);
		printf("Callback called: event code = GBMSAPI_AE_VALID_FRAME_ACQUIRED\n");
		printf("Frame Counter = %d\n",FrameCounter);
		printf("AcquisitionPhase = %d\n",AcquisitionPhase);

		int ClipPosX,ClipPosY;
		unsigned int ClipSX,ClipSY;

		GBMSAPI_GetClippingRegionPosition(&ClipPosX,&ClipPosY,&ClipSX,&ClipSY);
		printf("ClipPosX = %d ClipPosY = %d ClipSX = %d ClipSY = %d\n",ClipPosX,ClipPosY,ClipSX,ClipSY);
		FrameCounter ++;
		break;
	case GBMSAPI_AE_ACQUISITION_ERROR:
		printf("Callback called: event code = GBMSAPI_AE_ACQUISITION_ERROR\n");
		AcqStopped = GB_TRUE;
		break;
	case GBMSAPI_AE_ACQUISITION_END:
	{
		printf("Callback called: event code = GBMSAPI_AE_ACQUISITION_END\n");
		printf("Frame Counter = %d\n",FrameCounter);

		GBMSAPI_ImageFinalization(FramePtr);
		FILE *fp = fopen("FingPrint.raw","wb");
		if (fp != NULL)
		{
			fwrite(FramePtr,sizeof(unsigned char),FrameSizeX * FrameSizeY,fp);
			fclose(fp);
		}
		unsigned char FgpContrast;
		(void)GBMSAPI_GetFingerprintContrast(&FgpContrast);
		printf("\n\nIMAGE CONTRAST = %d\n",FgpContrast);

		int ClipPosX,ClipPosY;
		unsigned int ClipSX,ClipSY;

		GBMSAPI_GetClippingRegionPosition(&ClipPosX,&ClipPosY,&ClipSX,&ClipSY);
		printf("ClipPosX = %d ClipPosY = %d ClipSX = %d ClipSY = %d\n",ClipPosX,ClipPosY,ClipSX,ClipSY);
		AcqStopped = GB_TRUE;
		break;
	}
	default:
		break;
	}

	return 1;
}

int main(void)
{
	GBMSAPI_DeviceInfoStruct AttachedDeviceList[GBMSAPI_MAX_PLUGGED_DEVICE_NUM];
	int AttachedDeviceNumber;
	unsigned long USBErrorCode;
	unsigned char CurrDevID;
	char CurrDevSerNum[50];
	unsigned int Appoggio,Appoggio1;
	GB_BOOL GoOn;
	int ChoosenDeviceID;
	void *DS84DllPtr;
	unsigned char pippo[16];
	 char pipponame[16];

	DS84DllPtr = dlopen("libDS84U.so", RTLD_LAZY);
	if (DS84DllPtr == NULL)
	{
		printf("%s\n",dlerror());
	}
	else
	{
		printf("OK!!!!!!!!!!!!!!!!!!\n");
	}

	printf("Starting\n\n");
	// in case it was not correctly closed
	GBMSAPI_UnloadLibrary();

	GBMSAPI_LoadLibrary();

	GBMSAPI_GetUnderlyingLibraryDllVersion(&(pippo[0]), &(pippo[3]), &(pippo[6]), &(pippo[9]), &(pipponame[0]));

	int RetVal = GBMSAPI_GetAttachedDeviceList(
		AttachedDeviceList,
		&AttachedDeviceNumber,
		&USBErrorCode
	);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetAttachedDeviceList  returned = %d\n",RetVal);
//		printf("Quitting application...\n");
//		return 1;
	}

	printf("AttachedDeviceNumber = %d\n\n",AttachedDeviceNumber);

	for (int i= 0; i < AttachedDeviceNumber; i++)
	{
		printf("Device %d type = %d\n",i+1,AttachedDeviceList[i].DeviceID);
		printf("Device %d sn = %s\n",i+1,AttachedDeviceList[i].DeviceSerialNumber);
	}

	if (AttachedDeviceNumber <= 0)
	{
		printf("No devices on USB bus\n");
		printf("Quitting application...\n");
		return 1;
	}

	GoOn = FALSE;
	while (!GoOn)
	{
		printf("Choose current device from above list (-1 To Quit Application):\n");
		if (scanf("%d",&ChoosenDeviceID) == EOF)
		{
			GoOn = FALSE;
		}
		else
		{
			// check values
			if (ChoosenDeviceID < 0)
			{
				printf("Quitting application...\n");
				return 1;
			}
			else if (ChoosenDeviceID > AttachedDeviceNumber)
			{
				printf("Wrong selection\n");
				GoOn = FALSE;
			}
			else
			{
				GoOn = TRUE;
			}
		}
	}

	RetVal = GBMSAPI_SetCurrentDevice(AttachedDeviceList[ChoosenDeviceID - 1].DeviceID,
				AttachedDeviceList[ChoosenDeviceID - 1].DeviceSerialNumber);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_SetCurrentDevice returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	RetVal = GBMSAPI_GetCurrentDevice(
		&CurrDevID,
		CurrDevSerNum
	);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetCurrentDevice returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("CurrDevID = %d\n",CurrDevID);
	printf("CurrDevSerNum = %s\n\n",CurrDevSerNum);

	GBMSAPI_Sound (12 , 2, 1);

	RetVal = GBMSAPI_GetScannableTypes(&Appoggio);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetScannableTypes returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("Scannable Device mask = %d\n\n",Appoggio);

	RetVal = GBMSAPI_GetDeviceFeatures(&Appoggio);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetDeviceFeatures returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("Device Features mask = %d\n\n",Appoggio);

	RetVal = GBMSAPI_GetSupportedScanOptions(&Appoggio);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetSupportedScanOptions returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("Supported Scan Options mask = %d\n\n",Appoggio);

	RetVal = GBMSAPI_GetOptionalExternalEquipment(&Appoggio);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetOptionalExternalEquipment returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("OptionalExternalEquipment mask = %d\n\n",Appoggio);

	RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_SBT_FLAT_SINGLE_FINGER,&Appoggio);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetAvailableImageInfo returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("AvailableImageInfo mask = %d\n\n",Appoggio);

	RetVal = GBMSAPI_GetImageSize(GBMSAPI_SBT_FLAT_SINGLE_FINGER,TRUE,TRUE,&Appoggio,&Appoggio1);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetImageSize returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("Image size for flat is: x = %d, y = %d\n\n",Appoggio,Appoggio1);

	RetVal = GBMSAPI_GetMaxImageSize(&Appoggio,&Appoggio1);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_GetMaxImageSize returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	printf("Max Image size is: x = %d, y = %d\n\n",Appoggio,Appoggio1);

	RetVal = GBMSAPI_SetSelectImageTimeout(2000);

	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		printf("GBMSAPI_SetSelectImageTimeout returned = %d\n",RetVal);
		printf("Quitting application...\n");
		return 1;
	}

	// GO WITH ACQUISITION
	GoOn = GB_TRUE;

	while (GoOn == GB_TRUE)
	{
		AcqStopped = GB_FALSE;
		FrameCounter = 0;

		RetVal = GBMSAPI_StartAcquisition(GBMSAPI_SO_FLAT_RIGHT_THUMB,
				GBMSAPI_AO_AUTOCAPTURE,
				MyAcquisitionEventsManagerCallback,NULL,
				0,0,0);

		if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
		{
			GBMSAPI_StopAcquisition();
			printf("GBMSAPI_StartAcquisition returned = %d\n",RetVal);
			printf("Quitting application...\n");
			return 1;
		}

		while (AcqStopped == GB_FALSE)
		{
			// do nothing
			if (FrameCounter > 80)
				GBMSAPI_StopAcquisition();
			usleep(100000);
		}

		printf("Acquisition ended\n\n");

		usleep(100000);

		printf("Would you like to acquire another finger (Y,N)?\n");
		getchar();
		char UserResponse;

		if (scanf("%c",&UserResponse) == EOF)
		{
			printf("EOF from scanf\n");
			GoOn = FALSE;
		}
		else
		{
			// check values
			if ((UserResponse) != 'y' && (UserResponse) != 'Y')
			{
				printf("User response was (not in ASCII) %d\n",UserResponse);
				GoOn = FALSE;
			}
			else
			{
				printf("New acquisition\n");
				GoOn = TRUE;
			}
		}
	}

	// in case it was not correctly closed
	GBMSAPI_UnloadLibrary();

	printf("Quitting application...\n");
	return 0;
}

