﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using GBFINIMG_NET_WRAPPER;
using AN2K_NET_WRAPPER;

namespace GbFinImgExample_Cs
{
    public partial class Form1 : Form
    {
        bool SlowProcessOptionsEnabled;
        public uint ObjToDetect;

        public class ColorLegenda
        {
            public static Color IndexColor = Color.Green;
            public static Color MiddleColor = Color.Red;
            public static Color RingColor = Color.Blue;
            public static Color LittleColor = Color.Orange;
            public static Color ThumbColor = Color.Olive;
        }

        private int [] BBL; private int [] BBR;
        private int[] BBT; private int[] BBB;
        private Color[] FingerColorForLegenda;
        private int NumberOfSegmentsDetected;
        private bool ReadyForPaint;
        private bool ObjAllows180DegRotationCheck;
        private bool ObjAllowsIncompleteSlapDetection;
        private bool ObjAllowsMissingFingersMasking;

        public class ObjectTypes
        {
            public const String Left_4_Str = "LEFT 4";
            public const String Left_2_Str = "LEFT 2";
            public const String Right_4_Str = "RIGHT 4";
            public const String Right_2_Str = "RIGHT 2";
            public const String Thmb_2_Str = "THUMBS 2";
            public const String Single_Str = "SINGLE";
            public const String FlatSingle_Str = "FLAT SINGLE";
        }

        public Form1()
        {
            InitializeComponent();
            SlowProcessOptionsEnabled = true;

            NumberOfSegmentsDetected = 0;

            ObjAllows180DegRotationCheck = false;
            ObjAllowsIncompleteSlapDetection = false;
            ObjAllowsMissingFingersMasking = false;

            ObjTypesCombo.Items.Clear();
            ObjTypesCombo.Items.Add(ObjectTypes.Left_2_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.Left_4_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.Right_2_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.Right_4_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.Thmb_2_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.Single_Str);
            ObjTypesCombo.Items.Add(ObjectTypes.FlatSingle_Str);

            ObjTypesCombo.SelectedIndex = 0;

            this.IndexCheckBox.BackColor = ColorLegenda.IndexColor;
            this.MiddleCheckBox.BackColor = ColorLegenda.MiddleColor;
            this.RingCheckBox.BackColor = ColorLegenda.RingColor;
            this.LittleCheckBox.BackColor = ColorLegenda.LittleColor;

            this.ImageProcessGroupBox.Enabled = false;
            this.ProcessButton.Enabled = false;
            ObjTypesCombo.Enabled = false;
            ReadyForPaint = false;

            BBL = new int[4];
            BBR = new int[4];
            BBT = new int[4];
            BBB = new int[4];

            FingerColorForLegenda = new Color[4];
        }

        private void EnableSlowProcessOptions(bool Enable)
        {
            this.RotationCheckBox.Enabled = Enable;
            this.HaloCheckBox.Enabled = Enable;
            this.PalmQualCheckBox.Enabled = Enable;
            this.DeleteFingersCheckBox.Enabled = Enable;
            this.FastProcessWithSegmentCheckBox.Enabled = Enable;
            if (Enable)
            {
                if (this.FastProcessWithSegmentCheckBox.Checked)
                {
                    this.Check180RotationCheckBox.Enabled = false;
                    this.IncompleteSlapDataGenerationCheckBox.Enabled = false;
                }
                else
                {
                    this.Check180RotationCheckBox.Enabled = ObjAllows180DegRotationCheck;
                    this.IncompleteSlapDataGenerationCheckBox.Enabled = ObjAllowsIncompleteSlapDetection;
                }
            }
            else
            {
                this.Check180RotationCheckBox.Enabled = false;
                this.IncompleteSlapDataGenerationCheckBox.Enabled = false;
            }
            SlowProcessOptionsEnabled = Enable;
        }

        private void FastProcessCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (FastProcessCheckBox.Checked)
            {
                this.EnableSlowProcessOptions(false);
            }
            else
            {
                this.EnableSlowProcessOptions(true);
            }
        }

        private void FastProcessWithSegmentCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.FastProcessWithSegmentCheckBox.Checked)
            {
                this.Check180RotationCheckBox.Enabled = false;
                this.IncompleteSlapDataGenerationCheckBox.Enabled = false;
            }
            else
            {
                this.Check180RotationCheckBox.Enabled = ObjAllows180DegRotationCheck;
                this.IncompleteSlapDataGenerationCheckBox.Enabled = ObjAllowsIncompleteSlapDetection;
            }
        }

        uint GetProcessOptions()
        {
            uint RetVal = 0;
            if (SlowProcessOptionsEnabled)
            {
                if (this.RotationCheckBox.Enabled && this.RotationCheckBox.Checked)
                {
                    RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_SEGMENT_VERTICAL_ROTATION;
                }
                if (this.HaloCheckBox.Enabled && this.HaloCheckBox.Checked)
                {
                    RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_HALO_LATENT_ELIMINATION;
                }
                if (this.PalmQualCheckBox.Enabled && this.PalmQualCheckBox.Checked)
                {
                    RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION;
                }
                if (this.DeleteFingersCheckBox.Enabled && this.DeleteFingersCheckBox.Checked)
                {
                    RetVal |=
                        NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS;
                }
                if (this.FastProcessWithSegmentCheckBox.Checked)
                {
                    RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA;
                }
                if (this.IncompleteSlapDataGenerationCheckBox.Enabled && this.IncompleteSlapDataGenerationCheckBox.Checked)
                {
                    RetVal |=
                        NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION;
                    // NW_GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK must be used 
                    RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK;
                }
            }
            else
            {
                if (this.FastProcessCheckBox.Enabled && this.FastProcessCheckBox.Checked)
                {
                    RetVal |=
                        NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY;
                }
            }
            if (this.ImageRes1000DpiCheckBox.Checked)
            {
                RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;
            }
            if (this.LeftRightCheckBox.Checked)
            {
                RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK;
            }
			if (this.RefineDryFingerCheckBox.Enabled && this.RefineDryFingerCheckBox.Checked)
			{
				RetVal |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE;
			}
			if (this.RefineFpContrastCleanBgCheckBox.Enabled && this.RefineFpContrastCleanBgCheckBox.Checked)
			{
				RetVal |= 
					NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND;
			}
            return RetVal;
        }

        void EliminateStrideDistorsion(byte[] ImgBuffer,
            int bmpW, int bmpS, out int newWidth)
        {
            newWidth = (bmpW / 4) * 4;
            if (bmpW > bmpS)
                return;

            int IndexS = 0, IndexW = 0;
            while ((IndexS + bmpS) < ImgBuffer.Length)
            {
                Array.Copy(ImgBuffer, IndexS, ImgBuffer, IndexW, newWidth);
                IndexS += bmpS;
                IndexW += newWidth;
            }
        }

        private void LoadImageButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog FD = new OpenFileDialog();

            FD.ShowDialog();

            String fn = FD.FileName;
            if (fn != null && fn != "")
            {
                Bitmap x = new Bitmap(fn);

                if ((x != null)  && (x.PixelFormat == System.Drawing.Imaging.PixelFormat.Format8bppIndexed))
                {
                    Rectangle rect = new Rectangle(0, 0, x.Width, x.Height);
                    System.Drawing.Imaging.BitmapData bmpData =
                        x.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                        x.PixelFormat);

                    this.UserMessagesListBox.Items.Add("bitmap width: " + x.Width +
                        "bitmap stride: " + bmpData.Stride);

                    if (bmpData.Stride != x.Width)
                    {
                        IntPtr ptr = bmpData.Scan0;

                        int bytes = bmpData.Stride * x.Height;
                        byte[] rgbValues = new byte[bytes];

                        // Copy the RGB values into the array.
                        Marshal.Copy(ptr, rgbValues, 0, bytes);

                        int newW = 0;

                        EliminateStrideDistorsion(rgbValues, x.Width, bmpData.Stride, out newW);

                        Marshal.Copy(rgbValues, 0, ptr, x.Height * newW);

                        // Unlock the bits.
                        x.UnlockBits(bmpData);

                        x = new Bitmap(newW, x.Height, newW, System.Drawing.Imaging.PixelFormat.Format8bppIndexed, ptr);
                    }
                    else
                    {
                        // Unlock the bits.
                        x.UnlockBits(bmpData);
                    }             
                    System.Drawing.Imaging.ColorPalette pal = x.Palette;
                    for (int i = 0; i < pal.Entries.Length; i++)
                    {
                        pal.Entries[i] = Color.FromArgb(255, i, i, i);
                    }
                    x.Palette = pal;

                    this.UserMessagesListBox.Items.Add("bitmap width: " + x.Width);

                    this.FingerprintPictureBox.Image = x;
                    this.FingerprintPictureBox.SizeMode = PictureBoxSizeMode.StretchImage;

                    this.ImageProcessGroupBox.Enabled = true;

                    this.Text = FD.SafeFileName;

                    ReadyForPaint = false;
                    this.ProcessButton.Enabled = true;
                    ObjTypesCombo.Enabled = true;
                }
            }
        }

        private void ProcessButton_Click(object sender, EventArgs e)
        {
            // Load image from bmp
            Bitmap bmp = (Bitmap)this.FingerprintPictureBox.Image;

            this.UserMessagesListBox.Items.Clear();

            ReadyForPaint = false;
            this.FingerprintPictureBox.Image = bmp;
            this.Update();
            if (bmp.PixelFormat == System.Drawing.Imaging.PixelFormat.Format8bppIndexed)
            {
                this.UserMessagesListBox.Items.Add("After bmp.PixelFormat");
                // Lock the bitmap's bits.
                Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                System.Drawing.Imaging.BitmapData bmpData =
                    bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                    bmp.PixelFormat);

                // Get the address of the first line.
                IntPtr ptr = bmpData.Scan0;

                // Declare an array to hold the bytes of the bitmap.
                int bytes = bmpData.Stride * bmp.Height;
                byte[] rgbValues = new byte[bytes];

                // Copy the RGB values into the array.
                System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);

                // Unlock the bits.
                bmp.UnlockBits(bmpData);

                NW_GBFINIMG_InputData ID = new NW_GBFINIMG_InputData(
                    rgbValues, (uint)(bmp.Width), (uint)(bmp.Height),
                    ObjToDetect,
                    null, this.GetProcessOptions(), (uint)bmp.Width, (uint)bmp.Height);

                this.UserMessagesListBox.Items.Add("After NW_GBFINIMG_InputData");
                

                if (ID != null)
                {
                    try
                    {
                        NW_GBFINIMG_ProcessImage PI = new NW_GBFINIMG_ProcessImage(ID);

                        this.UserMessagesListBox.Items.Add("After new NW_GBFINIMG_ProcessImage");

                        // get information on missing fingers
                        if (this.MissingFingersGroupBox.Enabled)
                        {
                            Byte Mask = 0;
                            if (this.IndexCheckBox.Checked)
                            {
                                Mask |= ((Byte)(NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX));
                            }
                            if (this.MiddleCheckBox.Checked)
                            {
                                Mask |= ((Byte)(NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE));
                            }
                            if (this.RingCheckBox.Checked)
                            {
                                Mask |= ((Byte)(NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_RING));
                            }
                            if (this.LittleCheckBox.Checked)
                            {
                                Mask |= ((Byte)(NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE));
                            }

                            if (Mask != 0)
                            {
                                PI.NW_GBFINIMG_Slap4_MissingFingers(Mask);
                            }
                        }

                        NW_GBFINIMG_OutputData OD = null;

                        // get time
                        DateTime startTime = DateTime.Now;
                        if (this.FastProcessCheckBox.Enabled && this.FastProcessCheckBox.Checked)
                        {
                            OD = PI.OutputAfterFastProcess;
                        }
                        else
                        {
                            OD = PI.OutputAfterProcess;
                        }

                        this.UserMessagesListBox.Items.Add("After OutputAfterProcess");

                        DateTime stopTime = DateTime.Now;
                        TimeSpan duration = stopTime - startTime;
                        this.UserMessagesListBox.Items.Add("No Diagnostic: " +
                            ((OD.Diag.NW_GBFINIMG_NET_WRAPPER_NO_DIAG) ? "true" : "false").ToString());
                        this.UserMessagesListBox.Items.Add("Ambiguous Slap: " +
                            ((OD.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP) ? "true" : "false").ToString());
                        this.UserMessagesListBox.Items.Add("Incomplete Slap: " +
                            ((OD.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP) ? "true" : "false").ToString());
                        this.UserMessagesListBox.Items.Add("Left-Right mixup: " +
                            ((OD.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX) ? "true" : "false").ToString());
                        this.UserMessagesListBox.Items.Add("");
                        this.UserMessagesListBox.Items.Add("Total duration (in ms): " + duration.TotalMilliseconds);
                        this.UserMessagesListBox.Items.Add("");

                        this.UserMessagesListBox.Items.Add("Fingerprint Counter: " + PI.NW_GBFINIMG_GetFingerprintCounter());
                        this.UserMessagesListBox.Items.Add("");

                        if (OD.Segment_List != null)
                        {
                            this.UserMessagesListBox.Items.Add("Detected segments number: " + OD.Segment_List.Count);
                            this.UserMessagesListBox.Items.Add("");
                            if (!(this.FastProcessCheckBox.Enabled && this.FastProcessCheckBox.Checked))
                            {
                                for (int i = 0; i < OD.Segment_List.Count; i++)
                                {
                                    String fingID = OD.Segment_List[i].FingerID;

                                    BBL[i] = OD.Segment_List[i].SegBoundingBoxL;
                                    BBR[i] = OD.Segment_List[i].SegBoundingBoxR;
                                    BBT[i] = OD.Segment_List[i].SegBoundingBoxT;
                                    BBB[i] = OD.Segment_List[i].SegBoundingBoxB;
                                    if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.IndexColor;
                                        this.UserMessagesListBox.Items.Add("LEFT_INDEX");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.LittleColor;
                                        this.UserMessagesListBox.Items.Add("LEFT_LITTLE");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.MiddleColor;
                                        this.UserMessagesListBox.Items.Add("LEFT_MIDDLE");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.RingColor;
                                        this.UserMessagesListBox.Items.Add("LEFT_RING");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.ThumbColor;
                                        this.UserMessagesListBox.Items.Add("LEFT_THUMB");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.IndexColor;
                                        this.UserMessagesListBox.Items.Add("RIGHT_INDEX");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.LittleColor;
                                        this.UserMessagesListBox.Items.Add("RIGHT_LITTLE");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.MiddleColor;
                                        this.UserMessagesListBox.Items.Add("RIGHT_MIDDLE");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.RingColor;
                                        this.UserMessagesListBox.Items.Add("RIGHT_RING");
                                    }
                                    else if (fingID == AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB)
                                    {
                                        this.FingerColorForLegenda[i] = ColorLegenda.ThumbColor;
                                        this.UserMessagesListBox.Items.Add("RIGHT_THUMB");
                                    }
                                    else
                                    {
                                        this.FingerColorForLegenda[i] = Color.Gray;
                                    }


                                    this.UserMessagesListBox.Items.Add("Bounding Box: L " +
                                        BBL[i] +
                                        ", R " + BBR[i] +
                                        ", T " + BBT[i] +
                                        ", B " + BBB[i]);
                                    this.UserMessagesListBox.Items.Add("Inclination " + OD.Segment_List[i].Inclination);
                                    this.UserMessagesListBox.Items.Add("PatternCompletness " + OD.Segment_List[i].PatternCompletness);
                                    this.UserMessagesListBox.Items.Add("PatternValidity " + OD.Segment_List[i].PatternValidity);
                                    this.UserMessagesListBox.Items.Add("SegmQuality " + OD.Segment_List[i].SegQuality);
                                    this.UserMessagesListBox.Items.Add("Size " + OD.Segment_List[i].Size);
                                    this.UserMessagesListBox.Items.Add("");
                                }

                                ReadyForPaint = true;
                                NumberOfSegmentsDetected = (int)OD.Segment_List.Count;
                                this.FingerprintPictureBox.Image = bmp;
                                this.Update();
                            }
                        }
                        else
                        {
                            this.UserMessagesListBox.Items.Add("No segments detected (look carefully at diagnostics)");
                        }

                        if (this.Check180RotationCheckBox.Enabled && this.Check180RotationCheckBox.Checked)
                        {
                            int ProbabilityOf180DegRotation;
                            byte PatternDiagnostic;
                            PI.NW_GBFINIMG_CheckFingerRotation180Degree(out ProbabilityOf180DegRotation, out PatternDiagnostic);
                            this.UserMessagesListBox.Items.Add("");
                            this.UserMessagesListBox.Items.Add("");
                            this.UserMessagesListBox.Items.Add("Probability Of 180 Degrees Rotation = " + ProbabilityOf180DegRotation);
                            this.UserMessagesListBox.Items.Add("Pattern Diagnostic = " + PatternDiagnostic);
                        }
                    }
                    catch (Exception ex)
                    {
                        this.UserMessagesListBox.Items.Add("ProcessButton_Click exception: " + ex.Message );
                    }
                }
            }
        }

        private void ObjTypesCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            String SelIt = (String)(ObjTypesCombo.SelectedItem);
            ObjAllows180DegRotationCheck = false;

            switch (SelIt)
            {
                case (ObjectTypes.Left_2_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = false;
                        ObjAllowsMissingFingersMasking = false;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2;
                        ObjAllows180DegRotationCheck = true;
                        ObjAllowsIncompleteSlapDetection = false;
                        break;
                    }
                case (ObjectTypes.Left_4_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = true;
                        ObjAllowsMissingFingersMasking = true;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4;
                        ObjAllowsIncompleteSlapDetection = true;
                        break;
                    }
                case (ObjectTypes.Right_2_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = false;
                        ObjAllowsMissingFingersMasking = false;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2;
                        ObjAllows180DegRotationCheck = true;
                        ObjAllowsIncompleteSlapDetection = false;
                        break;
                    }
                case (ObjectTypes.Right_4_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = true;
                        ObjAllowsMissingFingersMasking = true;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
                        ObjAllowsIncompleteSlapDetection = true;
                        break;
                    }
                case (ObjectTypes.Single_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = false;
                        ObjAllowsMissingFingersMasking = false;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
                        ObjAllows180DegRotationCheck = true;
                        ObjAllowsIncompleteSlapDetection = false;
                        break;
                    }
                case (ObjectTypes.Thmb_2_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = false;
                        ObjAllowsMissingFingersMasking = false;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;
                        ObjAllows180DegRotationCheck = true;
                        ObjAllowsIncompleteSlapDetection = true;
                        break;
                    }
                case (ObjectTypes.FlatSingle_Str):
                    {
                        this.MissingFingersGroupBox.Enabled = false;
                        ObjAllowsMissingFingersMasking = false;
                        ObjToDetect = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER_FLAT;
                        ObjAllows180DegRotationCheck = true;
                        ObjAllowsIncompleteSlapDetection = false;
                        break;
                    }
            }
            this.FastProcessWithSegmentCheckBox_CheckedChanged(null, null);
        }

        private void FingerprintPictureBox_Paint(object sender, PaintEventArgs e)
        {
            if (ReadyForPaint)
            {
                Bitmap bmp = (Bitmap)this.FingerprintPictureBox.Image;
                // Adjust picture box to frame size
                float XFactor = ((float)(this.FingerprintPictureBox.Width) / bmp.Width);
                float YFactor = ((float)(this.FingerprintPictureBox.Height) / bmp.Height);

                for (int i = 0; i < this.NumberOfSegmentsDetected; i++)
                {

                    float SegBoundingBoxPosX = BBL[i] * XFactor;
                    float SegBoundingBoxPosY = BBT[i] * YFactor;
                    float SegBoundingBoxSizeX = (BBR[i] - BBL[i]) *
                        XFactor;
                    float SegBoundingBoxSizeY = (BBB[i] - BBT[i]) *
                        YFactor;


                    System.Drawing.Pen ClipPen = new Pen(this.FingerColorForLegenda[i], (float)3.0);
                    e.Graphics.DrawRectangle(
                        ClipPen,
                        SegBoundingBoxPosX,
                        SegBoundingBoxPosY,
                        SegBoundingBoxSizeX,
                        SegBoundingBoxSizeY
                    );
                }
            }
        }

        private void IncompleteSlapDataGenerationCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.IncompleteSlapDataGenerationCheckBox.Checked)
            {
                this.MissingFingersGroupBox.Enabled = false;
            }
            else
            {
                this.MissingFingersGroupBox.Enabled = ObjAllowsMissingFingersMasking;
            }
        }
    }
}
