using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using GBMSGUI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;

namespace GBMSDemo
{
    public partial class ConfigurationForm : Form
    {
        private DemoForm DemoFormRef;   // reference to main form

        public ConfigurationForm(DemoForm DemoFormRef)
        {
            // save reference to main form
            this.DemoFormRef = DemoFormRef;

            InitializeComponent();
        }

        // Standard used  for image sizes
        public class ScanModes
        {
            public const int FullHiRes = 0;
            public const int FullLowRes = 1;
            public const int RollIQS = 2;
            public const int RollGA = 3;
        }

        private void ConfigurationForm_Load(object sender, EventArgs e)
        {
            // acquisition options
            chkAutocapture.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.Autocapture);
            chkBlockAutocapture.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.BlockAutocapture);
            chkFullResPreview.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview);
            chkRotateFinger.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.RotateFinger);
            chkPalmPrintQualityCalculation.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation);
            if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.NoRollPreview))
                optNoRollPreview.Checked = true;
            else if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.RollPreviewManualStop))
                optRollPreviewManual.Checked = true;
            else if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.RollPreviewType))
                optRollPreviewCenter.Checked = true;
            else
                optRollPreviewSide.Checked = true;
            chkSound.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound);
            chkHighSpeedPreview.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.HighSpeedPreview);
            chkDetectInvalidPattern.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInvalidPattern);
            chkDetectIncompletePattern.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectIncompletePattern);
            chkDetectInclination.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInclination);
            chkAutoClearOutsideRoll.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.AutoClearOutsideRoll);
            chkSegmentsQualityEval.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.SegmentsQualityEval);
            chkAutoIncompleteSlapSegm.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.AutoIncompleteSlapSegm);

            // session options
            chkSequenceCheck.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SessionOptions, GBMSGUI.SessionOption.SequenceCheck);
            chkAskUnavailabilityReason.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason);
            chkDryFingerImageEnhancement.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SessionOptions, GBMSGUI.SessionOption.DryFingerImageEnhancement);

            // settings
            chkBlockAutocaptureContrast.Checked = DemoFormRef.DemoConfig.BlockAutocaptureContrast;
            chkBlockAutocaptureContrast.Enabled = chkBlockAutocapture.Checked;

            // image size
            // fill Standard's combobox
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.Custom, "Custom");
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.ANSI_NIST_ITL_1_2000, "ANSI/NIST-ITL 1-2000");
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.ANSI_NIST_ITL_1_2000_INTERPOL, "ANSI/NIST-ITL 1-2000 INTERPOL");
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.ISO_IEC_FCD_19794_4, "ISO-IEC FCD 19794-4");
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.ANSI_NIST_ITL_1_2007, "ANSI/NIST-ITL 1-2007");
            cmbStandard.Items.Insert(DemoForm.ImageSizeStandards.GA_CHINA, "GA (China)");
            // select current
            cmbStandard.SelectedIndex = DemoFormRef.DemoConfig.ImageSizeStandard;
            // show values
            ShowImageSizeValues();

            // IAFIS quality algorithm
            // fill combobox
            // subtract 1 to use ID as index...
            cmbQualityAlgorithm.Items.Insert(GBMSGUI.QualityAlgorithms.NFIQAlgorithm-1, "NIST Fingerprint Image Quality (NFIQ)");
            cmbQualityAlgorithm.Items.Insert(GBMSGUI.QualityAlgorithms.GBAlgorithm-1, "Green Bit proprietary");
            // select current
            cmbQualityAlgorithm.SelectedIndex = DemoFormRef.DemoConfig.IAFIsQualityAlgorithm - 1;

            // Image compression
            // 500 dpi
            if (DemoFormRef.DemoConfig.ImageCompression500 == DemoForm.ImageCompressions.WSQ)
                optWSQCompression500.Checked = true;
            else
                optJpeg2000Compression500.Checked = true;
            // 1000 dpi
            if (DemoFormRef.DemoConfig.ImageCompression1000 == DemoForm.ImageCompressions.WSQ)
                optWSQCompression1000.Checked = true;
            else
                optJpeg2000Compression1000.Checked = true;

            // compression rates
            // 500 dpi
            txtWSQBitRate500.Text = DemoFormRef.DemoConfig.WQSBitRate500.ToString();
            txtJPEGRate500.Text = DemoFormRef.DemoConfig.JPEGRate500.ToString();
            // 1000 dpi
            txtWSQBitRate1000.Text = DemoFormRef.DemoConfig.WQSBitRate1000.ToString();
            txtJPEGRate1000.Text = DemoFormRef.DemoConfig.JPEGRate1000.ToString();

            // fill devices list for frame rate
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26));
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32));
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I));
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84));
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C));
            lstDevice.Items.Add(DemoForm.GetDeviceName(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517));
            lstDevice.SelectedIndex = 0;

            // Roll area size
            if (DemoFormRef.DemoConfig.RollAreaSize == GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS)
                optRollAreaIQS.Checked = true;
            else
                optRollAreaGA.Checked = true;

            // Language
            cmbLanguage.Items.Insert(DemoForm.GUILanguages.English, "English");
            cmbLanguage.Items.Insert(DemoForm.GUILanguages.Italian, "Italian");
            cmbLanguage.Items.Insert(DemoForm.GUILanguages.Chinese, "Chinese");
            // select current
            cmbLanguage.SelectedIndex = DemoFormRef.DemoConfig.GUILanguage;

            // Finger contact evaluation mode
            // subtract 1 to use ID as index...
            cmbFingerContactEvaluationMode.Items.Insert(GBMSGUI.FingerContactEvaluationMode.DryWetWarning-1, "Dry/Wet warning");
            cmbFingerContactEvaluationMode.Items.Insert(GBMSGUI.FingerContactEvaluationMode.Contrast-1, "Contrast");
            // select current
            cmbFingerContactEvaluationMode.SelectedIndex = DemoFormRef.DemoConfig.FingerContactEvaluationMode-1;
        }

        private void ShowImageSizeValues()
        {
            // disable all (except flat finger)
            txtUpperPalmWidth.Enabled = false;
            txtUpperPalmHeight.Enabled = false;
            txtLowerPalmWidth.Enabled = false;
            txtLowerPalmHeight.Enabled = false;
            txtWritersPalmWidth.Enabled = false;
            txtWritersPalmHeight.Enabled = false;
            txtFourFingersWidth.Enabled = false;
            txtFourFingersHeight.Enabled = false;
            txtTwoThumbsWidth.Enabled = false;
            txtTwoThumbsHeight.Enabled = false;
            txtFlatThumbWidth.Enabled = false;
            txtFlatThumbHeight.Enabled = false;
            txtRolledThumbWidth.Enabled = false;
            txtRolledThumbHeight.Enabled = false;
            txtRolledIndexWidth.Enabled = false;
            txtRolledIndexHeight.Enabled = false;
            txtRolledMiddleWidth.Enabled = false;
            txtRolledMiddleHeight.Enabled = false;
            txtRolledRingWidth.Enabled = false;
            txtRolledRingHeight.Enabled = false;
            txtRolledLittleWidth.Enabled = false;
            txtRolledLittleHeight.Enabled = false;

            txtUpperPalmWidth.Text = DemoFormRef.DemoConfig.ImageSize.UpperPalmWidth.ToString();
            txtUpperPalmHeight.Text = DemoFormRef.DemoConfig.ImageSize.UpperPalmHeight.ToString();
            txtLowerPalmWidth.Text = DemoFormRef.DemoConfig.ImageSize.LowerPalmWidth.ToString();
            txtLowerPalmHeight.Text = DemoFormRef.DemoConfig.ImageSize.LowerPalmHeight.ToString();
            txtWritersPalmWidth.Text = DemoFormRef.DemoConfig.ImageSize.WritersPalmWidth.ToString();
            txtWritersPalmHeight.Text = DemoFormRef.DemoConfig.ImageSize.WritersPalmHeight.ToString();
            txtFourFingersWidth.Text = DemoFormRef.DemoConfig.ImageSize.FourFingersWidth.ToString();
            txtFourFingersHeight.Text = DemoFormRef.DemoConfig.ImageSize.FourFingersHeight.ToString();
            txtTwoThumbsWidth.Text = DemoFormRef.DemoConfig.ImageSize.TwoThumbsWidth.ToString();
            txtTwoThumbsHeight.Text = DemoFormRef.DemoConfig.ImageSize.TwoThumbsHeight.ToString();
            txtFlatThumbWidth.Text = DemoFormRef.DemoConfig.ImageSize.FlatThumbWidth.ToString();
            txtFlatThumbHeight.Text = DemoFormRef.DemoConfig.ImageSize.FlatThumbHeight.ToString();
            txtFlatFingerWidth.Text = DemoFormRef.DemoConfig.ImageSize.FlatFingerWidth.ToString();
            txtFlatFingerHeight.Text = DemoFormRef.DemoConfig.ImageSize.FlatFingerHeight.ToString();
            txtRolledThumbWidth.Text = DemoFormRef.DemoConfig.ImageSize.RolledThumbWidth.ToString();
            txtRolledThumbHeight.Text = DemoFormRef.DemoConfig.ImageSize.RolledThumbHeight.ToString();
            txtRolledIndexWidth.Text = DemoFormRef.DemoConfig.ImageSize.RolledIndexWidth.ToString();
            txtRolledIndexHeight.Text = DemoFormRef.DemoConfig.ImageSize.RolledIndexHeight.ToString();
            txtRolledMiddleWidth.Text = DemoFormRef.DemoConfig.ImageSize.RolledMiddleWidth.ToString();
            txtRolledMiddleHeight.Text = DemoFormRef.DemoConfig.ImageSize.RolledMiddleHeight.ToString();
            txtRolledRingWidth.Text = DemoFormRef.DemoConfig.ImageSize.RolledRingWidth.ToString();
            txtRolledRingHeight.Text = DemoFormRef.DemoConfig.ImageSize.RolledRingHeight.ToString();
            txtRolledLittleWidth.Text = DemoFormRef.DemoConfig.ImageSize.RolledLittleWidth.ToString();
            txtRolledLittleHeight.Text = DemoFormRef.DemoConfig.ImageSize.RolledLittleHeight.ToString();

            // enable only free values
            if ((DemoFormRef.DemoConfig.ImageSizeStandard == DemoForm.ImageSizeStandards.ANSI_NIST_ITL_1_2000)
                || (DemoFormRef.DemoConfig.ImageSizeStandard == DemoForm.ImageSizeStandards.ANSI_NIST_ITL_1_2000_INTERPOL))
            {
                txtTwoThumbsWidth.Enabled = true;
                txtTwoThumbsHeight.Enabled = true;
            }
            else if (DemoFormRef.DemoConfig.ImageSizeStandard == DemoForm.ImageSizeStandards.ISO_IEC_FCD_19794_4)
            {
                txtFlatThumbWidth.Enabled = true;
                txtFlatThumbHeight.Enabled = true;
            }
            else if (DemoFormRef.DemoConfig.ImageSizeStandard == DemoForm.ImageSizeStandards.Custom)
            {
                // enable all
                txtUpperPalmWidth.Enabled = true;
                txtUpperPalmHeight.Enabled = true;
                txtLowerPalmWidth.Enabled = true;
                txtLowerPalmHeight.Enabled = true;
                txtWritersPalmWidth.Enabled = true;
                txtWritersPalmHeight.Enabled = true;
                txtFourFingersWidth.Enabled = true;
                txtFourFingersHeight.Enabled = true;
                txtTwoThumbsWidth.Enabled = true;
                txtTwoThumbsHeight.Enabled = true;
                txtFlatThumbWidth.Enabled = true;
                txtFlatThumbHeight.Enabled = true;
                txtRolledThumbWidth.Enabled = true;
                txtRolledThumbHeight.Enabled = true;
                txtRolledIndexWidth.Enabled = true;
                txtRolledIndexHeight.Enabled = true;
                txtRolledMiddleWidth.Enabled = true;
                txtRolledMiddleHeight.Enabled = true;
                txtRolledRingWidth.Enabled = true;
                txtRolledRingHeight.Enabled = true;
                txtRolledLittleWidth.Enabled = true;
                txtRolledLittleHeight.Enabled = true;
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (chkAutocapture.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.Autocapture;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.Autocapture;

            if (chkBlockAutocapture.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.BlockAutocapture;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.BlockAutocapture;

            if (chkFullResPreview.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.FullResPreview;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.FullResPreview;

            if (chkRotateFinger.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.RotateFinger;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.RotateFinger;

            if (chkPalmPrintQualityCalculation.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation;

            if (optNoRollPreview.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.NoRollPreview;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.NoRollPreview;

            if (optRollPreviewManual.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.RollPreviewManualStop;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.RollPreviewManualStop;

            if (optRollPreviewCenter.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.RollPreviewType;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.RollPreviewType;

            if (chkSound.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.Sound;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.Sound;

            if (chkHighSpeedPreview.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.HighSpeedPreview;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.HighSpeedPreview;

            if (chkSequenceCheck.Checked)
                DemoFormRef.DemoConfig.SessionOptions |= GBMSGUI.SessionOption.SequenceCheck;
            else
                DemoFormRef.DemoConfig.SessionOptions &= ~GBMSGUI.SessionOption.SequenceCheck;
            
            if (chkAskUnavailabilityReason.Checked)
                DemoFormRef.DemoConfig.SessionOptions |= GBMSGUI.SessionOption.AskUnavailabilityReason;
            else
                DemoFormRef.DemoConfig.SessionOptions &= ~GBMSGUI.SessionOption.AskUnavailabilityReason;

            if (chkDetectInvalidPattern.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectInvalidPattern;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.DetectInvalidPattern;

            if (chkDetectIncompletePattern.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectIncompletePattern;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.DetectIncompletePattern;

            if (chkDetectInclination.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectInclination;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.DetectInclination;

            if (chkAutoClearOutsideRoll.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.AutoClearOutsideRoll;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.AutoClearOutsideRoll;

            if (chkSegmentsQualityEval.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.SegmentsQualityEval;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.SegmentsQualityEval;

            if (chkAutoIncompleteSlapSegm.Checked)
                DemoFormRef.DemoConfig.AcquisitionOptions |= GBMSGUI.AcquisitionOption.AutoIncompleteSlapSegm;
            else
                DemoFormRef.DemoConfig.AcquisitionOptions &= ~GBMSGUI.AcquisitionOption.AutoIncompleteSlapSegm;

            if (chkDryFingerImageEnhancement.Checked)
                DemoFormRef.DemoConfig.SessionOptions |= GBMSGUI.SessionOption.DryFingerImageEnhancement;
            else
                DemoFormRef.DemoConfig.SessionOptions &= ~GBMSGUI.SessionOption.DryFingerImageEnhancement;

            // settings
            DemoFormRef.DemoConfig.BlockAutocaptureContrast = chkBlockAutocaptureContrast.Checked;

            // image size
            if (!SaveImageSizeValues())
                return;

            // IAFIS quality
            DemoFormRef.DemoConfig.IAFIsQualityAlgorithm = cmbQualityAlgorithm.SelectedIndex + 1;

            // Image compression
            // 500 dpi
            if (optWSQCompression500.Checked)
                DemoFormRef.DemoConfig.ImageCompression500 = DemoForm.ImageCompressions.WSQ;
            else
                DemoFormRef.DemoConfig.ImageCompression500 = DemoForm.ImageCompressions.JPEG2000;
            // 1000 dpi
            if (optWSQCompression1000.Checked)
                DemoFormRef.DemoConfig.ImageCompression1000 = DemoForm.ImageCompressions.WSQ;
            else
                DemoFormRef.DemoConfig.ImageCompression1000 = DemoForm.ImageCompressions.JPEG2000;

            // compression rate
            // 500 dpi
            DemoFormRef.DemoConfig.WQSBitRate500 = Convert.ToDouble(txtWSQBitRate500.Text);
            DemoFormRef.DemoConfig.JPEGRate500 = Convert.ToInt32(txtJPEGRate500.Text);
            // 1000 dpi
            DemoFormRef.DemoConfig.WQSBitRate1000 = Convert.ToDouble(txtWSQBitRate1000.Text);
            DemoFormRef.DemoConfig.JPEGRate1000 = Convert.ToInt32(txtJPEGRate1000.Text);

            // roll area size
            if (optRollAreaIQS.Checked)
                DemoFormRef.DemoConfig.RollAreaSize = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS;
            else
                DemoFormRef.DemoConfig.RollAreaSize = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_GA;

            // Language
            DemoFormRef.DemoConfig.GUILanguage = cmbLanguage.SelectedIndex;

            // finger contact evaluation mode
            DemoFormRef.DemoConfig.FingerContactEvaluationMode = cmbFingerContactEvaluationMode.SelectedIndex + 1;

            // save configuration
            DemoForm.Configuration.Serialize(Path.ChangeExtension(Application.ExecutablePath, ".cfg"), DemoFormRef.DemoConfig);

            DialogResult = DialogResult.OK;
        }

        private void chkBlockAutocapture_CheckedChanged(object sender, EventArgs e)
        {
            chkBlockAutocaptureContrast.Enabled = chkBlockAutocapture.Checked;
        }

        private void cmbStandard_SelectedIndexChanged(object sender, EventArgs e)
        {
            DemoFormRef.DemoConfig.ImageSizeStandard = cmbStandard.SelectedIndex;
            // show values
            ShowImageSizeValues();

            if (cmbStandard.SelectedIndex == DemoForm.ImageSizeStandards.GA_CHINA)
                optRollAreaGA.Checked = true;
            else
                optRollAreaIQS.Checked = true;
        }

        private bool SaveImageSizeValues()
        {
            DemoForm.ImageSizes ImageSize = DemoFormRef.DemoConfig.ImageSize;

            // change only enabled fields
            if (txtUpperPalmWidth.Enabled)
                ImageSize.UpperPalmWidth = Convert.ToDouble(txtUpperPalmWidth.Text);
            if (txtUpperPalmHeight.Enabled)
                ImageSize.UpperPalmHeight = Convert.ToDouble(txtUpperPalmHeight.Text);
            if (txtLowerPalmWidth.Enabled)
                ImageSize.LowerPalmWidth = Convert.ToDouble(txtLowerPalmWidth.Text);
            if (txtLowerPalmHeight.Enabled)
                ImageSize.LowerPalmHeight = Convert.ToDouble(txtLowerPalmHeight.Text);
            if (txtWritersPalmWidth.Enabled)
                ImageSize.WritersPalmWidth = Convert.ToDouble(txtWritersPalmWidth.Text);
            if (txtWritersPalmHeight.Enabled)
                ImageSize.WritersPalmHeight = Convert.ToDouble(txtWritersPalmHeight.Text);
            if (txtFourFingersWidth.Enabled)
                ImageSize.FourFingersWidth = Convert.ToDouble(txtFourFingersWidth.Text);
            if (txtFourFingersHeight.Enabled)
                ImageSize.FourFingersHeight = Convert.ToDouble(txtFourFingersHeight.Text);
            if (txtTwoThumbsWidth.Enabled)
                ImageSize.TwoThumbsWidth = Convert.ToDouble(txtTwoThumbsWidth.Text);
            if (txtTwoThumbsHeight.Enabled)
                ImageSize.TwoThumbsHeight = Convert.ToDouble(txtTwoThumbsHeight.Text);
            if (txtFlatThumbWidth.Enabled)
                ImageSize.FlatThumbWidth = Convert.ToDouble(txtFlatThumbWidth.Text);
            if (txtFlatThumbHeight.Enabled)
                ImageSize.FlatThumbHeight = Convert.ToDouble(txtFlatThumbHeight.Text);
            if (txtFlatFingerWidth.Enabled)
                ImageSize.FlatFingerWidth = Convert.ToDouble(txtFlatFingerWidth.Text);
            if (txtFlatFingerHeight.Enabled)
                ImageSize.FlatFingerHeight = Convert.ToDouble(txtFlatFingerHeight.Text);
            if (txtRolledThumbWidth.Enabled)
                ImageSize.RolledThumbWidth = Convert.ToDouble(txtRolledThumbWidth.Text);
            if (txtRolledThumbHeight.Enabled)
                ImageSize.RolledThumbHeight = Convert.ToDouble(txtRolledThumbHeight.Text);
            if (txtRolledIndexWidth.Enabled)
                ImageSize.RolledIndexWidth = Convert.ToDouble(txtRolledIndexWidth.Text);
            if (txtRolledIndexHeight.Enabled)
                ImageSize.RolledIndexHeight = Convert.ToDouble(txtRolledIndexHeight.Text);
            if (txtRolledMiddleWidth.Enabled)
                ImageSize.RolledMiddleWidth = Convert.ToDouble(txtRolledMiddleWidth.Text);
            if (txtRolledMiddleHeight.Enabled)
                ImageSize.RolledMiddleHeight = Convert.ToDouble(txtRolledMiddleHeight.Text);
            if (txtRolledRingWidth.Enabled)
                ImageSize.RolledRingWidth = Convert.ToDouble(txtRolledRingWidth.Text);
            if (txtRolledRingHeight.Enabled)
                ImageSize.RolledRingHeight = Convert.ToDouble(txtRolledRingHeight.Text);
            if (txtRolledLittleWidth.Enabled)
                ImageSize.RolledLittleWidth = Convert.ToDouble(txtRolledLittleWidth.Text);
            if (txtRolledLittleHeight.Enabled)
                ImageSize.RolledLittleHeight = Convert.ToDouble(txtRolledLittleHeight.Text);

            DemoFormRef.DemoConfig.ImageSize =  ImageSize;

            return true;
        }

        private void SizeValidate(object sender, CancelEventArgs e)
        {
            // check only that value is ranged between 0.4 and 5
            if (!CheckDoubleField((TextBox)sender, 0.4, 5))
            {
                e.Cancel = true;
                return;
            }

            e.Cancel = false;
        }

        private void WSQBitRateValidate(object sender, CancelEventArgs e)
        {
            // check that value is ranged between 0 and 1
            if (!CheckDoubleField((TextBox)sender, 0, 1))
            {
                e.Cancel = true;
                return;
            }
            e.Cancel = false;
        }

        private void JPEGRateValidate(object sender, CancelEventArgs e)
        {
            // check only that value is ranged between 1 and 100
            if (!CheckIntField((TextBox)sender, 1, 100))
            {
                e.Cancel = true;
                return;
            }

            e.Cancel = false;
        }

        private bool CheckDoubleField(TextBox txtBox, double MinVal, double MaxVal)
        {
            double Value = 0;

            try
            {
                Value = Convert.ToDouble(txtBox.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtBox.Focus();
                return false;
            }

            // check only that value is ranged between MinVal and MaxVal
            if ((Value < MinVal) || (Value > MaxVal))
            {
                MessageBox.Show("Value out of range", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtBox.Focus();
                return false;
            }

            return true;
        }

        private bool CheckIntField(TextBox txtBox, int MinVal, int MaxVal)
        {
            int Value = 0;

            try
            {
                Value = Convert.ToInt32(txtBox.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtBox.Focus();
                return false;
            }

            // check  that value is ranged between MinVal and MaxVal
            if ((Value < MinVal) || (Value > MaxVal))
            {
                MessageBox.Show("Value out of range", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtBox.Focus();
                return false;
            }

            return true;
        }


        private void lstDevice_SelectedIndexChanged(object sender, EventArgs e)
        {
            // show modes list according to device
            int DeviceID = DemoForm.GetDeviceIDFromName(lstDevice.Items[lstDevice.SelectedIndex].ToString());
            int ScanMode;

            // save current selection
            if (optFullHiRes.Checked)
                ScanMode = ScanModes.FullHiRes;
            else if (optFullLowRes.Checked)
                ScanMode = ScanModes.FullLowRes;
            else if (optRollIQS.Checked)
                ScanMode = ScanModes.RollIQS;
            else
                ScanMode = ScanModes.RollGA;

            if ((DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C) ||
                (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517))
                optFullLowRes.Enabled = true;
            else
                optFullLowRes.Enabled = false;

            if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26)
                optRollIQS.Enabled = false;
            else
                optRollIQS.Enabled = true;

            if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517)
                optRollGA.Enabled = true;
            else
                optRollGA.Enabled = false;

            // restore selection (if possible)
            // or set selection to Full Hi res (always available)
            if ((ScanMode == ScanModes.FullLowRes) && optFullLowRes.Enabled)
                optFullLowRes.Checked = true;
            else if ((ScanMode == ScanModes.RollIQS) && optRollIQS.Enabled)
                optRollIQS.Checked = true;
            else if ((ScanMode == ScanModes.RollGA) && optRollGA.Enabled)
                optRollGA.Checked = true;
            else
                optFullHiRes.Checked = true;

            // refresh the view
            optScanMode_CheckedChanged(this, null);
        }

        private void optScanMode_CheckedChanged(object sender, EventArgs e)
        {
            int DeviceID = DemoForm.GetDeviceIDFromName(lstDevice.Items[lstDevice.SelectedIndex].ToString());
            int ScanMode;
            uint FrameRateOptions = 0;
		    double MaxFrameRate;
		    double MinFrameRate;
            double DefFrameRate;
            uint RollAreaStandard = 0;

            if (optFullHiRes.Checked)
            {
                ScanMode = ScanModes.FullHiRes;
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE;
            }
            else if (optFullLowRes.Checked)
            {
                ScanMode = ScanModes.FullLowRes;
            }
            else if (optRollIQS.Checked)
            {
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA;
                ScanMode = ScanModes.RollIQS;
                RollAreaStandard = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS;
            }
            else
            {
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA;
                ScanMode = ScanModes.RollGA;
                RollAreaStandard = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_GA;
            }

            GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global((byte)DeviceID, FrameRateOptions, RollAreaStandard,
                out MaxFrameRate, out MinFrameRate, out DefFrameRate);
            lblFrameRateRange.Text = "(" + MinFrameRate.ToString() + " - " + MaxFrameRate.ToString() + ")";

            switch (DeviceID)
            {
                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26:
                    txtFrameRate.Text = DemoFormRef.DemoConfig.DS26FrameRate.ToString();
                    //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS26_FLAT_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                    //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS26_FLAT_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS84FullFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_FLAT_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_FLAT_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.RollIQS:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS84PartialFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_ROLL_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_ROLL_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS40iFullFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_FLAT_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_FLAT_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.RollIQS:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS40iPartialFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_ROLL_ROLL_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_ROLL_ROLL_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS84cFullHiResFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.FullLowRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS84cFullLowResFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.RollIQS:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS84cPartialFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_ROLL_ROLL_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_ROLL_ROLL_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.MC517FullHiResFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_FULLRES_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_FULLRES_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.FullLowRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.MC517FullLowResFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE).ToString() + ")";
                            break;
                        case ScanModes.RollIQS:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.MC517PartialIQSFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_IQS).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_IQS).ToString() + ")";
                            break;
                        case ScanModes.RollGA:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.MC517PartialGAFrameRate.ToString();
                            //lblFrameRateRange.Text = "(" + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_GA).ToString()
                            //    + " - " + Convert.ToDouble(GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_GA).ToString() + ")";
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS32FullFrameRate.ToString();
                            break;
                        case ScanModes.RollIQS:
                            txtFrameRate.Text = DemoFormRef.DemoConfig.DS32PartialFrameRate.ToString();
                            break;
                    }
                    break;

            }
        }

        private void txtFrameRate_Validating(object sender, CancelEventArgs e)
        {
            double MinVal, MaxVal;
            uint FrameRateOptions = 0;
            double DefFrameRate;
            uint RollAreaStandard = 0;

            int DeviceID = DemoForm.GetDeviceIDFromName(lstDevice.Items[lstDevice.SelectedIndex].ToString());
            int ScanMode;

            if (optFullHiRes.Checked)
            {
                ScanMode = ScanModes.FullHiRes;
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE;
            }
            else if (optFullLowRes.Checked)
            {
                ScanMode = ScanModes.FullLowRes;
            }
            else if (optRollIQS.Checked)
            {
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA;
                ScanMode = ScanModes.RollIQS;
                RollAreaStandard = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS;
            }
            else
            {
                FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA;
                ScanMode = ScanModes.RollGA;
                RollAreaStandard = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_GA;
            }

            GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global((byte)DeviceID, FrameRateOptions, RollAreaStandard,
                out MaxVal, out MinVal, out DefFrameRate);

            /*
            switch (DeviceID)
            {
                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26:
                    MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS26_FLAT_FLAT_FULLRES_MIN_FRAME_RATE;
                    MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS26_FLAT_FLAT_FULLRES_MAX_FRAME_RATE;
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_FLAT_FLAT_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_FLAT_FLAT_FULLRES_MAX_FRAME_RATE;
                            break;
                        case ScanModes.RollIQS:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_ROLL_FLAT_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84_ROLL_FLAT_FULLRES_MAX_FRAME_RATE;
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_FLAT_FLAT_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_FLAT_FLAT_FULLRES_MAX_FRAME_RATE;
                            break;
                        case ScanModes.RollIQS:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_ROLL_ROLL_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS40I_ROLL_ROLL_FULLRES_MAX_FRAME_RATE;
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_FULLRES_MAX_FRAME_RATE;
                            break;
                        case ScanModes.FullLowRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE;
                            break;
                        case ScanModes.RollIQS:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_ROLL_ROLL_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_DS84C_ROLL_ROLL_FULLRES_MAX_FRAME_RATE;
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_FULLRES_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_FULLRES_MAX_FRAME_RATE;
                            break;
                        case ScanModes.FullLowRes:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE;
                            break;
                        case ScanModes.RollIQS:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_IQS;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_IQS;
                            break;
                        case ScanModes.RollGA:
                            MinVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_GA;
                            MaxVal = GBMSAPI_NET_FrameRateRangeLimits.GBMSAPI_NET_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_GA;
                            break;
                    }
                    break;
            }
            */

            // check value range
            if (!CheckDoubleField((TextBox)sender, MinVal, MaxVal))
            {
                e.Cancel = true;
                return;
            }

            e.Cancel = false;
        }

        private void txtFrameRate_Validated(object sender, EventArgs e)
        {
            int DeviceID = DemoForm.GetDeviceIDFromName(lstDevice.Items[lstDevice.SelectedIndex].ToString());
            int ScanMode;

            if (optFullHiRes.Checked)
                ScanMode = ScanModes.FullHiRes;
            else if (optFullLowRes.Checked)
                ScanMode = ScanModes.FullLowRes;
            else if (optRollIQS.Checked)
                ScanMode = ScanModes.RollIQS;
            else
                ScanMode = ScanModes.RollGA;

            switch (DeviceID)
            {
                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26:
                    DemoFormRef.DemoConfig.DS26FrameRate = Convert.ToDouble(txtFrameRate.Text);
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            DemoFormRef.DemoConfig.DS84FullFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollIQS:
                            DemoFormRef.DemoConfig.DS84PartialFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            DemoFormRef.DemoConfig.DS40iFullFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollIQS:
                            DemoFormRef.DemoConfig.DS40iPartialFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            DemoFormRef.DemoConfig.DS84cFullHiResFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.FullLowRes:
                            DemoFormRef.DemoConfig.DS84cFullLowResFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollIQS:
                            DemoFormRef.DemoConfig.DS84cPartialFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            DemoFormRef.DemoConfig.MC517FullHiResFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.FullLowRes:
                            DemoFormRef.DemoConfig.MC517FullLowResFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollIQS:
                            DemoFormRef.DemoConfig.MC517PartialIQSFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollGA:
                            DemoFormRef.DemoConfig.MC517PartialGAFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                    }
                    break;

                case GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32:
                    switch (ScanMode)
                    {
                        case ScanModes.FullHiRes:
                            DemoFormRef.DemoConfig.DS32FullFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                        case ScanModes.RollIQS:
                            DemoFormRef.DemoConfig.DS32PartialFrameRate = Convert.ToDouble(txtFrameRate.Text);
                            break;
                    }
                    break;
            }
        }

        private void optRollAreaGA_CheckedChanged(object sender, EventArgs e)
        {
            if (optRollAreaGA.Checked)
                cmbStandard.SelectedIndex = DemoForm.ImageSizeStandards.GA_CHINA;
        }

    }
}