using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using GBMSGUI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using WSQPACK_NET_WRAPPER;

namespace GBMSDemo
{
    // classes for managing configuration file
    public partial class DemoForm : Form
    {
        // type of objects to acquire in sequence
        public class SequenceType
        {
            public const uint Slaps = 0x00000001;
            public const uint SingleFlat = 0x00000002;
            public const uint Rolled = 0x00000004;
            public const uint Palms = 0x00000008;
        }

        // Standard used  for image sizes
        public class ImageSizeStandards
        {
            public const int Custom = 0;
            public const int ANSI_NIST_ITL_1_2000 = 1;
            public const int ANSI_NIST_ITL_1_2000_INTERPOL = 2;
            public const int ISO_IEC_FCD_19794_4 = 3;
            public const int ANSI_NIST_ITL_1_2007 = 4;
            public const int GA_CHINA = 5;
        }

        // Image sizes
        public struct ImageSizes
        {
            public double UpperPalmWidth;
            public double UpperPalmHeight;
            public double LowerPalmWidth;
            public double LowerPalmHeight;
            public double WritersPalmWidth;
            public double WritersPalmHeight;
            public double FourFingersWidth;
            public double FourFingersHeight;
            public double TwoThumbsWidth;
            public double TwoThumbsHeight;
            public double FlatThumbWidth;
            public double FlatThumbHeight;
            public double FlatFingerWidth;
            public double FlatFingerHeight;
            public double RolledThumbWidth;
            public double RolledThumbHeight;
            public double RolledIndexWidth;
            public double RolledIndexHeight;
            public double RolledMiddleWidth;
            public double RolledMiddleHeight;
            public double RolledRingWidth;
            public double RolledRingHeight;
            public double RolledLittleWidth;
            public double RolledLittleHeight;
        }

        // Image compressions
        public class ImageCompressions
        {
            public const int WSQ = 1;
            public const int JPEG2000 = 2;
        }

        // Languages
        public class GUILanguages
        {
            public const int English = 0;
            public const int Italian = 1;
            public const int Chinese = 2;
        }

        // Configuration class
        [Serializable]
        public class Configuration
        {
            int mVersion;
            public int Version
            {
                get { return mVersion; }
                set { mVersion = value; }
            }

            // options
            uint mSessionOptions;
            public uint SessionOptions
            {
                get { return mSessionOptions; }
                set { mSessionOptions = value; }
            }

            uint mAcquisitionOptions;
            public uint AcquisitionOptions
            {
                get { return mAcquisitionOptions; }
                set { mAcquisitionOptions = value; }
            }

            uint mSequenceTypes;
            public uint SequenceTypes
            {
                get { return mSequenceTypes; }
                set { mSequenceTypes = value; }
            }

            // settings
            bool mBlockAutocaptureContrast;
            public bool BlockAutocaptureContrast
            {
                get { return mBlockAutocaptureContrast; }
                set { mBlockAutocaptureContrast = value; }
            }

            // Statndard for image sizes
            int mImageSizeStandard;
            public int ImageSizeStandard
            {
                get { return mImageSizeStandard; }
                set
                {
                    mImageSizeStandard = value;
                    // set values
                    SetImageSizeStandardValues();
                }
            }

            // size of images (stored in inches)
            ImageSizes mImageSize;
            public ImageSizes ImageSize
            {
                get { return mImageSize; }
                set { mImageSize = value; }
            }


            // IAFIS quality algorithm
            int mIAFIsQualityAlgorithm;
            public int IAFIsQualityAlgorithm
            {
                get { return mIAFIsQualityAlgorithm; }
                set { mIAFIsQualityAlgorithm = value; }
            }

            // window size
            Rectangle mWindowSize;
            public Rectangle WindowSize
            {
                get { return mWindowSize; }
                set { mWindowSize = value; }
            }

            // window state
            bool mWindowMaximized;
            public bool WindowMaximized
            {
                get { return mWindowMaximized; }
                set { mWindowMaximized = value; }
            }

            // Image compression
            int mImageCompression500;
            public int ImageCompression500
            {
                get { return mImageCompression500; }
                set { mImageCompression500 = value; }
            }

            int mImageCompression1000;
            public int ImageCompression1000
            {
                get { return mImageCompression1000; }
                set { mImageCompression1000 = value; }
            }

            // WSQ Bit rate for 500 dpi
            double mWQSBitRate500;
            public double WQSBitRate500
            {
                get { return mWQSBitRate500; }
                set { mWQSBitRate500 = value; }
            }

            // WSQ Bit rate for 1000 dpi
            double mWQSBitRate1000;
            public double WQSBitRate1000
            {
                get { return mWQSBitRate1000; }
                set { mWQSBitRate1000 = value; }
            }

            // JPEG2000 compression rate for 500 dpi
            int mJPEGRate500;
            public int JPEGRate500
            {
                get { return mJPEGRate500; }
                set { mJPEGRate500 = value; }
            }

            // JPEG2000 compression rate for 1000 dpi
            int mJPEGRate1000;
            public int JPEGRate1000
            {
                get { return mJPEGRate1000; }
                set { mJPEGRate1000 = value; }
            }

            // DactyScan26 Frame Rate
            double mDS26FrameRate;
            public double DS26FrameRate
            {
                get { return mDS26FrameRate; }
                set { mDS26FrameRate = value; }
            }

            // DactyScan84 Full Frame Rate
            double mDS84FullFrameRate;
            public double DS84FullFrameRate
            {
                get { return mDS84FullFrameRate; }
                set { mDS84FullFrameRate = value; }
            }

            // DactyScan84 Partial Frame Rate
            double mDS84PartialFrameRate;
            public double DS84PartialFrameRate
            {
                get { return mDS84PartialFrameRate; }
                set { mDS84PartialFrameRate = value; }
            }

            // DactyScan40i Full Frame Rate
            double mDS40iFullFrameRate;
            public double DS40iFullFrameRate
            {
                get { return mDS40iFullFrameRate; }
                set { mDS40iFullFrameRate = value; }
            }

            // DactyScan40i Partial Frame Rate
            double mDS40iPartialFrameRate;
            public double DS40iPartialFrameRate
            {
                get { return mDS40iPartialFrameRate; }
                set { mDS40iPartialFrameRate = value; }
            }

            // DactyScan84c Full Low Resolution Frame Rate
            double mDS84cFullLowResFrameRate;
            public double DS84cFullLowResFrameRate
            {
                get { return mDS84cFullLowResFrameRate; }
                set { mDS84cFullLowResFrameRate = value; }
            }

            // DactyScan84c Full High Resolution Frame Rate
            double mDS84cFullHiResFrameRate;
            public double DS84cFullHiResFrameRate
            {
                get { return mDS84cFullHiResFrameRate; }
                set { mDS84cFullHiResFrameRate = value; }
            }

            // DactyScan84c Partial Frame Rate
            double mDS84cPartialFrameRate;
            public double DS84cPartialFrameRate
            {
                get { return mDS84cPartialFrameRate; }
                set { mDS84cPartialFrameRate = value; }
            }

            // MC517 Full Low Resolution Frame Rate
            double mMC517FullLowResFrameRate;
            public double MC517FullLowResFrameRate
            {
                get { return mMC517FullLowResFrameRate; }
                set { mMC517FullLowResFrameRate = value; }
            }

            // MC517 Full High Resolution Frame Rate
            double mMC517FullHiResFrameRate;
            public double MC517FullHiResFrameRate
            {
                get { return mMC517FullHiResFrameRate; }
                set { mMC517FullHiResFrameRate = value; }
            }

            // MC517 Partial Frame Rate IQS
            double mMC517PartialIQSFrameRate;
            public double MC517PartialIQSFrameRate
            {
                get { return mMC517PartialIQSFrameRate; }
                set { mMC517PartialIQSFrameRate = value; }
            }

            // MC517 Partial Frame Rate GA
            double mMC517PartialGAFrameRate;
            public double MC517PartialGAFrameRate
            {
                get { return mMC517PartialGAFrameRate; }
                set { mMC517PartialGAFrameRate = value; }
            }

            // DactyScan32 Full Frame Rate
            double mDS32FullFrameRate;
            public double DS32FullFrameRate
            {
                get { return mDS32FullFrameRate; }
                set { mDS32FullFrameRate = value; }
            }

            // DactyScan40i Partial Frame Rate
            double mDS32PartialFrameRate;
            public double DS32PartialFrameRate
            {
                get { return mDS32PartialFrameRate; }
                set { mDS32PartialFrameRate = value; }
            }



            // Roll Area size (MC517 only)
            uint mRollAreaSize;
            public uint RollAreaSize
            {
                get { return mRollAreaSize; }
                set { mRollAreaSize = value; }
            }

            int mGUILanguage;
            public int GUILanguage
            {
                get { return mGUILanguage; }
                set { mGUILanguage = value; }
            }

            int mFingerContactEvaluationMode;
            public int FingerContactEvaluationMode
            {
                get { return mFingerContactEvaluationMode; }
                set { mFingerContactEvaluationMode = value; }
            }

            public Configuration()
            {
                mVersion = 1;

                // default values
                mSessionOptions = 0;
                mSessionOptions |= GBMSGUI.SessionOption.SequenceCheck;

                mAcquisitionOptions = 0;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.Autocapture;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.BlockAutocapture;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.Sound;
                // no more active by default - they are time consuming...
                //mAcquisitionOptions |= GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation;
                //mAcquisitionOptions |= GBMSGUI.AcquisitionOption.RotateFinger;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectInclination;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectIncompletePattern;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.DetectInvalidPattern;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.RemoveHaloLatent;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.Segmentation;
                mAcquisitionOptions |= GBMSGUI.AcquisitionOption.SoundOnRollPreviewEnd;

                mSequenceTypes = 0;
                mSequenceTypes |= SequenceType.Slaps;
                mSequenceTypes |= SequenceType.Rolled;
                mSequenceTypes |= SequenceType.Palms;

                mBlockAutocaptureContrast = false;

                // image size
                // set first values not defined in all standards
                mImageSize.FlatFingerWidth = 1;
                mImageSize.FlatFingerHeight = 1;
                mImageSize.TwoThumbsWidth = 3.2;
                mImageSize.TwoThumbsHeight = 3;
                mImageSize.FlatThumbWidth = 1;
                mImageSize.FlatThumbHeight = 2;
                // set others
                mImageSizeStandard = ImageSizeStandards.ANSI_NIST_ITL_1_2007;
                SetImageSizeStandardValues();

                mIAFIsQualityAlgorithm = GBMSGUI.QualityAlgorithms.NFIQAlgorithm;

                mWindowSize = new Rectangle(0, 0, 1024, 768);
                mWindowMaximized = true;

                mImageCompression500 = ImageCompressions.WSQ;
                mWQSBitRate500 = 0.75;  // 15:1 compression
                //mWQSBitRate500 = NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
                mJPEGRate500 = 14;      // 14:1 compression
                mImageCompression1000 = ImageCompressions.JPEG2000;
                mWQSBitRate1000 = 0.75;  // 15:1 compression
                //mWQSBitRate1000 = NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
                mJPEGRate1000 = 14;     // 14:1 compression

                // default frame rates
                uint FrameRateOptions = 0;
                double MaxFrameRate;
                double MinFrameRate;
                double DefFrameRate;
                uint RollAreaStandard = 0;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS26FrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS84FullFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS84PartialFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS40iFullFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS40iPartialFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C, 0, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS84cFullLowResFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS84cFullHiResFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS84cPartialFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517, 0, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mMC517FullLowResFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mMC517FullHiResFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mMC517PartialIQSFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_GA, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mMC517PartialGAFrameRate = DefFrameRate;

                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS32FullFrameRate = DefFrameRate;
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_GetFrameRateRange_Global(GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32, GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA, 0, out MaxFrameRate, out MinFrameRate, out DefFrameRate);
                mDS32PartialFrameRate = DefFrameRate;



                // Roll Area size (MC517 only)
                mRollAreaSize = GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS;

                // Language
                mGUILanguage = GUILanguages.English;

                mFingerContactEvaluationMode = GBMSGUI.FingerContactEvaluationMode.DryWetWarning;
            }

            public static void Serialize(string file, Configuration c)
            {
                System.Xml.Serialization.XmlSerializer xs
                   = new System.Xml.Serialization.XmlSerializer(c.GetType());
                StreamWriter writer = File.CreateText(file);
                xs.Serialize(writer, c);
                writer.Flush();
                writer.Close();
                writer.Dispose();
            }
            public static Configuration Deserialize(string file)
            {
                System.Xml.Serialization.XmlSerializer xs
                   = new System.Xml.Serialization.XmlSerializer(
                      typeof(Configuration));
                StreamReader reader = File.OpenText(file);
                Configuration c = (Configuration)xs.Deserialize(reader);
                reader.Close();
                reader.Dispose();
                return c;
            }

            private void SetImageSizeStandardValues()
            {
                // sizes are expressed in inches
                switch (mImageSizeStandard)
                {
                    case ImageSizeStandards.ANSI_NIST_ITL_1_2000:
                        mImageSize.UpperPalmWidth = 5;
                        mImageSize.UpperPalmHeight = 5;
                        mImageSize.LowerPalmWidth = 5;
                        mImageSize.LowerPalmHeight = 5;
                        mImageSize.WritersPalmWidth = 1.8;
                        mImageSize.WritersPalmHeight = 5;
                        mImageSize.FourFingersWidth = 3.2;
                        mImageSize.FourFingersHeight = 2;
                        //mImageSize.TwoThumbsWidth = 3.2;
                        //mImageSize.TwoThumbsHeight = 2;
                        mImageSize.FlatThumbWidth = 1;
                        mImageSize.FlatThumbHeight = 2;
                        //mImageSize.FlatFingerWidth = 1;
                        //mImageSize.FlatFingerHeight = 1;
                        mImageSize.RolledThumbWidth = 1.6;
                        mImageSize.RolledThumbHeight = 1.5;
                        mImageSize.RolledIndexWidth = 1.6;
                        mImageSize.RolledIndexHeight = 1.5;
                        mImageSize.RolledMiddleWidth = 1.6;
                        mImageSize.RolledMiddleHeight = 1.5;
                        mImageSize.RolledRingWidth = 1.6;
                        mImageSize.RolledRingHeight = 1.5;
                        mImageSize.RolledLittleWidth = 1.6;
                        mImageSize.RolledLittleHeight = 1.5;
                        break;
                    case ImageSizeStandards.ANSI_NIST_ITL_1_2000_INTERPOL:
                        mImageSize.UpperPalmWidth = 5;
                        mImageSize.UpperPalmHeight = 5;
                        mImageSize.LowerPalmWidth = 5;
                        mImageSize.LowerPalmHeight = 5;
                        mImageSize.WritersPalmWidth = 1.752;
                        mImageSize.WritersPalmHeight = 5;
                        mImageSize.FourFingersWidth = 2.756;
                        mImageSize.FourFingersHeight = 2.56;
                        //mImageSize.TwoThumbsWidth = 2.756;
                        //mImageSize.TwoThumbsHeight = 2.56;
                        mImageSize.FlatThumbWidth = 1.182;
                        mImageSize.FlatThumbHeight = 2.166;
                        //mImageSize.FlatFingerWidth = 1.182;
                        //mImageSize.FlatFingerHeight = 2.166;
                        mImageSize.RolledThumbWidth = 1.6;
                        mImageSize.RolledThumbHeight = 1.574;
                        mImageSize.RolledIndexWidth = 1.574;
                        mImageSize.RolledIndexHeight = 1.574;
                        mImageSize.RolledMiddleWidth = 1.574;
                        mImageSize.RolledMiddleHeight = 1.574;
                        mImageSize.RolledRingWidth = 1.574;
                        mImageSize.RolledRingHeight = 1.574;
                        mImageSize.RolledLittleWidth = 1.3;
                        mImageSize.RolledLittleHeight = 1.574;
                        break;
                    case ImageSizeStandards.ISO_IEC_FCD_19794_4:
                        mImageSize.UpperPalmWidth = 5;
                        mImageSize.UpperPalmHeight = 5;
                        mImageSize.LowerPalmWidth = 5;
                        mImageSize.LowerPalmHeight = 5;
                        mImageSize.WritersPalmWidth = 1.8;
                        mImageSize.WritersPalmHeight = 5;
                        mImageSize.FourFingersWidth = 3.2;
                        mImageSize.FourFingersHeight = 3;
                        mImageSize.TwoThumbsWidth = 2;
                        mImageSize.TwoThumbsHeight = 3;
                        //mImageSize.FlatThumbWidth = 1;
                        //mImageSize.FlatThumbHeight = 2;
                        //mImageSize.FlatFingerWidth = 1;
                        //mImageSize.FlatFingerHeight = 1;
                        mImageSize.RolledThumbWidth = 1.6;
                        mImageSize.RolledThumbHeight = 1.5;
                        mImageSize.RolledIndexWidth = 1.6;
                        mImageSize.RolledIndexHeight = 1.5;
                        mImageSize.RolledMiddleWidth = 1.6;
                        mImageSize.RolledMiddleHeight = 1.5;
                        mImageSize.RolledRingWidth = 1.6;
                        mImageSize.RolledRingHeight = 1.5;
                        mImageSize.RolledLittleWidth = 1.6;
                        mImageSize.RolledLittleHeight = 1.5;
                        break;
                    case ImageSizeStandards.ANSI_NIST_ITL_1_2007:
                        mImageSize.UpperPalmWidth = 5;
                        mImageSize.UpperPalmHeight = 5;
                        mImageSize.LowerPalmWidth = 5;
                        mImageSize.LowerPalmHeight = 5;
                        mImageSize.WritersPalmWidth = 1.8;
                        mImageSize.WritersPalmHeight = 5;
                        mImageSize.FourFingersWidth = 3.2;
                        mImageSize.FourFingersHeight = 3;
                        mImageSize.TwoThumbsWidth = 3.2;
                        mImageSize.TwoThumbsHeight = 3;
                        mImageSize.FlatThumbWidth = 1;
                        mImageSize.FlatThumbHeight = 2;
                        //mImageSize.FlatFingerWidth = 1;
                        //mImageSize.FlatFingerHeight = 1;
                        mImageSize.RolledThumbWidth = 1.6;
                        mImageSize.RolledThumbHeight = 1.5;
                        mImageSize.RolledIndexWidth = 1.6;
                        mImageSize.RolledIndexHeight = 1.5;
                        mImageSize.RolledMiddleWidth = 1.6;
                        mImageSize.RolledMiddleHeight = 1.5;
                        mImageSize.RolledRingWidth = 1.6;
                        mImageSize.RolledRingHeight = 1.5;
                        mImageSize.RolledLittleWidth = 1.6;
                        mImageSize.RolledLittleHeight = 1.5;
                        break;
                    case ImageSizeStandards.GA_CHINA:
                        mImageSize.UpperPalmWidth = 4.608;
                        mImageSize.UpperPalmHeight = 4.608;
                        mImageSize.LowerPalmWidth = 4.608;
                        mImageSize.LowerPalmHeight = 4.608;
                        mImageSize.WritersPalmWidth = 2.048;
                        mImageSize.WritersPalmHeight = 3.936;
                        mImageSize.FourFingersWidth = 3.2;
                        mImageSize.FourFingersHeight = 3;
                        mImageSize.TwoThumbsWidth = 3.2;
                        mImageSize.TwoThumbsHeight = 3;
                        mImageSize.FlatThumbWidth = 1.28;
                        mImageSize.FlatThumbHeight = 1.28;
                        //mImageSize.FlatFingerWidth = 1;
                        //mImageSize.FlatFingerHeight = 1;
                        mImageSize.RolledThumbWidth = 1.28;
                        mImageSize.RolledThumbHeight = 1.28;
                        mImageSize.RolledIndexWidth = 1.28;
                        mImageSize.RolledIndexHeight = 1.28;
                        mImageSize.RolledMiddleWidth = 1.28;
                        mImageSize.RolledMiddleHeight = 1.28;
                        mImageSize.RolledRingWidth = 1.28;
                        mImageSize.RolledRingHeight = 1.28;
                        mImageSize.RolledLittleWidth = 1.28;
                        mImageSize.RolledLittleHeight = 1.28;
                        break;
                    default:
                        break;
                }
            }
        }

    }
}