using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Globalization;
using GBMSGUI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using AN2K_NET_WRAPPER;
using JPEG_NET_WRAPPER;
using WSQPACK_NET_WRAPPER;
using System.Text.RegularExpressions;

// UserDataForm: form for user images acquisition
namespace GBMSDemo
{
    public partial class UserDataForm : Form
    {
        public String ImagesPath;   // path where to store/read images
        public bool ViewMode;       // true = view mode; false = new mode

        private DemoForm.UserData UserData; // class where to store user data
        private DemoForm DemoFormRef;       // reference to main form
        private Byte[] ImageBuffer;         // buffer for images
        private GBMSGUI MyGUI = new GBMSGUI();  // GBMSGUI class
        private GCHandle gcUserDataFormHandle;

        // AfterStartCallback, if needed
        private static GBMSGUI.AfterStartCallback AfterStartCallbackRef = new GBMSGUI.AfterStartCallback(AfterStartCallback);

        private static GCHandle gcUserDataFormHandle_St;
        public static bool AfterStartCallback(IntPtr Param)
        {
            bool ret;

            // retrieve form reference
            if (!gcUserDataFormHandle_St.IsAllocated)
                gcUserDataFormHandle_St = GCHandle.FromIntPtr(Param);

            UserDataForm UserDataFormRef = (UserDataForm)gcUserDataFormHandle_St.Target;

            // call form method
            ret = UserDataFormRef.AfterStartCallback();

            return ret;
        }

        // class for scanned object list item
        private class ScanItem : Object
        {
            public uint ScanObjID;  // ID of the object
            public String Text;     // Descriptive text (used also to build filename)
            public PictureBox ImagePictureBox;  // picturebox where to show image
            public bool Supported;              // if object supported by the current scanner
            public ScanItemData ItemData = new ScanItemData();  // other data collected after scanning

            public ScanItem(uint ScanObjID, String Text, PictureBox ImagePictureBox)
            {
                this.ScanObjID = ScanObjID;
                this.Text = Text;
                this.ImagePictureBox = ImagePictureBox;
            }

            public override String ToString()
            {
                return Text;
            }
        }

        private ScanItem CurrentScanItem;

        // data for scan items
        private class ScanItemData
        {
	        public int Quality;
	        public int QualityAlgorithm;
            public int UnavailabilityReason;
        }

        // data for segments
        private class SegmentData
        {
            public ScanItemData ItemData = new ScanItemData();
            public Rectangle BoundingBox = new Rectangle();
        }
        private SegmentData[] LeftSlapSegmentsData = new SegmentData[4];
        private SegmentData[] RightSlapSegmentsData = new SegmentData[4];
        private SegmentData[] TwoThumbsSegmentsData = new SegmentData[2];

        public UserDataForm(DemoForm DemoFormRef)
        {
            // reference to main form
            this.DemoFormRef = DemoFormRef;

            InitializeComponent();

            // assign popupmenu to items
            pboxLeftFourFingers.ContextMenu = popAcquireItemMenu;
            pboxRightFourFingers.ContextMenu = popAcquireItemMenu;
            pboxTwoThumbs.ContextMenu = popAcquireItemMenu;
            pboxLeftRolledThumb.ContextMenu = popAcquireItemMenu;
            pboxLeftRolledIndex.ContextMenu = popAcquireItemMenu;
            pboxLeftRolledMiddle.ContextMenu = popAcquireItemMenu;
            pboxLeftRolledRing.ContextMenu = popAcquireItemMenu;
            pboxLeftRolledLittle.ContextMenu = popAcquireItemMenu;
            pboxRightRolledThumb.ContextMenu = popAcquireItemMenu;
            pboxRightRolledIndex.ContextMenu = popAcquireItemMenu;
            pboxRightRolledMiddle.ContextMenu = popAcquireItemMenu;
            pboxRightRolledRing.ContextMenu = popAcquireItemMenu;
            pboxRightRolledLittle.ContextMenu = popAcquireItemMenu;
            pboxLeftUpperPalm.ContextMenu = popAcquireItemMenu;
            pboxLeftLowerPalm.ContextMenu = popAcquireItemMenu;
            pboxLeftWritersPalm.ContextMenu = popAcquireItemMenu;
            pboxRightUpperPalm.ContextMenu = popAcquireItemMenu;
            pboxRightLowerPalm.ContextMenu = popAcquireItemMenu;
            pboxRightWritersPalm.ContextMenu = popAcquireItemMenu;
            pboxLeftFlatThumb.ContextMenu = popAcquireItemMenu;
            pboxLeftFlatIndex.ContextMenu = popAcquireItemMenu;
            pboxLeftFlatMiddle.ContextMenu = popAcquireItemMenu;
            pboxLeftFlatRing.ContextMenu = popAcquireItemMenu;
            pboxLeftFlatLittle.ContextMenu = popAcquireItemMenu;
            pboxRightFlatThumb.ContextMenu = popAcquireItemMenu;
            pboxRightFlatIndex.ContextMenu = popAcquireItemMenu;
            pboxRightFlatMiddle.ContextMenu = popAcquireItemMenu;
            pboxRightFlatRing.ContextMenu = popAcquireItemMenu;
            pboxRightFlatLittle.ContextMenu = popAcquireItemMenu;
            //V1.2 - DS40
            pboxLeftTwoFingers.ContextMenu = popAcquireItemMenu;
            pboxRightTwoFingers.ContextMenu = popAcquireItemMenu;
            pboxTwoIndexes.ContextMenu = popAcquireItemMenu;
        }

        private void UserDataForm_Load(object sender, EventArgs e)
        {
            byte DeviceID;

            // handle to the form instance to pass to callback
            gcUserDataFormHandle = GCHandle.Alloc(this, GCHandleType.Normal);

            // load scanned objects
            LoadScanObjectsList();

            // init arrays for slaps data
            int i;
            for (i = 0; i < 4; i++)
            {
                LeftSlapSegmentsData[i] = new SegmentData();
                RightSlapSegmentsData[i] = new SegmentData();
            }
            for (i = 0; i < 2; i++)
                TwoThumbsSegmentsData[i] = new SegmentData();

            if (!ViewMode)
            {
                // set form's caption
                Text = "New User";

                UserData = new DemoForm.UserData();

                // set device information
                if (DemoFormRef.Resolution1000Dpi)
                    UserData.AcquisitionDpi = 1000;
                else
                    UserData.AcquisitionDpi = 500;
                UserData.DeviceName = DemoForm.GetDeviceName(DemoFormRef.CurrentDevice.DeviceID);
                UserData.DeviceSerialNumber = DemoFormRef.CurrentDevice.DeviceSerialNumber;

                // allocate buffer for max image size from scanner
                ImageBuffer = new Byte[DemoFormRef.MaxImageBufferSize];

                // create a new folder name
                ImagesPath = Path.GetDirectoryName(Application.ExecutablePath) + Path.DirectorySeparatorChar + Path.GetRandomFileName();
                if (!Directory.Exists(ImagesPath))
                    Directory.CreateDirectory(ImagesPath);

                // load configuration for types for sequence
                chkSlaps.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SequenceTypes, DemoForm.SequenceType.Slaps);
                chkFlat.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SequenceTypes, DemoForm.SequenceType.SingleFlat);
                chkRolled.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SequenceTypes, DemoForm.SequenceType.Rolled);
                chkPalms.Checked = GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SequenceTypes, DemoForm.SequenceType.Palms);

                // enable only supported objects
                uint ObjectTypes;
                GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetScannableTypes(out ObjectTypes);
                foreach (ScanItem Item in lstScannedObjects.Items)
                {
                    if (GBMSGUI.CheckMask(ObjectTypes, GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(Item.ScanObjID)))
                        Item.Supported = true;
                    else
                        Item.Supported = false;
                }

                DeviceID = DemoFormRef.CurrentDevice.DeviceID;
                HideUnsupportedObjects(DeviceID, ObjectTypes);
                Application.DoEvents();

                if (DemoFormRef.LCDPresent)
                {
                    btnAcquire.Text = btnAcquire.Text + "\n(touch button on LCD)";

                    // enable start button on LCD
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen();
                    // start timer
                    TouchScreenTimer.Enabled = true;
                }
            }
            else
            {
                // set form's caption
                Text = "View User";
                txtSurname.ReadOnly = true;
                txtName.ReadOnly = true;
                btnConfig.Visible = false;
                grpObjects.Visible = false;
                btnAcquire.Visible = false;

                // read user data
                UserData = DemoForm.UserData.Deserialize(ImagesPath + Path.DirectorySeparatorChar + "UserData.xml");
                txtSurname.Text = UserData.Surname;
                txtName.Text = UserData.Name;

                // read other data
                ReadItemsData();
                ReadSegmentsData();

                DeviceID = DemoForm.GetDeviceIDFromName(UserData.DeviceName);
                HideUnsupportedObjects(DeviceID, 0xFFFFFFFF);
                Application.DoEvents();

                // load all images
                Cursor = Cursors.WaitCursor;
                LoadSavedImages();
                Cursor = Cursors.Default;
            }
        }

        private void HideUnsupportedObjects(byte DeviceID, uint ObjectTypes)
        {
            tabControl1.SelectedIndex = -1;

            // hide tabs of unsupported objects
            if (!GBMSGUI.CheckMask(ObjectTypes, GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_LOWER_HALF_PALM))
            {
                tabControl1.TabPages.Remove(tabPalmPrints);
                chkPalms.Enabled = false;
            }

            if (!GBMSGUI.CheckMask(ObjectTypes, GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_4))
            {
                tabControl1.TabPages.Remove(tabSlaps);
                chkSlaps.Enabled = false;
            }

            if (!GBMSGUI.CheckMask(ObjectTypes, GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_2))
            {
                tabControl1.TabPages.Remove(tabSlaps2);
            }

            if (!GBMSGUI.CheckMask(ObjectTypes, GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER))
            {
                tabControl1.TabPages.Remove(tabRolledFingerprints);
                chkRolled.Enabled = false;
            }

            tabControl1.SelectedIndex = 0;
        }

        private void LoadScanObjectsList()
        {
            ScanItem Item;

            lstScannedObjects.Items.Clear();

            // fill list with scanned objects
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT, "Left Flat Four Fingers", pboxLeftFourFingers);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT, "Right Flat Four Fingers", pboxRightFourFingers);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS, "Flat Two Thumbs", pboxTwoThumbs);
            lstScannedObjects.Items.Add(Item);
            //V1.2 - DS40
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT, "Left Flat Two Fingers", pboxLeftTwoFingers);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT, "Right Flat Two Fingers", pboxRightTwoFingers);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES, "Flat Two Indexes", pboxTwoIndexes);
            lstScannedObjects.Items.Add(Item);

            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB, "Left Flat Thumb", pboxLeftFlatThumb);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX, "Left Flat Index", pboxLeftFlatIndex);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE, "Left Flat Middle", pboxLeftFlatMiddle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING, "Left Flat Ring", pboxLeftFlatRing);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE, "Left Flat Little", pboxLeftFlatLittle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB, "Right Flat Thumb", pboxRightFlatThumb);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX, "Right Flat Index", pboxRightFlatIndex);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE, "Right Flat Middle", pboxRightFlatMiddle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING, "Right Flat Ring", pboxRightFlatRing);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE, "Right Flat Little", pboxRightFlatLittle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB, "Left Rolled Thumb", pboxLeftRolledThumb);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX, "Left Rolled Index", pboxLeftRolledIndex);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE, "Left Rolled Middle", pboxLeftRolledMiddle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING, "Left Rolled Ring", pboxLeftRolledRing);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE, "Left Rolled Little", pboxLeftRolledLittle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB, "Right Rolled Thumb", pboxRightRolledThumb);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX, "Right Rolled Index", pboxRightRolledIndex);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE, "Right Rolled Middle", pboxRightRolledMiddle);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING, "Right Rolled Ring", pboxRightRolledRing);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE, "Right Rolled Little", pboxRightRolledLittle);
            lstScannedObjects.Items.Add(Item);
            //Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT, "Left Upper Half Palm", pboxLeftUpperPalm);
            //lstScannedObjects.Items.Add(Item);
            //Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT, "Right Upper Half Palm", pboxRightUpperPalm);
            //lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT, "Left Lower Half Palm", pboxLeftLowerPalm);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT, "Right Lower Half Palm", pboxRightLowerPalm);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT, "Left Writer's Palm", pboxLeftWritersPalm);
            lstScannedObjects.Items.Add(Item);
            Item = new ScanItem(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT, "Right Writer's Palm", pboxRightWritersPalm);
            lstScannedObjects.Items.Add(Item);
        }

        private void btnAcquire_Click(object sender, EventArgs e)
        {
            // delete all files
            EmptyFolder(ImagesPath);
            // clear all pictureboxes
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                if (Item.ImagePictureBox.Image != null)
                    Item.ImagePictureBox.Image.Dispose();
                Item.ImagePictureBox.Image = null;
            }
            ResetScanItemsData();

            SelectAllScanItems();
            Acquire();
            // after acquisition, re-enable timer
            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen();
            TouchScreenTimer.Enabled = true;
        }

        private void Acquire()
        {
            int Index;
            int ret;
            int ImageSizeX, ImageSizeY;
            int Resolution;
            Bitmap bmp;
            uint AcqOptions = 0;
            uint SessionOpt = 0;
            String PersonID;
            int AcquisitionMode;
            int QualityThreshold1, QualityThreshold2;

            byte DeviceID = DemoFormRef.CurrentDevice.DeviceID;

            // disable TouchScreen
            TouchScreenTimer.Enabled = false;

            PersonID = txtSurname.Text + " " + txtName.Text;

            // set language
            SetLanguage();

            // set AfterStartCallback (if needed)
            //gcUserDataFormHandle = GCHandle.Alloc(this); // pass pointer to the form instance to callback
            MyGUI.SetAfterStartCallback(AfterStartCallbackRef, GCHandle.ToIntPtr(gcUserDataFormHandle));

            // set quality algorithm
            ret = MyGUI.SetQualityAlgorithm(DemoFormRef.DemoConfig.IAFIsQualityAlgorithm);
            if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
            {
                MessageBox.Show(MyGUI.GetErrorMessage() + " (SetQualityAlgorithm)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // set the thresholds according to the algorithm
            if (DemoFormRef.DemoConfig.IAFIsQualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
            {
                QualityThreshold1 = 3;
                QualityThreshold2 = 4;
            }
            else // GB algorithm
            {
                QualityThreshold1 = 50;
                QualityThreshold2 = 70;
            }

            // other settings - for demo a little more permissive
            MyGUI.SetArtefactsThresholds(15, 30);
            MyGUI.SetLowerPalmCompletenessThresholds(70, 80);
            MyGUI.SetBlockAutoCaptureContrast(DemoFormRef.DemoConfig.BlockAutocaptureContrast);
            MyGUI.SetPatternValidityThreshold(65);
            MyGUI.SetPatternCompletenessThreshold(75);

            // set window size and position
            ret = MyGUI.SetWindowSizeAndPosition(DemoFormRef.DemoConfig.WindowSize.X,
                DemoFormRef.DemoConfig.WindowSize.Y,
                DemoFormRef.DemoConfig.WindowSize.Width,
                DemoFormRef.DemoConfig.WindowSize.Height,
                DemoFormRef.DemoConfig.WindowMaximized);
            // ignore error
            /*
            if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
            {
                MessageBox.Show(MyGUI.GetErrorMessage() + " (SetWindowSizeAndPosition)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            */

            // because acquisition form shown by GBMSGUI is not modal, disable buttons
            btnAcquire.Enabled = false;
            mnuAcquireItem.Enabled = false;

            SessionOpt = DemoFormRef.DemoConfig.SessionOptions;
            // if acquiring single item, disable sequence check
            if (lstScannedObjects.CheckedItems.Count == 1)
            {
                SessionOpt &= ~GBMSGUI.SessionOption.SequenceCheck;
                AcquisitionMode = GBMSGUI.AcquisitionModes.SingleAcquisition;
            }
            else
            {
                AcquisitionMode = GBMSGUI.AcquisitionModes.MultipleAcquisition;
            }

            // start the acquisition session
            ret = MyGUI.StartSession(AcquisitionMode, PersonID, SessionOpt);
            if (ret != GBMSGUI.ReturnCodes.Ret_Success)
            {
                // re-enable our controls
                btnAcquire.Enabled = true;
                mnuAcquireItem.Enabled = true;

                if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                    MessageBox.Show(MyGUI.GetErrorMessage() + " (StartSession)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                else
                    MessageBox.Show("StartSession error " + ret.ToString(),
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            AcqOptions = DemoFormRef.DemoConfig.AcquisitionOptions;

            //MyGUI.SetFlatFingerOnRollArea(true);
            //MyGUI.SetIgnoreDiagnosticMask(
            //    GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN |
            //    GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT |
            //    GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT |
            //    GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP
            //    );

            MyGUI.SelectFingerContactEvaluationMode(DemoFormRef.DemoConfig.FingerContactEvaluationMode);

            // get object Id from list
            //foreach (ScanItem Item in lstScannedObjects.CheckedItems)
            ScanItem Item;
            for (Index = 0; Index < lstScannedObjects.CheckedItems.Count; Index++)
            {
                Item = (ScanItem)lstScannedObjects.CheckedItems[Index];

                // skip items not supported by the scanner
                if (!Item.Supported)
                    continue;

                // skip Two thumbs for DS40
                if ((DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I) &&
                    (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS))
                    continue;

                // save current item (used in AfterStartCallback)
                CurrentScanItem = Item;

                // set roll area size (MC517 only)
                if (GBMSGUI.IsRolled(Item.ScanObjID) &&
                    ((DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517) || 
                    (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MSC517)))
                {
                    if (DemoFormRef.DemoConfig.RollAreaSize == GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS)
                        GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_SetRollAreaStandard(GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS);
                    else
                        GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_SetRollAreaStandard(GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_GA);
                }

                // set image size
                ret = SetImageSize(Item.ScanObjID);
                if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                {
                    MessageBox.Show(MyGUI.GetErrorMessage() + " (SetImageSize)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        //MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //break;      // break sequence
                }

                // set size for segments
                if (GBMSGUI.IsSlap(Item.ScanObjID))
                {
                    ret = SetSegmentsImageSize(Item.ScanObjID);
                    if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                    {
                        MessageBox.Show(MyGUI.GetErrorMessage() + " (SetSegmentsImageSize)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        //MessageBoxButtons.OK, MessageBoxIcon.Error);
                        //break;      // break sequence
                    }
                }

                // quality thresholds for each object
                ret = MyGUI.SetQualityThresholds(Item.ScanObjID, QualityThreshold1, QualityThreshold2);
                if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                {
                    MessageBox.Show(MyGUI.GetErrorMessage() + " (SetQualityThresholds)",
                        Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;      // break sequence
                }

                if (GBMSGUI.IsSlap(Item.ScanObjID))
                {
                    // quality thresholds for segments
                    MyGUI.SetSegmentQualityThresholds(Item.ScanObjID, 0, QualityThreshold1, QualityThreshold2);
                    MyGUI.SetSegmentQualityThresholds(Item.ScanObjID, 1, QualityThreshold1, QualityThreshold2);
                    if ((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT))
                    {
                        MyGUI.SetSegmentQualityThresholds(Item.ScanObjID, 2, QualityThreshold1, QualityThreshold2);
                        MyGUI.SetSegmentQualityThresholds(Item.ScanObjID, 3, QualityThreshold1, QualityThreshold2);
                    }

                    // missing fingers
                    SetMissingFingers();
                }

                //if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                //    // TEST - set left ring as unavailable
                //    MyGUI.SetSegmentUnavailabilityReason(Item.ScanObjID, 2, GBMSGUI.UnavailabilityReason.Unprintable);

                // set frame rate
                uint DeviceFeatures;
                GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetDeviceFeatures(out DeviceFeatures);
                if (GBMSGUI.CheckMask(DeviceFeatures, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_FRAME_RATE_SETTING))
                {
                    double FrameRate = 5;
                    if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26)
                        FrameRate = DemoFormRef.DemoConfig.DS26FrameRate;
                    else if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84)
                    {
                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                            FrameRate = DemoFormRef.DemoConfig.DS84PartialFrameRate;
                        else
                            FrameRate = DemoFormRef.DemoConfig.DS84FullFrameRate;
                    }
                    else if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS40I)
                    {
                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                            FrameRate = DemoFormRef.DemoConfig.DS40iPartialFrameRate;
                        else
                            FrameRate = DemoFormRef.DemoConfig.DS40iFullFrameRate;
                    }
                    else if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS84C)
                    {
                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                            FrameRate = DemoFormRef.DemoConfig.DS84cPartialFrameRate;
                        else
                        {
                            // hi or low res
                            if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview))
                                FrameRate = DemoFormRef.DemoConfig.DS84cFullHiResFrameRate;
                            else
                                FrameRate = DemoFormRef.DemoConfig.DS84cFullLowResFrameRate;
                        }
                    }
                    else if ((DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC517) ||
                        (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MSC517))
                    {
                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                        {
                            if (DemoFormRef.DemoConfig.RollAreaSize == GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_ROLL_AREA_IQS)
                                FrameRate = DemoFormRef.DemoConfig.MC517PartialIQSFrameRate;
                            else
                                FrameRate = DemoFormRef.DemoConfig.MC517PartialGAFrameRate;
                        }
                        else
                        {
                            // hi or low res
                            if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview))
                                FrameRate = DemoFormRef.DemoConfig.MC517FullHiResFrameRate;
                            else
                                FrameRate = DemoFormRef.DemoConfig.MC517FullLowResFrameRate;
                        }
                    }
                    else if (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS32)
                    {
                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                            FrameRate = DemoFormRef.DemoConfig.DS32PartialFrameRate;
                        else
                            FrameRate = DemoFormRef.DemoConfig.DS32FullFrameRate;
                    }

                    uint FrameRateOptions = 0;
                    if (GBMSGUI.IsRolled(Item.ScanObjID))
                        FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_ROLL_AREA;
                    if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview))
                        FrameRateOptions |= GBMSAPI_NET_FrameRateOptions.GBMSAPI_NET_FRO_FULL_RESOLUTION_MODE;

                    ret = GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_SetFrameRateComplete(FrameRateOptions, FrameRate);
                    if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        MessageBox.Show(MyGUI.GetMSAPIErrorMessage("GBMSAPI_NET_SetFrameRateComplete", ret),
                                Application.ProductName,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;      // break sequence
                    }
                }

            Repeat:
                // acquire item
                ret = MyGUI.Acquire(Item.ScanObjID, AcqOptions, ImageBuffer, out ImageSizeX, out ImageSizeY, out Resolution);

                // if single acquisition (acquisition window is hidden after acquire), re-enable our window now
                if (AcquisitionMode == GBMSGUI.AcquisitionModes.SingleAcquisition)
                {
                    btnAcquire.Enabled = true;
                    mnuAcquireItem.Enabled = true;
                }

                if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                {
                    DialogResult Res = MessageBox.Show(MyGUI.GetErrorMessage() + " (Acquire)", 
                        Application.ProductName,
                        MessageBoxButtons.AbortRetryIgnore, MessageBoxIcon.Exclamation);
                    if (Res == DialogResult.Abort)
                        break;      // break sequence
                    else if (Res == DialogResult.Retry)
                    {
                        // repeat same acquisition
                        goto Repeat;
                    }
                    else if (Res == DialogResult.Ignore)
                        continue;   // go on
                }
                else if (ret == GBMSGUI.ReturnCodes.Ret_BreakSequence)
                {
                    break;
                }
                else if (ret == GBMSGUI.ReturnCodes.Ret_SkipImage)
                {
                    // if option is set, get unavailability reason
                    if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason))
                    {
                        MyGUI.GetUnavailabilityReason(Item.ScanObjID, out Item.ItemData.UnavailabilityReason);
                    }

                    continue;   // go on
                }
                else if (ret == GBMSGUI.ReturnCodes.Ret_GoBack)
                {
                    if (Index > 0)
                    {
                        Index--;
                        Index--;
                        continue;
                    }
                    else
                        break;
                }
                else if (ret == GBMSGUI.ReturnCodes.Ret_Success)
                {
                    // save image in Bmp format
                    bmp = GBMSGUI.RawImageToBitmap(ImageBuffer, ImageSizeX, ImageSizeY);
                    String FileName = BuildImageFileName(Item);
                    bmp.SetResolution((float)Resolution, (float)Resolution);
                    bmp.Save(FileName, ImageFormat.Bmp);
                    bmp.Dispose();
                    // display image
                    Item.ImagePictureBox.Load(FileName);

                    // save quality
                    ret = MyGUI.GetIAFISQuality(Item.ScanObjID, DemoFormRef.DemoConfig.IAFIsQualityAlgorithm,
                        out Item.ItemData.Quality);
                    Item.ItemData.QualityAlgorithm = DemoFormRef.DemoConfig.IAFIsQualityAlgorithm;

                    // if slap, save also segments
                    if (GBMSGUI.IsSlap(Item.ScanObjID))
                    {
                        int i, num;
                        int SegmLeft, SegmTop, SegmRight, SegmBottom, Quality, UnavailReason,
                            SegmSizeX, SegmSizeY;
                        bool SegmAvailable;
                        SegmentData[] SegmentsData;

                        if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        {
                            num = 4;
                            SegmentsData = LeftSlapSegmentsData;
                        }
                        else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        {
                            num = 4;
                            SegmentsData = RightSlapSegmentsData;
                        }
                        else //if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                        {
                            num = 2;
                            SegmentsData = TwoThumbsSegmentsData;
                        }

                        for (i = 0; i < num; i++)
                        {
                            ret = MyGUI.GetSegmentationResult(i, DemoFormRef.DemoConfig.IAFIsQualityAlgorithm,
                                out SegmAvailable, out SegmLeft, out SegmTop, out SegmRight, out SegmBottom,
                                out UnavailReason, out Quality,
                                ImageBuffer, out SegmSizeX, out SegmSizeY, out Resolution);
                            if (ret == GBMSGUI.ReturnCodes.Ret_Failure)
                            {
                                MessageBox.Show(MyGUI.GetErrorMessage() + " (GetSegmentationResult)",
                                    Application.ProductName,
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }

                            if (SegmAvailable)
                            {
                                // save segment image
                                bmp = GBMSGUI.RawImageToBitmap(ImageBuffer, SegmSizeX, SegmSizeY);
                                FileName = BuildSegmentFileName(Item, i);
                                bmp.SetResolution((float)Resolution, (float)Resolution);
                                bmp.Save(FileName, ImageFormat.Bmp);
                                bmp.Dispose();
                                // display image
                                PictureBox pbox = GetSegmentPictureBox(Item.ScanObjID, i);
                                pbox.Load(FileName);

                                // save segments data
                                SegmentsData[i].ItemData.Quality = Quality;
                                SegmentsData[i].ItemData.QualityAlgorithm = DemoFormRef.DemoConfig.IAFIsQualityAlgorithm;
                                SegmentsData[i].BoundingBox.X = SegmLeft;
                                SegmentsData[i].BoundingBox.Y = SegmTop;
                                SegmentsData[i].BoundingBox.Width = SegmRight - SegmLeft;
                                SegmentsData[i].BoundingBox.Height = SegmBottom - SegmTop;
                            }
                            else
                            {
                                // save unavailability reason
                                if (GBMSGUI.CheckMask(DemoFormRef.DemoConfig.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason))
                                    SegmentsData[i].ItemData.UnavailabilityReason = UnavailReason;
                            }
                        }
                    }
                }
            }

            // stop session
            MyGUI.StopSession();

            // save current window size and position
            int WLeft, WTop, WWidth, WHeight;
            bool WMaximized;
            MyGUI.GetWindowSizeAndPosition(out WLeft, out WTop, out WWidth, out WHeight, out WMaximized);
            Rectangle wRect = new Rectangle(WLeft, WTop, WWidth, WHeight);
            DemoFormRef.DemoConfig.WindowSize = wRect;
            DemoFormRef.DemoConfig.WindowMaximized = WMaximized;

            // re-enable our controls
            btnAcquire.Enabled = true;
            mnuAcquireItem.Enabled = true;
        }

        private int SetImageSize(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);
            int ret;
            double ImageWidth, ImageHeight; // size in inches
            uint PixelWidth, PixelHeight;   // size in pixels

            ImageWidth = ImageHeight = 0;

            // read from configuration the size for the current object
            if (GBMSGUI.IsUpperPalm(ScanObjID))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.UpperPalmWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.UpperPalmHeight;
            }
            else if (GBMSGUI.IsLowerPalm(ScanObjID))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.LowerPalmWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.LowerPalmHeight;
            }
            else if (GBMSGUI.IsWritersPalm(ScanObjID))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.WritersPalmWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.WritersPalmHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.FourFingersWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.FourFingersHeight;
            }
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.TwoThumbsWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.TwoThumbsHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.FlatThumbWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.FlatThumbHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.RolledThumbWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.RolledThumbHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.RolledIndexWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.RolledIndexHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.RolledMiddleWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.RolledMiddleHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.RolledRingWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.RolledRingHeight;
            }
            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.RolledLittleWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.RolledLittleHeight;
            }
            else if (GBMSGUI.IsFlatSingleFinger(ScanObjID))
            {
                ImageWidth = DemoFormRef.DemoConfig.ImageSize.FlatFingerWidth;
                ImageHeight = DemoFormRef.DemoConfig.ImageSize.FlatFingerHeight;
            }

            if ((ImageWidth != 0) && (ImageHeight != 0))
            {
                // convert to pixel
                if (DemoFormRef.Resolution1000Dpi)
                {
                    PixelWidth = (uint)(ImageWidth * 1000);
                    PixelHeight = (uint)(ImageHeight * 1000);
                }
                else
                {
                    PixelWidth = (uint)(ImageWidth * 500);
                    PixelHeight = (uint)(ImageHeight * 500);
                }
                ret = MyGUI.SetImageSize(ScanObjID, PixelWidth, PixelHeight);
                return ret;
            }

            return GBMSGUI.ReturnCodes.Ret_Success;
        }

        private int SetSegmentsImageSize(uint ScanObjID)
        {
            double SegmWidth, SegmHeight;     // sizes for segments
            uint PixelWidth, PixelHeight;
            int ret = GBMSGUI.ReturnCodes.Ret_Success;

            SegmWidth = SegmHeight = 0;

            if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT))
            {
                // set sizes for segments using flat finger setting
                SegmWidth = DemoFormRef.DemoConfig.ImageSize.FlatFingerWidth;
                SegmHeight = DemoFormRef.DemoConfig.ImageSize.FlatFingerHeight;
            }
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
            {
                // set sizes for segments using flat thumb setting
                SegmWidth = DemoFormRef.DemoConfig.ImageSize.FlatThumbWidth;
                SegmHeight = DemoFormRef.DemoConfig.ImageSize.FlatThumbHeight;
            }

            if ((SegmWidth != 0) && (SegmHeight != 0))
            {
                // convert to pixel
                if (DemoFormRef.Resolution1000Dpi)
                {
                    PixelWidth = (uint)(SegmWidth * 1000);
                    PixelHeight = (uint)(SegmHeight * 1000);
                }
                else
                {
                    PixelWidth = (uint)(SegmWidth * 500);
                    PixelHeight = (uint)(SegmHeight * 500);
                }

                int num;
                if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                    num = 2;
                else
                    num = 4;
                for (int i = 0; i < num; i++)
                {
                    ret = MyGUI.SetSegmentImageSize(ScanObjID, i, PixelWidth, PixelHeight);
                    //if (ret != GBMSGUI.ReturnCodes.Ret_Success)
                    //    break;
                }

                return ret;
            }

            return GBMSGUI.ReturnCodes.Ret_Success;
        }

        // get the correct picturebox where display segment image (in the place of the flat fingerprints)
        private PictureBox GetSegmentPictureBox(uint ScanObjID, int Index)
        {
            if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
            {
                if (Index == 0)
                    return pboxLeftFlatIndex;
                if (Index == 1)
                    return pboxLeftFlatMiddle;
                if (Index == 2)
                    return pboxLeftFlatRing;
                if (Index == 3)
                    return pboxLeftFlatLittle;
            }
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
            {
                if (Index == 0)
                    return pboxRightFlatIndex;
                if (Index == 1)
                    return pboxRightFlatMiddle;
                if (Index == 2)
                    return pboxRightFlatRing;
                if (Index == 3)
                    return pboxRightFlatLittle;
            }
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
            {
                if (Index == 0)
                    return pboxLeftFlatThumb;
                if (Index == 1)
                    return pboxRightFlatThumb;
            }
            //V1.1
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
            {
                if (Index == 0)
                    return pboxLeftFlatIndex;
                if (Index == 1)
                    return pboxLeftFlatMiddle;
            }
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT)
            {
                if (Index == 0)
                    return pboxRightFlatIndex;
                if (Index == 1)
                    return pboxRightFlatMiddle;
            }
            //V1.2
            else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
            {
                if (Index == 0)
                    return pboxLeftFlatIndex;
                if (Index == 1)
                    return pboxRightFlatIndex;
            }

            return null;
        }

        private void TouchScreenTimer_Tick(object sender, EventArgs e)
        {
            Byte PressedButton;

            TouchScreenTimer.Enabled = false;

            // check LCD Start button
            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_GetPressedButton(out PressedButton);
            if (PressedButton == GBMSAPI_NET_PressedButtonIDs.GBMSAPI_NET_VILCD_TOUCHSCREEN_START_BUTTON)
            {
                btnAcquire_Click(this, null);
                return;
            }

            TouchScreenTimer.Enabled = true;
        }

        private void EmptyFolder(String FolderName)
        {
            String[] Files = Directory.GetFiles(FolderName);

            foreach (String FileName in Files)
            {
                if (File.Exists(FileName))
                {
                    try
                    {
                        File.Delete(FileName);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
            }
        }

        private void UserDataForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel)
            {
                if (!ViewMode)
                {
                    // delete directory and contents
                    Directory.Delete(ImagesPath, true);
                }
            }

            TouchScreenTimer.Enabled = false;

            if (!ViewMode)
            {
                // save configuration of types for sequence
                if (chkSlaps.Checked)
                    DemoFormRef.DemoConfig.SequenceTypes |= DemoForm.SequenceType.Slaps;
                else
                    DemoFormRef.DemoConfig.SequenceTypes &= ~DemoForm.SequenceType.Slaps;
                if (chkFlat.Checked)
                    DemoFormRef.DemoConfig.SequenceTypes |= DemoForm.SequenceType.SingleFlat;
                else 
                    DemoFormRef.DemoConfig.SequenceTypes &= ~DemoForm.SequenceType.SingleFlat;
                if (chkRolled.Checked)
                    DemoFormRef.DemoConfig.SequenceTypes |= DemoForm.SequenceType.Rolled;
                else
                    DemoFormRef.DemoConfig.SequenceTypes &= ~DemoForm.SequenceType.Rolled;
                if (chkPalms.Checked)
                    DemoFormRef.DemoConfig.SequenceTypes |= DemoForm.SequenceType.Palms;
                else
                    DemoFormRef.DemoConfig.SequenceTypes &= ~DemoForm.SequenceType.Palms;

                // save configuration
                DemoForm.Configuration.Serialize(Path.ChangeExtension(Application.ExecutablePath, ".cfg"), DemoFormRef.DemoConfig);

                if (DemoFormRef.LCDPresent)
                    // return to Logo Screen
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetLogoScreen();
            }

            // free images
            foreach (ScanItem Item in lstScannedObjects.Items)
                if (Item.ImagePictureBox.Image != null)
                    Item.ImagePictureBox.Image.Dispose();
            lstScannedObjects.Dispose();

            if (gcUserDataFormHandle.IsAllocated)
                gcUserDataFormHandle.Free();

            if (UserDataForm.gcUserDataFormHandle_St.IsAllocated)
                UserDataForm.gcUserDataFormHandle_St.Free();
        }

        private void popAcquireItemMenu_Popup(object sender, EventArgs e)
        {
            // search current item
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                if (Item.Supported)
                    mnuAcquireItem.Visible = true;
                else
                    mnuAcquireItem.Visible = false;
                if (Item.ImagePictureBox == popAcquireItemMenu.SourceControl)
                {
                    if (Item.ImagePictureBox.Image != null)
                        mnuViewItem.Visible = true;
                    else
                        mnuViewItem.Visible = false;
                    break;
                }
                else
                {
                    // item not in the list
                    mnuAcquireItem.Visible = false;
                    mnuViewItem.Visible = false;
                }
            }
        }

        private void SelectAllScanItems()
        {
            // select all items
            for (int i = 0; i < lstScannedObjects.Items.Count; i++)
            {
                ScanItem Item = (ScanItem)lstScannedObjects.Items[i];

                if (GBMSGUI.IsSlap(Item.ScanObjID) && chkSlaps.Checked && Item.Supported)
                    lstScannedObjects.SetItemChecked(i, true);
                else if (GBMSGUI.IsRolled(Item.ScanObjID) && chkRolled.Checked && Item.Supported)
                    lstScannedObjects.SetItemChecked(i, true);
                else if (GBMSGUI.IsFlatSingleFinger(Item.ScanObjID) && chkFlat.Checked && Item.Supported)
                    lstScannedObjects.SetItemChecked(i, true);
                else if (GBMSGUI.IsPalm(Item.ScanObjID) && chkPalms.Checked && Item.Supported)
                    lstScannedObjects.SetItemChecked(i, true);
                else
                    lstScannedObjects.SetItemChecked(i, false);

                // check if finger is marked as missing
                if (GBMSGUI.IsSingleFinger(Item.ScanObjID))
                {
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE))
                        && chkMissingLeftLittle.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING))
                        && chkMissingLeftRing.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE))
                        && chkMissingLeftMiddle.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX))
                        && chkMissingLeftIndex.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB))
                        && chkMissingLeftThumb.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB))
                        && chkMissingRightThumb.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX))
                        && chkMissingRightIndex.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE))
                        && chkMissingRightMiddle.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING))
                        && chkMissingRightRing.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                    if (((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE) ||
                        (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE))
                        && chkMissingRightLittle.Checked)
                        lstScannedObjects.SetItemChecked(i, false);
                }
            }
        }

        private void UnselectAllScanItems()
        {
            // select all items
            for (int i = 0; i < lstScannedObjects.Items.Count; i++)
                lstScannedObjects.SetItemChecked(i, false);
        }

        private String BuildImageFileName(ScanItem Item)
        {
            String FileName = ImagesPath + Path.DirectorySeparatorChar + Item.Text + ".bmp";
            return FileName;
        }

        private String BuildSegmentFileName(ScanItem Item, int Index)
        {
            String FileName = ImagesPath + Path.DirectorySeparatorChar + Item.Text + "_" + Index.ToString() + ".bmp";
            return FileName;
        }

        // acquire a single item, called from context menu
        private void mnuAcquireItem_Click(object sender, EventArgs e)
        {
            UnselectAllScanItems();

            // search current item
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                if (Item.ImagePictureBox == popAcquireItemMenu.SourceControl)
                {
                    lstScannedObjects.SetItemChecked(lstScannedObjects.Items.IndexOf(Item), true);
                    // delete file
                    String FileName = BuildImageFileName(Item);
                    if (File.Exists(FileName))
                        File.Delete(FileName);
                    // clear picturebox
                    if (Item.ImagePictureBox.Image != null)
                        Item.ImagePictureBox.Image.Dispose();
                    Item.ImagePictureBox.Image = null;
                    // reset item data
                    Item.ItemData.Quality = 0;
                    Item.ItemData.QualityAlgorithm = 0;
                    Item.ItemData.UnavailabilityReason = 0;

                    // if slap, delete and clear also segments
                    if (GBMSGUI.IsSlap(Item.ScanObjID))
                    {
                        int i, num = 0;
                        if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        {
                            num = 4;
                            ResetSlapSegmentsData(LeftSlapSegmentsData, num);
                        }
                        else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        {
                            num = 4;
                            ResetSlapSegmentsData(RightSlapSegmentsData, num);
                        }
                        else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                        {
                            num = 2;
                            ResetSlapSegmentsData(TwoThumbsSegmentsData, num);
                        }

                        for (i = 0; i < num; i++)
                        {
                            FileName = BuildSegmentFileName(Item, i);
                            if (File.Exists(FileName))
                            {
                                File.Delete(FileName);
                                // clear image
                                PictureBox pbox = GetSegmentPictureBox(Item.ScanObjID, i);
                                pbox.Image = null;
                            }
                        }
                    }

                    // launch acquisition with a single item selected
                    Acquire();

                    // after acquisition, re-enable timer
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen();
                    TouchScreenTimer.Enabled = true;
                    break;
                }
            }

            // select all again
            SelectAllScanItems();
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (!ViewMode)
            {
                if (txtSurname.Text == "")
                {
                    MessageBox.Show("Specify at least Surname!", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

                UserData.Surname = txtSurname.Text;
                UserData.Name = txtName.Text;

                // save user data
                DemoForm.UserData.Serialize(ImagesPath + Path.DirectorySeparatorChar + "UserData.xml", UserData);

                // save other data
                SaveItemsData();
                SaveSegmentsData();

                // rename folder with User name
                String NewFolderName = txtSurname.Text;
                if (txtName.Text.Length != 0)
                    NewFolderName = NewFolderName + " " + txtName.Text;

                // remove invalid chars (if any)
                NewFolderName = MakeValidFileName(NewFolderName);

                NewFolderName = Path.GetDirectoryName(Application.ExecutablePath) + Path.DirectorySeparatorChar + NewFolderName;

                // check duplicate
                if (Directory.Exists(NewFolderName))
                {
                    MessageBox.Show("Duplicated name for folder! Specify a different Surname/Name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                try
                {
                    // rename folder
                    Directory.Move(ImagesPath, NewFolderName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            DialogResult = DialogResult.OK;
        }

        private static string MakeValidFileName(string name)
        {
            string invalidChars = Regex.Escape(new string(Path.GetInvalidFileNameChars()));
            string invalidReStr = string.Format(@"([{0}]*\.+$)|([{0}]+)", invalidChars);
            return Regex.Replace(name, invalidReStr, "_");
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            // Cancel code is in UserDataForm_FormClosing
        }

        private void mnuViewItem_Click(object sender, EventArgs e)
        {
            // search current item
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                if (Item.ImagePictureBox == popAcquireItemMenu.SourceControl)
                {
                    ViewImageForm ViewForm = new ViewImageForm();
                    ViewForm.Text = Item.Text;
                    ViewForm.bmpImage = Item.ImagePictureBox.Image;
                    ViewForm.ShowDialog();
                }
            }
        }

        private void LoadSavedImages()
        {
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                String FileName = BuildImageFileName(Item);
                if (File.Exists(FileName))
                {
                    if (Item.ImagePictureBox.Image != null)
                        Item.ImagePictureBox.Image.Dispose();
                    Item.ImagePictureBox.Load(FileName);
                    Application.DoEvents();
                }

                // load segments
                if (GBMSGUI.IsSlap(Item.ScanObjID))
                {
                    int i, num;
                    if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                        num = 2;
                    else
                        num = 4;
                    for (i = 0; i < num; i++)
                    {
                        FileName = BuildSegmentFileName(Item, i);
                        if (File.Exists(FileName))
                        {
                            // display image
                            PictureBox pbox = GetSegmentPictureBox(Item.ScanObjID, i);
                            pbox.Load(FileName);
                        }
                    }
                }
            }
        }

        private void btnConfig_Click(object sender, EventArgs e)
        {
            ConfigurationForm ConfigForm = new ConfigurationForm(DemoFormRef);

            ConfigForm.ShowDialog();
        }

        private bool AfterStartCallback()
        {
            // for example
            if (GBMSGUI.IsRolled(CurrentScanItem.ScanObjID))
                GBMSAPI_NET_ScannerStartedRoutines.GBMSAPI_NET_ROLL_SetPreviewTimeout(1500);

            return true;
        }

        // save acquired items data in a binary file
        private bool SaveItemsData()
        {
            int Count = 0;
            FileStream fs;
            BinaryWriter w;

            try
            {
                fs = new FileStream(ImagesPath + Path.DirectorySeparatorChar + "ItemsData.dat", FileMode.Create);
                w = new BinaryWriter(fs);

                // count items to be written
                foreach (ScanItem Item in lstScannedObjects.Items)
                    if ((Item.ItemData.Quality != 0) || (Item.ItemData.UnavailabilityReason != 0))
                        Count++;

                // write number of items
                w.Write(Count);

                foreach (ScanItem Item in lstScannedObjects.Items)
                {
                    if ((Item.ItemData.Quality != 0) || (Item.ItemData.UnavailabilityReason != 0))
                    {
                        // save this item
                        w.Write(Item.ScanObjID);
                        w.Write(Item.ItemData.Quality);
                        w.Write(Item.ItemData.QualityAlgorithm);
                        w.Write(Item.ItemData.UnavailabilityReason);
                    }
                }

                w.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        // read saved items data from a binary file
        private bool ReadItemsData()
        {
            FileStream fs;
            BinaryReader r;
            int Count = 0;
            uint ScanObjID;
            int Quality;
            int QualityAlgorithm;
            int UnavailabilityReason;
            ScanItem Item;

            try
            {
                fs = new FileStream(ImagesPath + Path.DirectorySeparatorChar + "ItemsData.dat", FileMode.Open);
                r = new BinaryReader(fs);

                // read number of items
                Count = r.ReadInt32();

                // read items
                for (int i = 0; i < Count; i++)
                {
                    ScanObjID = (uint)r.ReadInt32();
                    Quality = r.ReadInt32();
                    QualityAlgorithm = r.ReadInt32();
                    UnavailabilityReason = r.ReadInt32();

                    // assign values to ScanItem
                    Item = FindScanItem(ScanObjID);
                    if (Item != null)
                    {
                        Item.ItemData.Quality = Quality;
                        Item.ItemData.QualityAlgorithm = QualityAlgorithm;
                        Item.ItemData.UnavailabilityReason = UnavailabilityReason;
                    }
                }

                r.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        // save acquired segments data in a binary file
        private bool SaveSegmentsData()
        {
            int i;
            FileStream fs;
            BinaryWriter w;

            try
            {
                fs = new FileStream(ImagesPath + Path.DirectorySeparatorChar + "SegmentsData.dat", FileMode.Create);
                w = new BinaryWriter(fs);

                // left slap data
                for (i = 0; i < 4; i++)
                {
                    w.Write(LeftSlapSegmentsData[i].ItemData.Quality);
                    w.Write(LeftSlapSegmentsData[i].ItemData.QualityAlgorithm);
                    w.Write(LeftSlapSegmentsData[i].ItemData.UnavailabilityReason);
                    w.Write(LeftSlapSegmentsData[i].BoundingBox.X);
                    w.Write(LeftSlapSegmentsData[i].BoundingBox.Y);
                    w.Write(LeftSlapSegmentsData[i].BoundingBox.Width);
                    w.Write(LeftSlapSegmentsData[i].BoundingBox.Height);
                }

                // right slap data
                for (i = 0; i < 4; i++)
                {
                    w.Write(RightSlapSegmentsData[i].ItemData.Quality);
                    w.Write(RightSlapSegmentsData[i].ItemData.QualityAlgorithm);
                    w.Write(RightSlapSegmentsData[i].ItemData.UnavailabilityReason);
                    w.Write(RightSlapSegmentsData[i].BoundingBox.X);
                    w.Write(RightSlapSegmentsData[i].BoundingBox.Y);
                    w.Write(RightSlapSegmentsData[i].BoundingBox.Width);
                    w.Write(RightSlapSegmentsData[i].BoundingBox.Height);
                }

                // two thumbs data
                for (i = 0; i < 2; i++)
                {
                    w.Write(TwoThumbsSegmentsData[i].ItemData.Quality);
                    w.Write(TwoThumbsSegmentsData[i].ItemData.QualityAlgorithm);
                    w.Write(TwoThumbsSegmentsData[i].ItemData.UnavailabilityReason);
                    w.Write(TwoThumbsSegmentsData[i].BoundingBox.X);
                    w.Write(TwoThumbsSegmentsData[i].BoundingBox.Y);
                    w.Write(TwoThumbsSegmentsData[i].BoundingBox.Width);
                    w.Write(TwoThumbsSegmentsData[i].BoundingBox.Height);
                }

                w.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        // read saved segments data from a binary file
        private bool ReadSegmentsData()
        {
            int i;
            FileStream fs;
            BinaryReader r;

            try
            {
                fs = new FileStream(ImagesPath + Path.DirectorySeparatorChar + "SegmentsData.dat", FileMode.Open);
                r = new BinaryReader(fs);

                // left slap data
                for (i = 0; i < 4; i++)
                {
                    LeftSlapSegmentsData[i].ItemData.Quality = r.ReadInt32();
                    LeftSlapSegmentsData[i].ItemData.QualityAlgorithm = r.ReadInt32();
                    LeftSlapSegmentsData[i].ItemData.UnavailabilityReason = r.ReadInt32();
                    LeftSlapSegmentsData[i].BoundingBox.X = r.ReadInt32();
                    LeftSlapSegmentsData[i].BoundingBox.Y = r.ReadInt32();
                    LeftSlapSegmentsData[i].BoundingBox.Width = r.ReadInt32();
                    LeftSlapSegmentsData[i].BoundingBox.Height = r.ReadInt32();
                }

                // right slap data
                for (i = 0; i < 4; i++)
                {
                    RightSlapSegmentsData[i].ItemData.Quality = r.ReadInt32();
                    RightSlapSegmentsData[i].ItemData.QualityAlgorithm = r.ReadInt32();
                    RightSlapSegmentsData[i].ItemData.UnavailabilityReason = r.ReadInt32();
                    RightSlapSegmentsData[i].BoundingBox.X = r.ReadInt32();
                    RightSlapSegmentsData[i].BoundingBox.Y = r.ReadInt32();
                    RightSlapSegmentsData[i].BoundingBox.Width = r.ReadInt32();
                    RightSlapSegmentsData[i].BoundingBox.Height = r.ReadInt32();
                }

                // two thumbs data
                for (i = 0; i < 2; i++)
                {
                    TwoThumbsSegmentsData[i].ItemData.Quality = r.ReadInt32();
                    TwoThumbsSegmentsData[i].ItemData.QualityAlgorithm = r.ReadInt32();
                    TwoThumbsSegmentsData[i].ItemData.UnavailabilityReason = r.ReadInt32();
                    TwoThumbsSegmentsData[i].BoundingBox.X = r.ReadInt32();
                    TwoThumbsSegmentsData[i].BoundingBox.Y = r.ReadInt32();
                    TwoThumbsSegmentsData[i].BoundingBox.Width = r.ReadInt32();
                    TwoThumbsSegmentsData[i].BoundingBox.Height = r.ReadInt32();
                }

                r.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        // search an item in the list from the ID
        private ScanItem FindScanItem(uint ScanObjID)
        {
            foreach (ScanItem Item in lstScannedObjects.Items)
                if (Item.ScanObjID == ScanObjID)
                    return Item;

            return null;
        }

        // exports saved data into ANSI/NIST format
        private bool ExportEFTS(String FileName, bool UseTemplate, String TemplateName)
        {
            int ret;
            float Resolution;  // pixel per mm
            bool ImageAcquired;
            NW_AN2K_Type14_RECORD RecType14;
            NW_AN2K_Type15_RECORD RecType15;    // for Palms
            int ImageSizeX, ImageSizeY;
            Byte[] CompressedImage;
            int CompressedImageLength;
            String CompressionAlgorithm;
            String FingerPos;
            List<NW_AN2K_FING_AMP> AmpList = new List<NW_AN2K_FING_AMP>();         // amputation list
            List<NW_AN2K_FING_QUALITY> NFIQList;    // NIST quality list
            List<NW_AN2K_ALT_QUAL> AltQualList;     // alternate quality list
            NW_AN2K_ANSI_NIST ansi_nist;
            Byte[] ANSI_NIST_Array;
            NW_AN2K_Type2_RECORD RecType2;
            bool SlapAcquired = false;

            // show log window
            LogForm frmLog = new LogForm("Export to EFTS");
            frmLog.Show();
            this.Enabled = false;
            frmLog.LogMessage("Starting export to EFTS...");
            
            // convert resolution in pixel per mm
            Resolution = (float)UserData.AcquisitionDpi / (float)25.4;

            if (UseTemplate)
            {
                // read ansi/nist buffer from file
                ANSI_NIST_Array = File.ReadAllBytes(TemplateName);
                ansi_nist = new NW_AN2K_ANSI_NIST(ANSI_NIST_Array);

                // update record-1 with image resolution
                ansi_nist.NW_UpdateETFStype_1(Resolution, Resolution);
            }
            else
            {
                // Create ANSI_NIST structure
                ansi_nist = new NW_AN2K_ANSI_NIST(
                    NW_AN2K_VERSIONS.NW_VERSION_0400,
                    "Demo",
                    -1,
                    "Destination",
                    "Green Bit",
                    "000001",
                    null,
                    Resolution,
                    Resolution,
                    null);
                if (ansi_nist.Buffer == null)
                {
                    frmLog.LogMessage("Error creating ANSI/NIST structure");
                    frmLog.EnableClose();
                    this.Enabled = true;
                    return false;
                }
            }

            // create a Type-2 record for eventually unavailable fingers
            RecType2 = new NW_AN2K_Type2_RECORD();

            // cycle on items
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                // skip flat single fingers, if a slap is acquired
                // only information about bounding boxes will be inserted into record for slaps
                if (SlapAcquired)
                    if (GBMSGUI.IsFlatSingleFinger(Item.ScanObjID))
                        continue;

                //V1.2 - no slaps 2
                if ((Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT) ||
                    (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT) ||
                    (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES))
                    continue;

                // if image is present
                if (Item.ImagePictureBox.Image != null)
                {
                    ImageAcquired = true;

                    //// get raw image from bitmap
                    ImageSizeX = Item.ImagePictureBox.Image.Width;
                    ImageSizeY = Item.ImagePictureBox.Image.Height;

                    frmLog.LogMessage(Item.Text + ": compressing image...");

                    // compress image: JPEG2000 for 1000 dpi, WSQ for 500 dpi
                    // 07/10/2009 - image compression specified in configuration
                    int ImageCompression;
                    double WSQRate;
                    int JPEGRate;
                    if (UserData.AcquisitionDpi == 1000)
                    {
                        ImageCompression = DemoFormRef.DemoConfig.ImageCompression1000;
                        WSQRate = DemoFormRef.DemoConfig.WQSBitRate1000;
                        JPEGRate = DemoFormRef.DemoConfig.JPEGRate1000;
                    }
                    else
                    {
                        ImageCompression = DemoFormRef.DemoConfig.ImageCompression500;
                        WSQRate = DemoFormRef.DemoConfig.WQSBitRate500;
                        JPEGRate = DemoFormRef.DemoConfig.JPEGRate500;
                    }
                    if (ImageCompression == DemoForm.ImageCompressions.JPEG2000)
                    {
                        CompressedImage = CompressToJP2((Bitmap)(Item.ImagePictureBox.Image), JPEGRate);
                        CompressedImageLength = CompressedImage.Length;
                        CompressionAlgorithm = NW_AN2K_COMP_ALGORITHMS.NW_JPG2K_COMP;
                    }
                    else
                    {
                        CompressedImage = CompressToWSQ((Bitmap)(Item.ImagePictureBox.Image), WSQRate, UserData.AcquisitionDpi);
                        CompressedImageLength = CompressedImage.Length;
                        CompressionAlgorithm = NW_AN2K_COMP_ALGORITHMS.NW_COMP_WSQ;
                    }
                }
                else
                {
                    ImageAcquired = false;
                    CompressedImage = null;
                    CompressedImageLength = 0;
                    ImageSizeX = 0;
                    ImageSizeY = 0;
                    CompressionAlgorithm = null;
                }

                // if not acquired and no UnavailabilityReason specified, skip
                if (!ImageAcquired && (Item.ItemData.UnavailabilityReason == 0))
                    continue;

                if (GBMSGUI.IsSlap(Item.ScanObjID))
                {
                    SlapAcquired = true;

                    // Slaps
                    RecType14 = new NW_AN2K_Type14_RECORD();
                    SegmentData[] SegmentsData;
                    int i, num;
                    // Bounding box list
                    List<NW_AN2K_FING_SEGMENT> BoundingBoxList = new List<NW_AN2K_FING_SEGMENT>();
                    List<NW_AN2K_FING_AMP> SegmAmpList = new List<NW_AN2K_FING_AMP>();
                    NFIQList = new List<NW_AN2K_FING_QUALITY>();
                    AltQualList = new List<NW_AN2K_ALT_QUAL>();

                    if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                    {
                        num = 4;
                        SegmentsData = LeftSlapSegmentsData;
                    }
                    else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                    {
                        num = 4;
                        SegmentsData = RightSlapSegmentsData;
                    }
                    else // if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                    {
                        num = 2;
                        SegmentsData = TwoThumbsSegmentsData;
                    }

                    for (i = 0; i < num; i++)
                    {
                        FingerPos = SegmentIndexToAN2KFingerPos(Item.ScanObjID, i);

                        if (SegmentsData[i].ItemData.Quality != 0)
                        {
                            // add bounding box item
                            BoundingBoxList.Add(new NW_AN2K_FING_SEGMENT(FingerPos,
                                (short)SegmentsData[i].BoundingBox.Left,
                                (short)SegmentsData[i].BoundingBox.Right,
                                (short)SegmentsData[i].BoundingBox.Top,
                                (short)SegmentsData[i].BoundingBox.Bottom));

                            // add quality
                            if (SegmentsData[i].ItemData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                                NFIQList.Add(new NW_AN2K_FING_QUALITY(FingerPos, (byte)SegmentsData[i].ItemData.Quality));
                            else
                                AltQualList.Add(new NW_AN2K_ALT_QUAL(FingerPos, (byte)SegmentsData[i].ItemData.Quality,
                                    0x0040, // Green Bit Vendor ID
                                    0x0001)); // Green Bit quality algorithm ID
                        }
                        else
                        {
                            if (SegmentsData[i].ItemData.UnavailabilityReason != 0)
                                SegmAmpList.Add(new NW_AN2K_FING_AMP(FingerPos, GetAmpCode(SegmentsData[i].ItemData.UnavailabilityReason)));
                        }
                    }

                    frmLog.LogMessage("Adding record...");

                    // create record
                    ret = RecType14.NW_image2ETFStype_14(CompressedImage, CompressedImageLength,
                        ImageSizeX, ImageSizeY, 8, Resolution, CompressionAlgorithm,
                        NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_PLAN,
                        "GBMSDemo", "",
                        GBMSGUI.ScannedObjectIDToAN2KFingerID(Item.ScanObjID),
                        SegmAmpList,
                        BoundingBoxList,
                        NFIQList,
                        null,
                        AltQualList);
                    if (ret != 0)
                    {
                        frmLog.LogMessage("Error creating record type-14");
                    }

                    // add record to ANSI/NIST structure
                    ret = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(RecType14, ansi_nist.num_records);
                    if (ret != 0)
                    {
                        frmLog.LogMessage("Error adding record type-14");
                    }
                }
                else if (GBMSGUI.IsPalm(Item.ScanObjID))
                {
                    FingerPos = GBMSGUI.ScannedObjectIDToAN2KFingerID(Item.ScanObjID);
                    if (ImageAcquired)
                    {
                        // Palms
                        RecType15 = new NW_AN2K_Type15_RECORD();

                        // add quality
                        List <NW_AN2K_PALM_QUAL> PalmQualList = new List <NW_AN2K_PALM_QUAL>();
                        if (Item.ItemData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.GBAlgorithm)
                            PalmQualList.Add(new NW_AN2K_PALM_QUAL(FingerPos, 
                                    (byte)Item.ItemData.Quality,
                                    0x0040, // Green Bit Vendor ID
                                    0x0001)); // Green Bit quality algorithm ID

                        frmLog.LogMessage("Adding record...");

                        // create record
                        ret = RecType15.NW_image2ETFStype_15(CompressedImage, CompressedImageLength,
                            ImageSizeX, ImageSizeY, 8, Resolution, CompressionAlgorithm,
                            NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_PLAN,
                            "GBMSDemo", "",
                            FingerPos,
                            PalmQualList);
                        if (ret != 0)
                        {
                            frmLog.LogMessage("Error creating record type-15");
                        }
    
                        // add record to ANSI/NIST structure
                        ret = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(RecType15, ansi_nist.num_records);
                        if (ret != 0)
                        {
                            frmLog.LogMessage("Error adding record type-15");
                        }
                    }
                    else
                    {
                        if (Item.ItemData.UnavailabilityReason != 0)
                        {
                            // add to amputation list in type-2 record
                            AmpList.Add(new NW_AN2K_FING_AMP(FingerPos, GetAmpCode(Item.ItemData.UnavailabilityReason)));
                        }
                    }
                }
                //else if (GBMSGUI.IsRolled(Item.ScanObjID))
                else if (GBMSGUI.IsRolled(Item.ScanObjID) || GBMSGUI.IsFlatSingleFinger(Item.ScanObjID))
                {
                    FingerPos = GBMSGUI.ScannedObjectIDToAN2KFingerID(Item.ScanObjID);
                    int ImpressionCode;
                    // Rolled
                    if (ImageAcquired)
                    {
                        RecType14 = new NW_AN2K_Type14_RECORD();

                        NFIQList = new List<NW_AN2K_FING_QUALITY>();
                        AltQualList = new List<NW_AN2K_ALT_QUAL>();
                        // add quality
                        if (Item.ItemData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                            NFIQList.Add(new NW_AN2K_FING_QUALITY(FingerPos, (byte)Item.ItemData.Quality));
                        else
                            AltQualList.Add(new NW_AN2K_ALT_QUAL(FingerPos, 
                                    (byte)Item.ItemData.Quality,
                                    0x0040, // Green Bit Vendor ID
                                    0x0001)); // Green Bit quality algorithm ID

                        frmLog.LogMessage("Adding record...");

                        if (GBMSGUI.IsRolled(Item.ScanObjID))
                            ImpressionCode = NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_ROLLED;
                        else
                            ImpressionCode = NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_PLAN;

                        // create record
                        ret = RecType14.NW_image2ETFStype_14(CompressedImage, CompressedImageLength,
                            ImageSizeX, ImageSizeY, 8, Resolution, CompressionAlgorithm,
                            ImpressionCode,
                            "GBMSDemo", "",
                            FingerPos,
                            null,
                            null,
                            NFIQList,
                            null,
                            AltQualList);
                        if (ret != 0)
                        {
                            frmLog.LogMessage("Error creating record type-14");
                        }

                        // add record to ANSI/NIST structure
                        ret = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(RecType14, ansi_nist.num_records);
                        if (ret != 0)
                        {
                            frmLog.LogMessage("Error adding record type-14");
                        }
                    }
                    else
                    {
                        if (Item.ItemData.UnavailabilityReason != 0)
                        {
                            // add to amputation list in type-2 record
                            AmpList.Add(new NW_AN2K_FING_AMP(FingerPos, GetAmpCode(Item.ItemData.UnavailabilityReason)));
                        }
                    }
                }
            }

            if (!UseTemplate)
            {
                frmLog.LogMessage("Adding Type-2 record...");

                // add Type-2 record
                ret = RecType2.NW_CreateETFStype_2("Green Bit", UserData.DeviceName, UserData.DeviceSerialNumber, AmpList);
                if (ret != 0)
                {
                    frmLog.LogMessage("Error creating record type-2");
                }
                // add Type-2 record to ANSI/NIST structure
                ret = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(RecType2, ansi_nist.num_records);
                if (ret != 0)
                {
                    frmLog.LogMessage("Error adding record type-2");
                }
            }

            frmLog.LogMessage("Saving ANSI/NIST structure to file...");

            // save ansi_nist record
            ANSI_NIST_Array = ansi_nist.Buffer;
            //File.WriteAllBytes(ImagesPath + Path.DirectorySeparatorChar + "EFTS_Record.bin", ANSI_NIST_Array);
            File.WriteAllBytes(FileName, ANSI_NIST_Array);

            //frmLog.LogMessage("ANSI/NIST structure exported to EFTS_Record.bin");
            frmLog.LogMessage("ANSI/NIST structure exported to " + FileName);
            frmLog.EnableClose();
            this.Enabled = true;

            return true;
        }

        // converts slap ScanObjID and segment index into AN2K finger position
        private String SegmentIndexToAN2KFingerPos(uint ScanObjID, int Index)
        {
            switch (Index)
            {
                case 0:
                    if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                    break;

                case 1:
                    if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                    break;

                case 2:
                    if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                    break;

                case 3:
                    if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                    else if (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                        return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                    break;
            }

            return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN;
        }

        private String GetAmpCode(int Value)
        {
            if (Value == GBMSGUI.UnavailabilityReason.Amputated)
                return AN2K_NET_WRAPPER.NW_AN2K_AMPUTATION_CODES.NW_AMPUTATION;

            if (Value == GBMSGUI.UnavailabilityReason.Unprintable)
                return AN2K_NET_WRAPPER.NW_AN2K_AMPUTATION_CODES.NW_UNABLE_TO_PRINT;

            return "";
        }

        private void btnExportEFTS_Click(object sender, EventArgs e)
        {
            bool UseTemplate = false;
            String FileName;
            String TemplateName = "";

            openFileDialog1.Title = "Select ANSI/NIST Template file";
            openFileDialog1.Filter = "ANSI/NIST-ITL 1-200x files|*.nist;*.nst;*.eft;*.an2;*.an2k;*.bin";
            openFileDialog1.FileName = "";

            saveFileDialog1.Title = "Save ANSI/NIST file";
            saveFileDialog1.Filter = openFileDialog1.Filter;
            saveFileDialog1.DefaultExt = "nist";
            saveFileDialog1.FileName = "NISTFile";
            saveFileDialog1.OverwritePrompt = true;

            if (optEFTSTemplate.Checked)
            {
                // ask the user location and name of the template file
                openFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
                if (openFileDialog1.ShowDialog() == DialogResult.Cancel)
                    return;
                TemplateName = openFileDialog1.FileName;
                UseTemplate = true;
                saveFileDialog1.InitialDirectory = Path.GetDirectoryName(openFileDialog1.FileName);
            }
            else
                saveFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);

            // ask the user location and name of the file to create
            if (saveFileDialog1.ShowDialog() == DialogResult.Cancel)
                return;
            FileName = saveFileDialog1.FileName;

            Cursor = Cursors.WaitCursor;
            ExportEFTS(FileName, UseTemplate, TemplateName);
            Cursor = Cursors.Default;
        }

        private void ResetScanItemsData()
        {
            foreach (ScanItem Item in lstScannedObjects.Items)
            {
                Item.ItemData.Quality = 0;
                Item.ItemData.QualityAlgorithm = 0;
                Item.ItemData.UnavailabilityReason = 0;

                if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                    ResetSlapSegmentsData(LeftSlapSegmentsData, 4);
                else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                    ResetSlapSegmentsData(RightSlapSegmentsData, 4);
                else if (Item.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                    ResetSlapSegmentsData(TwoThumbsSegmentsData, 2);
            }
        }

        private void ResetSlapSegmentsData(SegmentData[] SegmentsData, int count)
        {
            for (int i = 0; i < count; i++)
            {
                SegmentsData[i].BoundingBox.X = 0;
                SegmentsData[i].BoundingBox.Y = 0;
                SegmentsData[i].BoundingBox.Width = 0;
                SegmentsData[i].BoundingBox.Height = 0;
                SegmentsData[i].ItemData.Quality = 0;
                SegmentsData[i].ItemData.QualityAlgorithm = 0;
                SegmentsData[i].ItemData.UnavailabilityReason = 0;
            }
        }

        private Byte[] CompressToWSQ(Bitmap bmpImage, double Rate, int Dpi)
        {
            Byte[] RawImage;
            int ImageSizeX, ImageSizeY;
            Byte[] CompressedImage;

            // get raw image from bitmap
            ImageSizeX = bmpImage.Width;
            ImageSizeY = bmpImage.Height;
            RawImage = new byte[ImageSizeX * ImageSizeY];
            GBMSGUI.BitmapToRawImage(bmpImage, RawImage);

            // create WSQ encoder
            NW_WSQPACK_Compress WSQEncoder = new NW_WSQPACK_Compress(RawImage, ImageSizeX, ImageSizeY);
            WSQEncoder.BitPerPixel = NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS.NW_WSQPACK_8_BITS_PER_PIXEL;
            //WSQEncoder.CompressionRate = NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
            WSQEncoder.CompressionRate = (float)Rate;
            WSQEncoder.ImageResolution = Dpi; // in DPI
            // encode
            CompressedImage = WSQEncoder.Encoded;

            return CompressedImage;
        }

        private Byte[] CompressToJP2(Bitmap bmpImage, int Rate)
        {
            Byte[] RawImage;
            int ImageSizeX, ImageSizeY;
            Byte[] CompressedImage;

            // get raw image from bitmap
            ImageSizeX = bmpImage.Width;
            ImageSizeY = bmpImage.Height;
            RawImage = new byte[ImageSizeX * ImageSizeY];
            GBMSGUI.BitmapToRawImage(bmpImage, RawImage);

            // create JPEG2000 encoder
            NW_GBJPEG_JPX_Encode JP2Encoder =
                new NW_GBJPEG_JPX_Encode(RawImage, (uint)ImageSizeX, (uint)ImageSizeY);
            JP2Encoder.BitPerPixel = NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_8_BITS_PER_PIXEL;
            JP2Encoder.CompressionRate = (ushort)Rate;
            JP2Encoder.CompressionAlgorithm = NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000;
            // encode
            CompressedImage = JP2Encoder.Encoded;

            return CompressedImage;
        }

        private void SetMissingFingers()
        {
            int Reason = GBMSGUI.UnavailabilityReason.Unprintable;

            if ((CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT) ||
                (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT))
            {
                if (chkMissingLeftIndex.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 0, Reason);
                if (chkMissingLeftMiddle.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 1, Reason);
                if (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                {
                    if (chkMissingLeftRing.Checked)
                        MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 2, Reason);
                    if (chkMissingLeftLittle.Checked)
                        MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 3, Reason);
                }
            }

            if ((CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT) ||
                (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT))
            {
                if (chkMissingRightIndex.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 0, Reason);
                if (chkMissingRightMiddle.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 1, Reason);
                if (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                {
                    if (chkMissingRightRing.Checked)
                        MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 2, Reason);
                    if (chkMissingRightLittle.Checked)
                        MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 3, Reason);
                }
            }

            if (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
            {
                if (chkMissingLeftThumb.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 0, Reason);
                if (chkMissingRightThumb.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 1, Reason);
            }

            if (CurrentScanItem.ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
            {
                if (chkMissingLeftIndex.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 0, Reason);
                if (chkMissingRightIndex.Checked)
                    MyGUI.SetSegmentUnavailabilityReason(CurrentScanItem.ScanObjID, 1, Reason);
            }
        }

        private void SetLanguage()
        {
            if (DemoFormRef.DemoConfig.GUILanguage == DemoForm.GUILanguages.Italian)
                MyGUI.SetLanguage(CultureInfo.CreateSpecificCulture("it"));
            else if (DemoFormRef.DemoConfig.GUILanguage == DemoForm.GUILanguages.Chinese)
                MyGUI.SetLanguage(CultureInfo.CreateSpecificCulture("zh-CN"));
            else
                MyGUI.SetLanguage(CultureInfo.CreateSpecificCulture("en"));
        }
    }
}