#ifndef   _WSQPACK_H_
#define   _WSQPACK_H_

#ifdef BUILD_DLL
#define WSQ_EXPORT __declspec( dllexport )
#else
#define WSQ_EXPORT __declspec( dllimport )
#endif

#define WSQPACK_OK					0

extern "C" {

// -----------------------------------------------------------------
//	WSQPACK_Compress - Compress a raw image buffer to WSQ format.
//
//	dest :				Pointer to the output buffer, allocated internally;
//						use WSQPACK_Free to free the pointer after use.
//	destLen:			Lenght of output buffer
//	r_bitrate:			Compression rate, number from 0 to 1;
//						it is recommended to use a compression of 0.75 (15:1)
//  source :			Pointer to the raw image to be compessed.
//  width :				source width
//	height :			source height
//	depth :				source depth (bits per pixel) (8,24)
//	ppi:				source resolution (pixel per inch)
//  comment_text :		optional comment, can be NULL
// -----------------------------------------------------------------
WSQ_EXPORT int __stdcall WSQPACK_Compress(
			unsigned char **dest, 
			int *destLen, 
			float r_bitrate,
			unsigned char *source, 
			int width, 
			int height, 
			int depth, 
			int ppi, 
			char *comment_text);
// -----------------------------------------------------------------

// -----------------------------------------------------------------
//	WSQPACK_Compress - Decompress image in WSQ format to raw format.
//
//	dest :				Pointer to the output buffer, allocated internally;
//						use WSQPACK_Free to free the pointer after use.
//  width :				dest width
//	height :			dest height
//	depth :				dest depth (bits per pixel) (8,24)
//	ppi:				dest resolution (pixel per inch)
//	lossyflag:			lossyflag (always 1)
//  source :			Pointer to the image to be decompessed.
//  sourceLen :			Lenght of source buffer
// -----------------------------------------------------------------
WSQ_EXPORT int __stdcall WSQPACK_Uncompress(
			unsigned char **dest,
			int *width,
			int *height,
			int *depth,
			int *ppi,
			int *lossyflag, 
			unsigned char *source,
			int sourceLen);
// -----------------------------------------------------------------

// -----------------------------------------------------------------
//	WSQPACK_Free - Free the pointers to memory allocated internally 
//	by the encoder/decoder functions.
//
//	Ptr :			Pointer to the memory to be freed.
// -----------------------------------------------------------------
WSQ_EXPORT void __stdcall WSQPACK_Free(unsigned char *dest);
// -----------------------------------------------------------------

} // extern "C"

#endif



