using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Reflection;
using System.IO;
using System.Resources;
using System.Globalization;
using System.Threading;
using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLEDsDefines;
using GBNFIQ_NET_WRAPPER;
using GBFINIMG_NET_WRAPPER;
using LFS_NET_WRAPPER;
using AN2K_NET_WRAPPER;
using BOZORTH_NET_WRAPPER;

namespace GBMSGUI_NET
{
    // class of the Acquisition form
    internal partial class AcquisitionForm : Form
    {
        public GBMSGUI MSGUIRef;    // reference to MSGUI instance
        public int ScanResult = -1; // form result
        public bool Acquiring = false;  // acquisition flag
        public string PersonID;
        public bool SingleAcquisition = false;
        public bool ForceClose = false; // flag for closing form only from StopSession
        public Bitmap bmpImage;     // bitmap for displaying final image
        public GBMSGUI.ScannedObject ScannedObject;  // object currently scanned
        public Byte[] FramePtr;        // pointer to raw frame
        public GCHandle gcAcqFormHandle;
        //----------------------------------------------------------------------------
        // callback for receiving GBMSAPI events
        private static GBMSAPI_NET_AcquisitionEventsManagerCallback AcqEvtMgrCallback = 
            new GBMSAPI_NET_AcquisitionEventsManagerCallback(AcquisitionEventsManagerCallback);
        private IntPtr AcqFormPtrParam;     // pointer to acquisition form passed to the callback
        private uint ImageSizeX;    // size of final image
        private uint ImageSizeY;
        private uint PreviewImageSizeX; // size of preview image
        private uint PreviewImageSizeY;
        private uint ScanOptions;       // options for current scanning
        private Bitmap bmpPreviewImage; // bitmap for displaying preview image
        //private Byte[] FramePtr;        // pointer to raw frame
        private Byte[] MarkerFrame;     // pointer to artefacts marker frame
        private Byte Contrast;          
        private Byte PalmCompleteness;
        private double AspectRatio;
        private double PreviewScale;
        private double ViewScaleFactor; // scale factor for image view (varies with resize)
        private bool FullResImage;
        private int FrameCounter;
        private bool PreviewPhase;
        private bool ToDoFinalization;  // 1.9.14.0
        // rectangles shown over the image (clipping, segment bounding boxes)
        private class ClipRect
        {
            public Rectangle RealRect = new Rectangle();
            public Rectangle ViewRect = new Rectangle();
            public bool Visible = false;
        }
        private enum SizeOperations     // size operations for clipping rectangle and bounding boxes
        {
            None,
            Right,
            Left,
            Top,
            Bottom,
            Move
        };
        private enum SizeOptions
        {
            AllowResize = 0x01,
            AllowMove = 0x02,
        };
        private ClipRect ClippingRect = new ClipRect();
        private ClipRect CurrRect;  // current rectangle (for mouse operations)
        private int CurrSegmIndex;  // current segment index (for context menu)
        // data for segments
        private class SegmentData
        {
            public string FingerID;     // one of NW_AN2K_FINGER_POSITIONS
            public int FingerCenterX;   // center of the finger inside image
            public int FingerCenterY;
            public String FingerText;   // text to display (finger name)
            public ClipRect BoundingBox = new ClipRect();   // segment bounding box
            public MenuItem FindSegmMenuItem;   // context menu for segment
            public Bitmap bmpImage;     // bitmap for segment image
            public int NFIQQuality;
            public int GBQuality;
            public Color QualityColor;  // evaluation of quality
            public bool IncompletePatternDetected = false;
            public bool InvalidPatternDetected = false;
            public bool InclinationDetected = false;
            public int UnavailabilityReason = -1;
            public bool Found;  // if the procedure found the segment
        }
        private List<SegmentData> SegmData = new List<SegmentData>();
        // variables for mouse operations inside pboxImage
        private bool mMouseDown = false;
        private SizeOperations mSizeOperation;
        private Point StartPoint;
        private Point SegmMenuPoint = new Point();    // point for finding segment

        // External device equipment
        private bool LCDPresent = false;
        private bool LedPresent = false;
        private bool SoundPresent = false;
        private bool PedalPresent = false;

        // Image Information
        private bool PalmCompletenessSupported = false;

        // LCD feature
        private bool LCDMidLimitSupported = false;

        // used by ListenerTimer
        private class PedalInformation
        {
            public int PressedTime = 0;
            public int ReleasedTime = 0;
            public bool PedalPressed;
            public int tStart;
            public int Condition;
        }
        private PedalInformation PedalInfo = new PedalInformation();
        // condition to test
        private static class PedatTestConditions
        {
            public const int ReleasePedal = 1;
            public const int PedalPressing = 2;
        }

        private bool IncompletePatternDetected = false;
        private bool InvalidPatternDetected = false;
        private bool InclinationDetected = false;
        private uint LastDiagnostic = 0;
        private uint PrevDiagnostic = 0;
        private bool SegmentationResult = false;
        private bool ProcessImageResult = false;

        // additional diagnostics for displaying (post-process)
        private uint PostProcessDiag = 0;
        private uint PrevPostProcessDiag = 0;
        private class PostProcessDiagnostic
        {
            public const uint Diag_Inclination = 0x00000001;
            public const uint Diag_IncompletePattern = 0x00000002;
            public const uint Diag_InvalidPattern = 0x00000004;
            public const uint Diag_SegmAmbiguousSlap = 0x00000008;
            public const uint Diag_SegmIncompleteSlap = 0x00000010;
            public const uint Diag_SegmPossibleIncompleteSlap = 0x00000020;
            public const uint Diag_LeftRightMixup = 0x00000040;
            public const uint Diag_PossibleLeftRightMixup = 0x00000080;
        }

        // moved global ver 1.3
        private NW_GBFINIMG_ProcessImage Segmentator = new NW_GBFINIMG_ProcessImage();

        // link to the library for beeping using DirectSound in OS not supporting Beep
        [DllImport("DSBeep.dll")]
        private static extern bool DSBeep(int Frequency, int Duration);

        public AcquisitionForm(GBMSGUI MSGUIRef)
        {
            // save reference to GBMSGUI instance
            this.MSGUIRef = MSGUIRef;

            // delegate
            AcqEventDelegate = new AcquisitionEvent(OnAcquisitionEvent);

            InitializeComponent();

            // initialize user interface strings
            SetGUIStrings();

            // init ImageSize to some value, to allow AdjustProportion to work
            ImageSizeX = 800;
            ImageSizeY = 800;
            AspectRatio = (double)ImageSizeX / (double)ImageSizeY;

            // check presence of optional devices
            uint ExternalEquipment;
            int ret;
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetOptionalExternalEquipment(out ExternalEquipment);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_GetOptionalExternalEquipment", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }

            if (MSGUIRef.Resolution1000Dpi)
            {
                // allocate buffer for rotation
                // get max size for object type on current scanner
                uint MaxImageSizeX, MaxImageSizeY;
                ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetMaxImageSize(out MaxImageSizeX, out MaxImageSizeY);
                uint Size = (uint)(MaxImageSizeX * MaxImageSizeY *
                    NW_GBFINIMG_ROTATION_FACTOR.NW_GBFINIMG_ROTATION_BUFFER_FACTOR);
                ret = Segmentator.NW_GBFINIMG_SetRotationBuffer1000DPI(Size);
                if (ret != NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_NO_ERROR)
                {
                    MSGUIRef.SetError("NW_GBFINIMG_SetRotationBuffer1000DPI", ret);
                    ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                    return;
                }
            }

            // check external equipment
            if (GBMSGUI.CheckMask(ExternalEquipment, GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_PEDAL))
                PedalPresent = true;

            if (GBMSGUI.CheckMask(ExternalEquipment, GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_VUI_LCD))
            {
                LCDPresent = true;
                // MC517 color LCD supports intermediate limit
                uint LCDFeatures;
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_GetLcdFeatures(out LCDFeatures);
                if (GBMSGUI.CheckMask(LCDFeatures, GBMSAPI_NET_DisplayFeatures.GBMSAPI_NET_VILCD_LF_INTERMEDIATE_LIMIT))
                    LCDMidLimitSupported = true;
            }

            if (GBMSGUI.CheckMask(ExternalEquipment, GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_VUI_LED))
                LedPresent = true;

            if (GBMSGUI.CheckMask(ExternalEquipment, GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_SOUND))
                SoundPresent = true;
        }

        private void AcquisitionForm_Load(object sender, EventArgs e)
        {
            gcAcqFormHandle = GCHandle.Alloc(this, GCHandleType.Normal);
            // pointer to AcquisitionForm instance to pass to callback
            AcqFormPtrParam = GCHandle.ToIntPtr(gcAcqFormHandle);

            // at first load, all disabled
            btnStop.Enabled = false;
            btnAccept.Enabled = false;
            btnRepeat.Enabled = false;
            btnSkip.Enabled = false;
            btnBreak.Enabled = false;
            btnBack.Enabled = false;

            if (SingleAcquisition)
            {
                // set "cancel" caption to break button
                toolTipButtons.SetToolTip(btnBreak, MSGUIRef.GetLocalizedString("btnCancel"));
            }

            pboxScannedObject.Image = imgLstHands.Images["Hands_empty.bmp"];

            lblPersonID.Text = PersonID;

            if (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.NoAdvancedView))
                chkAdvancedView.Visible = false;
            else
                chkAdvancedView.Visible = true;

            // 1.9.12.0
            if (MSGUIRef.DryFingerImageEnhancementSupported)
            {
                chkDryFingerImageEnhancement.Checked = GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.DryFingerImageEnhancement);
                chkDryFingerImageEnhancement.Enabled = true;
            }
            else
            {
                chkDryFingerImageEnhancement.Checked = false;
                chkDryFingerImageEnhancement.Enabled = false;
            }
        }

        // Acquire: called by GBMSGUI.Acquire
        public void Acquire()
        {
            int ret;
            bool FlatScanArea = true;

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
                FlatScanArea = false;
            else if (GBMSGUI.IsFlatSingleFinger(ScannedObject.ScannedObjectID)
                && MSGUIRef.CfgData.FlatFingerOnRollArea
                && GBMSGUI.CheckMask(MSGUIRef.SupportedScanOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
                FlatScanArea = false;

            // get image size based on ScannedObjectID
            // ask size for full resolution image
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetImageSize(
                GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScannedObject.ScannedObjectID),
                FlatScanArea,
                false,  // full resolution image
                out ImageSizeX, out ImageSizeY);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_GetImageSize", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }

            // now that ImageSize is changed, set aspect ratio
            AspectRatio = (double)ImageSizeX / (double)ImageSizeY;
            AdjustProportion();

            // inizialize draw object
            bmpImage = new Bitmap((int)ImageSizeX, (int)ImageSizeY, PixelFormat.Format8bppIndexed);

            // Set Bitmap ColorPalette to Grayscale
            GBMSGUI.SetGrayPalette(ref bmpImage);

            // ask size for preview image too
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetImageSize(
                GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScannedObject.ScannedObjectID),
                FlatScanArea,
                true,  // preview image
                out PreviewImageSizeX, out PreviewImageSizeY);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_GetImageSize", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }

            PreviewScale = (double)ImageSizeX / PreviewImageSizeX;

            // create bitmap for preview
            bmpPreviewImage = new Bitmap((int)PreviewImageSizeX, (int)PreviewImageSizeY, PixelFormat.Format8bppIndexed);

            // Set Bitmap ColorPalette to Grayscale
            GBMSGUI.SetGrayPalette(ref bmpPreviewImage);

            // if slap, set segments
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
                SetSegments();

            // show object name and picture
            //lblScannedObject.Text = GetScannedObjectName();
            //ShowScannedObjectPictogram();

            // contrast progressbar
            thresholdProgressBarContrast.MaxValue = 255;
            thresholdProgressBarContrast.SetThresholds(ScannedObject.ContrastThreshold1, ScannedObject.ContrastThreshold2);

            // show/hide specific UI parts according to object type
            if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID) ||
                (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID) && 
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation)))
            {
                // IAFIS quality
                panIAFISQuality.Visible = true;
                // direction depends on algorythm
                if (MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                    thresholdsLegendQuality.SetThresholds(ScannedObject.QualityThreshold1,
                        ScannedObject.QualityThreshold2, (int)ThresholdsLegend.Direction.SmallerThan);
                else
                    thresholdsLegendQuality.SetThresholds(ScannedObject.QualityThreshold1,
                        ScannedObject.QualityThreshold2, (int)ThresholdsLegend.Direction.GreatherThan);
            }
            else
                panIAFISQuality.Visible = false;

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                // artefacts for rolled
                panArtefacts.Visible = true;
                thresholdsLegendArtefacts.SetThresholds(MSGUIRef.CfgData.ArtefactsThreshold1, 
                    MSGUIRef.CfgData.ArtefactsThreshold2, (int)ThresholdsLegend.Direction.SmallerThan);
            }
            else
                panArtefacts.Visible = false;

            if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID))
            {
                // see if PalmCompleteness is supported
                uint ImageInfo;
                GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetAvailableImageInfo(
                    GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScannedObject.ScannedObjectID),
                    out ImageInfo);
                if (GBMSGUI.CheckMask(ImageInfo, GBMSAPI_NET_AvailableImageInfo.GBMSAPI_NET_AII_LOWER_HALF_PALM_COMPLETENESS))
                    PalmCompletenessSupported = true;
                else
                    PalmCompletenessSupported = false;
            }

            if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID) && PalmCompletenessSupported)
            {
                // palm completeness
                panScanningCompleteness.Visible = true;
                thresholdProgressBarCompleteness.MaxValue = 100;
                thresholdProgressBarCompleteness.SetThresholds(MSGUIRef.CfgData.LowerPalmCompletenessThreshold1,
                    MSGUIRef.CfgData.LowerPalmCompletenessThreshold2);
                //panIAFISQuality.Top = panScanningCompleteness.Bottom + 3;
                panIAFISQuality.Top = panScanningCompleteness.Bottom;

                // set completeness threshold for low level
                GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_SetLowerHalfPalmCompletenessThreshold(
                    (byte)MSGUIRef.CfgData.LowerPalmCompletenessThreshold2);

                if (LCDPresent && LCDMidLimitSupported)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimit(
                        (byte)MSGUIRef.CfgData.LowerPalmCompletenessThreshold1);
            }
            else
            {
                panScanningCompleteness.Visible = false;
            }

            // enable rotation option oly for lower palm
            if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID) && MSGUIRef.ImageRotationSupported)
            {
                chkRotate180.Enabled = true;
            }
            else
            {
                chkRotate180.Checked = false;
                chkRotate180.Enabled = false;
            }

            // 1.9.11.0 - moved in OnScannerStarted
            //btnSkip.Enabled = true;
            //btnBreak.Enabled = true;
            //if (!SingleAcquisition)
            //    btnBack.Enabled = true;

            // 1.9.12.0 - show Dry/wet diagnostic instead of contrast, if supported
            // and not setting changed
            if (MSGUIRef.DryFingerImageEnhancementSupported && 
                (MSGUIRef.FingerContactEvalMode == GBMSGUI.FingerContactEvaluationMode.DryWetWarning))
            {
                panDryWet.Visible = true;
                panContrast.Visible = false;
            }
            else
            {
                panDryWet.Visible = false;
                panContrast.Visible = true;
            }

            // start acquisition
            StartScanning();
        }

        // AcquireEnd: called by GBMSGUI.Acquire at the end of every acquisition
        public void AcquireEnd()
        {
            // free preview image
            if (bmpPreviewImage != null)
                bmpPreviewImage.Dispose();

            if (LedPresent)
                TurnOffLeds();

            ListenerTimer.Enabled = false;
        }

        private void ResetScannedObject()
        {
            // reset values
            ScannedObject.Acquired = false;
            ScannedObject.Completeness = 0;
            ScannedObject.GBQuality = 0;
            ScannedObject.NFIQQuality = 0;
            ScannedObject.RolledArtefactSize = 0;
        }

        private void StartScanning()
        {
            int ret;
            uint DisplayOptions = 0;

            ResetScannedObject();

            FullResImage = false;

            FrameCounter = 0;
            FramePtr = null;    // clear image
            pboxImage.Refresh();
            Contrast = 0;
            LastDiagnostic = 0;
            PrevDiagnostic = 0;
            PostProcessDiag = 0;
            PrevPostProcessDiag = 0;
            SegmentationResult = false;
            ProcessImageResult = false;
            IncompletePatternDetected = false;
            InvalidPatternDetected = false;
            InclinationDetected = false;

            // update interface
            lblStartMessage.Visible = true;
            ClippingRect.Visible = false;
            ResetSegments();
            DisplayMessage("", false);
            lblCurrentNominalFrameRate.Text = "";
            lblCurrentNominalFrameRate.BackColor = SystemColors.ButtonFace;
            thresholdProgressBarContrast.SetValue(Contrast);
            lblIAFISQualityValue.Text = "";
            lblIAFISQualityValue.BackColor = SystemColors.ButtonFace;
            //DisplayDiagnostic();
            lstDiag.Items.Clear();
            lstDiag.BackColor = SystemColors.Window;
            btnHelpDiag.Enabled = false;
            listViewMessages.Items.Clear();
            //grpScanningOptions.Enabled = false;
            panRotate180.Enabled = false;
            lblDryWet.Visible = false;

            // 1.9.11.0 - disable all buttons
            btnAccept.Enabled = false;
            btnRepeat.Enabled = false;
            btnSkip.Enabled = false;
            btnBack.Enabled = false;
            btnBreak.Enabled = false;

            // hide object name and picture (will be shown when first frame arrives)
            lblScannedObject.Text = "";
            pboxScannedObject.Image = imgLstHands.Images["Hands_empty.bmp"];

            // if option active, rotate 180 degrees
            if (chkRotate180.Enabled)   // 1.9.11.0
            {
                if (chkRotate180.Checked)
                    GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_SetImageRotation(GBMSAPI_NET_ImageRotationValues.GBMSAPI_NET_IMG_ROTATE_180);
                else
                    GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_SetImageRotation(GBMSAPI_NET_ImageRotationValues.GBMSAPI_NET_IMG_ROTATE_NO);
            }

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                lblArtefacsSize.Text = "";
                lblArtefacsSize.BackColor = SystemColors.ButtonFace;
                MarkerFrame = null;
            }

            if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID) && PalmCompletenessSupported)
            {
                panScanningCompleteness.Visible = true;
                thresholdProgressBarCompleteness.SetValue(0);
            }

            if (PedalPresent)
            {
                // check that pedal is not pressed before starting acquisition
                StartListenPedal(PedatTestConditions.ReleasePedal);
                while (PedalInfo.PedalPressed)
                {
                    Application.DoEvents();
                    Thread.Sleep(1);
                }
                // added 16/09/2009
                // disable timer before starting acquisition
                ListenerTimer.Enabled = false;
            }

            // let user interface refresh before slow operation (StartScanner)
            Application.DoEvents();

            // set current options
            SetScanningOptions(MSGUIRef.SupportedScanOptions);

            // set to go to Stop screen after acquisition
            DisplayOptions |= GBMSAPI_NET_DisplayOptions.GBMSAPI_NET_DO_FINAL_SCREEN;

            if (LCDPresent && LCDMidLimitSupported)
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimit(
                    (byte)ScannedObject.ContrastThreshold1);

            // 1.9.12.0
            // use this value to select if display contrast or dry/wet warning on LCD
            int ContrastThreshold = 0;
            if (MSGUIRef.DryFingerImageEnhancementSupported &&
                (MSGUIRef.FingerContactEvalMode == GBMSGUI.FingerContactEvaluationMode.DryWetWarning))
                ContrastThreshold = 0;
            else
                ContrastThreshold = ScannedObject.ContrastThreshold2;

            // start scanner
            ret = GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_StartAcquisition(
                ScannedObject.ScannedObjectID,
                ScanOptions,
                AcqEvtMgrCallback, // callback
                AcqFormPtrParam,
                DisplayOptions,
                //(byte)ScannedObject.ContrastThreshold2,
                (byte)ContrastThreshold,    // 1.9.12.0
                (byte)MSGUIRef.CfgData.LowerPalmCompletenessThreshold2);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_StartAcquisition", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }

            // information of acquisition started will arrive in AcquisitionEventsManagerCallback
            return;
        }

        // callback called by GBMSAPI on acquisition events
        public static GCHandle gcAcqFormHandle_St;
        public static Int32 AcquisitionEventsManagerCallback
        (
            UInt32 EventCode,
            Int32 GetFrameErrorCode,
            UInt32 EventInfo,
            Byte[] FramePtr,
            Int32 FrameSizeX,
            Int32 FrameSizeY,
            Double CurrentFrameRate,
            Double NominalFrameRate,
            UInt32 GB_Diagnostic,
            System.IntPtr UserDefinedParameters
        )
        {
            int ret;

            // retrieve AcquisitionForm reference
            if (!gcAcqFormHandle_St.IsAllocated)
                gcAcqFormHandle_St = GCHandle.FromIntPtr(UserDefinedParameters);
            AcquisitionForm AcqFormRef = (AcquisitionForm)gcAcqFormHandle_St.Target;

            // 1.9.8.0 - changed, because callback is called in a thread
            // call delegate
            ret = (int)AcqFormRef.Invoke(AcqFormRef.AcqEventDelegate,
                new Object[] { EventCode,
                    GetFrameErrorCode,
                    EventInfo,
                    FramePtr,
                    FrameSizeX,
                    FrameSizeY,
                    CurrentFrameRate,
                    NominalFrameRate,
                    GB_Diagnostic,
                    UserDefinedParameters });

            return ret;
        }

        // delegate for OnAcquisitionEvent
        public delegate int AcquisitionEvent(
            UInt32 EventCode,
            Int32 GetFrameErrorCode,
            UInt32 EventInfo,
            Byte[] FramePtr,
            Int32 FrameSizeX,
            Int32 FrameSizeY,
            Double CurrentFrameRate,
            Double NominalFrameRate,
            UInt32 GB_Diagnostic,
            System.IntPtr UserDefinedParameters);

        public AcquisitionEvent AcqEventDelegate;

        public int OnAcquisitionEvent(
            UInt32 EventCode,
            Int32 GetFrameErrorCode,
            UInt32 EventInfo,
            Byte[] FramePtr,
            Int32 FrameSizeX,
            Int32 FrameSizeY,
            Double CurrentFrameRate,
            Double NominalFrameRate,
            UInt32 GB_Diagnostic,
            System.IntPtr UserDefinedParameters)
        {
            int ret;

            this.FramePtr = FramePtr;

            // call form method appropriate for EventCode
            switch (EventCode)
            {
                case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_ACQUISITION_ERROR:
                    ret = OnAcquisitionError(GetFrameErrorCode);
                    break;

                case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_SCANNER_STARTED:
                    ret = OnScannerStarted();
                    break;

                case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_PREVIEW_PHASE_END:
                    ret = OnPreviewPhaseEnd();
                    break;

                case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_VALID_FRAME_ACQUIRED:
                    ret = OnValidFrameAcquired(EventInfo, FramePtr, FrameSizeX, FrameSizeY,
                        CurrentFrameRate, NominalFrameRate, GB_Diagnostic, false);
                    break;

                case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_ACQUISITION_END:
                    bool AutomaticStop = false;
                    bool Timeout = false;

                    // check stop type in EventInfo
                    if (GBMSGUI.CheckMask(EventInfo, GBMSAPI_NET_EventInfo.GBMSAPI_NET_EI_STOP_TYPE))
                        AutomaticStop = true;
                    if (GBMSGUI.CheckMask(EventInfo, GBMSAPI_NET_EventInfo.GBMSAPI_NET_EI_ACQUISITION_TIMEOUT))
                        Timeout = true;

                    if (!Timeout)
                    {
                        // contains also a valid frame
                        //V1.2 - FramePtr can be null
                        if (FramePtr != null)
                            ret = OnValidFrameAcquired(EventInfo, FramePtr, FrameSizeX, FrameSizeY,
                                CurrentFrameRate, NominalFrameRate, GB_Diagnostic, true);
                    }

                    ret = OnAcquisitionEnd(AutomaticStop, Timeout);
                    break;

                default:
                    ret = 0;
                    break;
            }

            return ret;
        }

        private void SetScanningOptions(uint SupportedOptions)
        {
            ScanOptions = 0;

            // verify that requested options are available; if not, show a warning
            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview))
            {
                if (GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FULL_RES_PREVIEW))
                    ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FULL_RES_PREVIEW;
                else
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Full_Res_Preview"), "Warning.bmp");
            }

            // Autocapture only for flat (for Rolled is always active)
            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Autocapture))
                {
                    if (GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE))
                    {
                        ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE;
                        listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Autocapture"), "Info.bmp");
                    }
                    else
                        listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Autocapture"), "Warning.bmp");
                }
                else
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Autocapture"), "Info.bmp");
            }

            // Roll preview; only for rolled
            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.NoRollPreview)
                    && GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_NO_ROLL_PREVIEW))
                {
                    ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_NO_ROLL_PREVIEW;
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Roll_Preview"), "Info.bmp");
                }
                else
                {
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RollPreviewManualStop))
                    {
                        if (GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_MANUAL_ROLL_PREVIEW_STOP))
                        {
                            ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_MANUAL_ROLL_PREVIEW_STOP;
                            listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Roll_Preview_Manual_Stop"), "Info.bmp");
                        }
                        else
                            listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Roll_Preview_Manual_Stop"), "Warning.bmp");
                    }
                    else // BUG FIX - the else was missing - 07/10/2009
                    {
                        // automatic roll preview modes are always supported
                        if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RollPreviewType))
                        {
                            ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_ROLL_PREVIEW_TYPE;
                            listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Roll_Preview_Center"), "Info.bmp");
                        }
                        else
                        {
                            listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Roll_Preview_Side"), "Info.bmp");
                        }
                    }
                }
            }

            // Flat finger on roll scanner
            if (GBMSGUI.IsFlatSingleFinger(ScannedObject.ScannedObjectID)
                && MSGUIRef.CfgData.FlatFingerOnRollArea
                && GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
                ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA;

            // High speed full res preview
            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.HighSpeedPreview))
            {
                if (GBMSGUI.CheckMask(SupportedOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_HIGH_SPEED_PREVIEW))
                    ScanOptions |= GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_HIGH_SPEED_PREVIEW;
                else
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_High_Speed_Preview"), "Warning.bmp");
            }

            // show other options

            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                // Block autocapture
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.BlockAutocapture)
                    && !MSGUIRef.BlockAutocaptureSupported)
                {
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_BlockAutocpature"), "Warning.bmp");
                }
            }

            // show only for Slap
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Segmentation_On"), "Info.bmp");
                else
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Segmentation_Off"), "Info.bmp");
            }

            if (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.SequenceCheck))
                listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Sequence_Check_ON"), "Info.bmp");
            else
                listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Sequence_Check_OFF"), "Info.bmp");

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                // Auto clear outside
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.AutoClearOutsideRoll))
                {
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Auto_Clear_Outside_Roll_Active"), "Info.bmp");
                }
            }

            // 1.9.12.0 - dry finger enhancement
            if (MSGUIRef.DryFingerImageEnhancementSupported)
            {
                if (!SetDryFingerImageEnhancementOption(chkDryFingerImageEnhancement.Checked))
                    return;
            }
            else
            {
                if (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.DryFingerImageEnhancement))
                {
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_No_Dry_Finger_Image_Enhancement"), "Warning.bmp");
                }
            }
        }


        private int OnAcquisitionError(int GetFrameErrorCode)
        {
            // turn off acquisition flag
            Acquiring = false;
            MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("Acquisition error", GetFrameErrorCode), GetFrameErrorCode);
            ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
            return 0;
        }

        private int OnScannerStarted()
        {
            int ret;

            // turn on acquisition flag
            Acquiring = true;
            PreviewPhase = true;

            // update user interface
            DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Preview"), false);
            // 1.9.11.0 - moved in StartScanning
            //btnAccept.Enabled = false;
            //btnRepeat.Enabled = false;
            // 1.9.11.0 - re-enable buttons
            btnSkip.Enabled = true;
            if (!SingleAcquisition)
                btnBack.Enabled = true;
            btnBreak.Enabled = true;

            btnStop.Enabled = true;
            btnStop.Focus();

            // set clipping
            ClippingRect.RealRect.Width = (int)ScannedObject.ImageSizeX;
            ClippingRect.RealRect.Height = (int)ScannedObject.ImageSizeY;

            // DEBUG
            //GBMSGUI.LogToFile("OnScannerStarted: ClippingRect width = " + ClippingRect.RealRect.Width.ToString() + 
            //    " ClippingRect height = " + ClippingRect.RealRect.Height.ToString());

            ret = GBMSAPI_NET_ScannerStartedRoutines.GBMSAPI_NET_SetClippingRegionSize((uint)ClippingRect.RealRect.Width, (uint)ClippingRect.RealRect.Height);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_SetClippingRegionSize", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                Acquiring = false;  //V1.2 added
                return 0;
            }

            // call afterstart callback, if defined
            if (MSGUIRef.AfterStartCallbackRef != null)
            {
                if (!MSGUIRef.AfterStartCallbackRef(MSGUIRef.AfterStartCallbackParam))
                {
                    MSGUIRef.SetError(MSGUIRef.GetLocalizedString("Msg_AfterStartCallback_Failed"), GBMSGUI.ErrorCodes.GBMSGUI_ERR_AfterStartCallback_Failed);
                    ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                    Acquiring = false;  //V1.2 added
                    return 0;
                }
            }

            return 1;
        }

        private int OnPreviewPhaseEnd()
        {
            // for rolled, roll direction indication is shown
            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Acquiring_Image"), false);
                Application.DoEvents();
            }

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) &&
                !GBMSGUI.CheckMask(ScanOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_NO_ROLL_PREVIEW))
            {
                // signal to start rolling
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.SoundOnRollPreviewEnd))
                {
                    if (SoundPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(12, 1, 1);
                    else
                        //Console.Beep(700, 200);
                        DSBeep(700, 200);
                }
            }

            return 1;
        }

        private int OnValidFrameAcquired(UInt32 EventInfo,
            Byte[] FramePtr,
            Int32 FrameSizeX,
            Int32 FrameSizeY,
            Double CurrentFrameRate,
            Double NominalFrameRate,
            UInt32 GB_Diagnostic,
            bool AcquisitionEnd)
        {
            // stop acquisition if form is closing for other reasons (skip, break, etc)
            if (ScanResult != -1)
                return 0;

            FrameCounter++;

            // check if preview or acquisition phase
            if (GBMSGUI.CheckMask(EventInfo, GBMSAPI_NET_EventInfo.GBMSAPI_NET_EI_ACQUISITION_PHASE))
                PreviewPhase = false;

            if (PreviewPhase)
            {
                // check if is full resolution image
                //if ((FrameSizeX == ImageSizeX) && (FrameSizeY == ImageSizeY))
                if (GBMSGUI.CheckMask(EventInfo, GBMSAPI_NET_EventInfo.GBMSAPI_NET_EI_PREVIEW_RES))
                    FullResImage = true;
            }
            else
            {
                // final image is always full resolution
                FullResImage = true;
            }

            // first frame
            if (lblStartMessage.Visible)
            {
                lblStartMessage.Visible = false;

                // show object name and picture
                lblScannedObject.Text = GetScannedObjectName();
                ShowScannedObjectPictogram();

                Application.DoEvents();
            }

            // show frame rate (if present)
            if (NominalFrameRate != 0)
                ShowFrameRate(CurrentFrameRate, NominalFrameRate);

            // save diagnostic for checking on stop
            LastDiagnostic = GB_Diagnostic;

            // clear masked diagnostic
            LastDiagnostic &= ~MSGUIRef.CfgData.IgnoreDiagnosticMask;

            // first, check roll direction indication inside diagnostic mask
            if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_ROLL_DIRECTION_LEFT))
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Roll_Left"), false);
            }
            else if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_ROLL_DIRECTION_RIGHT))
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Roll_Right"), false);
            }

            // check diagnostic
            //DisplayDiagnostic();  // moved at the end

            // if BlockAutocapture option
            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(ScanOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE)
                    && GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.BlockAutocapture)
                    && MSGUIRef.BlockAutocaptureSupported
                    && !AcquisitionEnd)
                {
                    if (GBMSGUI.CheckMask(GB_Diagnostic, MSGUIRef.CfgData.BlockAutoCaptureMask) ||
                        (MSGUIRef.CfgData.BlockAutoCaptureContrast && (Contrast < ScannedObject.ContrastThreshold2)))
                        BlockAutoCapture();
                    else
                        UnblockAutoCapture();
                }
            }

            // save frame pointer
            this.FramePtr = FramePtr;

            // draw frame
            if (!FullResImage)
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpPreviewImage);
            else
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);

            pboxImage.Refresh();

            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                // show contrast
                GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetFingerprintContrast(out Contrast);
                thresholdProgressBarContrast.SetValue(Contrast);
            }

            if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID) && PalmCompletenessSupported)
            {
                // show palm completeness
                GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetLowerHalfPalmCompleteness(out PalmCompleteness);
                thresholdProgressBarCompleteness.SetValue((int)PalmCompleteness);
            }

            // show clipping (for rolled, only at the end)
            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) || AcquisitionEnd)
            {
                // get clipping rectangle
                int ClipX, ClipY;
                uint ClipSizeX, ClipSizeY;
                GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetClippingRegionPosition(out ClipX, out ClipY, out ClipSizeX, out ClipSizeY);

                // in flat preview, clipping coordinates are relative to small image
                if (!FullResImage && (bmpPreviewImage.Width != bmpImage.Width))
                {
                    ClipX = Convert.ToInt32((double)ClipX * PreviewScale);
                    ClipY = Convert.ToInt32((double)ClipY * PreviewScale);
                }
                /*
                if (!FullResImage && (bmpPreviewImage.Width != bmpImage.Width))
                {
                    ClipX *= 3;
                    ClipY *= 3;
                    if (MSGUIRef.Resolution1000Dpi)
                    {
                        ClipX *= 2;
                        ClipY *= 2;
                    }
                }
                */

                ShowClipRectangle(ClipX, ClipY);
            }

            // live segmentation
	        if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID) &&
                !AcquisitionEnd &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation) &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.SegmentsQualityEval) &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.FullResPreview)
		        )
	        {
                if (Contrast == 0)
                {
                    ResetSegments();
                    PrevPostProcessDiag = PostProcessDiag;
                    PostProcessDiag = 0;
                }
                else
                {
                    bool res = LiveSegmentation();
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.BlockAutocapture)
                        && MSGUIRef.BlockAutocaptureSupported)
                    {
                        if (!res)
                            BlockAutoCapture();
                        else
                            UnblockAutoCapture();
                    }
                }
	        }

            // moved here because now displays both acquisition diag and post-processing diag (live segm)
            DisplayDiagnostic();
            
            return 1;
        }

        private void BlockAutoCapture()
        {
            ListViewItem Item;
            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_SetAutoCaptureBlocking(true);
            if (LedPresent)
                // notify also blinking led(s)
                GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_VUI_LED_BlinkDuringAcquisition(1);

            // show message in list
            Item = new ListViewItem(MSGUIRef.GetLocalizedString("Msg_Autocapture_Blocked"), "Warning.bmp");
            if (FindlistViewMessage(Item.Text) == -1)
                listViewMessages.Items.Add(Item);
        }

        private void UnblockAutoCapture()
        {
            ListViewItem Item;
            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_SetAutoCaptureBlocking(false);
            if (LedPresent)
                // stop blinking led(s)
                GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_VUI_LED_BlinkDuringAcquisition(0);

            // remove message
            Item = new ListViewItem(MSGUIRef.GetLocalizedString("Msg_Autocapture_Blocked"));
            int Index = FindlistViewMessage(Item.Text);
            if (Index != -1)
                listViewMessages.Items.RemoveAt(Index);
        }
							
        private int OnAcquisitionEnd(bool AutomaticStop, bool Timeout)
        {
            GBMSGUI.LogToFile("Entering OnAcquisitionEnd");

            // turn off acquisition flag
            Acquiring = false;

            ToDoFinalization = false;

            // if skip or break pressed, return immediately
            if (ScanResult != -1)
                return 0;

            // update user interface
            btnStop.Enabled = false;
            //btnRepeat.Enabled = true;
            //grpScanningOptions.Enabled = true;
            panRotate180.Enabled = true;

            DisplayMessage("", false);
            if (LCDPresent)
                // clear message on LCD
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                    GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_AND_DIAG_EMPTY,
                    GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);

            // check timeout
            if (Timeout)
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Acquisition_Timeout"), true);
                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_ACQUISITION_TIME_OUT,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                btnRepeat.Enabled = true;
                btnRepeat.Focus();

                // start listen pedal and LCD for Accept or Repeat actions
                if (PedalPresent)
                    StartListenPedal(PedatTestConditions.PedalPressing);

                if (LCDPresent)
                    ListenerTimer.Enabled = true;

                return 1;
            }

            // check stop type
            if (GBMSGUI.CheckMask(ScanOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE) && !AutomaticStop)
            {
                // stopped manually before autocapture
                listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Stop_Before_Autocapture"), "Warning.bmp");
            }

            // if roll is stopped manually, error
            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) && !AutomaticStop)
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Rotation_Not_Completed"), true);
                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_ROTATION_NO_COMPLETED,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                btnRepeat.Enabled = true;
                btnRepeat.Focus();

                // start listen pedal and LCD for Accept or Repeat actions
                if (PedalPresent)
                    StartListenPedal(PedatTestConditions.PedalPressing);

                if (LCDPresent)
                    ListenerTimer.Enabled = true;

                return 1;
            }

            // Contrast = 0 means no object detected
            if ((Contrast == 0) && !GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                ResetSegments();

                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_No_Object_Detected"), true);
                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_NO_FINGERPRINT_DETECTED,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                btnRepeat.Enabled = true;
                btnRepeat.Focus();

                // start listen pedal and LCD for Accept or Repeat actions
                if (PedalPresent)
                    StartListenPedal(PedatTestConditions.PedalPressing);

                if (LCDPresent)
                    ListenerTimer.Enabled = true;

                return 1;
            }

            if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
            {
                // notify acquisition process end (not for rolled, too many sounds...)
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound))
                {
                    if (SoundPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(50, 2, 1);
                    else
                        //Console.Beep(700, 100);
                        DSBeep(700, 100);
                }

                // 1.9.12.0 - restored check
                if (!GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA)
                    && !GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE))
                {
                    // 1.9.14.0 - set flag, image will be finalized after ImageProcessing
                    ToDoFinalization = true;
                    /*
                    // finalize image
                    GBMSAPI_NET_EndAcquisitionRoutines.GBMSAPI_NET_ImageFinalization(FramePtr);
                    GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
                    pboxImage.Refresh();
                    */
                }
            }

            // show contrast on LCD only for objects without any other information
            // if Quality option is active, quality is shown instead of contrast
            if (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID) && LCDPresent 
                && !GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation))
            {
                // 1.9.12.0 - except if dry/wet warning is selected
                if (!(MSGUIRef.DryFingerImageEnhancementSupported &&
                    (MSGUIRef.FingerContactEvalMode == GBMSGUI.FingerContactEvaluationMode.DryWetWarning)))
                {
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetContrastLimitOnStopScreen(
                        (byte)ScannedObject.ContrastThreshold2);
                    if (LCDMidLimitSupported)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(
                            (byte)ScannedObject.ContrastThreshold1);
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetContrastValueOnStopScreen(
                        (byte)Contrast);
                }
            }

            // Contrast=0 was here...

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
                GetRolledInfo();

            // call segmentation for slaps, ProcessImage for others (except Upper palm)
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
                {
                    // Slap4 now has automatic mode (GBFINIMG 3100)
                    /*
                    if (CheckUnavailableFingers())
                    {
                        // if some fingers are marked unavailable, switch to manual mode
                        listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Unavailable_Fingers"), "Warning.bmp");                        
                        SegmentationResult = false;
                    }
                    else
                    {
                    */

                    if (CheckUnavailableFingers())
                    {
                        // if some fingers are marked unavailable, notice
                        listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Unavailable_Fingers"), "Warning.bmp");
                    }

                    // show message on screen and LCD, because is a long operation
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Processing"), false);
                    Application.DoEvents();
                    if (LCDPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_PROCESSING,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);

                    // perform segmentation
                    ResetSegments();    // NEEDED when LiveSegmentation is active... to clear old data
                    SegmentationResult = DoSegmentation(false, false);

                    // center clipping rectangle with segment bounding boxes
                    if (ClippingRect.Visible && SegmentationResult)
                        CenterClipWithBoxes();
                }
            }
            else if (!GBMSGUI.IsUpperPalm(ScannedObject.ScannedObjectID))
            {
                // call GBFINIMG only if needed
                if ((MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.GBAlgorithm) ||
                    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInclination) ||
                    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectIncompletePattern) ||
                    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInvalidPattern) ||
                    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RemoveHaloLatent) ||
                    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RotateFinger))
                {
                    // show message on screen and LCD, because is a long operation
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Processing"), false);
                    Application.DoEvents();
                    if (LCDPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_PROCESSING,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                    // call ProcessImage
                    ProcessImageResult = ProcessImage();
                }
                else
                    // set to true for CheckResult
                    ProcessImageResult = true;
            }

            // 1.9.14.0 - call finalization after ProcessImage
            if (ToDoFinalization)
            {
                // finalize image
                GBMSAPI_NET_EndAcquisitionRoutines.GBMSAPI_NET_ImageFinalization(FramePtr);
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
                pboxImage.Refresh();
            }

            // clear message
            DisplayMessage("", false);
            if (LCDPresent)
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                    GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_AND_DIAG_EMPTY,
                    GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
            Application.DoEvents();

            // notify end of process
            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound))
            {
                if (SoundPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(50, 2, 1);
                else
                    //Console.Beep(500, 100);
                    DSBeep(500, 100);
            }

            // moved here from CheckResult
            //if (PostProcessDiag != 0)
                DisplayDiagnostic();

            btnRepeat.Enabled = true;

            // determine the result
            bool Result = CheckResult();
            
            // check Autoaccept
            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Autoaccept))
            {
                if (Result == true)
                {
                    btnAccept_Click(this, null);
                    return 1;
                }
                else
                {
                    // show message
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_No_Autoaccept"), true);
                    Application.DoEvents();
                    if (LCDPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_AUTOACEPT_CONDITION_NO_SATISFIED,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                }
            }

            GBMSGUI.LogToFile("Before starting ListenerTimer");

            // start listen pedal and LCD for Accept or Repeat actions
            if (PedalPresent)
                StartListenPedal(PedatTestConditions.PedalPressing);

            if (LCDPresent)
                ListenerTimer.Enabled = true;

            GBMSGUI.LogToFile("After starting ListenerTimer");

            return 1;
        }

        // check all conditions that determine the result
        private bool CheckResult()
        {
            bool Result = true;

            // clear message
            DisplayMessage("", false);

            // diagnostic
            if (LastDiagnostic != 0)
                Result = false;

            if (PostProcessDiag != 0)
            {
                //DisplayPostProcessingDiagnostic();
                Result = false;
            }

            List<NW_LFS_MINUTIA> Minutiae = null;

            // for single finger (flat or rolled), get quality and minutiae
            if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID))
            {
                ScannedObject.NFIQQuality = GetNFIQQuality(FramePtr, (int)ImageSizeX, (int)ImageSizeY, out Minutiae);

                // save minutiae for sequence check also for flat single fingers
                if (!GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
                {
                    GBMSGUI.SequenceCheckItem Item = new GBMSGUI.SequenceCheckItem();
                    Item.FingerID = GBMSGUI.ScannedObjectIDToAN2KFingerID(ScannedObject.ScannedObjectID);
                    Item.Minutiae = Minutiae;
                    MSGUIRef.SequenceCheckData.Add(Item);
                }
            }

            // for slaps, if segmentation option, check segments
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
                {
                    if (SegmentationResult)
                    {
                        // for LCD
                        Byte[] SegmResults = new Byte[4] 
                        {
                            GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK,
                            GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK,
                            GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK,
                            GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK 
                        };
                        // for Leds
                        uint IndicatorsRed = 0;
                        uint IndicatorsGreen = 0;

                        // check values on segments
                        for (int i = 0; i < SegmData.Count; i++)
                        {
                            if (SegmData[i].Found)
                            {
                                SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_OK;

                                // quality
                                if (!EvalSegmentIAFISQuality(i))
                                {
                                    Result = false;
                                    SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK;
                                }

                                // pattern validity
                                if (SegmData[i].InvalidPatternDetected)
                                {
                                    Result = false;
                                    SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK;
                                }
                                // pattenr completeness
                                if (SegmData[i].IncompletePatternDetected)
                                {
                                    Result = false;
                                    SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK;
                                }
                                // inclination
                                if (SegmData[i].InclinationDetected)
                                {
                                    Result = false;
                                    SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK;
                                }
                            }
                            else if (SegmData[i].UnavailabilityReason != -1)
                                // show as unavailable
                                SegmResults[i] = GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_UNAVAILABLE;

                            // set green and red masks for led indicators
                            if (SegmResults[i] == GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_OK)
                                IndicatorsGreen |= SetIndicatorMask(GBMSGUI.AN2KFingerIDToScannedObjectID(SegmData[i].FingerID));
                            else if (SegmResults[i] == GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_NOT_OK)
                                IndicatorsRed |= SetIndicatorMask(GBMSGUI.AN2KFingerIDToScannedObjectID(SegmData[i].FingerID));
                        }

                        if (LedPresent)
                        {
                            // turn on leds
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_SetFingerIndicatorVUIState(
                                IndicatorsGreen, GBMSAPI_NET_LEDColors.GBMSAPI_NET_COLOR_GREEN, false);
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_SetFingerIndicatorVUIState(
                                IndicatorsRed, GBMSAPI_NET_LEDColors.GBMSAPI_NET_COLOR_RED, false);
                        }

                        if (LCDPresent)
                        {
                            // show segmentation result on LCD
                            if (GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScannedObject.ScannedObjectID) 
                                == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_4)
                                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetSegmentationOnStopScreen(
                                    SegmResults[0], SegmResults[1], SegmResults[2], SegmResults[3]);
                            else
                                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetSegmentationOnStopScreen(
                                    GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY,
                                    SegmResults[0], SegmResults[1],
                                    GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY);
                        }

                        // verify position of segments bounding boxes
                        if (!CheckSegmentBoundingBoxPositions())
                            Result = false;

                        // verify overlap of segments bounding boxes
                        if (!CheckSegmentBoundingBoxOverlap())
                            Result = false;
                    }
                    else // if (SegmentationResult)
                    {
                        DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Segmentation_Failed"), true);
                        // inform about manual mode
                        // bug fix: add only once
                        ListViewItem Item = new ListViewItem(MSGUIRef.GetLocalizedString("Msg_Manual_Segm_Info"), "Info.bmp");
                        if (FindlistViewMessage(Item.Text) == -1)
                            listViewMessages.Items.Add(Item);
                        //listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Manual_Segm_Info"), "Info.bmp");
                        if (LCDPresent)
                        {
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                                GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SEGMENTATION_FAILED,
                                GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetSegmentationOnStopScreen(
                                GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY,
                                GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY,
                                GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY,
                                GBMSAPI_NET_SegmentationResultsValues.GBMSAPI_NET_VILCD_SEG_EMPTY);
                        }
                        btnRepeat.Focus();
                        return false;
                    }
                }
            }
            else // not slap
            {
                // check values on scanned object
                // Quality
                if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID) ||
                    (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID) && GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation)))
                    if (!EvalIAIFSQuality())
                        Result = false;

                // Artefacts
                if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID))
                    if (!EvalRolledArtefactsSize())
                        Result = false;

                // sequence check for rolled
                if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) &&
                    GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.SequenceCheck))
                {
                    if (!CheckSequence(Minutiae))
                        Result = false;
                }

                if (!GBMSGUI.IsUpperPalm(ScannedObject.ScannedObjectID))
                {
                    if (ProcessImageResult)
                    {
                        // Completeness
                        if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID))
                        {
                            if (PalmCompletenessSupported)
                            {
                                if (LCDPresent)
                                {
                                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetCompletenessLimitOnStopScreen(
                                        (byte)MSGUIRef.CfgData.LowerPalmCompletenessThreshold2);
                                    if (LCDMidLimitSupported)
                                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(
                                            (byte)MSGUIRef.CfgData.LowerPalmCompletenessThreshold1);
                                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetCompletenessValueOnStopScreen(
                                        (byte)PalmCompleteness);
                                }
                            }
                        }
                        else
                        {
                            if (IncompletePatternDetected)
                                Result = false;
                        }

                        // Validity
                        if (InvalidPatternDetected)
                            Result = false;

                        // Inclination
                        if (InclinationDetected)
                            Result = false;
                    }
                    else
                    {
                        Result = false;
                    }
                }

                if (LedPresent)
                {
                    // use led to show result
                    int Color;
                    if (Result)
                        Color = GBMSAPI_NET_LEDColors.GBMSAPI_NET_COLOR_GREEN;
                    else
                        Color = GBMSAPI_NET_LEDColors.GBMSAPI_NET_COLOR_RED;
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_SetFingerIndicatorVUIState(
                            SetIndicatorMask(ScannedObject.ScannedObjectID), Color, false);
                }
            }

            /*
            // sequence check for rolled
            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) && 
                GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.SequenceCheck))
            {
                if (!CheckSequence(Minutiae))
                    Result = false;
            }
            */

            // update user interface according to result
            if (Result)
            {
                // if all Ok, set focus on accept
                btnAccept.Enabled = true;
                btnAccept.Focus();

                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Success"), false);
                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_ACQUISITION_SUCCESSFUL,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);

                if (LCDPresent)
                    // enable OK button on LCD
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_EnableOKButtonOnStopScreen();
            }
            else
            {
                // enable accept anyway, but set focus on repeat
                btnAccept.Enabled = true;
                btnRepeat.Focus();

                // if not yet a message present
                if (lblMessage.Text == "")
                {
                    // tell the operator he should evaluate the result
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Check_Result"), false);
                    if (LCDPresent)
                    {
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_EVALUATE_THE_RESULT,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                    }
                }

                if (LCDPresent)
                    // enable OK button on LCD
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_EnableOKButtonOnStopScreen();

            }

            return Result;
        }

        // adjusts the image inside panImageSpace
        private void AdjustProportion()
        {
            int newWidth, newHeight, maxWidth, maxHeight;

            // consider border of panImageSpace (1 pixel per side)
            maxWidth = panImageSpace.Width - 2;
            maxHeight = panImageSpace.Height - 2;

            if (AspectRatio > 1.0)
            {
                // landscape image
                newWidth = maxWidth;
                newHeight = Convert.ToInt32(Math.Floor(newWidth / AspectRatio + 0.5));

                if (newHeight > maxHeight)
                {
                    newHeight = maxHeight;
                    newWidth = Convert.ToInt32(Math.Floor(newHeight * AspectRatio + 0.5));
                }
            }
            else
            {
                // portrait image
                newHeight = maxHeight;
                newWidth = Convert.ToInt32(Math.Floor(newHeight * AspectRatio + 0.5));

                if (newWidth > maxWidth)
                {
                    newWidth = maxWidth;
                    newHeight = Convert.ToInt32(Math.Floor(newWidth / AspectRatio + 0.5));
                }
            }

            panImage.Width = newWidth;
            panImage.Height = newHeight;

            // assign proportion value between real image and view
            ViewScaleFactor = (double)ImageSizeX / (double)newWidth;

            // center inside panImageSpace
            panImage.Left = panImageSpace.Width / 2 - panImage.Width / 2 - 1;
            panImage.Top = panImageSpace.Height / 2 - panImage.Height / 2 - 1;
        }

        private void panImageSpace_Resize(object sender, EventArgs e)
        {
            AdjustProportion();

            // adjust view of clipping rectangle and bounding boxes (if visible)
            if (ClippingRect.Visible)
                RealToViewRect(ClippingRect.RealRect, ref ClippingRect.ViewRect);

            // DEBUG
            //GBMSGUI.LogToFile("Resize: ClippingRect width = " + ClippingRect.RealRect.Width.ToString() +
            //    " ClippingRect height = " + ClippingRect.RealRect.Height.ToString());

            for (int i = 0; i < SegmData.Count; i++)
                if (SegmData[i].BoundingBox.Visible)
                    RealToViewRect(SegmData[i].BoundingBox.RealRect, ref SegmData[i].BoundingBox.ViewRect);

            pboxImage.Refresh();
        }

        private void StopScanning()
        {
            int ret;
            ret = GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_StopAcquisition();
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_StopAcquisition", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            int ret;

            // avoid double click
            btnStop.Enabled = false;

            if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) && 
                PreviewPhase &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RollPreviewManualStop))
            {
                ret = GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_ROLL_StopPreview();
                if (ret == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    // re-enable stop
                    btnStop.Enabled = true;
                    return;
                }
                // ignore GBMSAPI_NET_ERROR_CODE_METHOD_NOT_SUPPORTED error
                if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_METHOD_NOT_SUPPORTED)
                {
                    MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_ROLL_StopPreview", ret), ret);
                    ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                    return;
                }
            }

            StopScanning();
        }

        private void btnAccept_Click(object sender, EventArgs e)
        {
            //GBMSGUI.LogToFile("Entrata funzione btnAccept_Click");

            // avoid double click
            btnAccept.Enabled = false;

            // BUG FIX 1.9.12.0 - moved before image clipping (otherwise, if canceled, the image size is wrong)
            // for slap, check segments presence
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID) &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
            {
                //GBMSGUI.LogToFile("Check segments presence");
                if (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason))
                    if (!AreSegmentsAllFound())
                        if (!AskUnavailabilityReason())
                            return;

                GBMSGUI.LogToFile("CutBoundingBoxPositions");
                // ensure bounding boxes coordinates are inside the image
                //CutBoundingBoxPositions();
            }

            // return image to application
            if (ClippingRect.Visible)
            {
                // DEBUG
                //GBMSGUI.LogToFile("Clipping image: ClippingRect width = " + ClippingRect.RealRect.Width.ToString() +
                //    " ClippingRect height = " + ClippingRect.RealRect.Height.ToString());
                // clip image
                bmpImage = bmpImage.Clone(ClippingRect.RealRect, PixelFormat.Format8bppIndexed);
            }

            //GBMSGUI.LogToFile("Notify with Beep");
            // notify accept with sound
            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound))
            {
                //Console.Beep(800, 300);
                DSBeep(800, 300);
            }

            // show a message, while application saves image (can take long time for big images)
            DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Saving_Image"), false);
            if (LCDPresent)
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                    GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SAVING_IMAGE,
                    GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);

            ScanResult = GBMSGUI.ReturnCodes.Ret_Success;

            //GBMSGUI.LogToFile("Uscita funzione btnAccept_Click");
        }

        private void btnRepeat_Click(object sender, EventArgs e)
        {
            // avoid double click
            btnRepeat.Enabled = false;

            // inform about action
            DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Repeating"), false);
            Application.DoEvents();

            if (LCDPresent)
            {
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                    GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_REPEATING,
                    GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
            }

            if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound))
            {
                //Console.Beep(650, 200);
                DSBeep(650, 200);
            }

            // to avoid memory problems, force Grarbage collection
            GC.Collect();

            // restart acquisition
            StartScanning();
        }

        private void pboxImage_MouseDown(object sender, MouseEventArgs e)
        {
            int i;

            // during acquisition, nothing allowed
            if (Acquiring)
                return;

            // if right mouse click, popup menu for segment
            if (e.Button == MouseButtons.Right)
            {
                //if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID) &&
                //    GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
                //{
                    // check if mouse pointer is inside a segment bounding box
                    for (i = 0; i < SegmData.Count; i++)
                    {
                        if (SegmData[i].BoundingBox.Visible)
                            if (CheckMousePosition((Control)sender, e.X, e.Y, SegmData[i].BoundingBox.ViewRect, 0))
                            {
                                // popup menu for editing segment
                                CurrSegmIndex = i;    // save index
                                popEditSegmMenu.Show((Control)sender, e.Location);
                                return;
                            }
                    }

                    if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID) &&
                        GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
                    {
                        // poupu menu for finding segments
                        // save point
                        SegmMenuPoint = e.Location;
                        BuildSegmMenu();
                        popFindSegmMenu.Show((Control)sender, SegmMenuPoint);
                        return;
                    }
                //}
            }

            // start move or resize with left mouse button
            if (e.Button == MouseButtons.Left)
            {
                mMouseDown = true;
                StartPoint = ((Control)sender).PointToScreen(new Point(e.X, e.Y));
                // do not allow to exit from the control
                Cursor.Clip = ((Control)sender).RectangleToScreen(((Control)sender).ClientRectangle);
            }
        }

        private void pboxImage_MouseMove(object sender, MouseEventArgs e)
        {
            // during acquisition, nothing allowed
            if (Acquiring)
                return;

            int i;
            
            if (!mMouseDown)
            {
                // show appropriate cursor
                SizeOptions Options = 0;

                CurrRect = null;

                // check mouse position and determine size operation
                // check bounding boxes first, otherwise can be covered by clipping rectangle
                for (i = 0; i < SegmData.Count; i++)
                {
                    //Options = SizeOptions.AllowResize;
                    Options = SizeOptions.AllowResize | SizeOptions.AllowMove;
                    if (SegmData[i].BoundingBox.Visible)
                        if (CheckMousePosition((Control)sender, e.X, e.Y, SegmData[i].BoundingBox.ViewRect, Options))
                        {
                            // save affected rectangle
                            CurrRect = SegmData[i].BoundingBox;
                            return;
                        }
                }
                if (ClippingRect.Visible)
                {
                    // allow move of clipping rectangle
                    Options = SizeOptions.AllowMove;
                    if (CheckMousePosition((Control)sender, e.X, e.Y, ClippingRect.ViewRect, Options))
                    {
                        // save affected rectangle
                        CurrRect = ClippingRect;
                        return;
                    }
                }
            }
            else // mMouseDown
            {
                if (mSizeOperation == SizeOperations.None)
                    return;

                // perform size operation on selected rectangle
                DoSizeOperation((Control)sender, e.X, e.Y, ref CurrRect.ViewRect);
                // update real rectangle
                Rectangle tmpRect = new Rectangle();
                ViewToRealRect(CurrRect.ViewRect, ref tmpRect);
                if (CurrRect == ClippingRect)
                {
                    // if clipping rect, do not change size (can be approximation errors due to conversion)
                    CurrRect.RealRect.X = tmpRect.X;
                    CurrRect.RealRect.Y = tmpRect.Y;
                    // clipping coordinates MUST be even
                    CurrRect.RealRect.X &= ~0x01;
                    CurrRect.RealRect.Y &= ~0x01;
                }
                else
                {
                    CurrRect.RealRect.X = tmpRect.X;
                    CurrRect.RealRect.Y = tmpRect.Y;
                    CurrRect.RealRect.Width = tmpRect.Width;
                    CurrRect.RealRect.Height = tmpRect.Height;
                }
            }
        }

        private void pboxImage_MouseUp(object sender, MouseEventArgs e)
        {
            mMouseDown = false;
            // reset cursor clip
            Cursor.Clip = Rectangle.Empty;

            if ((CurrRect != null) && 
                GBMSGUI.IsSlap(ScannedObject.ScannedObjectID) &&
                GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Segmentation))
            {
                // clipping rectangle or bounding boxes are moved, check again
                CheckResult();
            }
        }

        private void pboxImage_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                // Create a local version of the graphics object for the PictureBox.
                Graphics g = e.Graphics;

                // optimize for speed
                g.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighSpeed;
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighSpeed;
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;

                if (FramePtr != null)
                {
                    if (FullResImage)
                        g.DrawImage(bmpImage, pboxImage.ClientRectangle);
                    else
                        g.DrawImage(bmpPreviewImage, pboxImage.ClientRectangle);
                }
                else
                {
                    g.Clear(SystemColors.Control);
                    return;
                }

                if (ClippingRect.Visible)
                {
                    Pen BluePen = new Pen(Color.Blue, 2);

                    // draw clipping rectangle in view
                    g.DrawRectangle(BluePen, ClippingRect.ViewRect);
                }

                // clipping is the only thing shown during acquisition
                // PROVA
                //if (Acquiring)
                //    return;

                // draw segments boxes and info
                int i;
                Pen DashedPen = new Pen(Color.Fuchsia, 1);
                DashedPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;

                for (i = 0; i < SegmData.Count; i++)
                {
                    if (SegmData[i].Found)
                    {
                        // box
                        g.DrawRectangle(DashedPen, SegmData[i].BoundingBox.ViewRect);
                        // draw text
                        DrawShadowedText(g, SegmData[i].FingerText, SegmData[i].BoundingBox.ViewRect);
                        // quality
                        if (MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                        {
                            if (SegmData[i].NFIQQuality != 0)
                                DrawSegmentQuality(g, SegmData[i].NFIQQuality, SegmData[i].QualityColor, SegmData[i].BoundingBox.ViewRect);
                        }
                        else
                        {
                            if (SegmData[i].GBQuality != 0)
                                DrawSegmentQuality(g, SegmData[i].GBQuality, SegmData[i].QualityColor, SegmData[i].BoundingBox.ViewRect);
                        }

                        // draw diagnostic (pattern, inclination)
                        if (SegmData[i].InvalidPatternDetected || SegmData[i].IncompletePatternDetected)
                        {
                            DrawSegmentDiagnostic(g, imgLstSegmentIcons.Images["Pattern.ico"], SegmData[i].BoundingBox.ViewRect);
                        }
                        else if (SegmData[i].InclinationDetected)
                        {
                            DrawSegmentDiagnostic(g, imgLstSegmentIcons.Images["Inclination.ico"], SegmData[i].BoundingBox.ViewRect);
                        }
                    }
                }

                if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) && (MarkerFrame != null)
                    && !GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.NoArtefactsDisplay))
                {
                    // draw artefacts marker
                    DrawMarkerFrame(g);
                }
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message + " in pboxImage_Paint", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                //pboxImage.Refresh();
                MSGUIRef.SetError(ex.Message + " in pboxImage_Paint", 0);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                Acquiring = false;
            }
        }

        private bool DoSegmentation(bool ManualMode, bool IncompleteSlap)
        {
            try
            {
                bool Failed = false;
                int i;
                int SegmSizeX, SegmSizeY;
                bool SingleObjectFlag = false;  // for slap 2 in case of missing finger

                //GBMSGUI.LogToFile("Entrata in DoSegmentation");

                PostProcessDiag = 0;

                // get maximum size requested for segments
                GetMaxSegmentsSize(out SegmSizeX, out SegmSizeY);

                List<NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO> SegmExtInfo;
                NW_GBFINIMG_OutputData SegmentationOutput;

                // ManualMode - operator selects segments
                if (ManualMode)
                {
                    SegmExtInfo = new List<NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO>();
                    // add items to SegmExtInfo from SegmData
                    NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO Info;
                    for (i = 0; i < SegmData.Count; i++)
                    {
                        if (SegmData[i].FingerCenterX != -1)
                        {
                            Info = new NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO();
                            //V1.2 - patch because 2 indexes for GBFINIMG does not exists
                            if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
                            {
                                if (SegmData[i].FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX)
                                    Info.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                                if (SegmData[i].FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX)
                                    Info.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                            }
                            else 
                                Info.FingerID = SegmData[i].FingerID;
                            Info.FingerCenterX = (uint)SegmData[i].FingerCenterX;
                            Info.FingerCenterY = (uint)SegmData[i].FingerCenterY;
                            SegmExtInfo.Add(Info);
                        }
                    }

                    // do not pass an empty external info
                    if (SegmExtInfo.Count == 0)
                    {
                        ResetSegments();
                        pboxImage.Refresh();
                        return false;
                    }
                }
                else
                    SegmExtInfo = null;

                // prepare input data
                uint ImageType = 0;
                if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
                else if ((ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                    //V1.2 - patch because 2 indexes for GBFINIMG does not exists
                    || (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES))
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;
                //V1.1
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2;
                  
                uint Options = 0;
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RemoveHaloLatent))
                {
                    if (ToDoFinalization)
                        // 1.9.14.0 - lighter processing if after ImageFinalization will be called
                        Options = NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_HALO_LATENT_ELIMINATION;
                    else
                        // 1.9.12.0 - new option substitute halo latent
                        Options = NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND;
                }
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RotateFinger))
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_SEGMENT_VERTICAL_ROTATION;
                if (MSGUIRef.Resolution1000Dpi)
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;
                if (IncompleteSlap)
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION |
                        NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK;


                // missing fingers
                if ((ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4) ||
                    (ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4))
                {
                    if (CheckUnavailableFingers())
                    {
                        byte MissingFingerMask = 0;

                        if (ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4)
                        {
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
                            if (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
                            if (MSGUIRef.SegmCfgData[2].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_RING;
                            if (MSGUIRef.SegmCfgData[3].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
                        }
                        else	// NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4
                        {
                            if (MSGUIRef.SegmCfgData[3].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
                            if (MSGUIRef.SegmCfgData[2].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
                            if (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_RING;
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
                        }

                        //GBFINIMG_Utilities.GBFINIMGdll_NetWrapper.GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
                        Segmentator.NW_GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
                    }
                }
                else    // Slap 2
                {
                    if (CheckUnavailableFingers())
                    {
                        // if both are set as unavailable, fail
                        if ((MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1) &&
                            (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1))
                        {
                            ResetSegments();
                            pboxImage.Refresh();
                            return false;
                        }
                        ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
                        SingleObjectFlag = true;
                    }
                }

                NW_GBFINIMG_InputData SegmInput = new NW_GBFINIMG_InputData(FramePtr,
                    ImageSizeX, ImageSizeY,
                    ImageType,
                    SegmExtInfo,
                    Options,
                    (uint)SegmSizeX, 
                    (uint)SegmSizeY);


                // perform segmentation
                Segmentator.InputToSet = SegmInput;
                SegmentationOutput = Segmentator.OutputAfterProcess;

                // DEBUG
                //if (SegmentationOutput.OutputFrame == null)
                //    GBMSGUI.LogToFile("SegmentationOutput.OutputFrame is null!");

                // update bitmap
                FramePtr = SegmentationOutput.OutputFrame;
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
                pboxImage.Refresh();

                // check if any segment is found
                if ((SegmentationOutput.Segment_List == null) || (SegmentationOutput.Segment_List.Count == 0))
                {
                    Failed = true;
                    // DEBUG
                    //GBMSGUI.LogToFile("Segment_List is NULL or empty");
                }

                // DEBUG
                //if (Failed)
                //    bmpImage.Save("SegmFailed.bmp", ImageFormat.Bmp);

                // check diagnostic
                if (!SegmentationOutput.Diag.NW_GBFINIMG_NET_WRAPPER_NO_DIAG)            
                {
                    // NEW - if incomplete or ambiguous slap, and option active, try automatic mode
                    if ((SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP ||
                        SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP) &&
                        GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.AutoIncompleteSlapSegm)
                        && !IncompleteSlap)	// avoid to call recursively...
                    {
                        //GBMSGUI.LogToFile("Calling DoSegmentation again for incomplete slap");
                        return DoSegmentation(false, true);
                    }

                    // add diagnostic messages
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
                    {
                        PostProcessDiag |= PostProcessDiagnostic.Diag_SegmAmbiguousSlap;
                    }
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP)
                    {
                        // different message depending if segments are found or not
                        if (Failed)
                            PostProcessDiag |= PostProcessDiagnostic.Diag_SegmIncompleteSlap;
                        else
                            PostProcessDiag |= PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap;
                    }
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
                    {
                        // different message depending if segments are found or not
                        if (Failed)
                            PostProcessDiag |= PostProcessDiagnostic.Diag_LeftRightMixup;
                        else
                            PostProcessDiag |= PostProcessDiagnostic.Diag_PossibleLeftRightMixup;
                    }

                    /*
                    if (IncompleteSlap)
                        Failed = false;
                    else
                        Failed = true;
                    */
                    // corrected - if really failed, even for incomplete slap, SegmentationOutput.Segment_List is null!
                    if (!IncompleteSlap)
                        Failed = true;
                }


                // check if segments found are identified
                // don't check in case of SingleObjectFlag
                if (!Failed && !SingleObjectFlag)
                {
                    // DEBUG
                    //if (SegmentationOutput.Segment_List == null)
                    //    GBMSGUI.LogToFile("Failed is FALSE but SegmentationOutput.Segment_List is NULL!!!");

                    for (i = 0; i < SegmentationOutput.Segment_List.Count; i++)
                    {
                        if ((SegmentationOutput.Segment_List[i].FingerID ==
                                NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN))
                        {
                            // failed to identify segments, needed manual identification
                            Failed = true;
                        }
                    }
                }

                if (!Failed)
                {
                    // DEBUG
                    //if (SegmentationOutput.Segment_List == null)
                    //    GBMSGUI.LogToFile("Failed is FALSE but SegmentationOutput.Segment_List is NULL!!!");

                    // read results

                    //GBMSGUI.LogToFile("Segments found: " + SegmentationOutput.Segment_List.Count.ToString());

                    // note that elements in list can be in different order; check FingerID
                    for (i = 0; i < SegmentationOutput.Segment_List.Count; i++)
                    {
                        int FngIndex;
                        if (SingleObjectFlag)
                        {
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason == -1)
                                FngIndex = 0;
                            else if (MSGUIRef.SegmCfgData[1].UnavailabilityReason == -1)
                                FngIndex = 1;
                            else
                                continue;
                        }
                        else
                        {
                            FngIndex = IndexOfFingerSegm(SegmentationOutput.Segment_List[i].FingerID);
                            if (FngIndex == -1)
                            {
                                // DEBUG
                                //GBMSGUI.LogToFile("Finger not found in Segment_List!");

                                continue;
                            }
                        }
                        SegmData[FngIndex].Found = true;
                        SegmData[FngIndex].FingerCenterX = (int)SegmentationOutput.Segment_List[i].FingerCenterX;
                        SegmData[FngIndex].FingerCenterY = (int)SegmentationOutput.Segment_List[i].FingerCenterY;
                        SegmData[FngIndex].BoundingBox.RealRect.X = SegmentationOutput.Segment_List[i].SegBoundingBoxL;
                        SegmData[FngIndex].BoundingBox.RealRect.Y = SegmentationOutput.Segment_List[i].SegBoundingBoxT;
                        SegmData[FngIndex].BoundingBox.RealRect.Width = SegmentationOutput.Segment_List[i].SegBoundingBoxR - SegmentationOutput.Segment_List[i].SegBoundingBoxL;
                        SegmData[FngIndex].BoundingBox.RealRect.Height = SegmentationOutput.Segment_List[i].SegBoundingBoxB - SegmentationOutput.Segment_List[i].SegBoundingBoxT;
                        RealToViewRect(SegmData[FngIndex].BoundingBox.RealRect, ref SegmData[FngIndex].BoundingBox.ViewRect);
                        SegmData[FngIndex].BoundingBox.Visible = true;
                        // create bitmap for segment
                        //GBMSGUI.LogToFile("Create bmpImage for FngIndex=" + FngIndex.ToString());
                        SegmData[FngIndex].bmpImage = GBMSGUI.RawImageToBitmap(
                            SegmentationOutput.Segment_List[i].SegmentImagePtr, SegmSizeX, SegmSizeY);
                        // DEBUG - check
                        //if (SegmData[FngIndex].bmpImage == null)
                        //    GBMSGUI.LogToFile("bmpImage is null");
                        // clip the segment image if necessary
                        if ((MSGUIRef.SegmCfgData[i].ImageSizeX < SegmSizeX) || (MSGUIRef.SegmCfgData[i].ImageSizeY < SegmSizeY))
                        {
                            // align to top, center horizontally
                            Rectangle Rect = new Rectangle();
                            Rect.Y = 0;
                            Rect.X = SegmSizeX/2 - MSGUIRef.SegmCfgData[i].ImageSizeX/2;
                            Rect.Width = MSGUIRef.SegmCfgData[i].ImageSizeX;
                            Rect.Height = MSGUIRef.SegmCfgData[i].ImageSizeY;
                            // clip
                            SegmData[FngIndex].bmpImage = SegmData[FngIndex].bmpImage.Clone(Rect, PixelFormat.Format8bppIndexed);
                        }
                        // quality
                        SegmData[FngIndex].GBQuality = SegmentationOutput.Segment_List[i].SegmQuality.quality;
                        // get NFIQ quality and minutiae for segment
                        List<NW_LFS_MINUTIA> Minutiae;
                        SegmData[FngIndex].NFIQQuality = GetNFIQQuality(SegmentationOutput.Segment_List[i].SegmentImagePtr, SegmSizeX, SegmSizeY, out Minutiae);
                        // add minutiae for sequence check
                        GBMSGUI.SequenceCheckItem Item = new GBMSGUI.SequenceCheckItem();
                        Item.FingerID = SegmentationOutput.Segment_List[i].FingerID;
                        // PATCH - if object is GBMSAPI_NET_SO_SLAP_2_INDEXES, replace FingerID (for GBFINIMG are thumbs...)
                        if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
                        {
                            if (Item.FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB)
                                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                            if (Item.FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB)
                                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                        }
                        Item.Minutiae = Minutiae;
                        MSGUIRef.SequenceCheckData.Add(Item);

                        // check diagnostic on segment (pattern, inclination)
                        if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectIncompletePattern) &&
                            SegmentationOutput.Segment_List[i].PatternCompletness < MSGUIRef.CfgData.PatternCompletenessThreshold)
                        {
                            SegmData[FngIndex].IncompletePatternDetected = true;
                            PostProcessDiag |= PostProcessDiagnostic.Diag_IncompletePattern;
                        }
                        if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInvalidPattern) &&
                            SegmentationOutput.Segment_List[i].PatternValidity < MSGUIRef.CfgData.PatternValidityThreshold)
                        {
                            SegmData[FngIndex].InvalidPatternDetected = true;
                            PostProcessDiag |= PostProcessDiagnostic.Diag_InvalidPattern;
                        }
                        if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInclination) &&
                            Math.Abs(SegmentationOutput.Segment_List[i].Inclination) > MSGUIRef.CfgData.InclinationThreshold)
                        {
                            SegmData[FngIndex].InclinationDetected = true;
                            PostProcessDiag |= PostProcessDiagnostic.Diag_Inclination;
                        }
                    }
                }

                if (Failed)
                {
                    ResetSegments();
                    pboxImage.Refresh();
                    return false;
                }

                pboxImage.Refresh();
                return true;
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Default;
                MessageBox.Show(ex.Message + " in DoSegmentation", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private int IndexOfFingerSegm(String FingerID)
        {
            int i;

            // returns index of SegmData array containing FingerID
            for (i = 0; i < SegmData.Count; i++)
            {
                //V1.2 - patch because 2 indexes for GBFINIMG does not exists
                if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
                {
                    if ((FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB) &&
                        (SegmData[i].FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX))
                        return i;
                    if ((FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB) &&
                        (SegmData[i].FingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX))
                        return i;
                }
                else
                {
                    if (SegmData[i].FingerID == FingerID)
                        return i;
                }
            }

            return -1;
        }

        private void BuildSegmMenu()
        {
            int i;

            popFindSegmMenu.MenuItems.Clear();

            // add menu for segments not yet found
            for (i = 0; i < SegmData.Count; i++)
            {
                if (!SegmData[i].Found && (MSGUIRef.SegmCfgData[i].UnavailabilityReason == -1))
                    popFindSegmMenu.MenuItems.Add(SegmData[i].FindSegmMenuItem);
            }
            
            // if incomplete slap, add possibility to try automatic
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmIncompleteSlap) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmAmbiguousSlap))
            {
                popFindSegmMenu.MenuItems.Add("-");
                popFindSegmMenu.MenuItems.Add(MSGUIRef.GetLocalizedString("Str_Incomplete_Slap_Segm"), OnIncompleteSlap_Click);
            }
        }

        private void SetSegments()
        {
            int i;

            // according to ScannedObjectID, set data for segments
            SegmData.Clear();
            SegmentData Item;

            if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Left");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Right");
                SegmData.Add(Item);
            }
            else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Little");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Ring");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Middle");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Index");
                SegmData.Add(Item);
            }
            else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Index");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Middle");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Ring");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Little");
                SegmData.Add(Item);
            }
            //V1.1
            else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Middle");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Index");
                SegmData.Add(Item);
            }
            else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Index");
                SegmData.Add(Item);
                
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Middle");
                SegmData.Add(Item);
            }
            //V1.2
            else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES)
            {
                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Left");
                SegmData.Add(Item);

                Item = new SegmentData();
                Item.FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                Item.FingerText = MSGUIRef.GetLocalizedString("Segm_Right");
                SegmData.Add(Item);
            }

            for (i = 0; i < SegmData.Count; i++)
            {
                // create menu item for finding segment
                SegmData[i].FindSegmMenuItem = new MenuItem(SegmData[i].FingerText);
                // set Tag to identify menu item in handler
                SegmData[i].FindSegmMenuItem.Tag = i;
                SegmData[i].FindSegmMenuItem.Click += new System.EventHandler(FindSegmMenuItem_Click);
            }
        }

        private void ResetSegments()
        {
            int i;

            for (i = 0; i < SegmData.Count; i++)
            {
                SegmData[i].Found = false;
                SegmData[i].FingerCenterX = -1;
                SegmData[i].FingerCenterY = -1;
                SegmData[i].BoundingBox.Visible = false;
                SegmData[i].IncompletePatternDetected = false;
                SegmData[i].InvalidPatternDetected = false;
                SegmData[i].InclinationDetected = false;
                SegmData[i].UnavailabilityReason = -1;
                SegmData[i].QualityColor = SystemColors.Window;
                if (SegmData[i].bmpImage != null)
                    SegmData[i].bmpImage.Dispose();
            }
        }

        private void FindSegmMenuItem_Click(object sender, System.EventArgs e)
        {
            int Index = (int)((MenuItem)sender).Tag;

            Rectangle tmpRect = new Rectangle(SegmMenuPoint, new Size(0, 0));
            // convert to real 
            ViewToRealRect(tmpRect, ref tmpRect);
            SegmData[Index].FingerCenterX = tmpRect.X;
            SegmData[Index].FingerCenterY = tmpRect.Y;

            // perform segmentation with manually added info
            SegmentationResult = DoSegmentation(true, false);
            CheckResult();
        }

        private void btnSkip_Click(object sender, EventArgs e)
        {
            // avoid double click
            btnSkip.Enabled = false;

            if (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason))
            {
                // if skip an entire slap, first reset segments
                if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
                {
                    ResetSegments();
                    pboxImage.Refresh();
                }

                // prompt operator for unavailability reason
                if (!AskUnavailabilityReason())
                    return;
            }
            else
            {
                // ask confirmation
                if (MessageBox.Show(MSGUIRef.GetLocalizedString("Msg_Confirm_Skip"),
                    MSGUIRef.GetLocalizedString("Dlg_Caption_Confirm"),
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                {
                    btnSkip.Enabled = true;
                    return;
                }
            }

            ScanResult = GBMSGUI.ReturnCodes.Ret_SkipImage;
        }

        private void btnBreak_Click(object sender, EventArgs e)
        {
            String Msg;

            // avoid double click
            btnBreak.Enabled = false;

            if (SingleAcquisition)
                Msg = MSGUIRef.GetLocalizedString("Msg_Confirm_Cancel");
            else
                Msg = MSGUIRef.GetLocalizedString("Msg_Confirm_Break");

            // ask confirmation
            if (MessageBox.Show(Msg, MSGUIRef.GetLocalizedString("Dlg_Caption_Confirm"),
                MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
            {
                btnBreak.Enabled = true;
                return;
            }

            // force acquisition flag
            //Acquiring = false;

            ScanResult = GBMSGUI.ReturnCodes.Ret_BreakSequence;
        }

        private void btnBack_Click(object sender, EventArgs e)
        {
            // avoid double click
            btnBack.Enabled = false;

            // ask confirmation
            if (MessageBox.Show(MSGUIRef.GetLocalizedString("Msg_Confirm_Back"),
                MSGUIRef.GetLocalizedString("Dlg_Caption_Confirm"),
                MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
            {
                btnBack.Enabled = true;
                return;
            }

            ScanResult = GBMSGUI.ReturnCodes.Ret_GoBack;
        }

        private void AcquisitionForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // form is closed only setting ForceClose to true in GBMSGUI.StopSession
            if (!ForceClose)
            {
                e.Cancel = true;

                // translate closing by "X" into Break
                btnBreak_Click(sender, e);
            }

            //if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RotateFinger)
            //    && MSGUIRef.Resolution1000Dpi)

            if (MSGUIRef.Resolution1000Dpi)
            {
                // release buffer for rotation
                Segmentator.NW_GBFINIMG_ReleaseRotationBuffer1000DPI();
            }
        }

        private void mnuRemoveSegm_Click(object sender, EventArgs e)
        {
            // remove segment
            SegmData[CurrSegmIndex].Found = false;
            SegmData[CurrSegmIndex].FingerCenterX = -1;
            SegmData[CurrSegmIndex].FingerCenterY = -1;
            SegmData[CurrSegmIndex].BoundingBox.Visible = false;

            // perform segmentation again
            SegmentationResult = DoSegmentation(true, false);
            CheckResult();
        }

        private void mnuViewSegm_Click(object sender, EventArgs e)
        {
            ViewSegmForm frmViewSegm = new ViewSegmForm();

            frmViewSegm.bmpImage = SegmData[CurrSegmIndex].bmpImage;
            frmViewSegm.Text = SegmData[CurrSegmIndex].FingerText;
            frmViewSegm.ShowDialog();
        }

        private void SetGUIStrings()
        {
            // load all controls text from resources
            this.Text = MSGUIRef.GetLocalizedString("Acquisition");
            toolTipButtons.SetToolTip(btnStop, MSGUIRef.GetLocalizedString("btnStop"));
            toolTipButtons.SetToolTip(btnAccept, MSGUIRef.GetLocalizedString("btnAccept"));
            toolTipButtons.SetToolTip(btnRepeat, MSGUIRef.GetLocalizedString("btnRepeat"));
            toolTipButtons.SetToolTip(btnSkip, MSGUIRef.GetLocalizedString("btnSkip"));
            toolTipButtons.SetToolTip(btnBreak, MSGUIRef.GetLocalizedString("btnBreak"));
            toolTipButtons.SetToolTip(btnBack, MSGUIRef.GetLocalizedString("btnBack"));
            grpPersonID.Text = MSGUIRef.GetLocalizedString("Person_ID");
            grpScannedObject.Text = MSGUIRef.GetLocalizedString("Scanned_Object");
            grpScanningInfo.Text = MSGUIRef.GetLocalizedString("Scanning_Info");
            lblFrameRate.Text = MSGUIRef.GetLocalizedString("Frame_Rate");
            lblContrast.Text = MSGUIRef.GetLocalizedString("Contrast");
            grpEvaluation.Text = MSGUIRef.GetLocalizedString("Image_Evaluation");
            lblIAFISQuality.Text = MSGUIRef.GetLocalizedString("IAFIS_Quality");
            lblArtefacs.Text = MSGUIRef.GetLocalizedString("Artefacts");
            lblScanningCompleteness.Text = MSGUIRef.GetLocalizedString("Completeness");
            listViewMessages.Columns[0].Text = MSGUIRef.GetLocalizedString("Message");
            grpDiagnostic.Text = MSGUIRef.GetLocalizedString("Diagnostic");
            mnuRemoveSegm.Text = MSGUIRef.GetLocalizedString("Mnu_Remove_Segm");
            mnuViewSegm.Text = MSGUIRef.GetLocalizedString("Mnu_View_Segm");
            mnuClearOutside.Text = MSGUIRef.GetLocalizedString("Mnu_Clear_Outside");
            lblStartMessage.Text = MSGUIRef.GetLocalizedString("lblStartMessage");
            chkAdvancedView.Text = MSGUIRef.GetLocalizedString("chkAdvancedView");
            grpScanningOptions.Text = MSGUIRef.GetLocalizedString("Scanning_Options");
            chkRotate180.Text = MSGUIRef.GetLocalizedString("Rotate_180");
            chkDryFingerImageEnhancement.Text = MSGUIRef.GetLocalizedString("Dry_Finger_Image_Enhancement");
        }

        private void DisplayMessage(String Message, bool IsError)
        {
            lblMessage.Text = Message;

            if (IsError)
            {
                lblMessage.BackColor = Color.Red;
                Application.DoEvents();

                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.Sound))
                {
                    if (SoundPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(12, 1, 4);
                    else
                        //Console.Beep(250, 1000);
                        DSBeep(250, 1000);
                }
            }
            else
                lblMessage.BackColor = SystemColors.Window;
        }

        private void ShowFrameRate(Double CurrentFrameRate, Double NominalFrameRate)
        {
            int FrameRateLimit = 80;

            //lblCurrentNominalFrameRate.Text = CurrentFrameRate.ToString("F") + "/" + NominalFrameRate.ToString("F");
            lblCurrentNominalFrameRate.Text = CurrentFrameRate.ToString("#0.##") + "/" + NominalFrameRate.ToString("#0.##");

            if (CurrentFrameRate < NominalFrameRate*FrameRateLimit/100)
                lblCurrentNominalFrameRate.BackColor = Color.Red;
            else
                lblCurrentNominalFrameRate.BackColor = SystemColors.ButtonFace;
            // PROVA
            Application.DoEvents();
        }

        private void ShowClipRectangle(int ClipX, int ClipY)
        {
            // show clipping rectangle if position is defined and size is not full image
            if ((ClipX >= 0) && (ClipY >= 0)
                && ((ClippingRect.RealRect.Width != ImageSizeX) || (ClippingRect.RealRect.Height != ImageSizeY)))
            {
                ClippingRect.RealRect.X = ClipX;
                ClippingRect.RealRect.Y = ClipY;
                // scale clipping rectangle for view
                RealToViewRect(ClippingRect.RealRect, ref ClippingRect.ViewRect);
                ClippingRect.Visible = true;
                //pboxImage.Refresh();  // NO. This slows down a lot!

                // DEBUG
                //if ((ClippingRect.RealRect.Width == 0) || (ClippingRect.RealRect.Height == 0))
                //    GBMSGUI.LogToFile("ShowClipRectangle: ClippingRect width = " + ClippingRect.RealRect.Width.ToString() +
                //        " ClippingRect height = " + ClippingRect.RealRect.Height.ToString());
            }
        }

        private void lstDiag_Click(object sender, EventArgs e)
        {
            if (lstDiag.SelectedIndex != -1)
                btnHelpDiag.Enabled = true;
        }

        private void lstDiag_DoubleClick(object sender, EventArgs e)
        {
            DiagnosticItem DiagItem;

            DiagItem = (DiagnosticItem)lstDiag.Items[lstDiag.SelectedIndex];

            MessageBox.Show(DiagItem.LongMessage, MSGUIRef.GetLocalizedString("Dlg_Caption_Information"), 
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void btnHelpDiag_Click(object sender, EventArgs e)
        {
            lstDiag_DoubleClick(sender, e);
        }

        private void GetRolledInfo()
        {
            int ret;
            byte[] NotWipedArtefactFrame;
            uint CompositeFingerprintSize;
            byte CompositeFingerprintContrast;
            uint FlatFingerprintSize;

            ret = GBMSAPI_NET_EndAcquisitionRoutines.GBMSAPI_NET_ROLL_GetCompositeImageInfo(
                out ScannedObject.RolledArtefactSize, out MarkerFrame, out NotWipedArtefactFrame,
                out CompositeFingerprintSize, out CompositeFingerprintContrast, out FlatFingerprintSize);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_ROLL_GetCompositeImageInfo", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return;
            }

            // show CompositeFingerprintContrast
            Contrast = CompositeFingerprintContrast;
            thresholdProgressBarContrast.SetValue(Contrast);

            // draw Marker frame
            pboxImage.Refresh();
        }

        private int GetNFIQQuality(Byte[] Frame, int SizeX, int SizeY, out List<NW_LFS_MINUTIA> Minutiae)
        {
            try
            {
                NW_GBNFIQ_ImageQuality_Info_InputData Input = new NW_GBNFIQ_ImageQuality_Info_InputData();
                NW_GBNFIQ_ImageQuality_Info Info;
                NW_GBNFIQ_ImageQuality_Info_OutputData Output;

                Minutiae = null;

                if (MSGUIRef.Resolution1000Dpi)
                {
                    // need to convert image to 500 dpi for passing it to GBNFIQ
                    int ret = GBMSAPI_NET_ImageProcessingUtilities.GBMSAPI_NET_ConvertImageFrom1000to500DPI(Frame, 
                        out Input.Frame, (uint)SizeX, (uint)SizeY);

                    if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_ConvertImageFrom1000to500DPI", ret), ret);
                        ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                        return 255;
                    }
                    Input.Width = SizeX / 2;
                    Input.Height = SizeY / 2;
                }
                else
                {
                    Input.Frame = Frame;
                    Input.Width = SizeX;
                    Input.Height = SizeY;
                }

                Info = new NW_GBNFIQ_ImageQuality_Info(Input);
                Output = Info.NW_GBNFIQ_GetImageQuality();

                Minutiae = Output.MinInfo;
                return Output.NFIQ_Quality;
            }
            catch (Exception ex)
            {
                Minutiae = null;
                return 255;    // attempt failed
            }
        }


        public int GetSegmentationResult(int Index, int QualityAlgorithmID,
            out bool SegmentAvailable, out int Left, out int Top, out int Right, out int Bottom,
            out int UnavailabilityReason, out int FingerprintQuality, 
            Byte[] ImagePtr, out int ImageSizeX, out int ImageSizeY)
        {
            int SegmentIndex;

            SegmentAvailable = false;
            Left = -1;
            Top = -1;
            Right = -1;
            Bottom = -1;
            FingerprintQuality = -1;
            UnavailabilityReason = -1;
            ImageSizeX = -1;
            ImageSizeY = -1;

            //GBMSGUI.LogToFile("Entrata funzione GetSegmentationResult");

            try
            {
                // in case of left slap, reverse index
                if ((ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                    || (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT))
                    SegmentIndex = (SegmData.Count - 1) - Index;
                else
                    SegmentIndex = Index;

                //GBMSGUI.LogToFile("SegmentIndex = " + SegmentIndex.ToString());

                ImageSizeX = -1;
                ImageSizeY = -1;

                // DEBUG - check index
                if ((SegmentIndex < 0) || (SegmentIndex >= SegmData.Count))
                {
                    MessageBox.Show("SegmentIndex out of bounds", "DEBUG",
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }

                if (SegmData[SegmentIndex].Found)
                {
                    //GBMSGUI.LogToFile("Set bounding box");

                    SegmentAvailable = true;
                    Left = SegmData[SegmentIndex].BoundingBox.RealRect.Left;
                    Top = SegmData[SegmentIndex].BoundingBox.RealRect.Top;
                    Right = SegmData[SegmentIndex].BoundingBox.RealRect.Right;
                    Bottom = SegmData[SegmentIndex].BoundingBox.RealRect.Bottom;

                    //GBMSGUI.LogToFile("Set quality");

                    if (QualityAlgorithmID == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                        FingerprintQuality = SegmData[SegmentIndex].NFIQQuality;
                    else
                        FingerprintQuality = SegmData[SegmentIndex].GBQuality;
                    UnavailabilityReason = -1;

                    if (ImagePtr != null)
                    {
                        //GBMSGUI.LogToFile("Set image");
                        // DEBUG - check bmpImage
                        //if (SegmData[SegmentIndex].bmpImage == null)
                        //{
                        //    GBMSGUI.LogToFile("bmpImage at SegmentIndex=" + SegmentIndex.ToString() + " is null");
                        //    return GBMSGUI.ReturnCodes.Ret_Failure;
                        //}

                        //GBMSGUI.LogToFile("Set image size");
                        // copy image
                        ImageSizeX = SegmData[SegmentIndex].bmpImage.Width;
                        ImageSizeY = SegmData[SegmentIndex].bmpImage.Height;

                        //GBMSGUI.LogToFile("Copy image");
                        GBMSGUI.BitmapToRawImage(SegmData[SegmentIndex].bmpImage, ImagePtr);
                    }
                    else
                    {
                        ImageSizeX = -1;
                        ImageSizeY = -1;
                    }
                }
                else
                {
                    SegmentAvailable = false;
                    Left = -1;
                    Top = -1;
                    Right = -1;
                    Bottom = -1;
                    FingerprintQuality = -1;
                    UnavailabilityReason = SegmData[SegmentIndex].UnavailabilityReason;
                    ImageSizeX = -1;
                    ImageSizeY = -1;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in GetSegmentationResult", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            //GBMSGUI.LogToFile("Uscita funzione GetSegmentationResult");

            return GBMSGUI.ReturnCodes.Ret_Success;
        }

        private void StartListenPedal(int Condition)
        {
            PedalInfo.PressedTime = 0;
            PedalInfo.ReleasedTime = 0;

            // call once to update the state
            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_CheckPedalState(out PedalInfo.PedalPressed);

            // get start time
            PedalInfo.tStart = Environment.TickCount;
            // set condition to check
            PedalInfo.Condition = Condition;
            
            // enable timer
            ListenerTimer.Enabled = true;
        }

        // timer for Pedal and LCD touchscreen buttons
        private void ListenerTimer_Tick(object sender, EventArgs e)
        {
            GBMSGUI.LogToFile("ListenerTimer fired!");

            bool Continue = true;

            ListenerTimer.Enabled = false;

            if (PedalPresent)
                Continue = CheckPedal();

            if (Continue && LCDPresent)
                Continue = CheckLCDButtons();

            if (Continue)
                ListenerTimer.Enabled = true;
        }

        private bool CheckPedal()
        {
            int tCheck;
            bool prevPedalState;
            int ret;

            GBMSGUI.LogToFile("Before call GBMSAPI_NET_CheckPedalState");

            // save last state
            prevPedalState = PedalInfo.PedalPressed;

            // get current state
            ret = GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_CheckPedalState(out PedalInfo.PedalPressed);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_CheckPedalState", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return false;
            }

            if ((PedalInfo.Condition == PedatTestConditions.ReleasePedal) && !PedalInfo.PedalPressed)
            {
                // inform about pedal released
                OnPedalReleased();
                return false;
            }

            // if state is not changed
            if (prevPedalState == PedalInfo.PedalPressed)
            {
                // increase the time
                tCheck = Environment.TickCount;

                if (PedalInfo.PedalPressed)
                    PedalInfo.PressedTime = (tCheck - PedalInfo.tStart);
                else
                    PedalInfo.ReleasedTime = (tCheck - PedalInfo.tStart);

                if (PedalInfo.Condition == PedatTestConditions.ReleasePedal)
                {
                    // if pressed for more than 2 seconds
                    if (PedalInfo.PressedTime >= 2000)
                    {
                        // Send Release pedal message
                        OnReleasePedal();
                    }
                }

                if ((PedalInfo.Condition == PedatTestConditions.PedalPressing) && PedalInfo.PedalPressed)
                {
                    if (PedalInfo.PressedTime >= 2000)
                    {
                        // inform about long pressing
                        if (OnLongPedalPressing() == true)
                            // exit from timer only if handled
                            return false;
                    }
                }
            }
            else
            {
                // pedal state has changed
                // if released, check for how much time has been pressed
                if ((PedalInfo.Condition == PedatTestConditions.PedalPressing) && !PedalInfo.PedalPressed)
                {
                    if (PedalInfo.PressedTime >= 100)
                    {
                        // inform about short pressing
                        if (OnShortPedalPressing() == true)
                            // exit from timer only if handled
                            return false;
                    }
                }

                // start counting
                PedalInfo.tStart = Environment.TickCount;
                PedalInfo.PressedTime = 0;
                PedalInfo.ReleasedTime = 0;
            }

            // continue checking
            return true;
        }

        private bool CheckLCDButtons()
        {
            Byte PressedButton;
            int ret;

            GBMSGUI.LogToFile("Before call GBMSAPI_NET_VUI_LCD_GetPressedButton");

            // get LCD touchscreen pressed button
            ret = GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_GetPressedButton(out PressedButton);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_VUI_LCD_GetPressedButton", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return false;
            }

            if (PressedButton == GBMSAPI_NET_PressedButtonIDs.GBMSAPI_NET_VILCD_TOUCHSCREEN_OK_BUTTON)
            {
                btnAccept_Click(this, null);
                return false;
            }
            else if (PressedButton == GBMSAPI_NET_PressedButtonIDs.GBMSAPI_NET_VILCD_TOUCHSCREEN_REPEAT_BUTTON)
            {
                btnRepeat_Click(this, null);
                return false;
            }

            // continue checking
            return true;
        }

        private void OnPedalReleased()
        {
            DisplayMessage("", false);
        }

        private bool OnShortPedalPressing()
        {
            if (btnAccept.Enabled)
            {
                btnAccept_Click(this, null);
                return true;
            }

            return false;
        }

        private bool OnLongPedalPressing()
        {
            btnRepeat_Click(this, null);
            return true;
        }

        private void OnReleasePedal()
        {
            DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Release_Pedal"), true);
            if (LCDPresent)
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                    GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_RELEASE_PEDAL,
                    GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
        }

        private bool ProcessImage()
        {
            try
            {
                NW_GBFINIMG_OutputData SegmentationOutput;
                uint ImageType = 0;

                PostProcessDiag = 0;

                // prepare input data
                if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID))
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_WRITER_PALM;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_WRITER_PALM;

                uint Options = 0;
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.RemoveHaloLatent))
                {
                    if (ToDoFinalization)
                    {
                        // 1.9.14.0 - lighter processing if after ImageFinalization will be called
                        Options = NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_HALO_LATENT_ELIMINATION;
                        if (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID) && !GBMSGUI.IsUpperPalm(ScannedObject.ScannedObjectID))
                            Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS;
                    }
                    else
                        // 1.9.12.0 - new option substitute halo latent
                        Options = NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND;
                }
                if (MSGUIRef.Resolution1000Dpi)
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation))
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION;

                NW_GBFINIMG_InputData SegmInput = new NW_GBFINIMG_InputData(FramePtr,
                    ImageSizeX, ImageSizeY,
                    ImageType,
                    null,
                    Options,
                    ImageSizeX, ImageSizeY);

                NW_GBFINIMG_ProcessImage ImgProcessor =
                    new NW_GBFINIMG_ProcessImage(SegmInput);

                Cursor = Cursors.WaitCursor;
                SegmentationOutput = ImgProcessor.OutputAfterProcess;
                Cursor = Cursors.Default;

                // check if any segment is found
                if ((SegmentationOutput.Segment_List == null) || (SegmentationOutput.Segment_List.Count == 0))
                {
                    Cursor = Cursors.Default;
                    MessageBox.Show(MSGUIRef.GetLocalizedString("Msg_Image_Processing_Failed"), 
                        MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                FramePtr = SegmentationOutput.OutputFrame;
                // update bitmap
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
                pboxImage.Refresh();

                // check diagnostic
                if (!SegmentationOutput.Diag.NW_GBFINIMG_NET_WRAPPER_NO_DIAG)
                {
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
                    {
                        PostProcessDiag |= PostProcessDiagnostic.Diag_LeftRightMixup;
                    }
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
                    {
                        // for single fingers, leave ambiguous slap
                        if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID))
                            PostProcessDiag |= PostProcessDiagnostic.Diag_SegmAmbiguousSlap;
                        else
                            PostProcessDiag |= PostProcessDiagnostic.Diag_PossibleLeftRightMixup;
                    }
                }

                // in case of ambiguous slap, show the segment bounding box
                if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID) &&
                    SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
                {
                    SegmData.Clear();
                    SegmentData Segm = new SegmentData();
                    Segm.FingerID = GBMSGUI.ScannedObjectIDToAN2KFingerID(ScannedObject.ScannedObjectID);
                    SegmData.Add(Segm);
                    SegmData[0].Found = true;
                    SegmData[0].FingerCenterX = (int)SegmentationOutput.Segment_List[0].FingerCenterX;
                    SegmData[0].FingerCenterY = (int)SegmentationOutput.Segment_List[0].FingerCenterY;
                    SegmData[0].BoundingBox.RealRect.X = SegmentationOutput.Segment_List[0].SegBoundingBoxL;
                    SegmData[0].BoundingBox.RealRect.Y = SegmentationOutput.Segment_List[0].SegBoundingBoxT;
                    SegmData[0].BoundingBox.RealRect.Width = SegmentationOutput.Segment_List[0].SegBoundingBoxR - SegmentationOutput.Segment_List[0].SegBoundingBoxL;
                    SegmData[0].BoundingBox.RealRect.Height = SegmentationOutput.Segment_List[0].SegBoundingBoxB - SegmentationOutput.Segment_List[0].SegBoundingBoxT;
                    RealToViewRect(SegmData[0].BoundingBox.RealRect, ref SegmData[0].BoundingBox.ViewRect);
                    SegmData[0].BoundingBox.Visible = true;
                    pboxImage.Refresh();

                    // only for rolled, if option active, clear outside automatically
                    if (GBMSGUI.IsRolled(ScannedObject.ScannedObjectID) &&
                        GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.AutoClearOutsideRoll))
                    {
                        ClearOutsideBoundingBox();
                        // remove diagnostic
                        PostProcessDiag &= ~PostProcessDiagnostic.Diag_SegmAmbiguousSlap;
                    }
                    else
                    {
                        // show message
                        ListViewItem Item = new ListViewItem(MSGUIRef.GetLocalizedString("Msg_Bounding_Box_Menu"), "Info.bmp");
                        if (FindlistViewMessage(Item.Text) == -1)
                            listViewMessages.Items.Add(Item);
                    }
                }

                // read results

                // GB quality
                ScannedObject.GBQuality = SegmentationOutput.Segment_List[0].SegmQuality.quality;
                // for palm objects, is valid only if option is active
                if (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID))
                {
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation))
                        // convert also to NFIQ quality
                        ScannedObject.NFIQQuality = GBQualityToNFIQ(ScannedObject.GBQuality);
                    else
                        ScannedObject.GBQuality = -1;
                }

                // check diagnostic (pattern, inclination)
                if (GBMSGUI.IsLowerPalm(ScannedObject.ScannedObjectID) && PalmCompletenessSupported)
                {
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectIncompletePattern) &&
                        PalmCompleteness < MSGUIRef.CfgData.LowerPalmCompletenessThreshold2)
                    {
                        IncompletePatternDetected = true;
                        PostProcessDiag |= PostProcessDiagnostic.Diag_IncompletePattern;
                    }
                }
                else 
                {
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectIncompletePattern) &&
                        SegmentationOutput.Segment_List[0].PatternCompletness < MSGUIRef.CfgData.PatternCompletenessThreshold)
                    {
                        IncompletePatternDetected = true;
                        PostProcessDiag |= PostProcessDiagnostic.Diag_IncompletePattern;
                    }
                }
                // for Palm, PatternValidity is present only if NW_GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION
                // option is activated;
                // for writer palm, no PatternValidity evaluation if left-right mix-up diagnostic is present
                if (!(
                    (GBMSGUI.IsPalm(ScannedObject.ScannedObjectID) && !GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.PalmPrintQualityCalculation))
                    || (GBMSGUI.IsWritersPalm(ScannedObject.ScannedObjectID) && (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX
                                || SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP))))
                {
                    if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInvalidPattern) &&
                        SegmentationOutput.Segment_List[0].PatternValidity < MSGUIRef.CfgData.PatternValidityThreshold)
                    {
                        InvalidPatternDetected = true;
                        PostProcessDiag |= PostProcessDiagnostic.Diag_InvalidPattern;
                    }
                }
                if (GBMSGUI.CheckMask(MSGUIRef.AcquisitionOptions, GBMSGUI.AcquisitionOption.DetectInclination) &&
                    Math.Abs(SegmentationOutput.Segment_List[0].Inclination) > MSGUIRef.CfgData.InclinationThreshold)
                {
                    InclinationDetected = true;
                    PostProcessDiag |= PostProcessDiagnostic.Diag_Inclination;
                }

                return true;
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Default;
                MessageBox.Show(ex.Message + " in ProcessImage", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private bool CheckSequence(List<NW_LFS_MINUTIA> Minutiae)
        {
            try
            {
                // convert ScannedObjectID to AN2K finger position
                String FingerID = GBMSGUI.ScannedObjectIDToAN2KFingerID(ScannedObject.ScannedObjectID);
                List<NW_LFS_MINUTIA> RefMinutiae = null;

                // search it in sequence check list
                foreach(GBMSGUI.SequenceCheckItem Item in MSGUIRef.SequenceCheckData)
                {
                    if (Item.FingerID == FingerID)
                    {
                        RefMinutiae = Item.Minutiae;
                        break;
                    }
                }

                if (RefMinutiae == null)
                {
                    // item not found, wrong sequence
                    listViewMessages.Items.Add(MSGUIRef.GetLocalizedString("Msg_Sequence_Check_Order"), "Warning.bmp");
                    if (LCDPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SEQUENCE_CHECK_FAILED,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                    // can't compare, return true
                    return true;
                }

                // compare minutiae
                NW_BOZORTH_MatchScoreCalculator Bozorth = new NW_BOZORTH_MatchScoreCalculator(Minutiae, RefMinutiae);
                Int32 Score;
                Score = Bozorth.NW_boz_direct_call(200);

                // minimum match score set to 24
                if (Score < 24)
                {
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Sequence_Check_Score"), true);
                    if (LCDPresent)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                            GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SEQUENCE_CHECK_FAILED,
                            GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                    return false;
                }

                return true;
            }
            catch (Exception ex)
            {
                //throw ex;
                MessageBox.Show(ex.Message + " in CheckSequence", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private bool AreSegmentsAllFound()
        {
            // check if segments are found or unavailability specified
            for (int i = 0; i < SegmData.Count; i++)
            {
                if (!SegmData[i].Found && 
                    (GBMSGUI.CheckMask(MSGUIRef.SessionOptions, GBMSGUI.SessionOption.AskUnavailabilityReason)
                        && (SegmData[i].UnavailabilityReason == -1)))
                    return false;
            }

            return true;
        }

        private bool AskUnavailabilityReason()
        {
            UnavailabilityReasonForm AskForm = new UnavailabilityReasonForm(this);
            int i;

            // add items to the grid
            if (!GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                GBMSGUI.UnavailabilityReasonListItem Item = new GBMSGUI.UnavailabilityReasonListItem(GetScannedObjectName(), 0);
                AskForm.UnavailabilityReasonList.Add(Item);
            }
            else
            {
                // add missing segments
                for (i = 0; i < SegmData.Count; i++)
                {
                    if (!SegmData[i].Found)
                    {
                        GBMSGUI.UnavailabilityReasonListItem Item = new GBMSGUI.UnavailabilityReasonListItem(SegmData[i].FingerText, 0);
                        AskForm.UnavailabilityReasonList.Add(Item);
                    }
                }
            }

            // show dialog
            if (AskForm.ShowDialog() == DialogResult.Cancel)
                return false;

            // read result
            if (!GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                ScannedObject.UnavailabilityReason = AskForm.UnavailabilityReasonList[0].Reason;
            }
            else
            {
                int j = 0;
                for (i = 0; i < SegmData.Count; i++)
                {
                    if (!SegmData[i].Found)
                    {
                        SegmData[i].UnavailabilityReason = AskForm.UnavailabilityReasonList[j].Reason;
                        j++;
                    }
                }
            }

            return true;
        }

        private void chkAdvancedView_CheckedChanged(object sender, EventArgs e)
        {
            // show/hide frame rate
            if (chkAdvancedView.Checked)
                grpScanningInfo.Visible = true;
            else
                grpScanningInfo.Visible = false;
        }

        private void mnuClearOutside_Click(object sender, EventArgs e)
        {
            ClearOutsideBoundingBox();
        }

        private void ClearOutsideBoundingBox()
        {
            FillImage(FramePtr, (int)ImageSizeX, (int)ImageSizeY,
                SegmData[0].BoundingBox.RealRect.Left, SegmData[0].BoundingBox.RealRect.Top,
                SegmData[0].BoundingBox.RealRect.Right, SegmData[0].BoundingBox.RealRect.Bottom,
                255);
            GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
            pboxImage.Refresh();

            ProcessImageResult = true;
            CheckResult();
        }

        private void popEditSegmMenu_Popup(object sender, EventArgs e)
        {
            if (GBMSGUI.IsSlap(ScannedObject.ScannedObjectID))
            {
                mnuViewSegm.Visible = true;
                mnuRemoveSegm.Visible = true;
                mnuClearOutside.Visible = false;
            }
            else if (GBMSGUI.IsSingleFinger(ScannedObject.ScannedObjectID))
            {
                mnuViewSegm.Visible = false;
                mnuRemoveSegm.Visible = false;
                mnuClearOutside.Visible = true;
            }
        }

        private bool LiveSegmentation()
        {
            bool QualityCheck = true;

            try
            {
                bool Failed = false;
                int i;
                int SegmSizeX, SegmSizeY;
                bool SingleObjectFlag = false;  // for slap 2 in case of missing finger

                PostProcessDiag = 0;

                // get maximum size requested for segments
                GetMaxSegmentsSize(out SegmSizeX, out SegmSizeY);

                NW_GBFINIMG_OutputData SegmentationOutput;

                // prepare input data
                uint ImageType = 0;
                if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
                else if ((ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                    //V1.2 - patch because 2 indexes for GBFINIMG does not exists
                    || (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES))
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;
                //V1.1
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2;
                else if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT)
                    ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2;

                uint Options = 0;
                if (MSGUIRef.Resolution1000Dpi)
                    Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;

                Options |= NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA;

                // missing fingers
                if ((ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4) ||
                    (ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4))
                {
                    if (CheckUnavailableFingers())
                    {
                        byte MissingFingerMask = 0;

                        if (ImageType == NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4)
                        {
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
                            if (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
                            if (MSGUIRef.SegmCfgData[2].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_RING;
                            if (MSGUIRef.SegmCfgData[3].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
                        }
                        else	// NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4
                        {
                            if (MSGUIRef.SegmCfgData[3].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
                            if (MSGUIRef.SegmCfgData[2].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
                            if (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_RING;
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1)
                                MissingFingerMask |= NW_GBFINIMG_MISSING_FINGER_SLAP4.NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
                        }

                        //GBFINIMG_Utilities.GBFINIMGdll_NetWrapper.GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
                        Segmentator.NW_GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
                    }
                }
                else    // Slap 2
                {
                    if (CheckUnavailableFingers())
                    {
                        // if both are set as unavailable, fail
                        if ((MSGUIRef.SegmCfgData[0].UnavailabilityReason != -1) &&
                            (MSGUIRef.SegmCfgData[1].UnavailabilityReason != -1))
                        {
                            ResetSegments();
                            pboxImage.Refresh();
                            return false;
                        }
                        ImageType = NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
                        SingleObjectFlag = true;
                    }
                }

                NW_GBFINIMG_InputData SegmInput = new NW_GBFINIMG_InputData(FramePtr,
                    ImageSizeX, ImageSizeY,
                    ImageType,
                    null,
                    Options,
                    (uint)SegmSizeX,
                    (uint)SegmSizeY);


                // perform segmentation
                Segmentator.InputToSet = SegmInput;
                SegmentationOutput = Segmentator.OutputAfterProcess;

                /*
                // update bitmap
                FramePtr = SegmentationOutput.OutputFrame;
                GBMSGUI.CopyRawImageIntoBitmap(FramePtr, ref bmpImage);
                pboxImage.Refresh();
                */

                // check if any segment is found
                if ((SegmentationOutput.Segment_List == null) || (SegmentationOutput.Segment_List.Count == 0))
                    Failed = true;

                // check diagnostic
                if (!SegmentationOutput.Diag.NW_GBFINIMG_NET_WRAPPER_NO_DIAG)
                {
                    // add diagnostic messages
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
                    {
                        PostProcessDiag |= PostProcessDiagnostic.Diag_SegmAmbiguousSlap;
                    }
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP)
                    {
                        // different message depending if segments are found or not
                        if (Failed)
                            PostProcessDiag |= PostProcessDiagnostic.Diag_SegmIncompleteSlap;
                        else
                            PostProcessDiag |= PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap;
                    }
                    if (SegmentationOutput.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
                    {
                        // different message depending if segments are found or not
                        if (Failed)
                            PostProcessDiag |= PostProcessDiagnostic.Diag_LeftRightMixup;
                        else
                            PostProcessDiag |= PostProcessDiagnostic.Diag_PossibleLeftRightMixup;
                    }
                }

                // check if segments found are identified
                // don't check in case of SingleObjectFlag
                if (!Failed && !SingleObjectFlag)
                {
                    for (i = 0; i < SegmentationOutput.Segment_List.Count; i++)
                    {
                        if ((SegmentationOutput.Segment_List[i].FingerID ==
                                NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN))
                        {
                            // failed to identify segments, needed manual identification
                            Failed = true;
                        }
                    }
                }

                if (!Failed)
                {
                    // read results
                    // note that elements in list can be in different order; check FingerID
                    for (i = 0; i < SegmentationOutput.Segment_List.Count; i++)
                    {
                        int FngIndex;
                        if (SingleObjectFlag)
                        {
                            if (MSGUIRef.SegmCfgData[0].UnavailabilityReason == -1)
                                FngIndex = 0;
                            else if (MSGUIRef.SegmCfgData[1].UnavailabilityReason == -1)
                                FngIndex = 1;
                            else
                                continue;
                        }
                        else
                        {
                            FngIndex = IndexOfFingerSegm(SegmentationOutput.Segment_List[i].FingerID);
                            if (FngIndex == -1)
                                continue;
                        }
                        SegmData[FngIndex].Found = true;
                        SegmData[FngIndex].FingerCenterX = (int)SegmentationOutput.Segment_List[i].FingerCenterX;
                        SegmData[FngIndex].FingerCenterY = (int)SegmentationOutput.Segment_List[i].FingerCenterY;
                        SegmData[FngIndex].BoundingBox.RealRect.X = SegmentationOutput.Segment_List[i].SegBoundingBoxL;
                        SegmData[FngIndex].BoundingBox.RealRect.Y = SegmentationOutput.Segment_List[i].SegBoundingBoxT;
                        SegmData[FngIndex].BoundingBox.RealRect.Width = SegmentationOutput.Segment_List[i].SegBoundingBoxR - SegmentationOutput.Segment_List[i].SegBoundingBoxL;
                        SegmData[FngIndex].BoundingBox.RealRect.Height = SegmentationOutput.Segment_List[i].SegBoundingBoxB - SegmentationOutput.Segment_List[i].SegBoundingBoxT;
                        RealToViewRect(SegmData[FngIndex].BoundingBox.RealRect, ref SegmData[FngIndex].BoundingBox.ViewRect);
                        SegmData[FngIndex].BoundingBox.Visible = true;
                        // quality
                        SegmData[FngIndex].GBQuality = SegmentationOutput.Segment_List[i].SegmQuality.quality;
                        // can't be calculated from image, convert it...
                        SegmData[FngIndex].NFIQQuality = GBQualityToNFIQ(SegmData[FngIndex].GBQuality);

                        if (!EvalSegmentIAFISQuality(FngIndex))
                            QualityCheck = false;
                    }
                }

                if (Failed)
                {
                    ResetSegments();
                    pboxImage.Refresh();
                    return false;
                }

                if (!QualityCheck)
                    return false;

                pboxImage.Refresh();
                return true;
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Default;
                MessageBox.Show(ex.Message + " in DoSegmentation", MSGUIRef.GetLocalizedString("Dlg_Caption_Error"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private void OnIncompleteSlap_Click(object sender, System.EventArgs e)
        {
            SegmentationResult = DoSegmentation(false, true);
            CheckResult();
        }

        private void chkDryFingerImageEnhancement_CheckedChanged(object sender, EventArgs e)
        {
            SetDryFingerImageEnhancementOption(chkDryFingerImageEnhancement.Checked);
        }

        private bool SetDryFingerImageEnhancementOption(bool Option)
        {
            int ret = GBMSAPI_NET_ScanSettingsRoutines.GBMSAPI_NET_EnableDrySkinImgEnhance(Option);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                MSGUIRef.SetError(MSGUIRef.GetMSAPIErrorMessage("GBMSAPI_NET_EnableDrySkinImgEnhance", ret), ret);
                ScanResult = GBMSGUI.ReturnCodes.Ret_Failure;
                return false;
            }

            return true;
        }
    }
}