using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Reflection;
using System.IO;
using System.Resources;
using System.Globalization;
using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLEDsDefines;

namespace GBMSGUI_NET
{
    // utility functions of the Acquisition form
    internal partial class AcquisitionForm : Form
    {
        // Item used for the diagnostic list
        private class DiagnosticItem : Object
        {
            public uint Value;          // diagostic value
            public String ShortMessage; // text shown in the list
            public String LongMessage;  // text shown in the messagebox

            public DiagnosticItem(uint Value, String ShortMessage, String LongMessage)
            {
                this.Value = Value;
                this.ShortMessage = ShortMessage;
                this.LongMessage = LongMessage;
            }

            public override String ToString()
            {
                return ShortMessage;
            }

            public override bool Equals(Object obj)
            {
                // If parameter is null return false.
                if (obj == null)
                    return false;

                // If parameter cannot be cast to DiagnosticItem return false.
                DiagnosticItem d = obj as DiagnosticItem;
                if ((System.Object)d == null)
                {
                    return false;
                }

                if (d.Value == this.Value)
                    return true;

                return false;
            }

            public override int GetHashCode()
            {
                return (int)Value;
            }

        }

        // translate a rectangle from view size to real size
        // ViewScaleFactor is calculates in AdjustProportion
        private void ViewToRealRect(Rectangle OrigRect, ref Rectangle TransRect)
        {
            double fLeft, fTop, fWidth, fHeight;

            fLeft = OrigRect.Left * ViewScaleFactor;
            fTop = OrigRect.Top * ViewScaleFactor;
            fWidth = OrigRect.Width * ViewScaleFactor;
            fHeight = OrigRect.Height * ViewScaleFactor;

            TransRect.X = (int)Math.Floor(fLeft + 0.5);
            TransRect.Y = (int)Math.Floor(fTop + 0.5);
            TransRect.Width = (int)Math.Floor(fWidth + 0.5);
            TransRect.Height = (int)Math.Floor(fHeight + 0.5);
        }

        // translate a rectangle from real size to view size
        // ViewScaleFactor is calculates in AdjustProportion
        private void RealToViewRect(Rectangle OrigRect, ref Rectangle TransRect)
        {
            double fLeft, fTop, fWidth, fHeight;

            fLeft = OrigRect.Left / ViewScaleFactor;
            fTop = OrigRect.Top / ViewScaleFactor;
            fWidth = OrigRect.Width / ViewScaleFactor;
            fHeight = OrigRect.Height / ViewScaleFactor;

            TransRect.X = (int)Math.Floor(fLeft + 0.5);
            TransRect.Y = (int)Math.Floor(fTop + 0.5);
            TransRect.Width = (int)Math.Floor(fWidth + 0.5);
            TransRect.Height = (int)Math.Floor(fHeight + 0.5);
        }

        private void DrawShadowedText(Graphics g, String Text, Rectangle Box)
        {
            try
            {
                // Create font and brush.
                Font drawFont = new Font("Arial", 12, FontStyle.Bold);
                SolidBrush drawBrush = new SolidBrush(Color.DeepSkyBlue);
                SolidBrush shadowBrush = new SolidBrush(Color.Black);
                Point ShadowLocation = new Point();
                Point Location = Box.Location;
                Location.Y -= 20;
                if (Location.Y < 0)
                    Location.Y = Box.Top;

                ShadowLocation.X = Location.X + 1;
                ShadowLocation.Y = Location.Y + 1;

                // draw text shadow
                g.DrawString(Text, drawFont, shadowBrush, ShadowLocation);
                // draw text
                g.DrawString(Text, drawFont, drawBrush, Location);

                drawFont.Dispose();
                drawBrush.Dispose();
                shadowBrush.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in DrawShadowedText");
                return;
            }
        }

        // check if mouse position inside rectangle and change cursor and size operation
        private bool CheckMousePosition(Control Ctrl, int X, int Y, Rectangle Rect, SizeOptions Options)
        {
            // if outside, return immediately
            if ((X < Rect.Left) || (X > Rect.Right)
                || (Y < Rect.Top) || (Y > Rect.Bottom))
            {
                Ctrl.Cursor = Cursors.Default;
                mSizeOperation = SizeOperations.None;
                return false;
            }

            if ((X > Rect.Left + 1) && (X < Rect.Right - 1)
                && (Y > Rect.Top + 1) && (Y < Rect.Bottom - 1)
                && GBMSGUI.CheckMask((uint)Options, (uint)SizeOptions.AllowMove))
            {
                Ctrl.Cursor = Cursors.SizeAll;
                mSizeOperation = SizeOperations.Move;
            }
            else if ((X >= Rect.Left - 1) && (X <= Rect.Left + 1)
                && GBMSGUI.CheckMask((uint)Options, (uint)SizeOptions.AllowResize))
            {
                Ctrl.Cursor = Cursors.SizeWE;
                mSizeOperation = SizeOperations.Left;
            }
            else if ((X >= Rect.Right - 1) && (X <= Rect.Right + 1)
                && GBMSGUI.CheckMask((uint)Options, (uint)SizeOptions.AllowResize))
            {
                Ctrl.Cursor = Cursors.SizeWE;
                mSizeOperation = SizeOperations.Right;
            }
            else if ((Y >= Rect.Top - 1) && (Y <= Rect.Top + 1)
                && GBMSGUI.CheckMask((uint)Options, (uint)SizeOptions.AllowResize))
            {
                Ctrl.Cursor = Cursors.SizeNS;
                mSizeOperation = SizeOperations.Top;
            }
            else if ((Y >= Rect.Bottom - 1) && (Y <= Rect.Bottom + 1)
                && GBMSGUI.CheckMask((uint)Options, (uint)SizeOptions.AllowResize))
            {
                Ctrl.Cursor = Cursors.SizeNS;
                mSizeOperation = SizeOperations.Bottom;
            }
            else
            {
                Ctrl.Cursor = Cursors.Default;
                mSizeOperation = SizeOperations.None;
            }

            return true;
        }

        // performs resize or move operation on affected rectangle
        private void DoSizeOperation(Control Ctrl, int X, int Y, ref Rectangle Rect)
        {
            Point EndPoint = Ctrl.PointToScreen(new Point(X, Y));
            int XShift = EndPoint.X - StartPoint.X;
            int YShift = EndPoint.Y - StartPoint.Y;
            StartPoint = EndPoint;

            switch (mSizeOperation)
            {
                case SizeOperations.Move:
                    Rect.X += XShift;
                    Rect.Y += YShift;
                    // do not allow to exit from the control
                    if (Rect.X < 0) Rect.X = 0;
                    if (Rect.Y < 0) Rect.Y = 0;
                    if (Rect.Right > Ctrl.Right) Rect.X = Ctrl.Right - Rect.Width;
                    if (Rect.Bottom > Ctrl.Bottom) Rect.Y = Ctrl.Bottom - Rect.Height;
                    break;

                case SizeOperations.Left:
                    Rect.X += XShift;
                    Rect.Width += -XShift;
                    break;

                case SizeOperations.Right:
                    Rect.Width += XShift;
                    break;

                case SizeOperations.Top:
                    Rect.Y += YShift;
                    Rect.Height += -YShift;
                    break;

                case SizeOperations.Bottom:
                    Rect.Height += YShift;
                    break;
            }

            Ctrl.Refresh();
        }

        // Evaluate quality of a segment, based on currently selected algorithm
        private bool EvalSegmentIAFISQuality(int Index)
        {
            bool Result = false;
            int Threshold1, Threshold2;

            // read thresholsd for the segment from configuration
            Threshold1 = MSGUIRef.SegmCfgData[Index].QualityThreshold1;
            Threshold2 = MSGUIRef.SegmCfgData[Index].QualityThreshold2;

            if ((Threshold1 == 0) || (Threshold2 == 0))
            {
                // no tresholds, no evaluation
                SegmData[Index].QualityColor = Color.Silver;
                Result = true;
            }
            else
            {
                // NFIQ quality
                if (MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                {
                    if (SegmData[Index].NFIQQuality <= Threshold1)
                    {
                        SegmData[Index].QualityColor = Color.Lime;
                        Result = true;
                    }
                    else if ((SegmData[Index].NFIQQuality > Threshold1) &&
                        (SegmData[Index].NFIQQuality <= Threshold2))
                    {
                        SegmData[Index].QualityColor = Color.Yellow;
                        Result = false;
                    }
                    else
                    {
                        SegmData[Index].QualityColor = Color.Red;
                        Result = false;
                    }
                }
                else
                {
                    // GB Quality
                    if (SegmData[Index].GBQuality >= Threshold2)
                    {
                        SegmData[Index].QualityColor = Color.Lime;
                        Result = true;
                    }
                    else if ((SegmData[Index].GBQuality < Threshold2) &&
                        (SegmData[Index].GBQuality >= Threshold1))
                    {
                        SegmData[Index].QualityColor = Color.Yellow;
                        Result = false;
                    }
                    else
                    {
                        SegmData[Index].QualityColor = Color.Red;
                        Result = false;
                    }
                }
            }

            pboxImage.Refresh();
            return Result;
        }

        // draw value of quality inside segment bounding box, using color determined by EvalSegmentIAFISQuality
        private void DrawSegmentQuality(Graphics g, int Quality, Color QualityColor, Rectangle Box)
        {
            try
            {
                Font drawFont = new Font("Arial", 10, FontStyle.Bold);
                SolidBrush drawBrush = new SolidBrush(Color.Black);
                SolidBrush backBrush;
                Rectangle Rect = new Rectangle();

                int RectWidth;
                int RectHeight = 20;
                if (MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                    RectWidth = 20;
                else
                    RectWidth = 30; // larger for GBQuality (up to 3 digits)

                Rect.X = Box.Right - RectWidth;
                Rect.Y = Box.Bottom - RectHeight;
                Rect.Width = RectWidth;
                Rect.Height = RectHeight;

                backBrush = new SolidBrush(QualityColor);
                g.FillRectangle(backBrush, Rect);
                StringFormat stringFormat = new StringFormat();
                stringFormat.Alignment = StringAlignment.Center;
                stringFormat.LineAlignment = StringAlignment.Center;
                if ((Quality == 254) || (Quality == 255))
                    g.DrawString("-", drawFont, drawBrush, Rect, stringFormat);
                else
                    g.DrawString(Quality.ToString(), drawFont, drawBrush, Rect, stringFormat);

                drawFont.Dispose();
                drawBrush.Dispose();
                backBrush.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in DrawSegmentQuality");
                return;
            }
        }

        // draw diagnostic icon (pattern, inclination) inside segment bounding box
        private void DrawSegmentDiagnostic(Graphics g, Image Icon, Rectangle Box)
        {
            try
            {
                Rectangle tmpRect = new Rectangle();

                tmpRect.X = Box.X;
                tmpRect.Y = Box.Y + Box.Height - Icon.Height;
                tmpRect.Width = Icon.Width;
                tmpRect.Height = Icon.Height;
                g.DrawImage(Icon, tmpRect);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in DrawSegmentDiagnostic");
                return;
            }
        }

        // show pictogram of the currently scanned object
        private void ShowScannedObjectPictogram()
        {
            switch (ScannedObject.ScannedObjectID)
            {
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Thumb.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Index.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Middle.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Ring.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Little.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Thumb.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Index.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Middle.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Ring.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Little.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Thumb.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Index.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Middle.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Ring.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Little.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Thumb.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Index.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Middle.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Ring.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Little.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS:
                    pboxScannedObject.Image = imgLstHands.Images["2_Thumbs.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT:
                    pboxScannedObject.Image = imgLstHands.Images["Left_4_Fingers.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT:
                    pboxScannedObject.Image = imgLstHands.Images["Right_4_Fingers.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Upper_Palm.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Upper_Palm.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Lower_Palm.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Lower_Palm.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Writer_Palm.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Writer_Palm.bmp"];
                    break;
                //V1.1 - DS40
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT:
                    pboxScannedObject.Image = imgLstHands.Images["Left_Slap_2.bmp"];
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT:
                    pboxScannedObject.Image = imgLstHands.Images["Right_Slap_2.bmp"];
                    break;
                //V1.2 - DS40
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES:
                    pboxScannedObject.Image = imgLstHands.Images["2_Indexes.bmp"];
                    break;
            }
        }

        // returns name of the currently scanned object
        private String GetScannedObjectName()
        {
            String ObjectName;

            switch (ScannedObject.ScannedObjectID)
            {
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Thumb");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Index");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Middle");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Ring");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Little");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Thumb");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Index");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Middle");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Ring");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Little");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Right_Thumb");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Right_Index");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Right_Middle");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Right_Ring");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Right_Little");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Left_Thumb");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Left_Index");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Left_Middle");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Left_Ring");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE:
                    ObjectName = MSGUIRef.GetLocalizedString("Rolled_Left_Little");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Two_Thumbs");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Left_Four_Fingers");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Right_Four_Fingers");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT:
                    ObjectName = MSGUIRef.GetLocalizedString("Left_Upper_Half_Palm");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT:
                    ObjectName = MSGUIRef.GetLocalizedString("Right_Upper_Half_Palm");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT:
                    ObjectName = MSGUIRef.GetLocalizedString("Left_Lower_Half_Palm");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT:
                    ObjectName = MSGUIRef.GetLocalizedString("Right_Lower_Half_Palm");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT:
                    ObjectName = MSGUIRef.GetLocalizedString("Left_Writers_Palm");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT:
                    ObjectName = MSGUIRef.GetLocalizedString("Right_Writers_Palm");
                    break;
                //V1.1 - DS40
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT:
                    ObjectName = MSGUIRef.GetLocalizedString("Left_Slap_2");
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT:
                    ObjectName = MSGUIRef.GetLocalizedString("Right_Slap_2");
                    break;
                //V1.2 - DS40
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES:
                    ObjectName = MSGUIRef.GetLocalizedString("Flat_Two_Indexes");
                    break;
                default:
                    ObjectName = "";
                    break;
            }

            return ObjectName;
        }

        // Dislpay scanning diagnostic in the diagnostic list
        private void DisplayDiagnostic()
        {
            DiagnosticItem DiagItem;
            int ItemIndex;

            // save the heaviest for the LCD (can show only one) (only PostProc)
            Byte LCDPPDiag = 0;

            // change method, due to Mono bugs...
            //lstDiag.Items.Clear();
            //lstDiag.BackColor = SystemColors.Window;

            // first, manage list

            // check firts heavy diagnostic conditions, so that they appear first in the list
            // check first if bit has changed

            // scanning diagnostics
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA,
                    MSGUIRef.GetLocalizedString("Diag_Scanner_Surface_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Scanner_Surface_Long"));
                // if present, add to list
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE,
                    MSGUIRef.GetLocalizedString("Diag_Scanner_Failure_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Scanner_Failure_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_COMPOSITION_SLOW))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_COMPOSITION_SLOW,
                    MSGUIRef.GetLocalizedString("Diag_Composition_Slow_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Composition_Slow_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_COMPOSITION_SLOW))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_SLIDING))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_SLIDING,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Sliding_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Finger_Sliding_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_SLIDING))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_EXT_LIGHT_TOO_STRONG))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_EXT_LIGHT_TOO_STRONG,
                    MSGUIRef.GetLocalizedString("Diag_External_Light_Short"),
                    MSGUIRef.GetLocalizedString("Diag_External_Light_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_EXT_LIGHT_TOO_STRONG))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_IMPROPER_ROLL))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_IMPROPER_ROLL,
                    MSGUIRef.GetLocalizedString("Diag_Improper_Roll_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Improper_Roll_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_IMPROPER_ROLL))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_FAST_ROLL))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_FAST_ROLL,
                    MSGUIRef.GetLocalizedString("Diag_Too_Fast_Roll_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Too_Fast_Roll_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_FAST_ROLL))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_NARROW_ROLL))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_NARROW_ROLL,
                    MSGUIRef.GetLocalizedString("Diag_Too_Narrow_Roll_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Too_Narrow_Roll_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_NARROW_ROLL))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }

            // Yellow diagnostics
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Left_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Flat_Finger_Out_Region_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Right_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Flat_Finger_Out_Region_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Top_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Flat_Finger_Out_Region_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Displaced_Down_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Flat_Finger_Out_Region_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_LEFT))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_LEFT,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Left_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Roll_Finger_Out_Region_Left_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_LEFT))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_RIGHT))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_RIGHT,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Right_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Roll_Finger_Out_Region_Right_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_RIGHT))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_TOP))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_TOP,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Out_Region_Top_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Roll_Finger_Out_Region_Top_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_TOP))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_DISPLACED_DOWN))
            {
                DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_DISPLACED_DOWN,
                    MSGUIRef.GetLocalizedString("Diag_Finger_Displaced_Down_Short"),
                    MSGUIRef.GetLocalizedString("Diag_Roll_Finger_Displaced_Down_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_DISPLACED_DOWN))
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                else
                    lstDiag.Items.Remove(DiagItem);
            }
            // 1.9.12.0 - wet/dry finger detection
            // this is not in the list, don't check PrevDiagnostic
            //if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER))
            //{
                // 1.9.13.0 - no more message, only icon
                //DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER,
                //    MSGUIRef.GetLocalizedString("Diag_Dry_Finger_Short"),
                //    MSGUIRef.GetLocalizedString("Diag_Dry_Finger_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER))
                {
                    //ItemIndex = lstDiag.Items.Add(DiagItem);
                    lblDryWet.Text = MSGUIRef.GetLocalizedString("Diag_Dry_Finger_Short");
                    // only if visible
                    if (panDryWet.Visible)
                    {
                        lblDryWet.Visible = true;

                        // 1.9.13.0 - show on LCD on stop screen
                        if (LCDPresent && !Acquiring)
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(
                                GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_DRY_FINGER);
                    }
                }
                else
                {
                    //lstDiag.Items.Remove(DiagItem);
                    //lblDryWet.Visible = false;
                }
            //}
            // this is not in the list, don't check PrevDiagnostic
            //if (GBMSGUI.CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER))
            //{
                // 1.9.13.0 - no more message, only icon
                //DiagItem = new DiagnosticItem(GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER,
                //    MSGUIRef.GetLocalizedString("Diag_Wet_Finger_Short"),
                //    MSGUIRef.GetLocalizedString("Diag_Wet_Finger_Long"));
                if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER))
                {
                    //ItemIndex = lstDiag.Items.Add(DiagItem);
                    lblDryWet.Text = MSGUIRef.GetLocalizedString("Diag_Wet_Finger_Short");
                    // only if visible
                    if (panDryWet.Visible)
                    {
                        lblDryWet.Visible = true;

                        // 1.9.13.0 - show on LCD on stop screen
                        if (LCDPresent && !Acquiring)
                            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(
                                GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_WET_FINGER);
                    }
                }
                else
                {
                    //lstDiag.Items.Remove(DiagItem);
                    //lblDryWet.Visible = false;
                }
            //}

            // lblDryWet is in common, hide only if both are false
            if (!GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER)
                && !GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER))
            {
                if (panDryWet.Visible)
                    lblDryWet.Visible = false;
            }

            // Post-processing diagnostic
            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_InvalidPattern))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_InvalidPattern,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Invalid_Pattern"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Invalid_Pattern"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_InvalidPattern))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INVALID_PATTERN_DETECTED;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_IncompletePattern))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_IncompletePattern,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Pattern"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Pattern"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_IncompletePattern))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INCOMPLETE_PATTERN_DETECTED;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_Inclination))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_Inclination,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Finger_Inclination"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Finger_Inclination"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_Inclination))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_FINGER_INCLINATION_DETECTED;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_SegmAmbiguousSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmAmbiguousSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Ambiguous_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Ambiguous_Slap"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmAmbiguousSlap))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_AMBIGUOUS_SLAP;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_SegmIncompleteSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmIncompleteSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Slap"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmIncompleteSlap))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INCOMPLETE_SLAP;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Incomplete_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Incomplete_Slap"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_POSSIBLE_INCOMPLETE_SLAP;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_LeftRightMixup))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_LeftRightMixup,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Left_Right_Mixup"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Left_Right_Mixup"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_LeftRightMixup))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_LEFT_RIGHT_MIXUP;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            if (GBMSGUI.CheckMask(PostProcessDiag ^ PrevPostProcessDiag, PostProcessDiagnostic.Diag_PossibleLeftRightMixup))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_PossibleLeftRightMixup,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Left_Right_Mixup"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Left_Right_Mixup"));
                // if present, add to list
                if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_PossibleLeftRightMixup))
                {
                    ItemIndex = lstDiag.Items.Add(DiagItem);
                    if (LCDPPDiag == 0)
                        LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_POSSIBLE_LEFT_RIGHT_MIXUP;
                }
                else
                    // else, remove it
                    lstDiag.Items.Remove(DiagItem);
            }

            // then, manage color
            Color DiagColor = SystemColors.Window;

            // Red diagnostics
            if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_COMPOSITION_SLOW) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_SLIDING) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_EXT_LIGHT_TOO_STRONG) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_IMPROPER_ROLL) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_FAST_ROLL) || 
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_TOO_NARROW_ROLL) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmAmbiguousSlap) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmIncompleteSlap) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_LeftRightMixup))
                    DiagColor = Color.Red;
            // Yellow diagnostics
            if (GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_LEFT) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_RIGHT) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_OUTSIDE_BORDER_TOP) ||
                GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_DISPLACED_DOWN) ||
                // 1.9.13.0 - no more message for wet/dry
                //GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER) ||
                //GBMSGUI.CheckMask(LastDiagnostic, GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_InvalidPattern) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_IncompletePattern) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_Inclination) ||
                GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap))
                    if (DiagColor != Color.Red)
                        DiagColor = Color.Yellow;

            lstDiag.BackColor = DiagColor;

            // show on LCD ONLY when acquisition is stopped
            if (LCDPresent && !Acquiring)
            {
                if (LCDPPDiag != 0)
                {
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        LCDPPDiag, GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_DIAG_AREA);
                }
            }

            // save previous diagnostic
            PrevDiagnostic = LastDiagnostic;
            PrevPostProcessDiag = PostProcessDiag;
        }

        // Dislpay post processing diagnostic in the diagnostic list
/*
        private void DisplayPostProcessingDiagnostic()
        {
            DiagnosticItem DiagItem;
            int ItemIndex;
            Color DiagColor = lstDiag.BackColor;

            // save the heaviest for the LCD (can show only one)
            Byte LCDPPDiag = 0;
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_InvalidPattern))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_InvalidPattern,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Invalid_Pattern"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Invalid_Pattern"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                if (DiagColor != Color.Red)
                    DiagColor = Color.Yellow;
                LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INVALID_PATTERN_DETECTED;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_IncompletePattern))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_IncompletePattern,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Pattern"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Pattern"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                if (DiagColor != Color.Red)
                    DiagColor = Color.Yellow;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INCOMPLETE_PATTERN_DETECTED;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_Inclination))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_Inclination,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Finger_Inclination"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Finger_Inclination"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                if (DiagColor != Color.Red)
                    DiagColor = Color.Yellow;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_FINGER_INCLINATION_DETECTED;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmAmbiguousSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmAmbiguousSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Ambiguous_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Ambiguous_Slap"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                DiagColor = Color.Red;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_AMBIGUOUS_SLAP;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmIncompleteSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmIncompleteSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Incomplete_Slap"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                DiagColor = Color.Red;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_INCOMPLETE_SLAP;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_SegmPossibleIncompleteSlap,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Incomplete_Slap"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Incomplete_Slap"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                if (DiagColor != Color.Red)
                    DiagColor = Color.Yellow;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_POSSIBLE_INCOMPLETE_SLAP;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_LeftRightMixup))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_LeftRightMixup,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Left_Right_Mixup"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Left_Right_Mixup"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                DiagColor = Color.Red;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_LEFT_RIGHT_MIXUP;
            }
            if (GBMSGUI.CheckMask(PostProcessDiag, PostProcessDiagnostic.Diag_PossibleLeftRightMixup))
            {
                DiagItem = new DiagnosticItem(PostProcessDiagnostic.Diag_PossibleLeftRightMixup,
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Left_Right_Mixup"),
                    MSGUIRef.GetLocalizedString("Segm_Diag_Possible_Left_Right_Mixup"));
                ItemIndex = lstDiag.Items.Add(DiagItem);
                if (DiagColor != Color.Red)
                    DiagColor = Color.Yellow;
                if (LCDPPDiag == 0)
                    LCDPPDiag = GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_POSSIBLE_LEFT_RIGHT_MIXUP;
            }

            lstDiag.BackColor = DiagColor;

            if (LCDPresent)
            {
                if (LCDPPDiag != 0)
                {
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        LCDPPDiag, GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_DIAG_AREA);
                }
            }
        }
*/

        // draw rolled artefacts borders
        private void DrawMarkerFrame(Graphics g)
        {
            try
            {
                int i, x, y;
                Rectangle OrigRect = new Rectangle();
                Rectangle DrawRect = new Rectangle();
                Pen RedPen = new Pen(Color.Red, 1);

                for (i = 0; i < (ImageSizeX * ImageSizeY); i++)
                {
                    if (MarkerFrame[i] == 0)
                        continue;

                    // draw only pixels marked as artefact border
                    if (GBMSGUI.CheckMask(MarkerFrame[i], GBMSAPI_NET_ArtefactRegionMarkers.GBMSAPI_NET_ARTEFACT_REGION_MARKED_BORDER))
                    {
                        // get real coordinates
                        y = i / (int)ImageSizeX;
                        x = i % (int)ImageSizeX;

                        OrigRect.X = x;
                        OrigRect.Y = y;

                        // transform into view coordinates
                        RealToViewRect(OrigRect, ref DrawRect);
                        DrawRect.Width = 1;
                        DrawRect.Height = 1;

                        // draw point
                        g.DrawRectangle(RedPen, DrawRect);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in DrawMarkerFrame");
                return;
            }
        }

        // Evaluate quality of scanned object, based on currently selected algorithm
        private bool EvalIAIFSQuality()
        {
            bool Result = false;

            // NFIQ Algorithm
            if (MSGUIRef.CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
            {
                if (ScannedObject.NFIQQuality == 255)
                {
                    lblIAFISQualityValue.Text = "--";
                    lblIAFISQualityValue.BackColor = Color.Red;
                    return false;
                }

                lblIAFISQualityValue.Text = ScannedObject.NFIQQuality.ToString();

                if ((ScannedObject.QualityThreshold1 == 0) || (ScannedObject.QualityThreshold2 == 0))
                {
                    // no thresholds, no evaluation
                    lblIAFISQualityValue.BackColor = SystemColors.ButtonFace;
                    Result = true;
                }
                else
                {
                    if (ScannedObject.NFIQQuality <= ScannedObject.QualityThreshold1)
                    {
                        lblIAFISQualityValue.BackColor = Color.Lime;
                        Result = true;
                    }
                    else if ((ScannedObject.NFIQQuality > ScannedObject.QualityThreshold1) &&
                        (ScannedObject.NFIQQuality <= ScannedObject.QualityThreshold2))
                    {
                        lblIAFISQualityValue.BackColor = Color.Yellow;
                        Result = false;
                    }
                    else
                    {
                        lblIAFISQualityValue.BackColor = Color.Red;
                        Result = false;
                    }
                }

                // show on LCD
                if (LCDPresent)
                {
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityLimitOnStopScreen(
                        (byte)ScannedObject.QualityThreshold1,
                        GBMSAPI_NET_LimitsDirection.GBMSAPI_NET_VILCD_SMALLER_THAN_LIMIT_DIRECTION);
                    if (LCDMidLimitSupported)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
                            (byte)ScannedObject.QualityThreshold2);
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityOnStopScreen(
                        (byte)ScannedObject.NFIQQuality);
                }

            }
            else
            {
                // GB Algorithm
                lblIAFISQualityValue.Text = ScannedObject.GBQuality.ToString();

                if ((ScannedObject.QualityThreshold1 == 0) || (ScannedObject.QualityThreshold2 == 0))
                {
                    // no thresholds, no evaluation
                    lblIAFISQualityValue.BackColor = SystemColors.ButtonFace;
                    Result = true;
                }
                else
                {
                    if (ScannedObject.GBQuality >= ScannedObject.QualityThreshold2)
                    {
                        lblIAFISQualityValue.BackColor = Color.Lime;
                        Result = true;
                    }
                    else if ((ScannedObject.GBQuality < ScannedObject.QualityThreshold2) &&
                        (ScannedObject.GBQuality >= ScannedObject.QualityThreshold1))
                    {
                        lblIAFISQualityValue.BackColor = Color.Yellow;
                        Result = false;
                    }
                    else
                    {
                        lblIAFISQualityValue.BackColor = Color.Red;
                        Result = false;
                    }
                }

                // show on LCD
                if (LCDPresent)
                {
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityLimitOnStopScreen(
                        (byte)ScannedObject.QualityThreshold2,
                        GBMSAPI_NET_LimitsDirection.GBMSAPI_NET_VILCD_GREATER_THAN_LIMIT_DIRECTION);
                    if (LCDMidLimitSupported)
                        GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
                            (byte)ScannedObject.QualityThreshold1);
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetQualityOnStopScreen(
                        (byte)ScannedObject.GBQuality);
                }
            }

            return Result;
        }

        // Evaluate rolled artefacts size
        private bool EvalRolledArtefactsSize()
        {
            bool Result = false;

            // show artefacts size
            lblArtefacsSize.Text = ScannedObject.RolledArtefactSize.ToString();

            if ((MSGUIRef.CfgData.ArtefactsThreshold1 == -1) || (MSGUIRef.CfgData.ArtefactsThreshold2 == -1))
            {
                // no thresholds, no evaluation
                lblArtefacsSize.BackColor = SystemColors.ButtonFace;
                Result = true;
            }
            else
            {
                if (ScannedObject.RolledArtefactSize <= MSGUIRef.CfgData.ArtefactsThreshold1)
                {
                    lblArtefacsSize.BackColor = Color.Lime;
                    Result = true;
                }
                if ((ScannedObject.RolledArtefactSize > MSGUIRef.CfgData.ArtefactsThreshold1) &&
                    (ScannedObject.RolledArtefactSize <= MSGUIRef.CfgData.ArtefactsThreshold2))
                {
                    lblArtefacsSize.BackColor = Color.Yellow;
                    Result = false;
                }
                if (ScannedObject.RolledArtefactSize > MSGUIRef.CfgData.ArtefactsThreshold2)
                {
                    lblArtefacsSize.BackColor = Color.Red;
                    Result = false;
                }
            }

            // show on LCD
            if (LCDPresent)
            {
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetArtefactLimitOnStopScreen(
                    MSGUIRef.CfgData.ArtefactsThreshold1,
                    GBMSAPI_NET_LimitsDirection.GBMSAPI_NET_VILCD_SMALLER_THAN_LIMIT_DIRECTION);
                if (LCDMidLimitSupported)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
                        MSGUIRef.CfgData.ArtefactsThreshold2);
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetArtefactOnStopScreen(
                    (int)ScannedObject.RolledArtefactSize);
            }

            return Result;
        }

        // return a mask corresponding to scanned object id, to use to set indicators
        private uint SetIndicatorMask(uint ScanObjID)
        {
            uint IndicatorMask = 0;

            if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_INDEX_LEFT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_LITTLE_LEFT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_MIDDLE_LEFT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_RING_LEFT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_THUMB_LEFT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_INDEX_RIGHT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_LITTLE_RIGHT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_MIDDLE_RIGHT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_RING_RIGHT;

            else if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB))
                    IndicatorMask |= GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_THUMB_RIGHT;

            return IndicatorMask;
        }

        // turns off all indicators
        private void TurnOffLeds()
        {
            uint IndicatorMask = GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_INDEX_LEFT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_INDEX_RIGHT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_LITTLE_LEFT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_LITTLE_RIGHT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_MIDDLE_LEFT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_MIDDLE_RIGHT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_RING_LEFT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_RING_RIGHT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_THUMB_LEFT |
                GBMSAPI_NET_LEDMaskValues.GBMSAPI_NET_VIL_THUMB_RIGHT;

            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_SetFingerIndicatorVUIState(
                IndicatorMask, GBMSAPI_NET_LEDColors.GBMSAPI_NET_COLOR_OFF, false);
        }

        // return the maximun size requested for segments, used for segmentation
        private void GetMaxSegmentsSize(out int SizeX, out int SizeY)
        {
            int SegmNum;
            if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                SegmNum = 2;
            else
                SegmNum = 4;

            SizeX = SizeY = 0;
            for (int i = 0; i < SegmNum; i++)
            {
                if (MSGUIRef.SegmCfgData[i].UnavailabilityReason == -1)
                {
                    if (MSGUIRef.SegmCfgData[i].ImageSizeX > SizeX)
                        SizeX = MSGUIRef.SegmCfgData[i].ImageSizeX;
                    if (MSGUIRef.SegmCfgData[i].ImageSizeY > SizeY)
                        SizeY = MSGUIRef.SegmCfgData[i].ImageSizeY;
                }
            }
        }

        // check if some fingers are set as unavailable from the application
        private bool CheckUnavailableFingers()
        {
            bool Result = false;
            int SegmNum;
            if (ScannedObject.ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS)
                SegmNum = 2;
            else
                SegmNum = 4;

            for (int i = 0; i < SegmNum; i++)
            {
                if (MSGUIRef.SegmCfgData[i].UnavailabilityReason != -1)
                {
                    SegmData[i].UnavailabilityReason = MSGUIRef.SegmCfgData[i].UnavailabilityReason;
                    Result = true;
                }
            }

            return Result;
        }

        // try to center clipping rectangle with bounding boxes
        private void CenterClipWithBoxes()
        {
            int i;
            int extTop, extLeft, extRight, extBottom;   // extremes

            extTop = (int)ImageSizeY + 1;
            extLeft = (int)ImageSizeX + 1;
            extRight = -1;
            extBottom = -1;

            // find bounding boxes extremes
            for (i = 0; i < SegmData.Count; i++)
            {
                if (!SegmData[i].Found)
                    continue;

                // top
                if (SegmData[i].BoundingBox.RealRect.Top < extTop)
                    extTop = SegmData[i].BoundingBox.RealRect.Top;
                // left
                if (SegmData[i].BoundingBox.RealRect.Left < extLeft)
                    extLeft = SegmData[i].BoundingBox.RealRect.Left;
                // right
                if (SegmData[i].BoundingBox.RealRect.Right > extRight)
                    extRight = SegmData[i].BoundingBox.RealRect.Right;
                // bottom
                if (SegmData[i].BoundingBox.RealRect.Bottom > extBottom)
                    extBottom = SegmData[i].BoundingBox.RealRect.Bottom;
            }

            // center clipping, with the same top
            ClippingRect.RealRect.Y = extTop;
            if (ClippingRect.RealRect.Y < 0)
                ClippingRect.RealRect.Y = 0;

            ClippingRect.RealRect.X = (extRight + extLeft) / 2 - ClippingRect.RealRect.Width / 2;
            if (ClippingRect.RealRect.X < 0)
                ClippingRect.RealRect.X = 0;

            if (ClippingRect.RealRect.Bottom > ImageSizeY)
            {
                ClippingRect.RealRect.Y = (int)ImageSizeY - ClippingRect.RealRect.Height;
            }

            if (ClippingRect.RealRect.Right > ImageSizeX)
            {
                ClippingRect.RealRect.X = (int)ImageSizeX - ClippingRect.RealRect.Width;
            }

            //GBMSGUI.LogToFile("CenterClipWithBoxes");
            //GBMSGUI.LogToFile("ClippingRect width = " + ClippingRect.RealRect.Width.ToString());
            //GBMSGUI.LogToFile("ClippingRect height = " + ClippingRect.RealRect.Height.ToString());

            // update clipping in view
            RealToViewRect(ClippingRect.RealRect, ref ClippingRect.ViewRect);
            pboxImage.Refresh();
        }

        private bool CheckSegmentBoundingBoxPositions()
        {
            int i;
            bool result = true;
            int Amount;
            int TestSize;
            int Limit = 10; // tolerance
            bool Unacceptable = false;

            // check that bounding boxes do not go out of clipping rectangle, or image,
            // more than 30% of its size
            for (i = 0; i < SegmData.Count; i++)
            {
                if (!SegmData[i].Found)
                    continue;

                if (ClippingRect.Visible)
                {
                    if (SegmData[i].BoundingBox.RealRect.Left < ClippingRect.RealRect.Left)
                    {
                        result = false;
                        Amount = ClippingRect.RealRect.Left - SegmData[i].BoundingBox.RealRect.Left;
                        TestSize = SegmData[i].BoundingBox.RealRect.Width;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Top < ClippingRect.RealRect.Top)
                    {
                        result = false;
                        Amount = ClippingRect.RealRect.Top - SegmData[i].BoundingBox.RealRect.Top;
                        TestSize = SegmData[i].BoundingBox.RealRect.Height;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Right > ClippingRect.RealRect.Right)
                    {
                        result = false;
                        Amount = SegmData[i].BoundingBox.RealRect.Right - ClippingRect.RealRect.Right;
                        TestSize = SegmData[i].BoundingBox.RealRect.Width;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Bottom > ClippingRect.RealRect.Bottom)
                    {
                        result = false;
                        Amount = SegmData[i].BoundingBox.RealRect.Bottom - ClippingRect.RealRect.Bottom;
                        TestSize = SegmData[i].BoundingBox.RealRect.Height;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                }
                else
                {
                    if (SegmData[i].BoundingBox.RealRect.Left < 0)
                    {
                        result = false;
                        Amount = 0 - SegmData[i].BoundingBox.RealRect.Left;
                        TestSize = SegmData[i].BoundingBox.RealRect.Width;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Top < 0)
                    {
                        result = false;
                        Amount = 0 - SegmData[i].BoundingBox.RealRect.Top;
                        TestSize = SegmData[i].BoundingBox.RealRect.Height;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Right > ImageSizeX)
                    {
                        result = false;
                        Amount = SegmData[i].BoundingBox.RealRect.Right - (int)ImageSizeX;
                        TestSize = SegmData[i].BoundingBox.RealRect.Width;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                    if (SegmData[i].BoundingBox.RealRect.Bottom > ImageSizeY)
                    {
                        result = false;
                        Amount = SegmData[i].BoundingBox.RealRect.Bottom - (int)ImageSizeY;
                        TestSize = SegmData[i].BoundingBox.RealRect.Height;
                        if (Amount > TestSize * Limit / 100)
                            Unacceptable = true;
                    }
                }
            }

            if (result == false)
            {
                if (Unacceptable)
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Bounding_Boxes_Out_Strong"), true);
                else
                    DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Bounding_Boxes_Out_Light"), true);

                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SEGMENT_BOUNDING_PROBLEM,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
            }

            // DEBUG
            //GBMSGUI.LogToFile("CheckSegmentBoundingBoxPositions: ClippingRect width = " + ClippingRect.RealRect.Width.ToString() +
            //    " ClippingRect height = " + ClippingRect.RealRect.Height.ToString());

            return result;
        }

        // check that bounding boxex don't overlap more than 15% of their size
        private bool CheckSegmentBoundingBoxOverlap()
        {
            int i,j;
            bool result = true;

/*
            for (i = 0; i < SegmData.Count-1; i++)
            {
                if (!SegmData[i].Found)
                    continue;

                if (AreRectOverlapping(SegmData[i].BoundingBox.RealRect, SegmData[i+1].BoundingBox.RealRect, 15))
                    result = false;
            }
*/
            // corrected 1.8.0.0 29/06/2010 - both segments must be assigned
            int Idx1;
            int Idx2;

            Idx1 = -1;
            for (i = 0; i < SegmData.Count - 1; i++)
            {
                if (!SegmData[i].Found)
                    continue;

                Idx1 = i;

                // search second segment
                Idx2 = -1;
                for (j = Idx1 + 1; j < SegmData.Count; j++)
                {
                    if (SegmData[j].Found)
                    {
                        Idx2 = j;
                        break;
                    }
                }

                if (Idx2 == -1)     // no neighbour found
                    continue;

                if (AreRectOverlapping(SegmData[Idx1].BoundingBox.RealRect, SegmData[Idx2].BoundingBox.RealRect, 15))
                {
                    result = false;
                    break;
                }
                else
                    continue;
            }

            if (Idx1 == -1)     // no segments assigned
                return true;
            
            if (!result)
            {
                DisplayMessage(MSGUIRef.GetLocalizedString("Msg_Bounding_Boxes_Overlap"), true);
                if (LCDPresent)
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        GBMSAPI_NET_PreloadedGeneralMessages.GBMSAPI_NET_VILCD_MSG_SEGMENT_BOUNDING_PROBLEM,
                        GBMSAPI_NET_PreloadedMessagesArea.GBMSAPI_NET_VILCD_MSG_AREA);
                return false;
            }

            return true;
        }

        private bool AreRectOverlapping(Rectangle r1, Rectangle r2, int Limit)
        {
            int HorzOverlap, VertOverlap;

            // check which starts first from left
            if (r1.Left < r2.Left)
                HorzOverlap = r2.Left - r1.Right;
            else
                HorzOverlap = r1.Left - r2.Right;

            // determine which one starts first from top
            if (r1.Top < r2.Top)
                VertOverlap = r2.Top - r1.Bottom;
            else
                VertOverlap = r1.Top - r2.Bottom;

            if ((HorzOverlap > 0) || (VertOverlap > 0))
            {
                // not overlapping
                return false;
            }

            // see if overlap more than Limit
            if ((HorzOverlap * VertOverlap) > (Math.Min(r1.Width * r1.Height, r2.Width * r2.Height) * Limit / 100))
            {
                return true;
            }

            return false;
        }

        private void CutBoundingBoxPositions()
        {
            int i;
            int TestWidth, TestHeight;

            // cut bounding box coordinates if they go out of clipping rectangle or image
            for (i = 0; i < SegmData.Count; i++)
            {
                if (!SegmData[i].Found)
                    continue;

                if (ClippingRect.Visible)
                {
                    // make coordinates relative to clipped image
                    SegmData[i].BoundingBox.RealRect.X -= ClippingRect.RealRect.X;
                    SegmData[i].BoundingBox.RealRect.Y -= ClippingRect.RealRect.Y;

                    TestWidth = ClippingRect.RealRect.Width;
                    TestHeight = ClippingRect.RealRect.Height;
                }
                else
                {
                    TestWidth = (int)ImageSizeX;
                    TestHeight = (int)ImageSizeY;
                }

                if (SegmData[i].BoundingBox.RealRect.X < 0)
                    SegmData[i].BoundingBox.RealRect.X = 0;

                if (SegmData[i].BoundingBox.RealRect.Y < 0)
                    SegmData[i].BoundingBox.RealRect.Y = 0;

                if (SegmData[i].BoundingBox.RealRect.Right > TestWidth)
                    SegmData[i].BoundingBox.RealRect.Width = TestWidth - SegmData[i].BoundingBox.RealRect.X;

                if (SegmData[i].BoundingBox.RealRect.Bottom > TestHeight)
                    SegmData[i].BoundingBox.RealRect.Height = TestHeight - SegmData[i].BoundingBox.RealRect.Y;
            }
        }

        // find an item by text in listViewMessages
        private int FindlistViewMessage(String Text)
        {
            int Index = -1;

            foreach (ListViewItem Item in listViewMessages.Items)
            {
                if (Item.Text == Text)
                    Index = Item.Index;
            }

            return Index;
        }

        // converts a Green Bit quality value lto NFIQ value
        private int GBQualityToNFIQ(int GBQuality)
        {
            int NFIQQuality;

            if (GBQuality == 100)
                NFIQQuality = 1;
            else
                NFIQQuality = 5 - GBQuality / 20;

            return NFIQQuality;
        }

        // fill image with FillColor, excluding Excluded area
        private void FillImage(Byte[] ImgBuff, int SizeX, int SizeY,
            int ExcludedLeft, int ExcludedTop, int ExcludedRight, int ExcludedBottom,
            byte FillColor)
        {
            int x, y;
            IntPtr pImage = Marshal.AllocHGlobal(SizeX * SizeY);
            Marshal.Copy(ImgBuff, 0, pImage, SizeX * SizeY);

            int pByte = (int)pImage;
            for (y = 0; y < SizeY; y++)
            {
                for (x = 0; x < SizeX; x++)
                {
                    // if outside Excluded area, set to FillColor
                    if ((y < ExcludedTop) || (y >= ExcludedBottom) ||
                        (x < ExcludedLeft) || (x >= ExcludedRight))
                        Marshal.WriteByte((IntPtr)pByte, FillColor);

                    pByte++;
                }
            }

            Marshal.Copy(pImage, ImgBuff, 0, SizeX * SizeY);
            Marshal.FreeHGlobal(pImage);
        }


        //effetti collaterali dell'immagine ruotata nella GUI (invece che nel basso livello dove e' logico che sia)
        //   - le coordinate del clipping sono sbagliate
        //   - la firma e' al contrario, quindi la gbfinimg fallisce        
        // e poi l'unsafe non va bene su linux....
        /*
        private unsafe void Rotate180(Byte[] ImgBuff, int SizeX, int SizeY)
        {
	        Byte tmpByte;
	        int i, half;
            half = SizeX * SizeY / 2;

            fixed (byte* pImgBuff = ImgBuff)
            {
                byte* pImg = pImgBuff;
                byte* pEndImg = pImgBuff + SizeX * SizeY - 1;

                for (i = 0; i < half; i++)
                {
                    tmpByte = *pImg;
                    *pImg = *pEndImg;
                    *pEndImg = tmpByte;
                    pImg++;
                    pEndImg--;
                }
            }
        }
        */
    }
}
