#include "ClipRectItem.h"

QClipRectItem::QClipRectItem(QColor PenColor, int PenWidth, Qt::PenStyle PenStyle)
{
    QGraphicsRectItem::QGraphicsRectItem();

    Text = "";
    QualityValue = 0;
    QualityColor = Qt::lightGray;

    mPen.setColor(PenColor);
    mPen.setWidth(PenWidth);
    mPen.setStyle(PenStyle);
    setPen(mPen);

    // non serve??? e' transparent di default???
    //setBrush(QBrush(Qt::transparent));

    setAcceptHoverEvents(true);
    setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
}

QClipRectItem::~QClipRectItem()
{
}

void QClipRectItem::SetSizeOption(SizeOptions SizeOption)
{
    mSizeOption = SizeOption;
}

QVariant QClipRectItem::itemChange ( GraphicsItemChange change, const QVariant & value )
{
    if (change == ItemPositionChange && scene())
    {
        // value is the new position.
        QPointF newPos = value.toPointF();
        QRectF rect = scene()->sceneRect();
        //if (!rect.contains(newPos))
        // check not only the top-left, but the whole rectangle...
        if (!rect.contains(QRectF(newPos, this->rect().size())))
        {
            // Keep the item inside the scene rect.
            newPos.setX(qMin(rect.right()-this->rect().width(), qMax(newPos.x(), rect.left())));
            newPos.setY(qMin(rect.bottom()-this->rect().height(), qMax(newPos.y(), rect.top())));
            return newPos;
        }
    }

    if (change == ItemPositionHasChanged)
    {
    }

    return QGraphicsItem::itemChange(change, value);
}

void QClipRectItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
    qreal x = event->pos().x();
    qreal y = event->pos().y();

    if (mEnabled)
    {
        // check cursor position for resize
        if ((mSizeOption & AllowMove) && (x > 1) && (x < rect().width() - 1) && (y > 1) && (y < rect().height() - 1))
        {
            this->setCursor(Qt::OpenHandCursor);
            mSizeOperation = SizeOperationMove;
        }
        else if ((mSizeOption & AllowResize) && (x >= -1) && (x <= 1))
        {
            this->setCursor(Qt::SizeHorCursor);
            mSizeOperation = SizeOperationLeft;
        }
        else if ((mSizeOption & AllowResize) && (x >= rect().width() - 1) && (x <= rect().width() + 1))
        {
            this->setCursor(Qt::SizeHorCursor);
            mSizeOperation = SizeOperationRight;
        }
        else if ((mSizeOption & AllowResize) && (y >= - 1) && (y <= 1))
        {
            this->setCursor(Qt::SizeVerCursor);
            mSizeOperation = SizeOperationTop;
        }
        else if ((mSizeOption & AllowResize) && (y >= rect().height() - 1) && (y <= rect().height() + 1))
        {
            this->setCursor(Qt::SizeVerCursor);
            mSizeOperation = SizeOperationBottom;
        }
        else
        {
            //this->setCursor(Qt::ArrowCursor);
            this->unsetCursor();
            mSizeOperation = SizeOperationNone;
        }
    }

    QGraphicsItem::hoverMoveEvent(event);
}

void QClipRectItem::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
    if (!mEnabled)
        return;

    if ((mSizeOperation != SizeOperationNone) && (mSizeOperation != SizeOperationMove))
        resizing = true;
    else
        resizing = false;

    QGraphicsItem::mousePressEvent(event);
}

void QClipRectItem::mouseMoveEvent ( QGraphicsSceneMouseEvent *event )
{
    if (!mEnabled)
        return;

    if(resizing)
    {
        // keep inside the scene
        QPointF scenePos = mapToScene(event->pos());

        if ((scenePos.x() < 0) || (scenePos.y() < 0) ||
                (scenePos.x() > scene()->width()) || (scenePos.y() > scene()->height()))
            return;

        qreal XShift = event->pos().x() - event->lastPos().x();
        qreal YShift = event->pos().y() - event->lastPos().y();
        QRectF newRect = rect();
        QPointF newPos = pos();
        //QRectF newRect = boundingRect();

        switch (mSizeOperation)
        {
            case SizeOperationLeft:
                //newRect.setLeft(rect().left() + XShift);
                newPos.setX(pos().x() + XShift);
                newRect.setWidth(rect().width() - XShift);
                break;
            case SizeOperationRight:
                newRect.setRight(rect().right() + XShift);
                break;
            case SizeOperationTop:
                //newRect.setTop(rect().top() + YShift);
                newPos.setY(pos().y() + YShift);
                newRect.setHeight(rect().height() - YShift);
                break;
            case SizeOperationBottom:
                newRect.setBottom(rect().bottom() + YShift);
                break;
        }

        // prevent to have a negative size
        if ((newRect.width() < 0) || (newRect.height() < 0))
            return;

        prepareGeometryChange();
        setRect(newRect);
        setPos(newPos);
        update();

        //qDebug() << "newRect" << newRect;
    }
    else
    {
        QGraphicsItem::mouseMoveEvent(event);
    }
}

void QClipRectItem::mouseReleaseEvent ( QGraphicsSceneMouseEvent *event )
{
    if (!mEnabled)
        return;

    resizing = false;

    emit Changed();

    QGraphicsItem::mouseReleaseEvent(event);
}

void QClipRectItem::SetEnabled(bool Enabled)
{
    mEnabled = Enabled;

    // la setFlag funziona solo se la chiamo DOPO
    setFlag(QGraphicsItem::ItemIsMovable, Enabled);
    setSelected(Enabled);
}

void QClipRectItem::SetText(QString Value)
{
    Text = Value;
}

void QClipRectItem::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget )
{
    if (Text != "")
    {
        painter->setFont(QFont(painter->font().family(), 11, QFont::Bold));
        QPoint p(rect().x() + 2, rect().y() - 2);

        // draw shadow
        //painter->setPen(Qt::darkGray);
        //painter->drawText(p, Text);

        // draw text
        painter->setPen(Qt::blue);
        p.setX(p.x() - 1);
        p.setY(p.y() - 1);
        painter->drawText(p, Text);
    }

    if (QualityValue)
    {
        QRect QualityRect(rect().topLeft().toPoint(), rect().bottomRight().toPoint());
        QualityRect.setLeft(QualityRect.right()-20);
        QualityRect.setTop(QualityRect.bottom()-20);

        // draw rectangle
        painter->fillRect(QualityRect, (QBrush(QualityColor)));

        // draw value
        QString strValue;
        if ((QualityValue == 254) || (QualityValue == 255))
            strValue = "-";
        else
            strValue.setNum(QualityValue);
        painter->setFont(QFont(painter->font().family(), 8, QFont::Bold));
        painter->setPen(Qt::black);
        painter->drawText(QualityRect, Qt::AlignCenter, strValue);
    }

    if (!DiagIcon.isNull())
    {
        QPointF DiagPoint = rect().bottomLeft();
        DiagPoint.setY(DiagPoint.y() - DiagIcon.height());
        painter->drawPixmap(DiagPoint, DiagIcon);
    }

    QGraphicsRectItem::paint(painter, option, widget);
}

void QClipRectItem::SetQuality(int Value, QColor Color)
{
    QualityValue = Value;
    QualityColor = Color;
}

void QClipRectItem::SetDiagIcon(QPixmap Icon)
{
    DiagIcon = Icon;

    if (!DiagIcon.isNull())
    {
        // set transparency
        DiagIcon.setMask(DiagIcon.createMaskFromColor(Qt::magenta));
    }
}

