#include "FrameItem.h"

QFrameItem::QFrameItem()
{
	DrawScaled = false;
}

QRectF QFrameItem::boundingRect() const
{
    if (!Image.isNull())
    {
        if (DrawScaled)
            return QRectF(0, 0, DrawSize.width(), DrawSize.height());
        else
            return QRectF(0, 0, Image.width(), Image.height());
    }
    else
        return QRectF(0, 0, 0, 0);
}

void QFrameItem::SetSize(QSize Size)
{
	DrawSize = Size;
	DrawScaled = true;
}

void QFrameItem::SetImage(QImage Image)
{
    this->Image = Image;
    update();
}

void QFrameItem::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
    if (Image.isNull())
        painter->eraseRect(boundingRect());
    else
	{
		painter->setRenderHint(QPainter::Antialiasing, false);

		if (DrawScaled)
		{
			// bo, non mi sembra + veloce...
			painter->drawImage(QRect(QPoint(0, 0), DrawSize), 
				Image, QRect(0, 0, Image.width(), Image.height()));
		}
		else
		{
			painter->drawImage(0, 0, Image);
		}
	}

    if (!MarkerPath.isEmpty())
    {
        painter->setPen( Qt::red );
        painter->drawPath(MarkerPath);
    }

    QGraphicsPixmapItem::paint(painter, option, widget);
}

void QFrameItem::ClearImage()
{
    Image = QImage();
    MarkerPath = QPainterPath();
    update();
}

void QFrameItem::SetMarkerPath(QPainterPath path)
{
    MarkerPath = path;
    update();
}
