#ifdef __cplusplus
extern "C"
{
#endif

//#pragma pack(1)
#pragma pack(push,1)

#define __stdcall

#ifdef LFS_COMPILE_DLL
	#ifndef DllExport_lfs
		#define DllExport_lfs
	#endif
#else
	#ifndef DllExport_lfs
		#define DllExport_lfs
	#endif
#endif

#ifndef _LFS_DLL_H
#define _LFS_DLL_H	

	/////////////////////////////////////
	// DATA STRUCTURES
	/////////////////////////////////////
	
	/************************************
	* MINUTIA data structure
	************************************/
	// MINUTIA structure contains all relevant information about a single minutia
	// inside a fingerprint image.
	typedef struct minutia{
		// the pixel x-coordinate of the detected minutia.
		int x;
		// the pixel y-coordinate of the detected minutia.
		int y;
		// reserved
		int ex;
		// reserved
		int ey;
		// the direction of the detected minutia. Minutia direction is starting at 
		// vertical pointing up with unit 0 and increasing clockwise in increments
		// of 11.25 degrees completing a full circle. Using this scheme, the angle
		// of a detected minutia is quantized into the range 0 to 31 with 8 
		// representing horizontal to the right, 16 representing vertical pointing
		// down, and 24 representing horizontal to the left.
		int direction;
		// The reliability measure assigned to the detected minutia is a floating
		// point value in the range 0.0 to 1.0, with 0.0 representing the lowest
		// minutia quality and 1.0 representing the highest minutia quality.
		double reliability;
		// It is the type of the detected minutia:
		// 0 - bifurcation
		// 1 - ridge ending
		int type;
		// reserved
		int appearing;
		// reserved
		int feature_id;
		// reserved
		int *nbrs;
		// reserved
		int *ridge_counts;
		// reserved
		int num_nbrs;
	} MINUTIA;

	/********************************************
	* MINUTIAE data structure
	********************************************/
	// MINUTIAE structure contains a list of MINUTIA records.
	typedef struct minutiae{
		// Number of MINUTIA records allocated
		int alloc;
		// Actual number of MINUTIA records contained in list.
		int num;
		// array of pointers to MINUTIA pointers. Its size is alloc and
		// number of valid records is num.
		MINUTIA **list;
	} MINUTIAE;
	
	///////////////////////////////////
	// LIBRARY FUNCTIONS
	///////////////////////////////////
	/*************************************************************************
	**************************************************************************
	#cat:   get_minutiae - Takes a grayscale fingerprint image, binarizes the input
	#cat:                image, and detects minutiae points using LFS Version 2.
	#cat:                The routine passes back the detected minutiae, the
	#cat:                binarized image, and a set of image quality maps.

	   Input:
		  idata    - grayscale fingerprint image data
		  iw       - width (in pixels) of the grayscale image
		  ih       - height (in pixels) of the grayscale image
		  id       - pixel depth (in bits) of the grayscale image
		  ppmm     - the scan resolution (in pixels/mm) of the grayscale image
		  
	   Output:
		  ominutiae         - points to a structure containing the
							  detected minutiae
	   Return Code:
		  Zero     - successful completion
		  Negative - system error
	**************************************************************************/
	DllExport_lfs int __stdcall get_minutiae(
		MINUTIAE **ominutiae,
		unsigned char *idata,
		const int iw,
		const int ih,
		const int id,
		const double ppmm
	);

	/*************************************************************************
	**************************************************************************
	#cat:   free_minutiae - frees memory previously allocated for a MINUTIAE *.

	   Input:
		  minutiae    - points to the memory to be freed
		  
	   Output:
		  ominutiae         - points to a structure containing the
							  detected minutiae
	   Return Code:
		  Zero     - successful completion
		  Negative - system error
	**************************************************************************/
	DllExport_lfs void __stdcall free_minutiae(MINUTIAE *minutiae);


#endif

#pragma pack(pop)

#ifdef __cplusplus
}

#endif
