#ifndef GBNFIQ_H_
#define GBNFIQ_H_

//----------------------------------------------------------------------------------------------------------
#define GBNFIQ_API __declspec(dllexport)
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
extern "C"
#endif
GBNFIQ_API void __stdcall GBNFIQ_GetVersion(unsigned char *VersionField1,
											unsigned char *VersionField2,
											unsigned char *VersionField3,
											unsigned char *VersionField4);
//----------------------------------------------------------------------------------------------------------
// Function obtains GBNFIQ.dll module version. 
// These fields are changed upon any modification of dll module.
//----------------------------------------------------------------------------------------------------------
// Parameters:
//		<VersionField1> - corresponds to the first field of module version
//		<VersionField2> - corresponds to the second field of module version
//		<VersionField3> - corresponds to the third field of module version
//		<VersionField4> - corresponds to the fourth field of module version
// Example: if module version is <1.2.3.4>, then VersionField1=1, VersionField2=2, etc. 
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
typedef struct 
{
	// the x-pixel coordinate of the detected minutia.
	int x;  
	// the y-pixel coordinate of the detected minutia.
	int y;  

	// The direction of the detected minutia. Minutia direction is starting at 
	//  vertical pointing up with unit 0 and increasing clockwise in increments of 11.25 degrees 
	//  completing a full circle. Using this scheme, the angle of a detected minutia is quantized 
	//  into the range 0 to 31 with 8 representing horizontal to the right, 16 representing 
	//  vertical pointing down, and 24 representing horizontal to the left.
	int direction; 

	// It is the type of the detected minutia: 0 - bifurcation, 1 - ridge ending
	int type;

	// The reliability measure assigned to the detected minutia is a floating point value in 
	//  the range 0.0 to 1.0, with 0.0 representing lowest minutia quality and 1.0 
	//  representing highest minutia quality.
	double reliability;
}
GBNFIQ_MINUTIA_DESCRIPTOR;

#define GBNFIQ_MAX_MINUTIAE 1000

typedef struct 
{
	GBNFIQ_MINUTIA_DESCRIPTOR  MinutiaeList[GBNFIQ_MAX_MINUTIAE];
	int MinutiaeNumber;
}
GBNFIQ_MINUTIAE_INFO;
//----------------------------------------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" 
#endif
GBNFIQ_API int __stdcall GBNFIQ_GetImageQuality(unsigned char			*ImagePtr,
											    int						ImageWidth,
											    int						ImageHeight,
											    int						*ImageQuality,
												GBNFIQ_MINUTIAE_INFO	*MinutiaeInfo);
//----------------------------------------------------------------------------------------------------------
// This is an implementation of the NIST Fingerprint Image Quality algorithm as described
//  in Fingerprint Image Quality, Technical Report 7151, (NISTIR 7151).
// This function implements a fingerprint image quality algorithm. It takes a fingerprint image
//  and analyzes the overall quality of the image returning an image quality number ranging from 1
//  for highest quality to 5 for lowest. The quality of the image can be extremely useful in knowing
//  the likely performance of a fingerprint matcher on that image.
// 
// Fingerprint image quality is a valuable tool in determining a fingerprint matchers ability to match
//  a fingerprint against other fingerprints. It has been shown that as fingerprint image quality
//  degrades so does matcher performance. Knowing the fingerprint image quality can
//  dramatically change the way fingerprints are currently processed. For example, multiple
//  attempts could be made to obtain a higher quality fingerprint at the time of image capture. If the
//  quality of fingerprints were know in advance, a system could handle the fingerprints of different
//  quality in different ways. For example, a faster matcher could be used on good quality images
//  without a drop in accuracy and a slower more accurate matcher could be used on lower quality
//  images to improve matching accuracy. This would greatly improve the overall performance of
//  the system.
// 
// NOTE:
// Please note that NFIQ has been designed to assign a level of quality to a grayscale fingerprint
//  image. It was not designed to distinguish fingerprint images from all possible non-fingerprint
//  images, and it will not necessarily assign non-fingerprint image types to the worst quality level
//  five.
//----------------------------------------------------------------------------------------------------------
// Return value:
//NFIQ SYSTEM ERROR					< 0	
#define GBNFIQ_NO_ERROR				0	
#define GBNFIQ_EMPTY_IMG			1 // empty image detected (feature vector set to 0's)
#define GBNFIQ_TOO_FEW_MINUTIAE		2 // too few minutiae detected from fingerprint image,
									  //   indicating poor quality fingerprint
//----------------------------------------------------------------------------------------------------------
// Parameters:
//	<*ImagePtr> -	pointer to memory buffer with ImageWidth*ImageHeight size
//					that contains input grayscale fingerprint image for quality evaluation; 
//				    Image parameters:  dark ridges, light valleys; 256 grayscales; 500 dpi.
//	<ImageWidth> -	image width, in pixels; 
//	<ImageHeight> - image height, in pixels; 
//	<*ImageQuality> - returns fingerprint image quality in range [1;5],
//					  where 1 is the highest quality and 5 is the lowest.
//	<*MinutiaeInfo> - if pointer non-zero, function fills minutiae info struct
//----------------------------------------------------------------------------------------------------------

#endif

