#ifdef __cplusplus
extern "C"
{
#endif


#ifdef LFS_COMPILE_DLL
	#ifndef DllExport_lfs
		#define DllExport_lfs __declspec( dllexport )
	#endif
#else
	#ifndef DllExport_lfs
		#define DllExport_lfs
	#endif
#endif

#ifndef _LFS_DLL_H
#define _LFS_DLL_H	

	typedef struct minutia{
		int x;
		int y;
		int ex;
		int ey;
		int direction;
		double reliability;
		int type;
		int appearing;
		int feature_id;
		int *nbrs;
		int *ridge_counts;
		int num_nbrs;
	} MINUTIA;

	typedef struct minutiae{
		int alloc;
		int num;
		MINUTIA **list;
	} MINUTIAE;

	/*************************************************************************
	**************************************************************************
	#cat:   get_minutiae - Takes a grayscale fingerprint image, binarizes the input
	#cat:                image, and detects minutiae points using LFS Version 2.

	   Input:
		  idata    - grayscale fingerprint image data
		  iw       - width (in pixels) of the grayscale image
		  ih       - height (in pixels) of the grayscale image
		  id       - pixel depth (in bits) of the grayscale image
		  ppmm     - the scan resolution (in pixels/mm) of the grayscale image
		  
	   Output:
		  ominutiae         - points to a structure containing the
							  detected minutiae
	   Return Code:
		  Zero     - successful completion
		  Negative - system error
	**************************************************************************/
	DllExport_lfs int __stdcall get_minutiae(MINUTIAE **ominutiae, unsigned char *idata, const int iw, const int ih, const int id, const double ppmm);

	/*************************************************************************
	**************************************************************************
	#cat:   free_minutiae - frees memory previously allocated for a MINUTIAE *.

	   Input:
		  minutiae    - points to the memory to be freed
		  
	   Output:
		  ominutiae         - points to a structure containing the
							  detected minutiae
	   Return Code:
		  Zero     - successful completion
		  Negative - system error
	**************************************************************************/
	DllExport_lfs void __stdcall free_minutiae(MINUTIAE *minutiae);


#endif

#ifdef __cplusplus
}
#endif