#include "SelectObjectForm.h"
#include "ui_SelectObjectForm.h"
#include "GBMSGUI_Qt.h"
#include "GBMSAPI_Library.h"
#include <QMessageBox>

SelectObjectForm::SelectObjectForm(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SelectObjectForm)
{
    ui->setupUi(this);

    unsigned int ScannableTypes;
    GBMSAPI_GetScannableTypes(&ScannableTypes);

    RollSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_ROLL_SINGLE_FINGER);
    FlatSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_SINGLE_FINGER);
    Slap4Supported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_SLAP_4);
    TwoThumbsSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_THUMBS_2);
    Slap2Supported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_SLAP_2);
    LowerPalmSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_LOWER_HALF_PALM);
    UpperPalmSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_UPPER_HALF_PALM);
    WritersPalmSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_WRITER_PALM);
    TwoIndexesSupported = CheckMask(ScannableTypes, GBMSAPI_SBT_FLAT_INDEXES_2);

	SetZOrder();
	FillObjectsList();
    SetHints();

    // signal mapper for shapes
    ShapeSignalMapper = new QSignalMapper(this);
    connect(ShapeSignalMapper, SIGNAL(mapped(QWidget *)), this, SLOT(OnShapeSelected(QWidget *)));

	// add shapes to mapping
	AddShapeMap(ui->shpLeftUpperPalm);
	AddShapeMap(ui->shpRightUpperPalm);
	AddShapeMap(ui->shpLeftLowerPalm);
	AddShapeMap(ui->shpRightLowerPalm);
	AddShapeMap(ui->shpLeftWritersPalm);
    AddShapeMap(ui->shpRightWritersPalm);
    AddShapeMap(ui->shpLeftSlap4);
    AddShapeMap(ui->shpRightSlap4);
    AddShapeMap(ui->shpTwoIndexes);
    AddShapeMap(ui->shpTwoThumbs);
    AddShapeMap(ui->shpLeftSlap2);
    AddShapeMap(ui->shpRightSlap2);

	// event for single finger selection
	connect(ui->chkLeftThumb, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkLeftIndex, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkLeftMiddle, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkLeftRing, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkLeftLittle, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkRightThumb, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkRightIndex, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkRightMiddle, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkRightRing, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));
	connect(ui->chkRightLittle, SIGNAL(clicked()), this, SLOT(OnSingleFingerSelected()));

    ObjectID = GBMSAPI_SBT_NO_OBJECT;
}

SelectObjectForm::~SelectObjectForm()
{
	delete ShapeSignalMapper;
    delete ui;
}

void SelectObjectForm::AddShapeMap(QWidget *widget)
{
	ShapeSignalMapper->setMapping(widget, widget);
	connect(widget, SIGNAL(Clicked()), ShapeSignalMapper, SLOT(map()));
}

void SelectObjectForm::FillObjectsList()
{
	ui->shpLeftSlap4->setVisible(Slap4Supported);
    ui->chkLeftSlap4->setVisible(Slap4Supported);
    ui->shpRightSlap4->setVisible(Slap4Supported);
    ui->chkRightSlap4->setVisible(Slap4Supported);

    ui->shpTwoThumbs->setVisible(TwoThumbsSupported);
    ui->chkTwoThumbs->setVisible(TwoThumbsSupported);

    ui->shpLeftSlap2->setVisible(Slap2Supported);
    ui->chkLeftSlap2->setVisible(Slap2Supported);
    ui->shpRightSlap2->setVisible(Slap2Supported);
    ui->chkRightSlap2->setVisible(Slap2Supported);

    ui->shpTwoIndexes->setVisible(TwoIndexesSupported);
    ui->chkTwoIndexes->setVisible(TwoIndexesSupported);

    ui->chkRolled->setVisible(RollSupported);

    ui->shpLeftUpperPalm->setVisible(UpperPalmSupported);
    ui->chkLeftUpperPalm->setVisible(UpperPalmSupported);
    ui->shpRightUpperPalm->setVisible(UpperPalmSupported);
    ui->chkRightUpperPalm->setVisible(UpperPalmSupported);

    ui->shpLeftLowerPalm->setVisible(LowerPalmSupported);
    ui->chkLeftLowerPalm->setVisible(LowerPalmSupported);
    ui->shpRightLowerPalm->setVisible(LowerPalmSupported);
    ui->chkRightLowerPalm->setVisible(LowerPalmSupported);

    ui->shpLeftWritersPalm->setVisible(WritersPalmSupported);
    ui->chkLeftWritersPalm->setVisible(WritersPalmSupported);
    ui->shpRightWritersPalm->setVisible(WritersPalmSupported);
    ui->chkRightWritersPalm->setVisible(WritersPalmSupported);

    // adjust shown object according to current device
    //if (DataModuleMain->DeviceID == GBMSAPI_DN_MS1000)
    if (Slap4Supported)
    {
        // hide Slap2
        ui->shpLeftSlap2->setVisible(false);
        ui->chkLeftSlap2->setVisible(false);
        ui->shpRightSlap2->setVisible(false);
        ui->chkRightSlap2->setVisible(false);
        ui->shpTwoIndexes->setVisible(false);
        ui->chkTwoIndexes->setVisible(false);
    }
}

void SelectObjectForm::done(int r)
{
    if(QDialog::Accepted == r)  // ok was pressed
    {
		ObjectID = GBMSAPI_SBT_NO_OBJECT;

		if (ui->chkLeftSlap4->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_4_LEFT;
		else if (ui->chkRightSlap4->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_4_RIGHT;
		else if (ui->chkTwoThumbs->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_2_THUMBS;
		else if (ui->chkLeftSlap2->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_2_LEFT;
		else if (ui->chkRightSlap2->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_2_RIGHT;
		else if (ui->chkTwoIndexes->isChecked())
			ObjectID = GBMSAPI_SO_SLAP_2_INDEXES;
		else if (ui->chkLeftUpperPalm->isChecked())
			ObjectID = GBMSAPI_SO_UPPER_HALF_PALM_LEFT;
		else if (ui->chkRightUpperPalm->isChecked())
			ObjectID = GBMSAPI_SO_UPPER_HALF_PALM_RIGHT;
		else if (ui->chkLeftLowerPalm->isChecked())
			ObjectID = GBMSAPI_SO_LOWER_HALF_PALM_LEFT;
		else if (ui->chkRightLowerPalm->isChecked())
			ObjectID = GBMSAPI_SO_LOWER_HALF_PALM_RIGHT;
		else if (ui->chkLeftWritersPalm->isChecked())
			ObjectID = GBMSAPI_SO_WRITER_PALM_LEFT;
		else if (ui->chkRightWritersPalm->isChecked())
			ObjectID = GBMSAPI_SO_WRITER_PALM_RIGHT;
		else
		{
			// combination of one or more fingers, or one + roll
			if (ui->chkLeftThumb->isChecked() && ui->chkRightThumb->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_2_THUMBS;
			else if (ui->chkLeftIndex->isChecked() && ui->chkRightIndex->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_2_INDEXES;
			else if (ui->chkLeftIndex->isChecked() && ui->chkLeftMiddle->isChecked() && ui->chkLeftRing->isChecked() && ui->chkLeftLittle->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_4_LEFT;
			else if (ui->chkRightIndex->isChecked() && ui->chkRightMiddle->isChecked() && ui->chkRightRing->isChecked() && ui->chkRightLittle->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_4_RIGHT;
			else if (ui->chkLeftIndex->isChecked() && ui->chkLeftMiddle->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_2_LEFT;
			else if (ui->chkRightIndex->isChecked() && ui->chkRightMiddle->isChecked())
				ObjectID = GBMSAPI_SO_SLAP_2_RIGHT;
			else if (ui->chkLeftThumb->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_LEFT_THUMB;
				else
					ObjectID = GBMSAPI_SO_FLAT_LEFT_THUMB;
			}
			else if (ui->chkLeftIndex->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_LEFT_INDEX;
				else
					ObjectID = GBMSAPI_SO_FLAT_LEFT_INDEX;
			}
			else if (ui->chkLeftMiddle->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_LEFT_MIDDLE;
				else
					ObjectID = GBMSAPI_SO_FLAT_LEFT_MIDDLE;
			}
			else if (ui->chkLeftRing->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_LEFT_RING;
				else
					ObjectID = GBMSAPI_SO_FLAT_LEFT_RING;
			}
			else if (ui->chkLeftLittle->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_LEFT_LITTLE;
				else
					ObjectID = GBMSAPI_SO_FLAT_LEFT_LITTLE;
			}
			else if (ui->chkRightThumb->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_RIGHT_THUMB;
				else
					ObjectID = GBMSAPI_SO_FLAT_RIGHT_THUMB;
			}
			else if (ui->chkRightIndex->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_RIGHT_INDEX;
				else
					ObjectID = GBMSAPI_SO_FLAT_RIGHT_INDEX;
			}
			else if (ui->chkRightMiddle->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_RIGHT_MIDDLE;
				else
					ObjectID = GBMSAPI_SO_FLAT_RIGHT_MIDDLE;
			}
			else if (ui->chkRightRing->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_RIGHT_RING;
				else
					ObjectID = GBMSAPI_SO_FLAT_RIGHT_RING;
			}
			else if (ui->chkRightLittle->isChecked())
			{
				if (ui->chkRolled->isChecked())
					ObjectID = GBMSAPI_SO_ROLL_RIGHT_LITTLE;
				else
					ObjectID = GBMSAPI_SO_FLAT_RIGHT_LITTLE;
			}
			else
			{
	            QMessageBox::warning(this, tr("Warning"), tr("Select an object"),  QMessageBox::Ok);
				return;
			}
		}
    }

    QDialog::done(r);
}

void SelectObjectForm::OnShapeSelected(QWidget *widget) // this is a slot
{    
	// disable roll
	ui->chkRolled->setEnabled(false);

    // intercept check from shape
    if (widget == ui->shpLeftSlap4)
        ui->chkLeftSlap4->setChecked(true);
    if (widget == ui->shpRightSlap4)
        ui->chkRightSlap4->setChecked(true);
    if (widget == ui->shpTwoThumbs)
        ui->chkTwoThumbs->setChecked(true);
    if (widget == ui->shpLeftSlap2)
        ui->chkLeftSlap2->setChecked(true);
    if (widget == ui->shpRightSlap2)
        ui->chkRightSlap2->setChecked(true);
    if (widget == ui->shpTwoIndexes)
        ui->chkTwoIndexes->setChecked(true);
    if (widget == ui->shpLeftUpperPalm)
        ui->chkLeftUpperPalm->setChecked(true);
    if (widget == ui->shpRightUpperPalm)
        ui->chkRightUpperPalm->setChecked(true);
    if (widget == ui->shpLeftLowerPalm)
        ui->chkLeftLowerPalm->setChecked(true);
    if (widget == ui->shpRightLowerPalm)
        ui->chkRightLowerPalm->setChecked(true);
    if (widget == ui->shpLeftWritersPalm)
        ui->chkLeftWritersPalm->setChecked(true);
    if (widget == ui->shpRightWritersPalm)
        ui->chkRightWritersPalm->setChecked(true);

	SetObjectSelection(ui->shpLeftSlap4, ui->chkLeftSlap4->isChecked());
    SetObjectSelection(ui->shpRightSlap4, ui->chkRightSlap4->isChecked());
    SetObjectSelection(ui->shpTwoThumbs, ui->chkTwoThumbs->isChecked());
    SetObjectSelection(ui->shpLeftSlap2, ui->chkLeftSlap2->isChecked());
    SetObjectSelection(ui->shpRightSlap2, ui->chkRightSlap2->isChecked());
    SetObjectSelection(ui->shpTwoIndexes, ui->chkTwoIndexes->isChecked());
    SetObjectSelection(ui->shpLeftUpperPalm, ui->chkLeftUpperPalm->isChecked());
    SetObjectSelection(ui->shpRightUpperPalm, ui->chkRightUpperPalm->isChecked());
    SetObjectSelection(ui->shpLeftLowerPalm, ui->chkLeftLowerPalm->isChecked());
    SetObjectSelection(ui->shpRightLowerPalm, ui->chkRightLowerPalm->isChecked());
    SetObjectSelection(ui->shpLeftWritersPalm, ui->chkLeftWritersPalm->isChecked());
    SetObjectSelection(ui->shpRightWritersPalm, ui->chkRightWritersPalm->isChecked());
}

void SelectObjectForm::OnSingleFingerSelected() // this is a slot
{
    ui->chkRolled->setEnabled(RollSupported);

    SetObjectSelection(ui->shpLeftSlap4, ui->chkLeftSlap4->isChecked());
    SetObjectSelection(ui->shpRightSlap4, ui->chkRightSlap4->isChecked());
    SetObjectSelection(ui->shpTwoThumbs, ui->chkTwoThumbs->isChecked());
    SetObjectSelection(ui->shpLeftSlap2, ui->chkLeftSlap2->isChecked());
    SetObjectSelection(ui->shpRightSlap2, ui->chkRightSlap2->isChecked());
    SetObjectSelection(ui->shpTwoIndexes, ui->chkTwoIndexes->isChecked());
    SetObjectSelection(ui->shpLeftUpperPalm, ui->chkLeftUpperPalm->isChecked());
    SetObjectSelection(ui->shpRightUpperPalm, ui->chkRightUpperPalm->isChecked());
    SetObjectSelection(ui->shpLeftLowerPalm, ui->chkLeftLowerPalm->isChecked());
    SetObjectSelection(ui->shpRightLowerPalm, ui->chkRightLowerPalm->isChecked());
    SetObjectSelection(ui->shpLeftWritersPalm, ui->chkLeftWritersPalm->isChecked());
    SetObjectSelection(ui->shpRightWritersPalm, ui->chkRightWritersPalm->isChecked());
}

void SelectObjectForm::SetObjectSelection(QLabel *lblShape, bool Selected)
{
    if (Selected)
    {
		lblShape->setLineWidth(2);
		lblShape->setStyleSheet("color:lime");
    }
    else
    {
		lblShape->setLineWidth(1);
		lblShape->setStyleSheet("color:blue");
    }
}

int SelectObjectForm::GetObjectID()
{
	return ObjectID;
}

void SelectObjectForm::SetZOrder()
{
    ui->shpLeftUpperPalm->raise();
    ui->shpRightUpperPalm->raise();
    ui->shpLeftLowerPalm->raise();
    ui->shpRightLowerPalm->raise();
	ui->shpLeftWritersPalm->raise();
    ui->shpRightWritersPalm->raise();
    ui->shpLeftSlap4->raise();
    ui->shpRightSlap4->raise();
    ui->shpTwoIndexes->raise();
    ui->shpTwoThumbs->raise();
    ui->shpLeftSlap2->raise();
    ui->shpRightSlap2->raise();

	// radiobuttons over all
	ui->chkLeftMiddle->raise();
    ui->chkLeftSlap4->raise();
    ui->chkLeftWritersPalm->raise();
    ui->chkRightUpperPalm->raise();
    ui->chkRightThumb->raise();
    ui->chkLeftIndex->raise();
    ui->chkRightMiddle->raise();
    ui->chkRightLittle->raise();
    ui->chkLeftThumb->raise();
    ui->chkRightIndex->raise();
    ui->chkLeftLittle->raise();
    ui->chkRightRing->raise();
    ui->chkRightSlap2->raise();
    ui->chkLeftUpperPalm->raise();
    ui->chkTwoIndexes->raise();
    ui->chkRightWritersPalm->raise();
    ui->chkLeftSlap2->raise();
    ui->chkLeftLowerPalm->raise();
    ui->chkRightSlap4->raise();
    ui->chkLeftRing->raise();
    ui->chkRightLowerPalm->raise();
    ui->chkTwoThumbs->raise();
    ui->chkRolled->raise();
}

void SelectObjectForm::SetHints()
{
	ui->shpLeftUpperPalm->setToolTip(tr("Left Upper Half Palm"));
    ui->shpRightUpperPalm->setToolTip(tr("Right Upper Half Palm"));
    ui->shpLeftLowerPalm->setToolTip(tr("Left Lower Half Palm"));
    ui->shpRightLowerPalm->setToolTip(tr("Right Lower Half Palm"));
	ui->shpLeftWritersPalm->setToolTip(tr("Left Writers Palm"));
    ui->shpRightWritersPalm->setToolTip(tr("Right Writers Palm"));
    ui->shpLeftSlap4->setToolTip(tr("Flat Left Four Fingers"));
    ui->shpRightSlap4->setToolTip(tr("Flat Right Four Fingers"));
    ui->shpTwoIndexes->setToolTip(tr("Flat Two Indexes"));
    ui->shpTwoThumbs->setToolTip(tr("Flat Two Thumbs"));
    ui->shpLeftSlap2->setToolTip(tr("Flat Left Two Fingers"));
    ui->shpRightSlap2->setToolTip(tr("Flat Right Two Fingers"));
}


