#include "ThresholdProgressBar.h"

QThresholdProgressBar::QThresholdProgressBar(QWidget *parent)
    : QWidget(parent)
{
    Threshold1 = 0;
    Threshold2 = 0;
    Value = 0;
    MaxValue = 100;
}

void QThresholdProgressBar::SetThresholds(int Th1, int Th2)
{
    Threshold1 = Th1;
    Threshold2 = Th2;

    repaint();
}

void QThresholdProgressBar::SetValue(int Val)
{
    Value = Val;

    repaint();
}

void QThresholdProgressBar::SetMaxValue(int Val)
{
    MaxValue = Val;
}

void QThresholdProgressBar::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    QColor newColor = palette().highlight().color();

    // thresholds
    if (Value < Threshold1)
        newColor = Qt::red;
    else if (((Value >= Threshold1) && (Value < Threshold2))
        && (Threshold1 != Threshold2))
        newColor = Qt::yellow;
    else if (Value >= Threshold2)
        newColor = Qt::green;

    if ((Threshold1 == 0) && (Threshold2 == 0))
        newColor = palette().highlight().color();

    // draw border
    QRect BorderRect(0, 0, event->rect().width()-1, 23);
    painter.drawRect(BorderRect);

    // draw progressbar
    QRect ProgressRect(1, 1, (BorderRect.width()-1) * Value / MaxValue, BorderRect.height()-1);
    painter.fillRect(ProgressRect, QBrush(newColor));

    // draw value
    painter.setFont(QFont(painter.font().family(), -1, QFont::Bold));
    painter.drawText(BorderRect, Qt::AlignCenter, QString().setNum(Value));

    if ((Threshold1 != 0) || (Threshold2 != 0))
    {
        // draw thresholds
        QPoint P1, P2;
        QRect TextRect;
        int TextWidth = 22;

        painter.setFont(QFont(painter.font().family(), painter.font().pointSize()-1, QFont::Normal));

        P1.setX(BorderRect.x() + BorderRect.width() * Threshold1 / MaxValue);
        P1.setY(BorderRect.y());
        P2.setX(P1.x());
        P2.setY(BorderRect.y() + BorderRect.height());
        painter.drawLine(P1, P2);
        TextRect.setX(P2.x() - TextWidth/2);
        TextRect.setY(P2.y() + 6);
        TextRect.setWidth(TextWidth);
        TextRect.setHeight(painter.font().pointSize() + 1);
        painter.drawText(TextRect, Qt::AlignCenter, QString().setNum(Threshold1));

        P1.setX(BorderRect.x() + BorderRect.width() * Threshold2 / MaxValue);
        P1.setY(BorderRect.y());
        P2.setX(P1.x());
        P2.setY(BorderRect.y() + BorderRect.height());
        painter.drawLine(P1, P2);
        TextRect.setX(P2.x() - TextWidth/2);
        TextRect.setY(P2.y() + 6);
        TextRect.setWidth(TextWidth);
        TextRect.setHeight(painter.font().pointSize() + 1);
        painter.drawText(TextRect, Qt::AlignCenter, QString().setNum(Threshold2));
    }

}
