#include "gbmsgui_class.h"
#include "SelectObjectForm.h"
#include <QApplication>
#ifdef WIN32
	#include "DSBeep.h"
#endif

static GBMSGUI *MSGui = 0;  // One and only GBMSGUI instance
#ifdef WIN32
static HMODULE hInstance=NULL;	// Dll handle
#endif

void __stdcall GBMSGUI_Load()
{
    if (MSGui != 0)
        return;

    MSGui = new GBMSGUI();
}

void __stdcall GBMSGUI_Unload()
{
    if (MSGui != 0)
	{
        delete MSGui;
		MSGui = 0;	// was missed - fixed in 1.5.3.0
	}
}

#ifdef WIN32
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
                                         )
{
        switch (ul_reason_for_call)
        {
        case DLL_PROCESS_ATTACH:
			hInstance = hModule;
			break;
        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
        case DLL_PROCESS_DETACH:
                break;
        }
    return TRUE;
}

QString GetDLLPath()
{
	wchar_t szModuleName[_MAX_PATH];

	GetModuleFileNameW(hInstance, szModuleName, _MAX_PATH);

	QString strPath = QString::fromWCharArray(szModuleName);
	strPath = QFileInfo(strPath).absolutePath();

	return strPath;
}

#endif

// these MUST be available all the lifetime of App
static int App_argc = 1;
static char *App_argv[] = { "GBMSGUI", NULL };

GBMSGUI::GBMSGUI()
{
    OnSession = false;
    OnAcquisition = false;
    SessionOptions = 0;
    AcquisitionOptions = 0;
    Resolution1000Dpi = false;
    BlockAutocaptureSupported = false;
    ImageRotationSupported = false;
	DryFingerImageEnhancementSupported = false;  // 1.5.4.0
    FingerContactEvalMode = GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING; // 1.5.4.0
    ErrorCode = 0;
    Parent = NULL;
    frmAcquisition = NULL;
    AfterStartCallbackRef = NULL;
    AfterStartCallbackParam = NULL;
    mApp = NULL;

#ifndef WIN32
    GBMSAPI_LoadLibrary();
#endif

    translator = new QTranslator();

    // get Application pointer
    if (QApplication::instance() == NULL)
    {
        // create Application object (if called from non-Qt app)
        mApp = new QApplication(App_argc, App_argv, true);
    }
    else
    {
        Parent = ((QApplication *)QApplication::instance())->activeWindow();
    }

	// needed to find the plugins directory
	//QString LibPath = QDir::currentPath() + "/plugins/";
	//QApplication::addLibraryPath(LibPath);

    // init CfgData
    CfgData.ArtefactsThreshold1 = 7;
    CfgData.ArtefactsThreshold2 = 14;
    CfgData.QualityAlgorithm = GBMSGUI_NFIQ_ALGORITHM;
    CfgData.PatternValidityThreshold = 75;
    CfgData.PatternCompletenessThreshold = 85;
    CfgData.LowerPalmCompletenessThreshold1 = 80;
    CfgData.LowerPalmCompletenessThreshold2 = 90;
    CfgData.BlockAutoCaptureMask = GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA
        | GBMSAPI_DM_FLAT_FINGER_SLIDING
        | GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT
        | GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT
        | GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP
        | GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN
		// 1.5.6.0
		| GBMSAPI_DM_DRY_FINGER
		| GBMSAPI_DM_WET_FINGER;
    CfgData.BlockAutoCaptureContrast = false;
    CfgData.InclinationThreshold = 35;
    CfgData.AcqWindowLeft = -1;
    CfgData.AcqWindowTop = -1;
    CfgData.AcqWindowWidth = -1;
    CfgData.AcqWindowHeight = -1;
    CfgData.AcqWindowMaximized = true;
    //CultureInfo Culture = null;
    CfgData.IgnoreDiagnosticMask = 0;
    CfgData.FlatFingerOnRollArea = false;
}

GBMSGUI::~GBMSGUI()
{
    delete translator;

    if (frmAcquisition)
    {
        delete frmAcquisition;
        frmAcquisition = NULL;
    }

#ifndef WIN32
    GBMSAPI_UnloadLibrary();
#endif

    if (mApp)
        delete mApp;
}

#ifdef WIN32
#define DllName L"GBMSGUI_Qt.dll"
int GetRCVersion(unsigned char *VersionField1,
                                  unsigned char *VersionField2,
                                  unsigned char *VersionField3,
                                  unsigned char *VersionField4)
{
	DWORD fvis=GetFileVersionInfoSize(DllName,NULL);
	if( fvis == 0 )
		return FALSE;

	unsigned char *vi=NULL;
	vi=new unsigned char[fvis];
	if( vi == NULL )
		return FALSE;

	if( GetFileVersionInfo(DllName,NULL,fvis,vi) == 0 )
	{
		delete[] vi;
		return FALSE;
	}

	VS_FIXEDFILEINFO *ffi;
	UINT len;
	if( VerQueryValue(vi,TEXT("\\"),(LPVOID*)&ffi,&len) == 0 )
	{
		delete[] vi;
		return FALSE;
	}
	if( len != sizeof(VS_FIXEDFILEINFO) )
	{
		delete[] vi;
		return FALSE;
	}

	if( ffi->dwFileVersionMS != ffi->dwProductVersionMS ||
	ffi->dwFileVersionLS != ffi->dwProductVersionLS )
	{
		delete[] vi;
		return FALSE;
	}

	if( VersionField1 ) *VersionField1=(unsigned char)(ffi->dwFileVersionMS>>16);
	if( VersionField2 ) *VersionField2=(unsigned char)(ffi->dwFileVersionMS);
	if( VersionField3 ) *VersionField3=(unsigned char)(ffi->dwFileVersionLS>>16);
	if( VersionField4 ) *VersionField4=(unsigned char)(ffi->dwFileVersionLS);

	delete[] vi;

	return TRUE;
}
#else
int GetRCVersion(unsigned char *VersionField1,
                                  unsigned char *VersionField2,
                                  unsigned char *VersionField3,
                                  unsigned char *VersionField4)
{
    char * version;
    char * rpath = (char *)malloc(PATH_MAX);
    char delims[] = ".";
    char *result = NULL;

    if (realpath("libGBMSGUI_Qt.so", rpath)==NULL) {
        free(rpath);
        return FALSE;
    }
    if ((version = strstr(rpath, ".so."))==NULL) {
        free(rpath);
        return FALSE;
    }
    version+=4;

    if((result = strtok( version, delims ))==NULL) {
        free(rpath);
        return FALSE;
    }
    *VersionField1 = (unsigned char)atoi(result);
    if((result = strtok( NULL, delims ))==NULL) {
        free(rpath);
        return FALSE;
    }
    *VersionField2 = (unsigned char)atoi(result);
    if((result = strtok( NULL, delims ))==NULL) {
        free(rpath);
        return FALSE;
    }
    *VersionField3 = (unsigned char)atoi(result);
    /*
    if((result = strtok( NULL, delims ))==NULL) {
        free(rpath);
        return FALSE;
    }
    *VersionField4 = (unsigned char)atoi(result);
    if((result = strtok( NULL, delims ))==NULL) {
        free(rpath);
        return FALSE;
    }
    */

    *VersionField4 = 0;

    free(rpath);
    return TRUE;
}
#endif

void __stdcall GBMSGUI_GetVersion(int *Major, int *Minor, int *Build, int *Revision)
{
    unsigned char Ver1, Ver2, Ver3, Ver4;

    int ret = GetRCVersion(&Ver1, &Ver2, &Ver3, &Ver4);
    if (ret == TRUE)
    {
        if (Major)
            *Major = Ver1;
        if (Minor)
            *Minor = Ver2;
        if (Build)
            *Build = Ver3;
        if (Revision)
            *Revision = 0;
    }
}

int __stdcall GBMSGUI_SetLanguage(const char *LangName)
{        
    return MSGui->SetLanguage(LangName);
}
int GBMSGUI::SetLanguage(const char *LangName)
{
    if (LangName == NULL)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (QString::fromAscii(LangName) != "en")
	{
		QString LangFile = "GBMSGUI_" + QString::fromAscii(LangName);
#ifdef WIN32
		// add Dll path
		QString DLLPath = GetDLLPath();
		LangFile = DLLPath + "/" + LangFile;
#endif

		if (translator->load(LangFile))
		{
			if (mApp)
				mApp->installTranslator(translator);
			else
				QApplication::instance()->installTranslator(translator);
		}
	}
	else
    {
		// restore English (default)
		if (mApp)
			mApp->removeTranslator(translator);
		else
			QApplication::instance()->removeTranslator(translator);
    }

    // if LCD supports multilanguage, set language to LCD too
    uint ExternalEquipment;
    int ret = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetOptionalExternalEquipment", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }

    // not yet supported in linux
#ifdef WIN32
    if (CheckMask(ExternalEquipment, GBMSAPI_OED_VUI_LCD))
    {
        uint LCDFeatures;
        GBMSAPI_VUI_LCD_GetLcdFeatures(&LCDFeatures);
        if (CheckMask(LCDFeatures, GBMSAPI_VILCD_LF_LANGUAGE_SETTING))
            GBMSAPI_VUI_LCD_SetLanguage((char *)LangName);
    }
#endif

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetWindowSizeAndPosition(int Left, int Top, int Width, int Height, bool Maximized)
{
    return MSGui->SetWindowSizeAndPosition(Left, Top, Width, Height, Maximized);
}
int GBMSGUI::SetWindowSizeAndPosition(int Left, int Top, int Width, int Height, bool Maximized)
{
    // set size anyway, error is like a warning
    CfgData.AcqWindowLeft = Left;
    CfgData.AcqWindowTop = Top;
    CfgData.AcqWindowWidth = Width;
    CfgData.AcqWindowHeight = Height;
    CfgData.AcqWindowMaximized = Maximized;

    if (Width < 1024)
    {
        Width = 1024;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Height < 768)
    {
        Height = 768;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetArtefactsThresholds(int Threshold1, int Threshold2)
{
    return MSGui->SetArtefactsThresholds(Threshold1, Threshold2);
}

int GBMSGUI::SetArtefactsThresholds(int Threshold1, int Threshold2)
{
    if ((Threshold1 < -1) || (Threshold1 > Threshold2))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.ArtefactsThreshold1 = Threshold1;
    CfgData.ArtefactsThreshold2 = Threshold2;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetQualityAlgorithm(int AlgorithmID)
{
    return MSGui->SetQualityAlgorithm(AlgorithmID);
}

int GBMSGUI::SetQualityAlgorithm(int AlgorithmID)
{
    if ((AlgorithmID != GBMSGUI_NFIQ_ALGORITHM)
        && (AlgorithmID != GBMSGUI_GB_ALGORITHM))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.QualityAlgorithm = AlgorithmID;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetPatternValidityThreshold(int Threshold)
{
    return MSGui->SetPatternValidityThreshold(Threshold);
}

int GBMSGUI::SetPatternValidityThreshold(int Threshold)
{
    if ((Threshold < 0) || (Threshold > 100))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.PatternValidityThreshold = Threshold;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetPatternCompletenessThreshold(int Threshold)
{
    return MSGui->SetPatternCompletenessThreshold(Threshold);
}

int GBMSGUI::SetPatternCompletenessThreshold(int Threshold)
{
    if ((Threshold < 0) || (Threshold > 100))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.PatternCompletenessThreshold = Threshold;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetLowerPalmCompletenessThresholds(int Threshold1, int Threshold2)
{
    return MSGui->SetLowerPalmCompletenessThresholds(Threshold1, Threshold2);
}
int GBMSGUI::SetLowerPalmCompletenessThresholds(int Threshold1, int Threshold2)
{
    if ((Threshold1 < 0) || (Threshold1 > 100) ||
        (Threshold2 < 0) || (Threshold2 > 100) ||
        (Threshold1 > Threshold2))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.LowerPalmCompletenessThreshold1 = Threshold1;
    CfgData.LowerPalmCompletenessThreshold2 = Threshold2;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetInclinationThreshold(int Threshold)
{
    return MSGui->SetInclinationThreshold(Threshold);
}

int GBMSGUI::SetInclinationThreshold(int Threshold)
{
    if ((Threshold < 0) || (Threshold > 55))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    CfgData.InclinationThreshold = Threshold;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetBlockAutoCaptureMask(unsigned int Mask)
{
    return MSGui->SetBlockAutoCaptureMask(Mask);
}

int GBMSGUI::SetBlockAutoCaptureMask(unsigned int Mask)
{
    CfgData.BlockAutoCaptureMask = Mask;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetBlockAutoCaptureContrast(bool Flag)
{
    return MSGui->SetBlockAutoCaptureContrast(Flag);
}

int GBMSGUI::SetBlockAutoCaptureContrast(bool Flag)
{
    CfgData.BlockAutoCaptureContrast = Flag;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetIgnoreDiagnosticMask(unsigned int IgnoreDiagnosticMask)
{
    return MSGui->SetIgnoreDiagnosticMask(IgnoreDiagnosticMask);
}

int GBMSGUI::SetIgnoreDiagnosticMask(unsigned int IgnoreDiagnosticMask)
{
    CfgData.IgnoreDiagnosticMask = IgnoreDiagnosticMask;

    // "scanner error" and "object on scanner" cannot be masked
    CfgData.IgnoreDiagnosticMask &= ~GBMSAPI_DM_SCANNER_FAILURE;
    CfgData.IgnoreDiagnosticMask &= ~GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetFlatFingerOnRollArea(bool FlatFingerOnRollArea)
{
    return MSGui->SetFlatFingerOnRollArea(FlatFingerOnRollArea);
}

int GBMSGUI::SetFlatFingerOnRollArea(bool FlatFingerOnRollArea)
{
    CfgData.FlatFingerOnRollArea = FlatFingerOnRollArea;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_StartSession(int AcquisitionMode, wchar_t *PersonID, unsigned int Options)
{
    return MSGui->StartSession(AcquisitionMode, PersonID, Options);
}
int GBMSGUI::StartSession(int AcquisitionMode, wchar_t *PersonID, unsigned int Options)
{
    int ret;

    if (OnAcquisition)
        return GBMSGUI_RET_BUSY_ACQUIRING;

    if (OnSession)
    {
        SetError(QCoreApplication::tr("Session already open"), GBMSGUI_ERR_SESSION_ALREADY_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if ((AcquisitionMode != GBMSGUI_AM_SINGLE_ACQUISITION) &&
        (AcquisitionMode != GBMSGUI_AM_MULTIPLE_ACQUISITION))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    // if device is not defined, return error
    unsigned char DeviceID;
    char SerialNum[GBMSAPI_MAX_SN_LENGHT];
    ret = GBMSAPI_GetCurrentDevice(&DeviceID, SerialNum);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_GetCurrentDevice", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }

    // check serial number to see if Full Enhanced SDK is supported
	char CharToCheck = SerialNum[2];
    if ((toupper(CharToCheck) == 'B') || (toupper(CharToCheck) == 'L'))
    {
        SetError(QCoreApplication::tr("Full Enhanced SDK not supported"), GBMSGUI_ERR_FULL_ENHANCED_SDK_NOT_SUPPORTED);
        return GBMSGUI_RET_FAILURE;
    }

	// 1.5.7.0 - check GBFINIMG license
	GBFINIMG_LICENSE_USB_SCANNER_CHECK GBFINIMGLicense;
	GBFINIMG_GetLicenseInfo(&GBFINIMGLicense);
	if (GBFINIMGLicense.Result != GBSCAN_NO_ERROR)
	{
		if (GBFINIMGLicense.Result == GBSCAN_ERROR_USB_DEVICE_NOT_FOUND)
		{
			SetError(QCoreApplication::tr("Full Enhanced SDK not supported"), GBMSGUI_ERR_FULL_ENHANCED_SDK_NOT_SUPPORTED);
			return GBMSGUI_RET_FAILURE;
		}
		if (GBFINIMGLicense.Result == GBSCAN_ERROR_USB_DRIVER)
		{
		    QString ErrMsg;
            ErrMsg = QCoreApplication::tr("USB Driver error");
			ErrMsg = ErrMsg + QString(" (%1)").arg(GBFINIMGLicense.UsbDriverErrorCodeDetail, 8, 16);
			SetError(ErrMsg, GBMSGUI_ERR_FULL_ENHANCED_SDK_NOT_SUPPORTED);	// Boh???
			return GBMSGUI_RET_FAILURE;
		}
	}

    // get characteristics of current device
    unsigned int Features;
    ret = GBMSAPI_GetDeviceFeatures(&Features);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_GetAdditionalDeviceFeatures", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }

    if (CheckMask(Features, GBMSAPI_DF_1000DPI_RESOLUTION))
        Resolution1000Dpi = true;
    else
        Resolution1000Dpi = false;

    if (CheckMask(Features, GBMSAPI_DF_AUTO_CAPTURE_BLOCKING))
        BlockAutocaptureSupported = true;
    else
        BlockAutocaptureSupported = false;

    if (CheckMask(Features, GBMSAPI_DF_IMAGE_ROTATION))
        ImageRotationSupported = true;
    else
        ImageRotationSupported = false;

    // 1.5.4.0
    if (CheckMask(Features, GBMSAPI_DF_DRY_SKIN_IMG_ENHANCE))
        DryFingerImageEnhancementSupported = true;
    else
        DryFingerImageEnhancementSupported = false;

    // get supported scan options
    ret = GBMSAPI_GetSupportedScanOptions(&SupportedScanOptions);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_GetSupportedScanOptions", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }

    // see if PalmCompleteness is supported
    uint ImageInfo;
    GBMSAPI_GetAvailableImageInfo(GBMSAPI_SBT_FLAT_LOWER_HALF_PALM, &ImageInfo);
    if (CheckMask(ImageInfo, GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS))
        PalmCompletenessSupported = true;
    else
        PalmCompletenessSupported = false;

    // check presence of optional devices
    uint ExternalEquipment;
    ret = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_GetOptionalExternalEquipment", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }
    // check external equipment
    PedalPresent = CheckMask(ExternalEquipment, GBMSAPI_OED_PEDAL);
    LCDPresent = CheckMask(ExternalEquipment, GBMSAPI_OED_VUI_LCD);
    LedPresent = CheckMask(ExternalEquipment, GBMSAPI_OED_VUI_LED);
    SoundPresent = CheckMask(ExternalEquipment, GBMSAPI_OED_SOUND);

    // not yet supported in linux
#ifdef WIN32
    if (LCDPresent)
	{
		// MC517 color LCD supports intermediate limit
		uint LCDFeatures;
		GBMSAPI_VUI_LCD_GetLcdFeatures(&LCDFeatures);
		LCDMidLimitSupported = CheckMask(LCDFeatures, GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT);
	}
#endif

    SessionOptions = Options;

    // init scanned objects list
    InitScannedObjectsList();
    SetDefaultSegmentsConfiguration();

    // create acquisition form
    // pass reference to current instance of GBMSGUI (this) to acquisition form
    frmAcquisition = new AcquisitionForm(this, Parent);
    frmAcquisition->setAttribute( Qt::WA_DeleteOnClose, false );

    this->AcquisitionMode = AcquisitionMode;

    frmAcquisition->SetPersonID(QString::fromWCharArray(PersonID));

    // window size and position
    if (CfgData.AcqWindowWidth != -1)
    {
		// there is a different behaviour in Ubuntu and Windows...
#ifdef WIN32
        frmAcquisition->resize(CfgData.AcqWindowWidth, CfgData.AcqWindowHeight);
        frmAcquisition->move(CfgData.AcqWindowLeft, CfgData.AcqWindowTop);
#else
        frmAcquisition->setGeometry(CfgData.AcqWindowLeft, CfgData.AcqWindowTop,
                                    CfgData.AcqWindowWidth, CfgData.AcqWindowHeight);
#endif
    }

    OnSession = true;
    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetImageSize(unsigned int ScannedObjectID, unsigned int Width, unsigned int Height)
{
    return MSGui->SetImageSize(ScannedObjectID, Width, Height);
}
int GBMSGUI::SetImageSize(unsigned int ScannedObjectID, unsigned int Width, unsigned int Height)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    bool Result = true;
    unsigned int MaxImageSizeX, MaxImageSizeY;
    bool FlatScanArea = true;

    // get max size for object type on current scanner
    if (GBMSGUI::IsRolled(ScannedObjectID))
        FlatScanArea = false;
    else if (GBMSGUI::IsFlatSingleFinger(ScannedObjectID)
        && CfgData.FlatFingerOnRollArea
        && CheckMask(SupportedScanOptions, GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
        FlatScanArea = false;

    int ret = GBMSAPI_GetImageSize(GBMSAPI_GetTypeFromObject(ScannedObjectID),
        FlatScanArea, false, &MaxImageSizeX, &MaxImageSizeY);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        SetError(GetMSAPIErrorMessage("GBMSAPI_GetImageSize", ret), ret);
        return GBMSGUI_RET_FAILURE;
    }

    // min size
    unsigned int MinImageSizeX, MinImageSizeY;
    if (Resolution1000Dpi)
        MinImageSizeX = MinImageSizeY = 400;
    else
        MinImageSizeX = MinImageSizeY = 200;
    if ((Width < MinImageSizeX) || (Height < MinImageSizeY))
    {
        // set anyway to min size, so error can be treated as a warning
        if (Width < MinImageSizeX)
            Width = MinImageSizeX;
        if (Height < MinImageSizeY)
            Height = MinImageSizeY;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        Result = false;
    }

    // max size
    if ((Width > MaxImageSizeX) || (Height > MaxImageSizeY))
    {
        // set anyway to max size, so error can be treated as a warning
        if (Width > MaxImageSizeX)
            Width = MaxImageSizeX;
        if (Height > MaxImageSizeY)
            Height = MaxImageSizeY;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        Result = false;
    }

    if (!Result)
        return GBMSGUI_RET_FAILURE;

    int Index = IndexOfScannedObject(ScannedObjectID);
    ScannedObjectsList[Index].ImageSizeX = Width;
    ScannedObjectsList[Index].ImageSizeY = Height;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetContrastThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2)
{
    return MSGui->SetContrastThresholds(ScannedObjectID, Threshold1, Threshold2);
}

int GBMSGUI::SetContrastThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if ((Threshold1 < 0) || (Threshold2 < 0) || (Threshold1 > 255) || (Threshold2 > 255)
        || (Threshold1 > Threshold2))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int Index = IndexOfScannedObject(ScannedObjectID);
    ScannedObjectsList[Index].ContrastThreshold1 = Threshold1;
    ScannedObjectsList[Index].ContrastThreshold2 = Threshold2;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetQualityThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2)
{
    return MSGui->SetQualityThresholds(ScannedObjectID, Threshold1, Threshold2);
}

int GBMSGUI::SetQualityThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int MaxThreshold;
    if (CfgData.QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
        MaxThreshold = 5;
    else
        MaxThreshold = 100;

    // 0 is permitted; both 0 means no thresholds
    if ((Threshold1 < 0) || (Threshold1 > MaxThreshold) ||
        (Threshold2 < 0) || (Threshold2 > MaxThreshold) ||
        (Threshold1 > Threshold2))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int Index = IndexOfScannedObject(ScannedObjectID);
    ScannedObjectsList[Index].QualityThreshold1 = Threshold1;
    ScannedObjectsList[Index].QualityThreshold2 = Threshold2;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetSegmentImageSize(unsigned int ScannedObjectID, int Index, unsigned int Width, unsigned int Height)
{
    return MSGui->SetSegmentImageSize(ScannedObjectID, Index, Width, Height);
}

int GBMSGUI::SetSegmentImageSize(unsigned int ScannedObjectID, int Index, unsigned int Width, unsigned int Height)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Index < 0 || Index > 3)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int SegmentIndex;
    // in case of left slap, reverse index
    if (ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        SegmentIndex = 3 - Index;
    else if (ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
        SegmentIndex = 1 - Index;
    else
        SegmentIndex = Index;

    bool Result = true;

    // min size
    unsigned int MinImageSizeX, MinImageSizeY;
    if (Resolution1000Dpi)
        MinImageSizeX = MinImageSizeY = 400;
    else
        MinImageSizeX = MinImageSizeY = 200;
    if ((Width < MinImageSizeX) || (Height < MinImageSizeY))
    {
        // set anyway to min size, so error can be treated as a warning
        if (Width < MinImageSizeX)
            Width = MinImageSizeX;
        if (Height < MinImageSizeY)
            Height = MinImageSizeY;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        Result = false;
    }

    // check that size requested is not bigger than slap
    int i = IndexOfScannedObject(ScannedObjectID);
    if ((Width > ScannedObjectsList[i].ImageSizeX) || (Height > ScannedObjectsList[i].ImageSizeY))
    {
        // set anyway to max size, so error can be treated as a warning
        if (Width > ScannedObjectsList[i].ImageSizeX)
            Width = ScannedObjectsList[i].ImageSizeX;
        if (Height > ScannedObjectsList[i].ImageSizeY)
            Height = ScannedObjectsList[i].ImageSizeY;
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        Result = false;
    }

    SegmCfgData[SegmentIndex].ImageSizeX = (int)Width;
    SegmCfgData[SegmentIndex].ImageSizeY = (int)Height;

    if (!Result)
        return GBMSGUI_RET_FAILURE;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetSegmentQualityThresholds(unsigned int ScannedObjectID, int Index, int Threshold1, int Threshold2)
{
    return MSGui->SetSegmentQualityThresholds(ScannedObjectID, Index, Threshold1, Threshold2);
}

int GBMSGUI::SetSegmentQualityThresholds(unsigned int ScannedObjectID, int Index, int Threshold1, int Threshold2)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Index < 0 || Index > 3)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int SegmentIndex;
    // in case of left slap, reverse index
    if (ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        SegmentIndex = 3 - Index;
    else if (ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
        SegmentIndex = 1 - Index;
    else
        SegmentIndex = Index;

    int MaxThreshold;
    if (CfgData.QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
        MaxThreshold = 5;
    else
        MaxThreshold = 100;

    // 0 is permitted; both 0 means no thresholds
    if ((Threshold1 < 0) || (Threshold1 > MaxThreshold) ||
        (Threshold2 < 0) || (Threshold2 > MaxThreshold) ||
        (Threshold1 > Threshold2))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    SegmCfgData[SegmentIndex].QualityThreshold1 = Threshold1;
    SegmCfgData[SegmentIndex].QualityThreshold2 = Threshold2;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_SetSegmentUnavailabilityReason(unsigned int ScannedObjectID, int Index, int Reason)
{
    return MSGui->SetSegmentUnavailabilityReason(ScannedObjectID, Index, Reason);
}

int GBMSGUI::SetSegmentUnavailabilityReason(unsigned int ScannedObjectID, int Index, int Reason)
{
    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Index < 0 || Index > 3)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int SegmentIndex;
    // in case of left slap, reverse index
    if (ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        SegmentIndex = 3 - Index;
    else if (ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
        SegmentIndex = 1 - Index;
    else
        SegmentIndex = Index;

    if ((Reason != GBMSGUI_UR_AMPUTATED)
        && (Reason != GBMSGUI_UR_UNPRINTABLE))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    SegmCfgData[SegmentIndex].UnavailabilityReason = Reason;

    return GBMSGUI_RET_SUCCESS;
}

void __stdcall GBMSGUI_SetAfterStartCallback(GBMSGUI_AfterStartFuncPtr AfterStartFunc, void *AfterStartParam)
{
    MSGui->SetAfterStartCallback(AfterStartFunc, AfterStartParam);
}

void GBMSGUI::SetAfterStartCallback(GBMSGUI_AfterStartFuncPtr AfterStartFunc, void *AfterStartParam)
{
    AfterStartCallbackRef = AfterStartFunc;
    AfterStartCallbackParam = AfterStartParam;
}

int __stdcall GBMSGUI_Acquire(unsigned int ScannedObjectID, unsigned int Options,
            unsigned char *Image, unsigned long ImageBufferSize,
            int *ImageSizeX, int *ImageSizeY, int *Dpi)
{
    return MSGui->Acquire(ScannedObjectID, Options, Image, ImageBufferSize, ImageSizeX, ImageSizeY, Dpi);
}
int GBMSGUI::Acquire(unsigned int ScannedObjectID, unsigned int Options,
            unsigned char *Image, unsigned long ImageBufferSize,
            int *ImageSizeX, int *ImageSizeY, int *Dpi)
{
    if ((ImageSizeX == NULL) || (ImageSizeY == NULL) || (Dpi == NULL))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    *ImageSizeX = -1;
    *ImageSizeY = -1;

    if (Resolution1000Dpi)
        *Dpi = 1000;
    else
        *Dpi = 500;

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Image == NULL)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (OnAcquisition)
        return GBMSGUI_RET_BUSY_ACQUIRING;

    OnAcquisition = true;

    AcquisitionOptions = Options;

    // retrieve object in the list
    int Index = IndexOfScannedObject(ScannedObjectID);
    frmAcquisition->SetScannedObject(&ScannedObjectsList[Index]);

    // show acquisition form
    if (!frmAcquisition->isVisible())
    {
        frmAcquisition->setModal(true);
        if (CfgData.AcqWindowMaximized)
            frmAcquisition->showMaximized();
        else
            frmAcquisition->showNormal();
    }
    // call Acquire method of the form
    frmAcquisition->Acquire();
    // wait for result
    while(!frmAcquisition->Finished() || frmAcquisition->IsAcquiring())
    {
        QApplication::processEvents();
        // do not let application use 100% of CPU
#ifdef WIN32
        Sleep(1);
#else
        usleep(1000);
#endif
    }

    frmAcquisition->AcquireEnd();

    int Result = frmAcquisition->GetScanResult();
    if (Result == GBMSGUI_RET_SUCCESS)
    {
        // copy image into buffer
        frmAcquisition->GetAcquiredImageBuffer(Image, ImageSizeX, ImageSizeY);
        ScannedObjectsList[Index].Acquired = true;
    }

    if (AcquisitionMode == GBMSGUI_AM_SINGLE_ACQUISITION)
    {
        CfgData.AcqWindowMaximized = frmAcquisition->isMaximized();
        frmAcquisition->hide();
    }

    // after slap, clear segments settings
    if (IsSlap(ScannedObjectID))
        SetDefaultSegmentsConfiguration();

    OnAcquisition = false;
    return Result;
}

int __stdcall GBMSGUI_GetIAFISQuality(unsigned int ScannedObjectID, int QualityAlgorithm, int *Quality)
{
    return MSGui->GetIAFISQuality(ScannedObjectID, QualityAlgorithm, Quality);
}

int GBMSGUI::GetIAFISQuality(unsigned int ScannedObjectID, int QualityAlgorithm, int *Quality)
{
    if (Quality == NULL)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    *Quality = -1;

    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int Index = IndexOfScannedObject(ScannedObjectID);

    if (QualityAlgorithm == GBMSGUI_GB_ALGORITHM)
        *Quality = ScannedObjectsList[Index].GBQuality;
    else
        *Quality = ScannedObjectsList[Index].NFIQQuality;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_GetUnavailabilityReason(unsigned int ScannedObjectID, int *Reason)
{
    return MSGui->GetUnavailabilityReason(ScannedObjectID, Reason);
}

int GBMSGUI::GetUnavailabilityReason(unsigned int ScannedObjectID, int *Reason)
{
    if (Reason == NULL)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }
    *Reason = -1;

    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (!IsValidScannedObjectID(ScannedObjectID))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    int Index = IndexOfScannedObject(ScannedObjectID);

    *Reason = ScannedObjectsList[Index].UnavailabilityReason;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_GetWindowSizeAndPosition(int *Left, int *Top, int *Width, int *Height, bool *Maximized)
{
    return MSGui->GetWindowSizeAndPosition(Left, Top, Width, Height, Maximized);
}
int GBMSGUI::GetWindowSizeAndPosition(int *Left, int *Top, int *Width, int *Height, bool *Maximized)
{
    if ((Left == NULL) || (Top == NULL) || (Width == NULL) || (Height == NULL) || (Maximized == NULL))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    *Left = CfgData.AcqWindowLeft;
    *Top = CfgData.AcqWindowTop;
    *Width = CfgData.AcqWindowWidth;
    *Height = CfgData.AcqWindowHeight;
    *Maximized = CfgData.AcqWindowMaximized;

    return GBMSGUI_RET_SUCCESS;
}

int __stdcall GBMSGUI_StopSession()
{
    return MSGui->StopSession();
}
int GBMSGUI::StopSession()
{
    if (OnAcquisition)
        return GBMSGUI_RET_BUSY_ACQUIRING;

    // save window size and position
    CfgData.AcqWindowLeft = frmAcquisition->x();
    CfgData.AcqWindowTop = frmAcquisition->y();
    CfgData.AcqWindowWidth = frmAcquisition->geometry().width();
    CfgData.AcqWindowHeight = frmAcquisition->geometry().height();

	if (CheckMask(frmAcquisition->windowState(), Qt::WindowMaximized))
        CfgData.AcqWindowMaximized = true;
    else
        CfgData.AcqWindowMaximized = false;

    frmAcquisition->SetForceClose();
    frmAcquisition->close();

    for (int i=0; i<SequenceCheckData.count(); i++)
        free_minutiae(SequenceCheckData[i].Minutiae);
    SequenceCheckData.clear();

    delete frmAcquisition;
    frmAcquisition = NULL;

    if (LCDPresent)
		GBMSAPI_VUI_LCD_SetLogoScreen();

    OnSession = false;
    return GBMSGUI_RET_SUCCESS;
}


int __stdcall GBMSGUI_GetSegmentationResult(int Index, int QualityAlgorithmID,
            bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
            int *UnavailabilityReason, int *FingerprintQuality,
            unsigned char *ImageBuffer, int ImageBufferSize, int *ImageSizeX, int *ImageSizeY, int *Dpi)
{
    return MSGui->GetSegmentationResult(Index, QualityAlgorithmID,
                            SegmentAvailable, Left, Top, Right, Bottom,
                            UnavailabilityReason, FingerprintQuality,
                            ImageBuffer, ImageBufferSize, ImageSizeX, ImageSizeY, Dpi);
}

int GBMSGUI::GetSegmentationResult(int Index, int QualityAlgorithmID,
                                    bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
                                    int *UnavailabilityReason, int *FingerprintQuality,
                                    unsigned char *ImageBuffer, int ImageBufferSize, int *ImageSizeX, int *ImageSizeY, int *Dpi)
{
    if ((SegmentAvailable == NULL) || (Left == NULL) || (Top == NULL) || (Right == NULL) ||
            (Bottom == NULL) || (FingerprintQuality == NULL) || (UnavailabilityReason == NULL) ||
            (ImageSizeX == NULL) || (ImageSizeY == NULL) || (Dpi == NULL))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    *SegmentAvailable = false;
    *Left = -1;
    *Top = -1;
    *Right = -1;
    *Bottom = -1;
    *FingerprintQuality = -1;
    *UnavailabilityReason = -1;
    *ImageSizeX = -1;
    *ImageSizeY = -1;
    *Dpi = -1;

    if (!OnSession)
    {
        SetError(QCoreApplication::tr("Session not open"), GBMSGUI_ERR_SESSION_NOT_OPEN);
        return GBMSGUI_RET_FAILURE;
    }

    if (Index < 0 || Index > 3)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if ((QualityAlgorithmID != GBMSGUI_NFIQ_ALGORITHM)
        && (QualityAlgorithmID != GBMSGUI_GB_ALGORITHM))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if (Resolution1000Dpi)
        *Dpi = 1000;
    else
        *Dpi = 500;

    return frmAcquisition->GetSegmentationResult(Index, QualityAlgorithmID,
        SegmentAvailable, Left, Top, Right, Bottom,
        UnavailabilityReason, FingerprintQuality,
        ImageBuffer, ImageSizeX, ImageSizeY);
}

void GBMSGUI::InitScannedObjectsList()
{
    //int ListCount = 32;

    //ScannedObjectsList.resize(ListCount);

    int Index = 0;
    InitScannedObject(GBMSAPI_SO_FLAT_RIGHT_THUMB, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_RIGHT_INDEX, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_RIGHT_MIDDLE, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_RIGHT_RING, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_RIGHT_LITTLE, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_LEFT_THUMB, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_LEFT_INDEX, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_LEFT_MIDDLE, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_LEFT_RING, Index++);
    InitScannedObject(GBMSAPI_SO_FLAT_LEFT_LITTLE, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_RIGHT_THUMB, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_RIGHT_INDEX, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_RIGHT_MIDDLE, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_RIGHT_RING, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_RIGHT_LITTLE, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_LEFT_THUMB, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_LEFT_INDEX, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_LEFT_MIDDLE, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_LEFT_RING, Index++);
    InitScannedObject(GBMSAPI_SO_ROLL_LEFT_LITTLE, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_2_THUMBS, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_4_LEFT, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_4_RIGHT, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_2_LEFT, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_2_RIGHT, Index++);
    InitScannedObject(GBMSAPI_SO_SLAP_2_INDEXES, Index++);
    InitScannedObject(GBMSAPI_SO_UPPER_HALF_PALM_LEFT, Index++);
    InitScannedObject(GBMSAPI_SO_UPPER_HALF_PALM_RIGHT, Index++);
    InitScannedObject(GBMSAPI_SO_LOWER_HALF_PALM_LEFT, Index++);
    InitScannedObject(GBMSAPI_SO_LOWER_HALF_PALM_RIGHT, Index++);
    InitScannedObject(GBMSAPI_SO_WRITER_PALM_LEFT, Index++);
    InitScannedObject(GBMSAPI_SO_WRITER_PALM_RIGHT, Index++);
}

void GBMSGUI::InitScannedObject(unsigned int ObjID, int Index)
{
    unsigned int ObjType = GBMSAPI_GetTypeFromObject(ObjID);
    ScannedObjectsList[Index].ScannedObjectID = ObjID;

    // set default values according to ObjID

    // image size: use values defined in ANSI/NIST ITL 1-2007
    // NOTE: size is initialized at 500 dpi; after, in InitScannedObjectsList (called in StartSession),
    // is adapted to current scanner resolution
    if (IsSlap(ObjID))
    {
        // for slap 2 (DS40), set 800x800
        if ((ObjType == GBMSAPI_SBT_FLAT_SLAP_2) ||
            (ObjType == GBMSAPI_SBT_FLAT_INDEXES_2))
        {
            ScannedObjectsList[Index].ImageSizeX = 800;
            ScannedObjectsList[Index].ImageSizeY = 800;
        }
        else
        {
            ScannedObjectsList[Index].ImageSizeX = 1600;
            ScannedObjectsList[Index].ImageSizeY = 1500;
        }
    }
    else if (IsRolled(ObjID))
    {
        ScannedObjectsList[Index].ImageSizeX = 800;
        ScannedObjectsList[Index].ImageSizeY = 750;
    }
    else if ((ObjType == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM) ||
        (ObjType == GBMSAPI_SBT_FLAT_UPPER_HALF_PALM))
    {
        ScannedObjectsList[Index].ImageSizeX = 2500;
        ScannedObjectsList[Index].ImageSizeY = 2500;
    }
    else if (ObjType == GBMSAPI_SBT_FLAT_WRITER_PALM)
    {
        ScannedObjectsList[Index].ImageSizeX = 900;
        ScannedObjectsList[Index].ImageSizeY = 2500;
    }
    else if ((ObjID == GBMSAPI_SO_FLAT_LEFT_THUMB) ||
        (ObjID == GBMSAPI_SO_FLAT_RIGHT_THUMB))
    {
        ScannedObjectsList[Index].ImageSizeX = 500;
        ScannedObjectsList[Index].ImageSizeY = 1000;
    }
    else // remaining flat single fingers
    {
        ScannedObjectsList[Index].ImageSizeX = 500;
        ScannedObjectsList[Index].ImageSizeY = 500;
    }

    // adjust default image sizes if resolution is 1000 Dpi
    if (Resolution1000Dpi)
    {
        ScannedObjectsList[Index].ImageSizeX *= 2;
        ScannedObjectsList[Index].ImageSizeY *= 2;
    }

    // anyway, limit to the max image size
    unsigned int MaxImageSizeX, MaxImageSizeY;
    GBMSAPI_GetMaxImageSize(&MaxImageSizeX, &MaxImageSizeY);
    if (ScannedObjectsList[Index].ImageSizeX > MaxImageSizeX)
        ScannedObjectsList[Index].ImageSizeX = MaxImageSizeX;
    if (ScannedObjectsList[Index].ImageSizeY > MaxImageSizeY)
        ScannedObjectsList[Index].ImageSizeY = MaxImageSizeY;

    // Default contrast thresholds
    ScannedObjectsList[Index].ContrastThreshold1 = 120;
    ScannedObjectsList[Index].ContrastThreshold2 = 150;

    // Default Quality thresholds for NFIQ
    ScannedObjectsList[Index].QualityThreshold1 = 2;
    ScannedObjectsList[Index].QualityThreshold2 = 3;

    ScannedObjectsList[Index].Acquired = false;
    ScannedObjectsList[Index].NFIQQuality = 254;  // not calculated
    ScannedObjectsList[Index].GBQuality = 0;
    ScannedObjectsList[Index].RolledArtefactSize = 0;
    ScannedObjectsList[Index].Completeness = 0;
    ScannedObjectsList[Index].UnavailabilityReason = -1;

}

int GBMSGUI::IndexOfScannedObject(unsigned int ObjID)
{
    int i;

    //for (i=0; i<ScannedObjectsList.count(); i++)
    for (i=0; i<SCANNED_OBJECTS_NUMBER; i++)
    {
        if (ScannedObjectsList[i].ScannedObjectID == ObjID)
            return i;
    }

    return -1;
}

bool __stdcall GBMSGUI_IsSlap(unsigned int ScanObjID)
{
    return GBMSGUI::IsSlap(ScanObjID);
}

bool GBMSGUI::IsSlap(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if ((ObjectType == GBMSAPI_SBT_FLAT_SLAP_4) ||
        (ObjectType == GBMSAPI_SBT_FLAT_THUMBS_2) ||
        (ObjectType == GBMSAPI_SBT_FLAT_SLAP_2) ||
        (ObjectType == GBMSAPI_SBT_FLAT_INDEXES_2))
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsRolled(unsigned int ScanObjID)
{
    return GBMSGUI::IsRolled(ScanObjID);
}

bool GBMSGUI::IsRolled(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if (ObjectType == GBMSAPI_SBT_ROLL_SINGLE_FINGER)
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsSingleFinger(unsigned int ScanObjID)
{
    return GBMSGUI::IsSingleFinger(ScanObjID);
}

bool GBMSGUI::IsSingleFinger(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if ((ObjectType == GBMSAPI_SBT_ROLL_SINGLE_FINGER) ||
        (ObjectType == GBMSAPI_SBT_FLAT_SINGLE_FINGER))
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsFlatSingleFinger(unsigned int ScanObjID)
{
    return GBMSGUI::IsFlatSingleFinger(ScanObjID);
}

bool GBMSGUI::IsFlatSingleFinger(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if (ObjectType == GBMSAPI_SBT_FLAT_SINGLE_FINGER)
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsPalm(unsigned int ScanObjID)
{
    return GBMSGUI::IsPalm(ScanObjID);
}

bool GBMSGUI::IsPalm(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if ((ObjectType == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM) ||
        (ObjectType == GBMSAPI_SBT_FLAT_UPPER_HALF_PALM) ||
        (ObjectType == GBMSAPI_SBT_FLAT_WRITER_PALM))
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsLowerPalm(unsigned int ScanObjID)
{
    return GBMSGUI::IsLowerPalm(ScanObjID);
}

bool GBMSGUI::IsLowerPalm(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if (ObjectType == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM)
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsUpperPalm(unsigned int ScanObjID)
{
    return GBMSGUI::IsUpperPalm(ScanObjID);
}

bool GBMSGUI::IsUpperPalm(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if (ObjectType == GBMSAPI_SBT_FLAT_UPPER_HALF_PALM)
        return true;

    return false;
}

bool __stdcall GBMSGUI_IsWritersPalm(unsigned int ScanObjID)
{
    return GBMSGUI::IsWritersPalm(ScanObjID);
}

bool GBMSGUI::IsWritersPalm(unsigned int ScanObjID)
{
    unsigned int ObjectType = GBMSAPI_GetTypeFromObject(ScanObjID);

    if (ObjectType == GBMSAPI_SBT_FLAT_WRITER_PALM)
        return true;

    return false;
}

void GBMSGUI::SetError(QString ErrMsg, int ErrCode)
{
    ErrorMessage = ErrMsg;
    ErrorCode = ErrCode;
}

static wchar_t *MSAPIErrorMessage = NULL;
const wchar_t * __stdcall GBMSGUI_GetMSAPIErrorMessage(wchar_t *FuncName, int ErrCode)
{
    if (MSAPIErrorMessage)
        free (MSAPIErrorMessage);
    QString Msg = MSGui->GetMSAPIErrorMessage(QString::fromWCharArray(FuncName), ErrCode);
    MSAPIErrorMessage = (wchar_t *)malloc(sizeof(wchar_t)*(Msg.length() + 1));
    Msg.toWCharArray(MSAPIErrorMessage);
    MSAPIErrorMessage[Msg.length()] = 0;

    return MSAPIErrorMessage;
}
// set error string for GBMSAPI error code
QString GBMSGUI::GetMSAPIErrorMessage(QString FuncName, int ErrCode)
{
    QString ErrMsg;

    switch (ErrCode)
    {
        case GBMSAPI_ERROR_CODE_USB_DRIVER:
            ErrMsg = QCoreApplication::tr("USB Driver error");
            unsigned long USBErr;
            GBMSAPI_GetUSBErrorCode(&USBErr);
            ErrMsg = ErrMsg + QString(" (%1)").arg(USBErr, 8, 16);
            break;
        case GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND:
            ErrMsg = QCoreApplication::tr("Device not found");
            break;
        case GBMSAPI_ERROR_CODE_USB_THREAD:
            ErrMsg = QCoreApplication::tr("USB thread error");
            break;
        case GBMSAPI_ERROR_CODE_PARAMETER:
            ErrMsg = QCoreApplication::tr("Parameter error");
            break;
        case GBMSAPI_ERROR_CODE_SCANNER_NOT_CONFIGURED:
            ErrMsg = QCoreApplication::tr("Scanner not configured");
            break;
        case GBMSAPI_ERROR_CODE_DEVICE_NOT_RESPONDING:
            ErrMsg = QCoreApplication::tr("Device not responding");
            break;
        case GBMSAPI_ERROR_CODE_SCANNER_COMMUNICATION:
            ErrMsg = QCoreApplication::tr("Scanner communication error");
            break;
        case GBMSAPI_ERROR_CODE_UNAVAILABLE_OPTION:
            ErrMsg = QCoreApplication::tr("Unavailable option");
            break;
        case GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED:
            ErrMsg = QCoreApplication::tr("Specific DLL not loaded");
            break;
        case GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
            ErrMsg = QCoreApplication::tr("Method not supported");
            break;
        case GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
            ErrMsg = QCoreApplication::tr("Object type not supported");
            break;
        case GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED:
            ErrMsg = QCoreApplication::tr("Scan area not supported");
            break;
        case GBMSAPI_ERROR_CODE_ACQUISITION_THREAD:
            ErrMsg = QCoreApplication::tr("Acquisition thread error");
            break;
        case GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
            ErrMsg = QCoreApplication::tr("Acquisition already started");
            break;
        case GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
            ErrMsg = QCoreApplication::tr("Feature not supported");
            break;
        case GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
            ErrMsg = QCoreApplication::tr("Current device not set");
            break;
        case GBMSAPI_ERROR_CODE_MEMORY_ALLOCATION:
            ErrMsg = QCoreApplication::tr("Memory allocation error");
            break;
        case GBMSAPI_ERROR_CODE_GENERIC:
            ErrMsg = QCoreApplication::tr("Generic error");
            break;
        case GBMSAPI_ERROR_CODE_INTERNAL:
            ErrMsg = QCoreApplication::tr("Internal error");
            break;
        default:
            ErrMsg = QCoreApplication::tr("Unknown error") + " (" + ErrCode + ")";
            break;
    }

    ErrMsg = ErrMsg + " (" + FuncName + ")";

    return ErrMsg;
}

static wchar_t *ErrorMessage = NULL;
const wchar_t * __stdcall GBMSGUI_GetErrorMessage()
{
    if (ErrorMessage)
        free (ErrorMessage);

    QString Msg = MSGui->GetErrorMessage();
    ErrorMessage = (wchar_t *)malloc(sizeof(wchar_t)*(Msg.length() + 1));
    Msg.toWCharArray(ErrorMessage);
    ErrorMessage[Msg.length()] = 0;

    return ErrorMessage;
}
QString GBMSGUI::GetErrorMessage()
{
    return ErrorMessage;
}

int __stdcall GBMSGUI_GetErrorCode()
{
    return MSGui->GetErrorCode();
}
int GBMSGUI::GetErrorCode()
{
    return ErrorCode;
}

bool GBMSGUI::IsValidScannedObjectID(unsigned int ScannedObjectID)
{
    bool ret;

    switch (ScannedObjectID)
    {
        case GBMSAPI_SO_FLAT_LEFT_INDEX:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_LEFT_LITTLE:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_LEFT_MIDDLE:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_LEFT_RING:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_LEFT_THUMB:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_RIGHT_INDEX:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_RIGHT_LITTLE:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_RIGHT_MIDDLE:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_RIGHT_RING:
            ret = true;
            break;
        case GBMSAPI_SO_FLAT_RIGHT_THUMB:
            ret = true;
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_LEFT:
            ret = true;
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_RIGHT:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_LEFT_INDEX:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_LEFT_LITTLE:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_LEFT_MIDDLE:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_LEFT_RING:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_LEFT_THUMB:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_RIGHT_INDEX:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_RIGHT_LITTLE:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_RIGHT_MIDDLE:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_RIGHT_RING:
            ret = true;
            break;
        case GBMSAPI_SO_ROLL_RIGHT_THUMB:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_2_THUMBS:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_4_LEFT:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_4_RIGHT:
            ret = true;
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_LEFT:
            ret = true;
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_RIGHT:
            ret = true;
            break;
        case GBMSAPI_SO_WRITER_PALM_LEFT:
            ret = true;
            break;
        case GBMSAPI_SO_WRITER_PALM_RIGHT:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_2_LEFT:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_2_RIGHT:
            ret = true;
            break;
        case GBMSAPI_SO_SLAP_2_INDEXES:
            ret = true;
            break;
        default:
            ret = false;
            break;
    }

    return ret;
}

bool GBMSGUI::IsResolution1000Dpi()
{
    return Resolution1000Dpi;
}

int GBMSGUI::GetAcquisitionMode()
{
    return AcquisitionMode;
}

unsigned int GBMSGUI::GetSessionOptions()
{
    return SessionOptions;
}

unsigned int GBMSGUI::GetAcquisitionOptions()
{
    return AcquisitionOptions;
}

unsigned int GBMSGUI::GetSupportedScanOptions()
{
    return SupportedScanOptions;
}

bool GBMSGUI::IsBlockAutocaptureSupported()
{
    return BlockAutocaptureSupported;
}

bool GBMSGUI::IsPalmCompletenessSupported()
{
    return PalmCompletenessSupported;
}

ConfigurationData * GBMSGUI::GetCfgData()
{
    return &CfgData;
}

bool GBMSGUI::IsLCDPresent()
{
    return LCDPresent;
}

bool GBMSGUI::IsLedPresent()
{
    return LedPresent;
}

bool GBMSGUI::IsSoundPresent()
{
    return SoundPresent;
}

bool GBMSGUI::IsPedalPresent()
{
    return PedalPresent;
}

bool GBMSGUI::IsImageRotationSupported()
{
	return ImageRotationSupported;
}

bool GBMSGUI::IsLCDMidLimitSupported()
{
	return LCDMidLimitSupported;
}

SegmentsConfigurationData * GBMSGUI::GetSegmCfgData(int index)
{
    return &SegmCfgData[index];
}

GBMSGUI_AfterStartFuncPtr GBMSGUI::GetAfterStartCallback()
{
    return AfterStartCallbackRef;
}

void *GBMSGUI::GetAfterStartCallbackParam()
{
    return AfterStartCallbackParam;
}

void GBMSGUI::AddSequenceCheckItem(int ObjectID, MINUTIAE *Minutiae)
{
    int i;

    // if item already present, delete it first
    for (i=0; i<SequenceCheckData.count(); i++)
        if (SequenceCheckData[i].ObjectID == ObjectID)
        {
            SequenceCheckData.removeAt(i);
            break;
        }

    SequenceCheckItem *Item = new SequenceCheckItem;
    Item->ObjectID = ObjectID;
    Item->Minutiae = Minutiae;

    SequenceCheckData.append(*Item);
}

int GBMSGUI::FindSequenceCheckSameFinger(int ObjectID)
{
    int i;
    int MatchingObjectID;

    for (i=0; i<SequenceCheckData.count(); i++)
    {
        MatchingObjectID = SequenceCheckData[i].ObjectID;

        if (((ObjectID == GBMSAPI_SO_ROLL_LEFT_THUMB) || (ObjectID == GBMSAPI_SO_FLAT_LEFT_THUMB))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_LEFT_THUMB))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_LEFT_INDEX) || (ObjectID == GBMSAPI_SO_FLAT_LEFT_INDEX))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_LEFT_INDEX))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_LEFT_MIDDLE) || (ObjectID == GBMSAPI_SO_FLAT_LEFT_MIDDLE))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_LEFT_MIDDLE))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_LEFT_RING) || (ObjectID == GBMSAPI_SO_FLAT_LEFT_RING))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_LEFT_RING))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_LEFT_LITTLE) || (ObjectID == GBMSAPI_SO_FLAT_LEFT_LITTLE))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_LEFT_LITTLE))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_RIGHT_THUMB) || (ObjectID == GBMSAPI_SO_FLAT_RIGHT_THUMB))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_RIGHT_THUMB))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_RIGHT_INDEX) || (ObjectID == GBMSAPI_SO_FLAT_RIGHT_INDEX))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_RIGHT_INDEX))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_RIGHT_MIDDLE) || (ObjectID == GBMSAPI_SO_FLAT_RIGHT_MIDDLE))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_RIGHT_MIDDLE))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_RIGHT_RING) || (ObjectID == GBMSAPI_SO_FLAT_RIGHT_RING))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_RIGHT_RING))
            return i;
        if (((ObjectID == GBMSAPI_SO_ROLL_RIGHT_LITTLE) || (ObjectID == GBMSAPI_SO_FLAT_RIGHT_LITTLE))
                && (MatchingObjectID == GBMSAPI_SO_FLAT_RIGHT_LITTLE))
            return i;
    }

    return -1;
}

MINUTIAE *GBMSGUI::GetItemMinutiae(int Index)
{
    return SequenceCheckData[Index].Minutiae;
}

void GBMSGUI::Beep(int Frequency, int Duration)
{
#ifdef WIN32
	DSBeep(Frequency, Duration);
#else
	QApplication::beep();   // NON FUNZIONA!!!
/*
    beep_parms_t params;
    memset(&params, 0, sizeof(beep_parms_t));
    params.freq = Frequency;
    params.length = Duration;
    play_beep(params);
*/
#endif
}

void GBMSGUI::SetDefaultSegmentsConfiguration()
{
    for (int i = 0; i < 4; i++)
    {
        SegmCfgData[i].ImageSizeX = 500;
        SegmCfgData[i].ImageSizeY = 500;
        if (Resolution1000Dpi)
        {
            SegmCfgData[i].ImageSizeX *= 2;
            SegmCfgData[i].ImageSizeY *= 2;
        }
        SegmCfgData[i].UnavailabilityReason = -1;
        SegmCfgData[i].QualityThreshold1 = 2;
        SegmCfgData[i].QualityThreshold2 = 3;
    }
}

int __stdcall GBMSGUI_SelectScannedObject(int *ScannedObjectID)
{        
    return MSGui->SelectScannedObject(ScannedObjectID);
}
int GBMSGUI::SelectScannedObject(int *ScannedObjectID)
{
    if (ScannedObjectID == NULL)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

	SelectObjectForm frmSelectObject;

	frmSelectObject.exec();

	*ScannedObjectID = frmSelectObject.GetObjectID();

    return GBMSGUI_RET_SUCCESS;
}


int __stdcall GBMSGUI_SelectFingerContactEvaluationMode(int Mode)
{
	return MSGui->SelectFingerContactEvaluationMode(Mode);
}

int GBMSGUI::SelectFingerContactEvaluationMode(int Mode)
{
    if ((Mode != GBMSGUI_FINGER_CONTACT_EVAL_MODE_CONTRAST)
        && (Mode != GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING))
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

    if ((Mode == GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING) &&
        !DryFingerImageEnhancementSupported)
    {
        SetError(QCoreApplication::tr("Invalid parameter"), GBMSGUI_ERR_INVALID_PARAMETER);
        return GBMSGUI_RET_FAILURE;
    }

	// 1.5.6.0
    if (Mode == GBMSGUI_FINGER_CONTACT_EVAL_MODE_CONTRAST)
    {
        // if Dry/wet warning is not shown, can't block autocapture...
        CfgData.BlockAutoCaptureMask &= ~GBMSAPI_DM_DRY_FINGER;
        CfgData.BlockAutoCaptureMask &= ~GBMSAPI_DM_WET_FINGER;
    }
    else
    {
        CfgData.BlockAutoCaptureMask |= GBMSAPI_DM_DRY_FINGER;
        CfgData.BlockAutoCaptureMask |= GBMSAPI_DM_WET_FINGER;
    }

    FingerContactEvalMode = Mode;
    return GBMSGUI_RET_SUCCESS;
}

bool GBMSGUI::IsDryFingerImageEnhancementSupported()
{
	return DryFingerImageEnhancementSupported;
}

int GBMSGUI::GetFingerContactEvalMode()
{
	return FingerContactEvalMode;
}


