#ifndef GBMSGUI_CLASS_H
#define GBMSGUI_CLASS_H

#include "GBMSGUI_Qt.h"
#include "GBMSAPI_Library.h"
#include "acquisitionform.h"
#include "lfs_dll.h"

// forward declarations
class AcquisitionForm;

struct ScannedObject
{
    unsigned int ScannedObjectID;
    unsigned int ImageSizeX;     // requested image size
    unsigned int ImageSizeY;
    int ContrastThreshold1;
    int ContrastThreshold2;
    int QualityThreshold1;
    int QualityThreshold2;
    // output values (filled after acquisition)
    bool Acquired;
    int NFIQQuality;
    int GBQuality;
    unsigned int RolledArtefactSize;
    unsigned int Completeness;    // for lower palm
    int UnavailabilityReason;
};

struct ConfigurationData
{
    int ArtefactsThreshold1;
    int ArtefactsThreshold2;
    int QualityAlgorithm;
    int PatternValidityThreshold;
    int PatternCompletenessThreshold;
    int LowerPalmCompletenessThreshold1;
    int LowerPalmCompletenessThreshold2;
    unsigned int BlockAutoCaptureMask;
    bool BlockAutoCaptureContrast;
    int InclinationThreshold;
    int AcqWindowLeft;
    int AcqWindowTop;
    int AcqWindowWidth;
    int AcqWindowHeight;
    bool AcqWindowMaximized;
    //CultureInfo Culture;
    uint IgnoreDiagnosticMask;
    bool FlatFingerOnRollArea;
};

struct SegmentsConfigurationData
{
    int ImageSizeX;
    int ImageSizeY;
    int UnavailabilityReason;
    int QualityThreshold1;
    int QualityThreshold2;
};

// data for sequence check
struct SequenceCheckItem
{
    int ObjectID;
    MINUTIAE *Minutiae;
};

#define SCANNED_OBJECTS_NUMBER 32

//class GBMSGUISHARED_EXPORT GBMSGUI {
class GBMSGUI {
public:
    GBMSGUI();
    ~GBMSGUI();
    int SetLanguage(const char *LangName);
    int SetWindowSizeAndPosition(int Left, int Top, int Width, int Height, bool Maximized);
    int SetArtefactsThresholds(int Threshold1, int Threshold2);
    int SetQualityAlgorithm(int AlgorithmID);
    int SetPatternValidityThreshold(int Threshold);
    int SetPatternCompletenessThreshold(int Threshold);
    int SetLowerPalmCompletenessThresholds(int Threshold1, int Threshold2);
    int SetInclinationThreshold(int Threshold);
    int SetBlockAutoCaptureMask(unsigned int Mask);
    int SetBlockAutoCaptureContrast(bool Flag);
    int SetIgnoreDiagnosticMask(unsigned int IgnoreDiagnosticMask);
    int SetFlatFingerOnRollArea(bool FlatFingerOnRollArea);
    int StartSession(int AcquisitionMode, wchar_t *PersonID, unsigned int Options);
    int SetImageSize(unsigned int ScannedObjectID, unsigned int Width, unsigned int Height);
    int SetContrastThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2);
    int SetQualityThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2);
    int SetSegmentImageSize(unsigned int ScannedObjectID, int Index, unsigned int Width, unsigned int Height);
    int SetSegmentQualityThresholds(unsigned int ScannedObjectID, int Index, int Threshold1, int Threshold2);
    int SetSegmentUnavailabilityReason(unsigned int ScannedObjectID, int Index, int Reason);
    void SetAfterStartCallback(GBMSGUI_AfterStartFuncPtr AfterStartFunc, void *AfterStartParam);
    int Acquire(unsigned int ScannedObjectID, unsigned int Options,
                unsigned char *Image, unsigned long ImageBufferSize,
                int *ImageSizeX, int *ImageSizeY, int *Dpi);
    int GetIAFISQuality(unsigned int ScannedObjectID, int QualityAlgorithm, int *Quality);
    int GetUnavailabilityReason(unsigned int ScannedObjectID, int *Reason);
    int GetWindowSizeAndPosition(int *Left, int *Top, int *Width, int *Height, bool *Maximized);
    int StopSession();
    int GetSegmentationResult(int Index, int QualityAlgorithmID,
                            bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
                            int *UnavailabilityReason, int *FingerprintQuality,
                            unsigned char *ImageBuffer, int ImageBufferSize, int *ImageSizeX, int *ImageSizeY, int *Dpi);
    QString GetErrorMessage();
    int GetErrorCode();
    QString GetMSAPIErrorMessage(QString FuncName, int ErrCode);
	int SelectScannedObject(int *ScannedObjectID);
	int SelectFingerContactEvaluationMode(int Mode);

    void SetError(QString ErrMsg, int ErrCode);
    bool IsResolution1000Dpi();
    int GetAcquisitionMode();
    unsigned int GetSessionOptions();
    unsigned int GetAcquisitionOptions();
    unsigned int GetSupportedScanOptions();
    ConfigurationData *GetCfgData();
    bool IsBlockAutocaptureSupported();
    bool IsPalmCompletenessSupported();
    bool IsLCDPresent();
    bool IsLedPresent();
    bool IsSoundPresent();
    bool IsPedalPresent();
	bool IsImageRotationSupported();
	bool IsLCDMidLimitSupported();
    SegmentsConfigurationData * GetSegmCfgData(int index);
    GBMSGUI_AfterStartFuncPtr GetAfterStartCallback();
    void *GetAfterStartCallbackParam();
    void AddSequenceCheckItem(int ObjectID, MINUTIAE *Minutiae);
    int FindSequenceCheckSameFinger(int ObjectID);
    MINUTIAE *GetItemMinutiae(int Index);
    void Beep(int Frequency, int Duration);
	bool IsDryFingerImageEnhancementSupported();
	int GetFingerContactEvalMode();

    static bool IsSlap(unsigned int ScanObjID);
    static bool IsRolled(unsigned int ScanObjID);
    static bool IsSingleFinger(unsigned int ScanObjID);
    static bool IsFlatSingleFinger(unsigned int ScanObjID);
    static bool IsPalm(unsigned int ScanObjID);
    static bool IsLowerPalm(unsigned int ScanObjID);
    static bool IsUpperPalm(unsigned int ScanObjID);
    static bool IsWritersPalm(unsigned int ScanObjID);
    static bool IsValidScannedObjectID(unsigned int ScannedObjectID);

private:
    QApplication *mApp;
    AcquisitionForm *frmAcquisition;
    QWidget *Parent;    // application window
    QTranslator *translator;
    QString ErrorMessage;  // last error message
    int ErrorCode;         // last error code
    bool OnSession;
    bool OnAcquisition;
    ScannedObject ScannedObjectsList[SCANNED_OBJECTS_NUMBER];
    unsigned int SessionOptions;
    int AcquisitionMode;
    unsigned int AcquisitionOptions;
    bool Resolution1000Dpi;
    bool BlockAutocaptureSupported;
    bool ImageRotationSupported;
    bool PalmCompletenessSupported;
    unsigned int SupportedScanOptions;       // options supported by current scanner
	bool LCDMidLimitSupported;			// LCD feature
    // External device equipment
    bool LCDPresent;
    bool LedPresent;
    bool SoundPresent;
    bool PedalPresent;
    ConfigurationData CfgData;
    SegmentsConfigurationData SegmCfgData[4];
    GBMSGUI_AfterStartFuncPtr AfterStartCallbackRef;
    void *AfterStartCallbackParam;
    QList<SequenceCheckItem> SequenceCheckData;
	bool DryFingerImageEnhancementSupported;
	int FingerContactEvalMode;

    void InitScannedObjectsList();
    void InitScannedObject(unsigned int ObjID, int Index);
    int IndexOfScannedObject(unsigned int ObjID);
    void SetDefaultSegmentsConfiguration();
};


#endif // GBMSGUI_CLASS_H
