#include <stdafx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//#include "windows.h"
#include "GBUtil.h"
#include "ImageProcessing.h"

#pragma warning( disable : 4996 4793)

using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;

#define BM (unsigned short)(0x4d42) // ANSI code for "BM"
#define GB_BI_RGB        0L

// bitmap data structures: are the same as windows
// for them byte alignment is specified in order to read directly from file
#pragma pack(1)
struct GB_BMPFILEHEADER { 
  unsigned short bfType; 
  unsigned int   bfSize; 
  unsigned short bfReserved1; 
  unsigned short bfReserved2; 
  unsigned int   bfOffBits; 
};

typedef struct GB_BMPFILEHEADER GB_BmpFileHeader;

#pragma pack(1)
struct GB_BMPINFOHEADER{
  unsigned int   biSize; 
  unsigned int   biWidth; 
  unsigned int   biHeight; 
  unsigned short biPlanes; 
  unsigned short biBitCount; 
  unsigned int   biCompression; 
  unsigned int   biSizeImage; 
  unsigned int   biXPelsPerMeter; 
  unsigned int   biYPelsPerMeter; 
  unsigned int   biClrUsed; 
  unsigned int   biClrImportant; 
} ;

typedef struct GB_BMPINFOHEADER GB_BmpInfoHeader;

#pragma pack(1)
struct GB_RGB {
  unsigned char    rgbBlue; 
  unsigned char    rgbGreen; 
  unsigned char    rgbRed; 
  unsigned char    rgbReserved; 
};
typedef struct GB_RGB GB_rgb;


void	FreeImage(unsigned char *pImage)
{
	try {
		delete [] pImage;
	}
	catch(System::Exception ^ex)
	{
		MessageBox::Show(ex->Message);
	}
}

int		LoadImageFromObject(unsigned char ObjID,unsigned char **pFrame,unsigned int *SizeX,unsigned int *SizeY)
{
	switch (ObjID)
	{
	case LEFT_SLAP:
		return ReadSingleBMP(LEFT_SLAP_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_SLAP:
		return ReadSingleBMP(RIGHT_SLAP_F_NAME,pFrame,SizeX,SizeY);
		break;
	case TWO_THUMBS:
		return ReadSingleBMP(TWO_THUMBS_F_NAME,pFrame,SizeX,SizeY);
		break;
	case LEFT_THUMB:
		return ReadSingleBMP(LEFT_THUMB_F_NAME,pFrame,SizeX,SizeY);
		break;
	case LEFT_INDEX:
		return ReadSingleBMP(LEFT_INDEX_F_NAME,pFrame,SizeX,SizeY);
		break;
	case LEFT_MIDDLE:
		return ReadSingleBMP(LEFT_MIDDLE_F_NAME,pFrame,SizeX,SizeY);
		break;
	case LEFT_RING:
		return ReadSingleBMP(LEFT_RING_F_NAME,pFrame,SizeX,SizeY);
		break;
	case LEFT_LITTLE:
		return ReadSingleBMP(LEFT_LITTLE_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_THUMB:
		return ReadSingleBMP(RIGHT_THUMB_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_INDEX:
		return ReadSingleBMP(RIGHT_INDEX_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_MIDDLE:
		return ReadSingleBMP(RIGHT_MIDDLE_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_RING:
		return ReadSingleBMP(RIGHT_RING_F_NAME,pFrame,SizeX,SizeY);
		break;
	case RIGHT_LITTLE:
		return ReadSingleBMP(RIGHT_LITTLE_F_NAME,pFrame,SizeX,SizeY);
		break;
	default:
		return GB_EXAMPLE_PARAM_ERROR;
	}
	return GB_EXAMPLE_PARAM_ERROR;
}



int ReadSingleBMP(char *FName,unsigned char **pFrame,unsigned int *SizeX,unsigned int *SizeY){
	FILE *fp;
	GB_BmpFileHeader FHead;
	GB_BmpInfoHeader FInfo;
	GB_rgb *Palette;
	unsigned int ElemToRead,i,StartFrame,StartAppoggio;
	unsigned char *appoggio,*Frame;	

	fp = fopen(FName,"rb");
	if (fp == NULL)
	{
		return GB_EXAMPLE_SYS_ERROR;
	}	

	// read file header
	fread(&FHead,sizeof(GB_BmpFileHeader),1,fp);
	if (FHead.bfType != BM)
	{
		fclose(fp);
		return GB_EXAMPLE_PARAM_ERROR;
	}


	// read info header
	fread(&FInfo,1,sizeof(GB_BmpInfoHeader),fp);
	// check pixel depth
	if (FInfo.biBitCount != 8)
	{
		fclose(fp);
		return GB_EXAMPLE_PARAM_ERROR;
	}
	// check colors and compression
	if (FInfo.biClrUsed != 256 || FInfo.biCompression != GB_BI_RGB)
	{
		fclose(fp);
		return GB_EXAMPLE_PARAM_ERROR;
	}

	// read palette
	ElemToRead = 256 * sizeof(GB_rgb);
	Palette = (GB_rgb *)malloc(ElemToRead);

	fread(Palette,1,ElemToRead,fp);

	// read frame	
	ElemToRead = ((FInfo.biWidth) * (FInfo.biHeight)) * sizeof(unsigned char);

	Frame = (unsigned char *)malloc(ElemToRead);
	if (Frame == NULL)
	{
		fclose(fp);
		free(Palette);
		return GB_EXAMPLE_MEM_ERROR;
	}
	appoggio = (unsigned char *)malloc(ElemToRead);
	if (appoggio == NULL)
	{
		fclose(fp);
		free(Palette);
		free(Frame);
		return GB_EXAMPLE_MEM_ERROR;
	}

	fread(appoggio,1,ElemToRead,fp);
	fclose(fp);

	// flip up-down frame
	for (i=0;i<FInfo.biHeight;i++){
		StartFrame = FInfo.biWidth * i;
		StartAppoggio = FInfo.biWidth * (FInfo.biHeight - i - 1);

		memcpy(Frame + StartFrame,appoggio + StartAppoggio,FInfo.biWidth);
	}

	// free memory
	free(appoggio);
	free(Palette);

	// copy into output
	*(pFrame) = Frame;
	*SizeX = FInfo.biWidth;
	*SizeY = FInfo.biHeight;

	// return normally
	return GB_EXAMPLE_NO_ERROR;
}