#ifndef GB_MINUTIAE_H
#define GB_MINUTIAE_H

#include <lfs_dll.h> 

using namespace System;

/********************************************************************************************/
// This function returns the index inside
extern MINUTIAE *FlatMinutiae[10];
// array from the ObjID passed as parameter.
//
// Parameters:
// - ObjID	: ID of the slap to be processed. Must be one of those defined in 
//		ImageProcessing.h
//
// Return:
// - a value >= 0 if valid index
// - a value < 0 in case of error
/********************************************************************************************/
int		GetFlatMinutiaeIndexFromObjID(int ObjID);

/********************************************************************************************/
// This function shows how should correctly be used the lfs tool. Simply calculates minutiae
// for the finger whose ID parameters were passed as input.
// Operations performed:
//
// 1) Get the pointer to the list where calculated minutiae should be stored (see
//		GetFlatMinutiaeIndexFromObjID(...))
// 2) Get the segment where taking the image buffer from (see SearchSlapIndexFromFingerID(...)
//		in GB_Segmentation.h
// 3) Calculate minutiae (lfs tool)
//
// Parameters:
// - FingerID	: ID of the finger as defined in ImageProcessing.h. It will be passed to
//		GetFlatMinutiaeIndexFromObjID(...)
// - GBFINIMG_FingerID: ID of the finger as defined in GBFINIMG.h. It will be passed to
//		SearchSlapIndexFromFingerID(...) together with the Hand id if needed
/********************************************************************************************/
void	CalcMinForFingerInSlap(int FingerID, int GBFINIMG_FingerID);

/********************************************************************************************/
// This function simply calls CalcMinForFingerInSlap(...) for each object in the slap whose
// ID is passed as parameter.
//
// Parameters:
// - ObjID	: ID of the slap to be processed
/********************************************************************************************/
int		CalculateSlapMinutiae(unsigned char ObjID);

/********************************************************************************************/
// This function shows how should correctly be used the lfs  and bozorth tools.
// Operations performed:
//
// 1) Calculate minutiae (lfs tool)
// 2) Get list of the slap corresponding minutiae(see GetFlatMinutiaeIndexFromObjID(...))
// 3) Calculates matching score (bozorth tool)
// 4) If matching score is below the GB_MIN_COMPARISON_SCORE (see ImageProcessing.h) 
//		threshold a warning message is displayed
//
// Parameters:
// - frame			: image buffer
// - SizeX, SizeY	: image dimensions
// - ObjID			: ID of the slap to be processed
/********************************************************************************************/
void	CalculateRollMinutiaeAndCompare(Byte *frame,int SizeX,int SizeY, int ObjID);

#endif GB_MINUTIAE_H