#ifndef GB_SEGM_H
#define GB_SEGM_H

// To avoid compilation errors, redefine the
// followings: FAR, PASCAL and BYTE
// before including GBFINIMG.h
#ifndef FAR
#define GB_FAR_DEFINED
#define FAR
#define PASCAL __stdcall
#define BYTE unsigned char
#endif	FAR
#include <GBFINIMG.h>
#ifdef GB_FAR_DEFINED // if are defined externally keep them defined
#undef FAR
#undef PASCAL
#undef BYTE
#endif GB_FAR_DEFINED

/********************************************************************************************/
// Initializes the external info list with the approximate coordinates of left fingers
/********************************************************************************************/
void	InitExternalInfoList(void);

/********************************************************************************************/
// Allocates memory for image buffers inside the segment descriptor structures.
// Returns memory error if unsuccessful
/********************************************************************************************/
int		InitSegmentDescriptorList(void);

/********************************************************************************************/
// Searches index of the segment identified by FingerID and HandID (for thumbs) inside the
// CompleteSlapList array.
// Returns negative if unsuccessful
/********************************************************************************************/
int		SearchSlapIndexFromFingerID(int FingerID,int HandID = -1);

/********************************************************************************************/
// Processes a slap image and performs a fully automatic segmentation, by using the following
// global structure array, where segmentation results are stored:
//extern GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR CompleteSlapList[4];
// Operations performed:
//
// 1) Call GBFINIMG_ProcessImage passing NULL as external info list
// 2) Shows how should correctly interpreted the results (RetVal, Diagnostic, etc...)
//
// Parameters:
// - pImage	: image buffer
// - SizeX, SizeY	: image dimensions
// - ObjID	: Object currently processed
/********************************************************************************************/
int		CalculateAutomaticSegmentation(unsigned char *pImage,unsigned int SizeX,
										  unsigned int SizeY,Byte ObjID);

/********************************************************************************************/
// Processes a slap image and performs a semi automatic segmentation, by using the following
// global structure arrays, where segmentation results are stored:
extern GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR CompleteSlapList[4];
// and where an external info list is passed to GBFINIMG_ProcessImage(...)
extern GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO LeftSlapExternInfoList[3];
// In this last vector approximate coordinates of finger center into left slap image (taken
// from file Smaples\LeftSlap.bmp) are stored. They have been got simply by opening the image
// file with an imaging appliication (for example Microsoft (R) Paint) and looking to
// mouse coordinates after having pointed it near to the center of each finger. These
// coordinates have been saved as defines:
/*
#define LITTLE_CENTER_X	(UInt32)473
#define LITTLE_CENTER_Y	(UInt32)971
#define MIDDLE_CENTER_X	(UInt32)1005
#define MIDDLE_CENTER_Y	(UInt32)654
#define INDEX_CENTER_X	(UInt32)1272
#define INDEX_CENTER_Y	(UInt32)787
*/
// Operations performed:
//
// 1) Call GBFINIMG_ProcessImage passing LeftSlapExternInfoList as external info list
// 2) Shows how should correctly interpreted the results (RetVal, Diagnostic, etc...)
//
// Parameters: (no need of getting ObjID because it's for left slap)
// - pImage	: image buffer
// - SizeX, SizeY	: image dimensions
/********************************************************************************************/
int		CalculateManualSegmentation(unsigned char *pImage,unsigned int SizeX,
									   unsigned int SizeY);

/********************************************************************************************/
// This functions shows information about segments obtained by 
// CalculateAutomaticSegmentation(...) or CalculateManualSegmentation(...)
//
// Operations performed:
//
// 1) Allocs memory for the NIST quality list by calling:
extern int		BuildQualityListFromObjID(int ObjID);
// 2) After having choosen the parameters, for each segment calls 
//		DisplaySegmInfoForFingerInSlap(...)
//
// Parameters: (no need of getting ObjID because it's for left slap)
// - ObjID	: object currently processed
/********************************************************************************************/
void	ShowSegmentationData(unsigned char ObjID);

/********************************************************************************************/
// Once that Segmentation has been performed, for each finger inside slap is called this
// function, that:
//
// 1) Shows how should correctly interpreted the results of segmentation operation(GB_Quality,
//		Pattern Validity and Completeness, Finger Inclination), as indicated in the following
//		defines (see ImageProcessing.h):
//#define GB_PATTERN_COMPLETENESS_MIN_THRESH	85
//#define GB_PATTERN_VAIDITY_MIN_THRESH			75
//#define GB_PATTERN_QUALITY_MIN_THRESH			70
//#define GB_PATTERN_INCLINATION_MAX_THRESH		35
//#define GB_PATTERN_INCLINATION_MIN_THRESH		-35
//
// 2) Calculates NIST Quality and shows it after having stored into:
extern FOUR_FING_QUALITY *NIST_QualityList;
//		that will be loaded as a parameter into record 14
//
//
// Parameters:
// - FingerID	: ID of the finger, choosen between defines in GBFINIMG.h
// - NISTQualityListIndex	: Index of NIST_QualityList where NIST quality must be stored,
//		choosen by the calling function following this table:
//
// -------------------------------------------------------------------------------------------
//	OBJ ID		|	FingerID				|	HandID				|	NISTQualityListIndex |
// -------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_INDEX	|	Don't care			|	0                    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_MIDDLE	|	Don't care			|	1                    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_RING	|	Don't care			|	NOTHING (Missing)    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_LITTLE	|	Don't care			|	2                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_INDEX	|	Don't care			|	0                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_MIDDLE	|	Don't care			|	1                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_RING	|	Don't care			|	2                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_LITTLE	|	Don't care			|	3                    |
// -------------------------------------------------------------------------------------------
//	TWO_THUMBS	|	GBFINIMG_FINGER_THUMB	|	GBFINIMG_HAND_LEFT	|	0                    |
// -------------------------------------------------------------------------------------------
//	TWO_THUMBS	|	GBFINIMG_FINGER_THUMB	|	GBFINIMG_HAND_RIGHT	|	1                    |
// -------------------------------------------------------------------------------------------
//
// - ANSI_NIST_FingID	: Finger Identifier choosen between the values in an2k_dll.h and
//		corresponding to values of finger for the currently processed object (for example,
//		if current process is LEFT_SLAP and FingerID is GBFINIMG_FINGER_INDEX then
//		ANSI_NIST_FingID should be ETFS_14_FGP_LEFT_INDEX)
// - HandID	: ID of thehand, choosen between defines in GBFINIMG.h. It's needed only for
//		2 thumbs object, because for slaps all fingers belong to the same hand
/********************************************************************************************/
void	DisplaySegmInfoForFingerInSlap(int FingerID, int NISTQualityListIndex,
									   char *ANSI_NIST_FingID, int HandID);

#endif	GB_SEGM_H