#include <stdafx.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ImageProcessing.h"
#include "GB_Segmentation.h"
#include "GBUtil.h"
#include <an2k_dll.h>
#include "GB_AN2K.h"
#include "GB_Minutiae.h"
#include <gbnfiq.h>
#include <lfs_dll.h>
#include <WSQpack.H>
// To avoid compilation errors, redefine the
// followings: FAR, PASCAL and BYTE
#ifndef FAR
#define GB_FAR_DEFINED
#define FAR
#define PASCAL __stdcall
#define BYTE unsigned char
#endif	FAR
#include <GBFINIMG.h>
#include <GBJpeg.h>
#ifdef GB_FAR_DEFINED // if are defined externally keep them defined
#undef FAR
#undef PASCAL
#undef BYTE
#endif GB_FAR_DEFINED

// to avoid warning with strcpy and so on
#pragma warning( disable : 4996 4793)

using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;



// recommended r_bitrate for WSQ algorithm
#define GB_RECOMMENDED_WSQ_BITRATE (float)0.75

// used rate for JP2 algorithm (compression rate = 1/(rate))
#define GB_USED_JP2_RATE	(UInt16)1

/****************************************************************************/
/************* GLOBAL VARIABLES *********************************************/
/****************************************************************************/
Boolean CanUseSlapForSequenceCheck[3] = {false,false,false};



/****************************************************************************/
/************* ROUTINES *****************************************************/
/****************************************************************************/

void	SetUsabilityForSequenceCheck(Byte ObjID,Boolean Usability)
{
	// check input
	if (ObjID > TWO_THUMBS) return;

	// it's intended that slaps are defined from 0 to 2
	CanUseSlapForSequenceCheck[ObjID] = Usability; 
}

int		InitGlobals(void)
{
	Int32 RetVal;

	// init minutiae list
	for (int i = 0; i < 10; i++)
	{
		FlatMinutiae[i] = nullptr;
	}

	// init external info list
	InitExternalInfoList();

	// init segment descriptors list
	RetVal = InitSegmentDescriptorList();
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		return RetVal;
	}

	/* init amputation list */
	RetVal = InitAmputationList();
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		return RetVal;
	}

	// init quality list
	NIST_QualityList = nullptr;

	return GB_EXAMPLE_NO_ERROR;
}

int		ProcessRoll(Byte ObjID,RECORD **pRecord)
{
	Int32	RetVal;
	Byte	*frame;
	UInt32	SizeX,SizeY;

	/* Load Image */
	Console::WriteLine("Loading Image...");
	RetVal = LoadImageFromObject(ObjID,&frame,&SizeX,&SizeY);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("LoadImageFromObject returned " + RetVal + " : cannot load...");
		// return error
		return RetVal;
	}
	
	// calculate NIST Quality and show it for each finger
	Console::WriteLine("Image Loaded...\n\nCalculating Quality...");

	int ImageNISTQuality;
	RetVal = GBNFIQ_GetImageQuality(frame,SizeX,SizeY, // image parameters (input)
		&ImageNISTQuality, // image quality (output)
		nullptr // minutiae info (output)
		);
	// check retval
	String ^ MessageToDisplay;
	if (RetVal == GBNFIQ_EMPTY_IMG)
	{
		// not found
		MessageToDisplay = ("Finger not found...\n");
	}
	if (RetVal == GBNFIQ_TOO_FEW_MINUTIAE)
	{
		// not found
		MessageToDisplay = ("Too much Low NIST Quality image...\n");
	}
	else
	{
		// no error
		MessageToDisplay = ("Finger NIST Image Quality = " + ImageNISTQuality + "...");
	}
	Console::WriteLine(MessageToDisplay);

	// calculate minutiae
	Console::WriteLine("Quality succesfully calculated...\n\nCalculating minutiae...");
	CalculateRollMinutiaeAndCompare(frame,SizeX,SizeY,ObjID);


	/* compress image to JPEG 2000 */
	Console::WriteLine ("");
	Console::WriteLine ("Compressing Image in JP2 format...");
	Byte *ImgCompressed;
	unsigned long ImgCompLen;
	char ImgCompAlg[20];	
	
	RetVal = GBJPEG_JP2_Encode(&ImgCompressed, // compressed image buffer pointer (output)
		&ImgCompLen, // compressed image buffer length (output)
		frame,(unsigned long)SizeX,(unsigned long)SizeY, // image frame and size (input)
		8, // image pixel depth (input)
		GB_USED_JP2_RATE // 1/(Compression Rate) - input
		);

	// check retval
	if (RetVal != GBJPEG_OK)
	{
		Console::WriteLine ("GBJPEG_JP2_Encode returned " + RetVal + " :");
		Console::WriteLine ("Cannot compress image: record will be saved with image in original format...");
		ImgCompressed = frame;
		ImgCompLen = SizeX * SizeY;
		strcpy(ImgCompAlg,COMP_NONE);
	}
	else
	{
		Console::WriteLine ("Image Compression in JP2 format OK...");
		strcpy(ImgCompAlg,BIN_COMP_JPEG2K);
	}

	// create Rec 4
	RetVal = CreateRec4FromObjID(ObjID,
		pRecord,
		ImgCompressed,ImgCompLen,SizeX,SizeY,
		ImgCompAlg);
	

	// free memory
	FreeImage(frame);
	if (ImgCompressed != frame)
	{
		GBJPEG_Free(ImgCompressed);
	}

	return RetVal;
}
int		ProcessSlap(unsigned char ObjID,RECORD **pRecord)
{
	Int32	RetVal;
	Byte	*frame;
	UInt32	SizeX,SizeY;

	// check parameters
	if (ObjID > TWO_THUMBS)
	{
		*pRecord = nullptr;
		return GB_EXAMPLE_PARAM_ERROR;
	}

	/* Load Image */
	Console::WriteLine ("");
	Console::WriteLine("Loading Image...");
	RetVal = LoadImageFromObject(ObjID,&frame,&SizeX,&SizeY);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		// set wrong sequence check for slap
		SetUsabilityForSequenceCheck(ObjID,false);
		Console::WriteLine("LoadImageFromObject returned " + RetVal + " : cannot use for sequence check...");
		// return error
		return RetVal;
	}
	
	/* Calculate segmentation */
	Console::WriteLine("Image Loaded...\nCalculating segmentation...");
	if (ObjID == LEFT_SLAP)
	{
		// manual segmentation
		RetVal = CalculateManualSegmentation(frame,SizeX,SizeY);
	}
	else
	{
		// automatic segmentation
		RetVal = CalculateAutomaticSegmentation(frame,SizeX,SizeY,ObjID);
	}
	

	if (RetVal != GB_EXAMPLE_NO_ERROR && RetVal != GB_EXAMPLE_ACCEPTABLE_SEGM_RES_ERROR)
	{
		// set wrong sequence check for slap
		SetUsabilityForSequenceCheck(ObjID,false);
		Console::WriteLine("Segments calculation returned " + RetVal + " : cannot use for sequence check...");
		// return error
		return RetVal;
	}

	
	/* Show segmentation data and NIST Quality for each finger */
	Console::WriteLine("Segmentation OK...\n\nShowing Segmentation Data and NIST Quality...");	
	ShowSegmentationData(ObjID);
	// calculate minutiae and tell if they can be used for sequence check
	Console::WriteLine("Quality succesfully calculated...\n\nCalculating minutiae...");
	RetVal = CalculateSlapMinutiae(ObjID);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		// set wrong sequence check for slap
		SetUsabilityForSequenceCheck(ObjID,false);
		Console::WriteLine("CalculateSlapMinutiae returned " + RetVal + " : cannot use for sequence check...");
	}
	else
	{
		// set right sequence check for slap
		SetUsabilityForSequenceCheck(ObjID,true);
		Console::WriteLine("CalculateMinutiae returned " + RetVal + " : OK...");
	}
	
	Console::WriteLine("");
	/* compress image to WSQ */
	Console::WriteLine("Compressing image in WSQ Format...");
	Byte *ImgCompressed;
	int ImgCompLen;
	char ImgCompAlg[20];
	
	RetVal = WSQPACK_Compress(
		&ImgCompressed, // dest
		&ImgCompLen, // dest len
		GB_RECOMMENDED_WSQ_BITRATE, // r_bitrate
		frame, // source
		(int)SizeX, 
		(int)SizeY, 
		8, // pixel depth
		500, // dpi
		nullptr // comment
		);

	// check retval
	if (RetVal != WSQPACK_OK)
	{
		Console::WriteLine ("Cannot compress image: record will be saved with image in original format...");
		ImgCompressed = frame;
		ImgCompLen = SizeX * SizeY;
		strcpy(ImgCompAlg,COMP_NONE);
	}
	else
	{
		Console::WriteLine ("Image Compression in WSQ format OK...");
		strcpy(ImgCompAlg,COMP_WSQ);
	}
	
	Console::WriteLine("");

	/* Create Lists for Record 14 */
	RetVal = CreateRec14FromObjID(ObjID,
		pRecord,
		ImgCompressed,ImgCompLen,SizeX,SizeY,
		ImgCompAlg);

	/* Return */
	// free memory
	FreeImage(frame);
	if (ImgCompressed != frame)
	{
		WSQPACK_Free(ImgCompressed);
	}
	

	return RetVal;
}

