/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package testwsq_java;

import testwsq_java.TestWSQ_LogMessageOptions;

import java.util.logging.Logger;
import java.io.*;
import GBWSQ_JAVA_Defines.GBWSQ_ReturnCodes;
import GBWSQ_JAVA_Functions.GBWSQ_DllWrapper.GBWsq_Library;
import com.sun.jna.Pointer;
import com.sun.jna.Library;
import com.sun.jna.Native;
import com.sun.jna.Platform;
import com.sun.jna.Pointer;
import com.sun.jna.Callback;
import com.sun.jna.Memory;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.LongByReference;
import com.sun.jna.ptr.ByteByReference;
import com.sun.jna.ptr.DoubleByReference;

/**
 *
 * @author andrea
 */
public class TestWSQ_Java {

	public static int RawImageSizeX = 224;
	public static int RawImageSizeY = 328;
	public static String InputFileName = "sample.raw";
	public static String OutWSQFileName = "sample_JAVA.wsq";
	public static String OutImageFileName = "sample_JAVA.raw";

	private static void WriteAllBytes(String fileName, byte[] BufferToSave) {
		FileOutputStream fos = null;
		DataOutputStream ds = null;
		try {
			fos = new FileOutputStream(fileName);
			ds = new DataOutputStream(fos);
			ds.write(BufferToSave);
		} catch (Exception e) {
			System.out.println("Error in SaveOnFile");
		} finally {
			try {
				ds.close();
			} catch (IOException ex) {
				System.out.println("Error in closing ds");
			}
			try {
				fos.close();
			} catch (IOException ex) {
				System.out.println("Error in closing fos");
			}
		}
	}

	private static byte[] loadFile(File file) {
		FileInputStream fin = null;

		byte[] fileContent = null;
		try {
			fin = new FileInputStream(file);
			fileContent = new byte[(int) file.length()];
			fin.read(fileContent);



		} catch (IOException ex) {
			System.out.println("Error in getting file: ");
		} finally {
			try {
				fin.close();


			} catch (IOException ex) {
				System.out.println("Error in closing file: ");
			}

		}
		return fileContent;
	}

	private static byte[] ReadAllBytes(String fileName) {
		File file = new File(fileName);
		return loadFile(file);
	}

	public static void LogMessage(int Option, String message) {
		if (Option == TestWSQ_LogMessageOptions.HugeMessage) {
			System.out.println();
			System.out.println("********************************************");
			System.out.println("** " + message + " **");
			System.out.println("********************************************");
			System.out.println();
		} else if (Option == TestWSQ_LogMessageOptions.ShortMessage) {
			System.out.println(message);
		} else if (Option == TestWSQ_LogMessageOptions.ShortMessageWithStars) {
			System.out.println("** " + message + " **");
		}
	}

	public static void main(String[] args) {
		/////////////////////////////////////////////
		// LOADING INPUT
		/////////////////////////////////////////////
		Pointer dest = new Memory(100);
		IntByReference destLen = new IntByReference(),
				pwidth = new IntByReference(),
				pheight = new IntByReference(),
				pdepth = new IntByReference(),
				pppi = new IntByReference(),
				plossy = new IntByReference();

		LogMessage(TestWSQ_LogMessageOptions.HugeMessage, "Loading Input file");
		byte[] ImgBuffer = ReadAllBytes(InputFileName);
		LogMessage(TestWSQ_LogMessageOptions.ShortMessageWithStars, "Input file loaded");


		/////////////////////////////////////////////
		// COMPRESSION
		/////////////////////////////////////////////
		LogMessage(TestWSQ_LogMessageOptions.HugeMessage, "Compressing buffer");

		int ret = GBWsq_Library.INSTANCE.WSQPACK_Compress(
				dest,
				destLen,
				(float) 0.75, ImgBuffer, RawImageSizeX, RawImageSizeY, 8, 500, null);
		if (ret != GBWSQ_ReturnCodes.WSQPACK_OK)
		{
			LogMessage (TestWSQ_LogMessageOptions.ShortMessage, "Error in compression");
				return;
		}

		byte[] compressedBuffer = dest.getPointer(0).getByteArray(0, destLen.getValue());

		LogMessage(TestWSQ_LogMessageOptions.ShortMessageWithStars, "After compression:");
		LogMessage(TestWSQ_LogMessageOptions.ShortMessage, "Encoded buffer len: "
				+ destLen.getValue());
		
		GBWsq_Library.INSTANCE.WSQPACK_Free(dest.getPointer(0));

		/////////////////////////////////////////////
		// WSQ SAVING
		/////////////////////////////////////////////
		LogMessage(TestWSQ_LogMessageOptions.HugeMessage, "Saving wsq file");
		WriteAllBytes("sample_java.wsq", compressedBuffer);
		LogMessage(TestWSQ_LogMessageOptions.ShortMessageWithStars, "File saved in "
				+ OutWSQFileName);


		/////////////////////////////////////////////
		// DECOMPRESSION
		/////////////////////////////////////////////
		LogMessage(TestWSQ_LogMessageOptions.HugeMessage, "Uncompressing buffer");
		ret = GBWsq_Library.INSTANCE.WSQPACK_Uncompress(
				dest,
				pwidth, pheight, pdepth, pppi, plossy,
				compressedBuffer,
				compressedBuffer.length);
		if (ret != GBWSQ_ReturnCodes.WSQPACK_OK)
		{
			LogMessage (TestWSQ_LogMessageOptions.ShortMessage, "Error in decompression");
				return;
		}
		LogMessage(TestWSQ_LogMessageOptions.ShortMessageWithStars, "After decompression:");
		LogMessage(TestWSQ_LogMessageOptions.ShortMessage,
				"BitsPerPixel: " + pdepth.getValue());
		LogMessage(TestWSQ_LogMessageOptions.ShortMessage,
				"ImageHeight: " + pheight.getValue());
		LogMessage(TestWSQ_LogMessageOptions.ShortMessage,
				"ImageWidth: " + pwidth.getValue());
		LogMessage(TestWSQ_LogMessageOptions.ShortMessage,
				"ImageResolution: " + pppi.getValue());

		/////////////////////////////////////////////
		// SAVING DECOMPRESSED IMAGE
		/////////////////////////////////////////////
		byte[] decompressedBuffer = dest.getPointer(0).getByteArray(0, 
				pwidth.getValue() * pheight.getValue());
		WriteAllBytes(OutImageFileName, decompressedBuffer);
		LogMessage(TestWSQ_LogMessageOptions.ShortMessageWithStars,
				"File saved in " + OutImageFileName);
		GBWsq_Library.INSTANCE.WSQPACK_Free(dest.getPointer(0));

	}
}
