/*
 * Copyright (c) 2010, Oracle. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * * Neither the name of Oracle nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package Test_GBFIR_GUI;

import javax.swing.JFileChooser;
import java.io.File;
import javax.swing.filechooser.FileFilter;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import javax.swing.JOptionPane;
import javax.swing.ComboBoxModel;
import java.awt.image.ColorModel;
import java.awt.color.ColorSpace;
import javax.swing.DefaultListModel;
import javax.swing.DefaultComboBoxModel;

import com.sun.jna.Library;
import com.sun.jna.Native;
import com.sun.jna.Pointer;
import com.sun.jna.NativeLong;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.ByteByReference;
import com.sun.jna.ptr.PointerByReference;
import com.sun.jna.ptr.DoubleByReference;
import com.sun.jna.ptr.FloatByReference;
import com.sun.jna.ptr.ShortByReference;
import com.sun.jna.WString;
import java.nio.ByteBuffer;

import Test_GBFIR_Utils.ImageFilter;
import Test_GBFIR_Utils.TestFIR_Globals;
import Test_GBFIR_Utils.Utils;

import GBFIR_JAVA_Defines.*;
import GBFIR_JAVA_Functions.GBFIR_DllWrapper;
import Test_GBFIR_Utils.FirFileFilter;

public class GBFIR_MainWindow extends javax.swing.JFrame {

	private static final long serialVersionUID = 1L;
	private DefaultListModel MessageListModel;
	private DefaultComboBoxModel StoreOptionsListModel;

	/**
	 * Creates new form GBFIR_MainWindow
	 */
	public GBFIR_MainWindow() {
		initComponents();

		MessageListModel = new DefaultListModel();
		this.jMessagesList.setModel(MessageListModel);

		StoreOptionsListModel = new DefaultComboBoxModel();

		StoreOptionsListModel.addElement(TestFIR_Globals.ISOIEC);
		StoreOptionsListModel.addElement(TestFIR_Globals.ANSI);
		this.jStoreOptionsComboBox.setModel(StoreOptionsListModel);
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jFileChooser1 = new javax.swing.JFileChooser();
        jControlPanel = new javax.swing.JPanel();
        jStoreOptionsComboBox = new javax.swing.JComboBox();
        jCreateFIRButton = new javax.swing.JButton();
        jReadFIRButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jMessagesList = new javax.swing.JList();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Test_GBFIR_GUI_Java");

        jControlPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(" Control"));

        jCreateFIRButton.setText("Create FIR");
        jCreateFIRButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jCreateFIRButtonMouseClicked(evt);
            }
        });

        jReadFIRButton.setText("Read FIR");
        jReadFIRButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jReadFIRButtonMouseClicked(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jControlPanelLayout = new org.jdesktop.layout.GroupLayout(jControlPanel);
        jControlPanel.setLayout(jControlPanelLayout);
        jControlPanelLayout.setHorizontalGroup(
            jControlPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jControlPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(jControlPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jReadFIRButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jCreateFIRButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 91, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 121, Short.MAX_VALUE)
                .add(jStoreOptionsComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 323, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jControlPanelLayout.setVerticalGroup(
            jControlPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jControlPanelLayout.createSequentialGroup()
                .add(jControlPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jStoreOptionsComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jCreateFIRButton))
                .add(18, 18, 18)
                .add(jReadFIRButton)
                .addContainerGap(32, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(" Messages "));

        jScrollPane1.setViewportView(jMessagesList);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 535, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 355, Short.MAX_VALUE)
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jControlPanel, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jControlPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        jControlPanel.getAccessibleContext().setAccessibleName(" Controls ");

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void LogMessage(String messageToShow) {
		MessageListModel.addElement(messageToShow);
	}

	private void ShowError(String Function, int RetVal) {
		String MessageToShow = "Function " + Function + " returned " + RetVal;
		JOptionPane.showMessageDialog(this, MessageToShow);
		LogMessage(MessageToShow);
	}

	public void ShowException(String Function, String exMessage) {
		String MessageToShow = "Exception in function " + Function + ": " + exMessage;
		JOptionPane.showMessageDialog(this, MessageToShow);
		LogMessage(MessageToShow);
	}
	private void jCreateFIRButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jCreateFIRButtonMouseClicked
		// TODO add your handling code here:
		JFileChooser FD = new JFileChooser();
		FD.setFileFilter(new ImageFilter());
		FD.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int RetVal = FD.showOpenDialog(this);
		MessageListModel.clear();

		if (RetVal == JFileChooser.APPROVE_OPTION) {
			File file = FD.getSelectedFile();

			String DirectoryName = Utils.getDirectoryName(file),
					PureFileName = Utils.getFileNameWithoutExtension(file);

			LogMessage("Directory: " + Utils.getDirectoryName(file));
			LogMessage("File Name: " + Utils.getFileNameWithoutExtension(file));
			LogMessage("Extension: " + Utils.getExtension(file));
			LogMessage("getAbsolutePath: " + file.getAbsolutePath());
			try {
				LogMessage("getCanonicalPath: " + file.getCanonicalPath());
			} catch (Exception e) {
				ShowException("ImageIO.read", "Exception in opening the image: " + e.getMessage());
				return;
			}
			LogMessage("getPath: " + file.getPath());
			LogMessage("toString: " + file.toString());

			BufferedImage img = null;
			try {
				img = ImageIO.read(file);
			} catch (Exception e) {
				ShowException("ImageIO.read", "Exception in opening the image: " + e.getMessage());
				return;
			}

			ColorModel cm = img.getColorModel();
			LogMessage("cm pixel size: " + cm.getPixelSize());
			LogMessage("cm color space type: " + cm.getColorSpace().getType());

			int ImgWidth = img.getWidth();
			int ImgHeight = img.getHeight();
			LogMessage("Extension for compression: " + Utils.getExtension(file));
			byte CompressionAlgorithm = (Utils.getExtension(file)).equals("jpg")
					? GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG
					: GBFIR_CompressionAlgorithms.GBFIR_CA_PNG;
			LogMessage("CompressionAlgorithm: " + CompressionAlgorithm);

			if (cm.getPixelSize() != 8 || cm.getColorSpace().getType() != ColorSpace.TYPE_GRAY) {
				ShowException("jCreateFIRButtonMouseClicked", "INCORRECT PIXEL FORMAT");
				return;
			}

			///////////////////////////////////////////////
			// GET WHOLE FILE CONTENTS
			///////////////////////////////////////////////
			byte[] TempBuffer = Utils.ReadAllBytes(file.toString());


			///////////////////////////////////////////////
			// CREATE FIR
			///////////////////////////////////////////////
			int FirType = (StoreOptionsListModel.getIndexOf(
					StoreOptionsListModel.getSelectedItem())) == 0
					? GBFIR_Formats.GBFIR_FORMAT_ISO
					: GBFIR_Formats.GBFIR_FORMAT_ANSI;
			RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_CreateFIR(
					FirType,
					(short) 31,
					null,
					(short) 0,
					GBFIR_ScaleUnits.GBFIR_SU_PIXEL_PER_INCH,
					(short) 500,
					(short) 500,
					(short) 500,
					(short) 500,
					(byte) 8,
					CompressionAlgorithm);

			if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
				ShowError("GBFIR_CreateFIR", RetVal);
				return;
			} else {
				LogMessage("FIR created succesfully");
			}


			///////////////////////////////////////////////
			// ADD IMAGE TO FIR
			///////////////////////////////////////////////				
			RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_AddImageToFIR(
					TempBuffer,
					TempBuffer.length,
					ImgWidth, ImgHeight,
					(byte) 1, (byte) 1, 100,
					GBFIR_FingerPositions.GBFIR_FP_UNKNOWN,
					GBFIR_ImpressionTypes.GBFIR_IT_LIVE_SCAN_PLAIN);

			if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
				ShowError("GBFIR_AddImageToFIR", RetVal);
				return;
			} else {
				LogMessage("image added to FIR succesfully");
			}

			///////////////////////////////////////////////
			// SAVE FIR ON FILE
			///////////////////////////////////////////////	
			int FIRBuffLen = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetFIRLen();
			LogMessage("FIRBuffLen = " + FIRBuffLen);

			TempBuffer = new byte[FIRBuffLen];

			RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_WriteFIRToBuffer(
					TempBuffer);

			if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
				ShowError("GBFIR_WriteFIRToBuffer", RetVal);
				return;
			} else {
				LogMessage("FIR written to buffer succesfully");
			}
			GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_FreeFIR();
			String extension =
					(FirType == GBFIR_Formats.GBFIR_FORMAT_ISO) ? ".fir-iso" : ".fir-ansi";

			String outputFileName = DirectoryName + "/"
					+ PureFileName
					+ extension;
			Utils.WriteAllBytes(outputFileName, TempBuffer);
			LogMessage("FIR saved in " + outputFileName);
		} else {
			ShowException("jCreateFIRButtonMouseClicked", "NO FILE TO BE OPENED");
			return;
		}
	}//GEN-LAST:event_jCreateFIRButtonMouseClicked

	private void jReadFIRButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jReadFIRButtonMouseClicked
		try {
			JFileChooser FD = new JFileChooser();
			FD.setFileFilter(new FirFileFilter());
			FD.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int RetVal = FD.showOpenDialog(this);
			MessageListModel.clear();

			if (RetVal == JFileChooser.APPROVE_OPTION) {
				File file = FD.getSelectedFile();
				String extension = Utils.getExtension(file);
				LogMessage("extension: " + extension);
				int FIRType;
				if (extension.equals(Utils.fir_iso)) {
					FIRType = GBFIR_Formats.GBFIR_FORMAT_ISO;
				} else if (extension.equals(Utils.fir_ansi)) {
					FIRType = GBFIR_Formats.GBFIR_FORMAT_ANSI;
				} else {
					ShowException("OpenFIRButton_Click", "file extension not allowed");
					return;
				}

				byte[] TempBuffer = Utils.ReadAllBytes(file.toString());
				RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_ReadFIRFromBuffer(
						TempBuffer, TempBuffer.length, FIRType);

				if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
					ShowError("GBFIR_ReadFIRFromBuffer", RetVal);
					return;
				} else {
					LogMessage("FIR read from buffer succesfully");
				}

				ShortByReference ImageAcquisitionLevel = new ShortByReference(),
						CaptureDeviceID = new ShortByReference(),
						ScanResolutionH = new ShortByReference(),
						ScanResolutionV = new ShortByReference(),
						ImageResolutionH = new ShortByReference(),
						ImageResolutionV = new ShortByReference();
				byte[] CBEFFProductId = new byte[GBFIR_Constants.CBEFF_PRODUCT_ID_LEN];
				ByteByReference ScaleUnits = new ByteByReference(),
						PixelDepth = new ByteByReference(),
						ImageCompressionAlgorithm = new ByteByReference();

				RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetFIRHeaderFields(
						ImageAcquisitionLevel,
						CBEFFProductId,
						CaptureDeviceID,
						ScaleUnits,
						ScanResolutionH,
						ScanResolutionV,
						ImageResolutionH,
						ImageResolutionV,
						PixelDepth,
						ImageCompressionAlgorithm);

				if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
					ShowError("GBFIR_GetFIRHeaderFields", RetVal);
					return;
				} else {
					LogMessage("FIR fields got succesfully");
				}

				LogMessage("FIR ImageAcquisitionLevel: " + ImageAcquisitionLevel.getValue());
				LogMessage("FIR CaptureDeviceID: " + CaptureDeviceID.getValue());
				LogMessage("FIR ScaleUnits: " + ScaleUnits.getValue());
				LogMessage("FIR ScanResolutionH: " + ScanResolutionH.getValue());
				LogMessage("FIR ScanResolutionV: " + ScanResolutionV.getValue());
				LogMessage("FIR ImageResolutionH: " + ImageResolutionH.getValue());
				LogMessage("FIR ImageResolutionV: " + ImageResolutionV.getValue());
				LogMessage("FIR PixelDepth: " + PixelDepth.getValue());
				LogMessage("FIR ImageCompressionAlgorithm: " + ImageCompressionAlgorithm.getValue());

				int NumFingers = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetNumberOfFingers();
				IntByReference ImageBufferSize = new IntByReference();
				byte[] ImageBuffer;
				IntByReference ImageWidth = new IntByReference(),
						ImageHeight = new IntByReference(),
						ImageQuality = new IntByReference(),
						FingerPosition = new IntByReference(),
						ImpressionType = new IntByReference();
				ByteByReference CountView = new ByteByReference(),
						ViewNumber = new ByteByReference();

				LogMessage("Number of records = " + (NumFingers));

				for (int i = 0; i < NumFingers; i++) {
					LogMessage("### Record " + (i));

					RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetFingerRecordHeaderFields(i,
							CountView,
							ViewNumber,
							ImageQuality,
							FingerPosition,
							ImpressionType);


					if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
						ShowError("GBFIR_GetFingerRecordHeaderFields", RetVal);
						return;
					}

					LogMessage("    Count View = " + (CountView.getValue()));
					LogMessage("    View Number = " + (ViewNumber.getValue()));
					LogMessage("    Image Quality = " + (ImageQuality.getValue()));
					LogMessage("    FingerPosition = " + (FingerPosition.getValue()));
					LogMessage("    ImpressionType = " + (ImpressionType.getValue()));

					RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetFingerImageSize(i,
							ImageBufferSize);
					if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
						ShowError("GBFIR_GetFingerImageSize", RetVal);
						return;
					}
					LogMessage("    ImpressionType = " + (ImageBufferSize.getValue()));

					ImageBuffer = new byte[ImageBufferSize.getValue()];
					RetVal = GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_GetFingerImage(
							i, ImageBuffer,
							ImageWidth, ImageHeight);
					if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS) {
						ShowError("GBFIR_GetFingerImage", RetVal);
						return;
					}

					LogMessage("    Image Width = " + (ImageWidth.getValue()));
					LogMessage("    Image Height = " + (ImageHeight.getValue()));


					String outFName = Utils.getDirectoryName(file) + '/'
							+ Utils.getFileNameWithoutExtension(file) + '_' + i;

					if (ImageCompressionAlgorithm.getValue()
							== GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG) {
						outFName += ".jpg";
					} else if (ImageCompressionAlgorithm.getValue()
							== GBFIR_CompressionAlgorithms.GBFIR_CA_PNG) {
						outFName += ".png";
					} else if (ImageCompressionAlgorithm.getValue()
							== GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_WSQ) {
						outFName += ".wsq";
					} else if (ImageCompressionAlgorithm.getValue()
							== GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG2000) {
						outFName += ".j2k";
					}

					Utils.WriteAllBytes(outFName, ImageBuffer);

					LogMessage("    Image file saved: " + outFName);
				}
				GBFIR_DllWrapper.GBFIR_Library.INSTANCE.GBFIR_FreeFIR();
			} else {
				ShowException("OpenFIRButton_Click", "Cannot open file");
			}
		} catch (Exception ex) {
			ShowException("OpenFIRButton_Click", ex.getMessage());
		}

	}//GEN-LAST:event_jReadFIRButtonMouseClicked

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
		//<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels = javax.swing.UIManager.getInstalledLookAndFeels();
			for (int idx = 0; idx < installedLookAndFeels.length; idx++) {
				if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
					javax.swing.UIManager.setLookAndFeel(installedLookAndFeels[idx].getClassName());
					break;






				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(GBFIR_MainWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(GBFIR_MainWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(GBFIR_MainWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(GBFIR_MainWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		//</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {

			public void run() {
				new GBFIR_MainWindow().setVisible(true);
			}
		});
	}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JPanel jControlPanel;
    private javax.swing.JButton jCreateFIRButton;
    private javax.swing.JFileChooser jFileChooser1;
    private javax.swing.JList jMessagesList;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton jReadFIRButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JComboBox jStoreOptionsComboBox;
    // End of variables declaration//GEN-END:variables
}
