#ifndef GBJPEG_H
#define GBJPEG_H

#ifdef BUILD_DLL
#define DLL_EXPORT __declspec( dllexport )
#else
#define DLL_EXPORT __declspec( dllimport )
#endif

#define GBJPEG_8_BITS_PER_PIXEL                           8
#define GBJPEG_24_BITS_PER_PIXEL                          24

#define GBJPEG_RATE_LOSSLESS                              1

#define GBJPEG_OK                                         0
#define GBJPEG_ERROR_INVALID_PARAMETERS                   1
#define GBJPEG_ERROR_CREATE_JPEG_STREAM					  2
#define GBJPEG_ERROR_UNSUPPORTED_BITSPERPIXEL             3
#define GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE               4
#define GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE_COMPONENT     5
#define GBJPEG_ERROR_MATRIX_MEMORY_ALLOCATION             6
#define GBJPEG_ERROR_UNABLE_TO_ENCODE_IMAGE               7
#define GBJPEG_ERROR_INITIALIZATION                       8
#define GBJPEG_ERROR_UNABLE_TO_DECODE_IMAGE               9
#define GBJPEG_ERROR_MISSING_IMAGE_CHANNEL                10
#define GBJPEG_ERROR_COLOR_SPACE_MODEL_UNSUPPORTED        11
#define GBJPEG_ERROR_IRREGULAR_CHANNEL_GEOMETRY           12
#define GBJPEG_ERROR_MEMORY_ALLOCATION                    13
#define GBJPEG_ERROR_UNKNOWN_FORMAT                       14



extern "C" {
// -----------------------------------------------------------------
//	GBJPEG_JP2_Encode - Encode image in JPEG-2000 JP2 File Format Syntax (ISO/IEC 15444-1)
//
//	ImgBuffer :			Pointer to the output buffer, allocated internally;
//						use GBJPEG_Free to free the pointer after use.
//	BuffLen:			Lenght of output buffer
//  RawImage :			Pointer to the raw image to be compessed.
//  ImageWidth :		RawImage width
//	ImageHeight :		RawImage height
//	BitsPerPixel :		RawImage bits per pixel (8,24)
//  Rate :				1/Rate will be real compression Rate 
//									Rate = 1 - lossless compression
//									Rate = 100 - 1/100 effective rate
// -----------------------------------------------------------------
DLL_EXPORT int __stdcall GBJPEG_JP2_Encode(unsigned char **ImgBuffer,
											unsigned long *BuffLen,
                                            unsigned char *RawImage,
                                            unsigned long ImageWidth,
                                            unsigned long ImageHeight,
                                            BYTE BitsPerPixel,
                                            unsigned short Rate);

// -----------------------------------------------------------------
//	GBJPEG_JP2_Decode - Decode image in JPEG-2000 JP2 File Format Syntax (ISO/IEC 15444-1)
//
//	ImgBuffer :			Pointer to the input buffer
//	BuffLen:			Lenght of input buffer
//  RawImage :			Pointer to the output raw image, allocated internally;
//						use GBJPEG_Free to free the pointer after use.
//  ImageWidth :		RawImage width
//	ImageHeight :		RawImage height
//	BitsPerPixel :		RawImage bits per pixel (8,24)
// -----------------------------------------------------------------
DLL_EXPORT int __stdcall GBJPEG_JP2_Decode(unsigned char *ImgBuffer,
											unsigned long BuffLen,
                                            unsigned char **RawImage,
                                            unsigned long *ImageWidth,
                                            unsigned long *ImageHeight,
                                            BYTE *BitsPerPixel);

// -----------------------------------------------------------------
//	GBJPEG_JPG_Encode - Encode image in JPEG JPG File Format Syntax
//
//	ImgBuffer :			Pointer to the output buffer, allocated internally;
//						use GBJPEG_Free to free the pointer after use.
//	BuffLen:			Lenght of output buffer
//  RawImage :			Pointer to the raw image to be compessed.
//  ImageWidth :		RawImage width
//	ImageHeight :		RawImage height
//	BitsPerPixel :		RawImage bits per pixel (8,24)
//	DPI:				Dpi of raw image (dots-per-inch)
//  Q :					Quality (0-100)
// -----------------------------------------------------------------
DLL_EXPORT int __stdcall GBJPEG_JPG_Encode(unsigned char **ImgBuffer,
											unsigned long *BuffLen,
                                            unsigned char *RawImage,
                                            unsigned long ImageWidth,
                                            unsigned long ImageHeight,
                                            BYTE BitsPerPixel,
											unsigned short DPI,
                                            unsigned short Q);

// -----------------------------------------------------------------
//	GBJPEG_JPG_Encode - Encode image in JPEG JPG File Format Syntax
//
//	ImgBuffer :			Pointer to the input buffer
//	BuffLen:			Lenght of input buffer
//  RawImage :			Pointer to the raw image, allocated internally;
//						use GBJPEG_Free to free the pointer after use.
//  ImageWidth :		RawImage width
//	ImageHeight :		RawImage height
//	BitsPerPixel :		RawImage bits per pixel (8,24)
// -----------------------------------------------------------------
DLL_EXPORT int __stdcall GBJPEG_JPG_Decode(unsigned char *ImgBuffer,
											unsigned long BuffLen,
                                            unsigned char **RawImage,
                                            unsigned long *ImageWidth,
                                            unsigned long *ImageHeight,
                                            BYTE *BitsPerPixel);

// -----------------------------------------------------------------
//	GBJPEG_Free - Free the pointers to memory allocated internally 
//	by the encoder/decoder functions.
//
//	Ptr :			Pointer to the memory to be freed.
// -----------------------------------------------------------------
DLL_EXPORT void __stdcall GBJPEG_Free(void *Ptr);

} // extern "C"

#endif

