﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using GBFINIMG_Utilities;
using GBFINIMG_NET_WRAPPER;
using AN2K_NET_WRAPPER;

namespace GBFINIMG_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Structures definition: PACK 1 for compatibility with
     * C structures
     * 
     ************************************************************
    ************************************************************/
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR
    {
        public IntPtr SegmentImagePtr; // byte * 
        public UInt32 FingerCenterX;
        public UInt32 FingerCenterY;
        public UInt32 Quality;
        public UInt32 Hand;
        public UInt32 Finger;

        public Int32 BoundingBoxL;
        public Int32 BoundingBoxR;
        public Int32 BoundingBoxT;
        public Int32 BoundingBoxB;

        public UInt32 PatternValidity;
        public UInt32 PatternCompleteness;
        public UInt32 Size;

        public Int32 Inclination;	
    };

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO
    {
        public UInt32 FingerCenterX;
        public UInt32 FingerCenterY;
        public UInt32 Hand;
        public UInt32 Finger; 
    };

    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    internal class GBFINIMGdll_NetWrapper
    {
        /// <summary>
        /// GBFINIMG_SetRotationBuffer1000DPI
        /// </summary>
        /// <param name="RotationBuffer"></param>
        /// <param name="RotationBufferSize"></param>
        [DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_SetRotationBuffer1000DPI")]
        public static extern void GBFINIMG_SetRotationBuffer1000DPI(
             IntPtr RotationBuffer,    // unsigned char	*
             UInt32	RotationBufferSize
        );
        /// <summary>
        /// GBFINIMG_Slap4_MissingFingers
        /// </summary>
        /// <param name="MissingFingerMask"></param>
        [DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_Slap4_MissingFingers")]
        public static extern void GBFINIMG_Slap4_MissingFingers(Byte MissingFingerMask);

        /// <summary>
        /// GBFINIMG_ProcessImage
        /// </summary>
        /// <param name="InputImage"></param>
        /// <param name="InputSizeX"></param>
        /// <param name="InputSizeY"></param>
        /// <param name="InputImageType"></param>
        /// <param name="ImageProcessingOption"></param>
        /// <param name="SegmentImageList"></param>
        /// <param name="SegmentImageSizeX"></param>
        /// <param name="SegmentImageSizeY"></param>
        /// <param name="DetectedSegmentImageNumber"></param>
        /// <param name="DetectionFailureDiagnostic"></param>
        /// <param name="ExternalInfoList"></param>
        /// <param name="ExternalInfoListSize"></param>
        /// <param name="TempBuffer"></param>
        /// <param name="TempBufferSize"></param>
        /// <returns></returns>
        [DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_ProcessImage")]
        public static extern int GBFINIMG_ProcessImage(
            IntPtr InputImage,	//unsigned char	*
            UInt32	InputSizeX,
            UInt32	InputSizeY,
            UInt32	InputImageType,
            UInt32	ImageProcessingOption,

            [In, Out] GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR [] SegmentImageList, // GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR * to a list

            UInt32	SegmentImageSizeX, 
            UInt32	SegmentImageSizeY, 
            out UInt32	DetectedSegmentImageNumber,
            out UInt32	DetectionFailureDiagnostic,

            [In, Out] GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO[] ExternalInfoList, // GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  *to a list
            UInt32	ExternalInfoListSize,

            IntPtr TempBuffer,
            UInt32 TempBufferSize);

        /// <summary>
        /// GBFINIMG_GetFingerprintCounter
        /// </summary>
        /// <returns></returns>
        [DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_GetFingerprintCounter")]
        public static extern int GBFINIMG_GetFingerprintCounter();

        /// <summary>
        /// GBFINIMG_CheckFingerRotation180Degree
        /// </summary>
        /// <returns></returns>
        [DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_CheckFingerRotation180Degree")]
        public static extern void GBFINIMG_CheckFingerRotation180Degree(out int Probability, out byte PatternDiagnostic);

		/// <summary>
		/// VER 2.2.0.2
		/// </summary>
		/// <param name="ParameterType"></param>
		/// <param name="ParameterValue"></param>
		[DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_SetInternalParameterValue")]
		public static extern void GBFINIMG_SetInternalParameterValue(int ParameterType, int ParameterValue);

		// VER 2.2.0.3
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
		public struct GBFINIMG_LICENSE_USB_SCANNER_CHECK
		{
			public Int32 Result;
			public UInt32 UsbDriverErrorCodeDetail;
		}

		[DllImport("GBFINIMG.dll", EntryPoint = "GBFINIMG_GetLicenseInfo")]
		public static extern void GBFINIMG_GetLicenseInfo(
			ref GBFINIMG_LICENSE_USB_SCANNER_CHECK ScannerBasedLicense
			);
		// end VER 2.2.0.3
    }

    /************************************************************
     ************************************************************
     *
     * Constants
     * 
     ************************************************************
    ************************************************************/
    public class GBFINIMG_DiagConstants
    {
        public const UInt32 GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX = 0x00000001;
        public const UInt32 GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP = 0x00000002;
        public const UInt32 GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP = 0x00000004;
    }

    public class GBFINIMG_FingerConstants
    {
        public const UInt32 GBFINIMG_FINGER_UNDEFINED = 0;
        public const UInt32 GBFINIMG_FINGER_THUMB = 1;
        public const UInt32 GBFINIMG_FINGER_INDEX = 2;
        public const UInt32 GBFINIMG_FINGER_MIDDLE = 3;
        public const UInt32 GBFINIMG_FINGER_RING = 4;
        public const UInt32 GBFINIMG_FINGER_LITTLE = 5;
    }

    public class GBFINIMG_HandConstants
    {
        public const UInt32 GBFINIMG_HAND_UNDEFINED = 0;
        public const UInt32 GBFINIMG_HAND_LEFT = 1;
        public const UInt32 GBFINIMG_HAND_RIGHT = 2;
    }

    public class GreenBit_IBIA_Algorithms
    {
        // Green Bit Biometric Organization Identifier at IBIA
        public const UInt16 GB_IBIA_IDENT = 0x0040;
        // Green Bit algorithm IDs
        public const UInt16 GB_QUALITY_ALG_ID = 0x0001;
        public const UInt16 GB_SEGMENTATION_ALG_ID = 0x0002;
        public const UInt16 GB_WSQ_ALG_ID = 0x0003;
        public const UInt16 GB_JPEG_ALG_ID = 0x0004;
        public const UInt16 GB_JPEG2000_ALG_ID = 0x0005;
    }

    public class GBTempBuffConstants
    {
		// VER 2.2.0.2: changed to 21299031
		public const int GBFINIMG_MEMORY_BUFFER_SIZE = 21299031;
    }

    /************************************************************
     ************************************************************
     *
     * Globals
     * 
     ************************************************************
    ************************************************************/
    public class GBFINIMG_Globals
    {
        public static IntPtr RotationBuffer1000Dpi = IntPtr.Zero;
        public static UInt32 RotationBufferSize = 0;
    }

    /************************************************************
     ************************************************************
     *
     * General utilities
     * 
     ************************************************************
    ************************************************************/
    public class GBFINIMG_Utils
    {
        public static String GetAn2kFingPos(UInt32 Hand,UInt32 Finger)
        {
            if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_UNDEFINED ||
                    Hand == GBFINIMG_HandConstants.GBFINIMG_HAND_UNDEFINED)
            {
                return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN;
            }
            
            // case right hand
            if (Hand == GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT)
            {
                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_THUMB)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_INDEX)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_MIDDLE)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_RING)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_LITTLE)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                }
            }

            // case left hand
            if (Hand == GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT)
            {
                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_THUMB)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_INDEX)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_MIDDLE)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_RING)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                }

                if (Finger == GBFINIMG_FingerConstants.GBFINIMG_FINGER_LITTLE)
                {
                    return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                }
            }
            
            // no match
            return NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN;
        }

        public static void SetHandFingFromAn2kFingerPos(String value, out UInt32 Hand, out UInt32 Finger)
        {
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_UNDEFINED;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_UNDEFINED;
                return;
            }
            //////////////
            // left hand
            //////////////
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_INDEX;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_LITTLE;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_MIDDLE;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_RING;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB ||
                value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_PLAIN_LEFT_THUMB)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_LEFT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_THUMB;
                return;
            }

            //////////////
            // right hand
            //////////////
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_INDEX;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_LITTLE;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_MIDDLE;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_RING;
                return;
            }
            if (value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB ||
                value == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_PLAIN_RIGHT_THUMB)
            {
                Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_RIGHT;
                Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_THUMB;
                return;
            }

            Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_UNDEFINED;
            Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_UNDEFINED;
            return;
        }
    }
}

namespace GBFINIMG_NET_WRAPPER
{
    
	/**********************************************************************/
	/**********************************************************************/
	//**********			SECTION 1: CONSTANTS		******************//
	/**********************************************************************/
	/**********************************************************************/
	/// <summary> Error codes </summary>
	public  class NW_GBFINIMG_ErrorCodes
	{
        public static Int32	NW_GBFINIMG_NO_ERROR					= 0;
		public static Int32	NW_GBFINIMG_ERROR_PARAMETER				= 1;
		public static Int32	NW_GBFINIMG_ERROR_UNAVAILABLE_OPTION	= 2;
		// VER 2000
		public static Int32	NW_GBFINIMG_ERROR_NO_MEMORY				= 3;
		// VER 2.2.0.3
		public static Int32 NW_GBFINIMG_ERROR_LICENSE_NOT_GRANTED = 4;
	};

	// VER 2.2.0.3
	/// <summary> License info error details </summary>
	public class NW_GBFINIMG_UsbDriverErrorCodeValues
	{
		public const UInt32 NoError = 0;
		public const UInt32 UsbDriver = 1;
		public const UInt32 UsbDeviceNotFound = 2;
	}
	// end VER 2.2.0.3

	/// <summary> Limits for input parameters </summary>
	public  class NW_GBFINIMG_PARAMETER_LIMITS
	{
		/// <summary> in pixel: 3.3" at 500dpi </summary>
        public static Int32 NW_GBFINIMG_MAX_INPUT_SIZE_X = 2520;
		// VER 2000
        public static Int32 NW_GBFINIMG_MAX_INPUT_SIZE_X_1000DPI = NW_GBFINIMG_MAX_INPUT_SIZE_X * 2;
		/// <summary> in pixel: 3.0" at 500dpi </summary>
        public static Int32 NW_GBFINIMG_MAX_INPUT_SIZE_Y = 2520;
		// VER 2000
        public static Int32 NW_GBFINIMG_MAX_INPUT_SIZE_Y_1000DPI = NW_GBFINIMG_MAX_INPUT_SIZE_Y * 2;
	};

	/// <summary> Values for the input image type </summary>
	public  class NW_GBFINIMG_IMAGE_TYPES
	{
		/// <summary> LEFT HAND SLAP 4:  INDEX+MIDDLE+RING+LITTLE </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4	= 1;
		/// <summary> RIGHT HAND SLAP 4: INDEX+MIDDLE+RING+LITTLE </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4	= 2;		
		/// <summary> THUMB LEFT + THUMB RIGHT </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2		= 3;
		/// <summary> FLAT OR ROLLED </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER	= 4;		
		/// <summary> LEFT HAND SLAP 2:  INDEX+MIDDLE </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2	= 5;
		/// <summary> RIGHT HAND SLAP 2: INDEX+MIDDLE </summary>
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2	= 6;

		// VER 2000
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM	= 7;
		// VER 2000
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM	= 8;
		// VER 2000
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_WRITER_PALM	= 9;
		// VER 2000
		public static UInt32	NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_WRITER_PALM	= 10;
        // VER 2102
        public static UInt32    NW_GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER_FLAT = 11;
	};

	/// <summary> Segmentation options </summary>
	public  class NW_GBFINIMG_SEGMENTATION_OPTIONS
	{
		/// <summary> Wipes halo-effect/latent-prints around fingerprint segments in the input image. </summary>
        public static UInt32 NW_GBFINIMG_HALO_LATENT_ELIMINATION = 0x00000001;
		/// <summary> Rotates segmented images to the vertical position in the segmented images. </summary>
        public static UInt32 NW_GBFINIMG_SEGMENT_VERTICAL_ROTATION = 0x00000002;

		// VER 2000
        public static UInt32 NW_GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS = 0x00000004;
		// VER 2000
        public static UInt32 NW_GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION = 0x00000008;
		// VER 2000
        public static UInt32 NW_GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING = 0x00000010;
		// VER 2003
        public static UInt32 NW_GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY = 0x00000020;
		// VER 2005
        public static UInt32 NW_GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK = 0x00000040;
		// VER 2006
        public static UInt32 NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA = 0x00000080;
        // VER 2.2.0.1 2201
        public static UInt32 NW_GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION = 0x00000100;
		// VER 2.2.0.2
		public static UInt32 NW_GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE = 0x00000200;
		// VER 2.2.0.2
		public static UInt32 NW_GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE = 0x00000400;
		// VER 2.2.0.2
		public static UInt32 NW_GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND = 0x00000800;
	};

	/// <summary>
	/// VER 2006
	/// Missing Fingers in a Slap-4 object
	/// </summary>
	public  class NW_GBFINIMG_MISSING_FINGER_SLAP4
	{
		public static Byte	NW_GBFINIMG_MISSING_FINGER_SLAP4_INDEX	= 0x01;
        public static Byte NW_GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE = 0x02;
        public static Byte NW_GBFINIMG_MISSING_FINGER_SLAP4_RING = 0x04;
        public static Byte NW_GBFINIMG_MISSING_FINGER_SLAP4_LITTLE = 0x08;
	};

	// VER 2000
	public  class NW_GBFINIMG_ROTATION_FACTOR
	{
        public static Double NW_GBFINIMG_ROTATION_BUFFER_FACTOR = 3.3;
	};

	/// <summary> Diagnostic word </summary>
	/// <remarks>
	/// Properties of this class have no direct equivalent in the C version of the library.
	/// It corresponds to the "DetectionFailureDiagnostic" output parameter of the "GBFINIMG_ProcessImage" in the C version,
	/// and the properties, if true, state that the correspondent bit (see detailed explanation for each property) in
	/// "DetectionFailureDiagnostic" is set.
	/// </remarks>
    public class NW_GBFINIMG_SegmentationFailureDiagnostic
    {
        protected UInt32 DiagnosticWord;

        /// <summary> Default Constructor </summary>
        public NW_GBFINIMG_SegmentationFailureDiagnostic()
        {
            this.DiagnosticWord = 0;
        }
        /// <summary> Overloaded Constructor
        /// <para>
        /// <paramref name="DW" />Diagnostic Word to be set
        /// </para>
        /// </summary>
        public NW_GBFINIMG_SegmentationFailureDiagnostic(UInt32 DW)
        {
            this.DiagnosticWord = DW;
        }
        /// <summary> possible left 2 right hand mix </summary>
        /// <remarks>
        /// It's true in all the cases when the "GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX" bit is set in"DetectionFailureDiagnostic"
        /// </remarks>
        public Boolean NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX
        {
            get
            {
                if ((DiagnosticWord & GBFINIMG_DiagConstants.GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX) != 0)
                {
                    return true;
                }
                else return false;
            }
        }
        /// <summary> one finger(s) of slap have not been detected </summary>
        /// <remarks>
        /// It's true in all the cases when the "GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP" bit is set in"DetectionFailureDiagnostic"
        /// </remarks>
        public Boolean NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP
        {
            get
            {
                if ((DiagnosticWord & GBFINIMG_DiagConstants.GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP) != 0)
                {
                    return true;
                }
                else return false;
            }
        }
        /// <summary> ambiguous segment naming; probably, scanned object does not correspond to the requested one </summary>
        /// <remarks>
        /// It's true in all the cases when the "GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP" bit is set in"DetectionFailureDiagnostic"
        /// </remarks>
        public Boolean NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP
        {
            get
            {
                if ((DiagnosticWord & GBFINIMG_DiagConstants.GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP) != 0)
                {
                    return true;
                }
                else return false;
            }
        }

        public Boolean NW_GBFINIMG_NET_WRAPPER_NO_DIAG
        {
            get
            {
                if (this.DiagnosticWord == 0)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }
    }

    

	/**********************************************************************/
	/**********************************************************************/
	//**********			SECTION 2: CLASSES		**********************//
	/**********************************************************************/
	/**********************************************************************/
	/// <summary> External Info:
	/// This is the equivalent of the "GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO" structure in the C version of the GBFINIMG library
	/// <para>
	/// This structure is used to pass externally specified fingerprint segment position and name to run 
	/// GBFINIMG_ProcessImage() in SEMI-AUTOMATIC mode.
	/// This position should refer to the top phalange of a fingerprint.
	/// Application is responsible to provide correct fingerprint segment position and naming
	/// (normally, an  operator points to the segment inside image using some graphical pointing tool).
	/// </para>
	/// </summary>
	public  class NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO
	{
	    public UInt32 Hand;
		public UInt32 Finger;
		public UInt32 FCX;
		public UInt32 FCY;

		/// <summary> Default Constructor </summary>
        public NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO()
        {
            this.Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_UNDEFINED;
            this.Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_UNDEFINED;
            this.FCX = 0;
            this.FCY = 0;
        }

		/// <summary> Sets and gets the current FingerID </summary>
		/// <remarks>
		/// It is the equivalent of the "Hand" and "Finger" members of the "GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO" structure in the 
		/// C version taken together. The values that this class can assume are taken from the 
		/// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS" /> class.
		/// For example, if in a "GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO"
		/// structure "Hand" equals to "GBFINIMG_HAND_LEFT" and "Finger" equals to "GBFINIMG_FINGER_THUMB", than
		/// <see cref = "FingerID"/> equals to <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB"/>.
		/// </remarks>
		public String FingerID {
			/// <summary>
			/// <returns> a value from <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS" /> </returns>
			/// </summary>
			get
            {
                return GBFINIMG_Utils.GetAn2kFingPos(this.Hand, this.Finger);
            }
			/// <summary>
			/// <param name="FingPos" >
			/// Should be one from <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS" />
			/// </param>
			/// </summary>
			set
            {
                GBFINIMG_Utils.SetHandFingFromAn2kFingerPos(value, out this.Hand, out this.Finger);
            }
		}

		/// <summary> Sets and gets the current X coordinate of the finger center </summary>
		public UInt32 FingerCenterX {
			/// <summary>
			/// <returns> the current X coordinate of the finger center </returns>
			/// </summary>
			get
            {
                return this.FCX;
            }
			/// <summary>
			/// <param name="FingCX">
			/// Value of the current X coordinate of the finger center
			/// </param>
			/// </summary>
			set
            {
                this.FCX = value;
            }
		}

		/// <summary> Sets and gets the current Y coordinate of the finger center </summary>
		public UInt32 FingerCenterY {
			/// <summary>
			/// <returns> the current Y coordinate of the finger center </returns>
            /// </summary>
            get
            {
                return this.FCY;
            }
			/// <summary>
			/// <param name="FingCY"> 
			/// Value of the current Y coordinate of the finger center
			/// </param>
			/// </summary>
			set
            {
                this.FCY = value;
            }
		}
	};

    /// <summary> Segment Image Descriptor:
    /// This is the equivalent of the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR" structure in the C version of the GBFINIMG library
    /// <para>
    /// This structure is used to describe segmented image of a fingerprint generated by 
    /// GBFINIMG_ProcessImage() as a result of multi-or-single-finger input image processing. 
    /// All information refers to the top phalange of a fingerprint.
    /// </para>
    /// </summary>
    public class NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR
    {
        public UInt32			Hand;
        public UInt32			Finger;
        public UInt32			FCX;
        public UInt32			FCY;
        public Byte[]	        Frame;
        public UInt32			mm2Size;
        public Int32			SegInclination;
        public UInt32			SegPatternValidity;
        public UInt32			SegPatternCompletness;
        public UInt32			SegQuality;
        // bounding box data is generated in correspondence with 
        // ELECTRONIC FINGERPRINT TRANSMISSION SPECIFICATION (May 2, 2005) Appendix N.
        public Int32			SegBoundingBoxL;
        public Int32			SegBoundingBoxR;
        public Int32			SegBoundingBoxT;
        public Int32			SegBoundingBoxB;

        /// <summary> Default Constructor </summary>
        public NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR()
        {
            this.Finger = GBFINIMG_FingerConstants.GBFINIMG_FINGER_UNDEFINED;
            this.Hand = GBFINIMG_HandConstants.GBFINIMG_HAND_UNDEFINED;
            this.FCX = 0;
            this.FCY = 0;
            this.Frame = null;
            this.mm2Size = 0;
            this.SegInclination = 0;
            this.SegPatternCompletness = 0;
            this.SegPatternValidity = 0;
            this.SegQuality = 0;
            this.SegBoundingBoxB = 0;
            this.SegBoundingBoxL = 0;
            this.SegBoundingBoxR = 0;
            this.SegBoundingBoxT = 0;
            return;
        }

        /// <summary> Gets the current FingerID </summary>
        /// <remarks>
        /// It is the equivalent of the "Hand" and "Finger" members of the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR" structure in the 
        /// C version taken together. The values that this class can assume are taken from the 
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS" /> class.
        /// For example, if in a "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR"
        /// structure "Hand" equals to "GBFINIMG_HAND_LEFT" and "Finger" equals to "GBFINIMG_FINGER_THUMB", than
        /// <see cref = "FingerID"/> equals to <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB"/>.
        /// </remarks>
        public String  FingerID {
            /// <summary>
            /// <returns> a value from <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS" /> </returns>
            /// </summary>
            get
            {
                return GBFINIMG_Utils.GetAn2kFingPos(this.Hand, this.Finger);
            }
        }

        /// <summary> Gets the current X coordinate of the finger center </summary>
        public UInt32 FingerCenterX {
            /// <summary>
            /// <returns> the current X coordinate of the finger center </returns>
            /// </summary>
            get
            {
                return this.FCX;
            }
        }

        /// <summary> Gets the current Y coordinate of the finger center </summary>
        public UInt32 FingerCenterY {
            /// <summary>
            /// <returns> the current Y coordinate of the finger center </returns>
            /// </summary>
            get
            {
                return this.FCY;
            }
        }

        /// <summary> Gets the current Segment Image </summary>
        public Byte []  SegmentImagePtr {
            /// <summary>
            /// <returns> the current Segment Image </returns>
            /// </summary>
            get
            {
                return this.Frame;
            }
        }

        /// <summary> Gets the current Segment Size in mm2 </summary>
        public UInt32 Size {
            /// <summary>
            /// <returns> the current Segment Size in mm2 </returns>
            /// </summary>
            get
            {
                return this.mm2Size;
            }
        }

        /// <summary> Gets the current Segment Inclination in degrees </summary>
        public Int32 Inclination {
            /// <summary>
            /// <returns> the current Segment Inclination in degrees </returns>
            /// </summary>
            get
            {
                return this.SegInclination;
            }
        }

        /// <summary> Gets the current Pattern Validity </summary>
        public UInt32 PatternValidity {
            /// <summary>
            /// <returns> the current Pattern Validity in range [0-100] </returns>
            /// </summary>
            get
            {
                return this.SegPatternValidity;
            }
        }

        /// <summary> Gets the current Pattern Completness </summary>
        public UInt32 PatternCompletness {
            /// <summary>
            /// <returns> the current Pattern Completness in range [0-100] </returns>
            /// </summary>
            get
            {
                return this.SegPatternCompletness;
            }
        }

        /// <summary> Gets the current Segmentation quality structure </summary>
        /// <remarks>
        /// The <see cref = "SegmQuality"/> member can be considered as the equivalent of the 
        /// "Quality" member in the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR" of the C version. It is defined
        /// as a <see cref = "AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL"/> class, where the
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL.fing_id"/> member is equal to
        /// <see cref = "FingerID"/> and the 
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL.quality"/> member is equal to
        /// the "Quality" member of the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR" of the C version.
        /// </remarks>
        public AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL SegmQuality {
            /// <summary>
            /// <returns> the current Segmentation quality structure </returns>
            /// <para> Values for vendor and algorithm are from Green Bit identifiers within IBIA </para>
            /// </summary>
            get
            {
                AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL RetVal = new NW_AN2K_ALT_QUAL(
                    this.FingerID,
                    (Byte)(this.SegQuality),
                    GreenBit_IBIA_Algorithms.GB_IBIA_IDENT,
                    GreenBit_IBIA_Algorithms.GB_QUALITY_ALG_ID
                );
                
                return RetVal;
            }
        }

        /// <summary> Gets the current Segment bounding box coordinates </summary>
        /// <remarks>
        /// The <see cref = "SegmQuality"/> member can be considered as the equivalent of the 
        /// "BoundingBoxL", "BoundingBoxR", "BoundingBoxT", "BoundingBoxB" members in the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR"
        /// of the C version taken together. It is defined
        /// as a <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT"/> class, where the
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT.fing_id"/> member is equal to
        /// <see cref = "FingerID"/> and the 
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT.bottom_bound"/>,
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT.left_bound"/>,
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT.right_bound"/>,
        /// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT.top_bound"/>
        /// members are equal respectively to the "BoundingBoxB","BoundingBoxL", "BoundingBoxR", "BoundingBoxT"
        /// members of the "GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR" of the C version.
        /// </remarks>
        public AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT SegmBound {
            /// <summary>
            /// <returns> the current Segment bounding box coordinates </returns>
            /// </summary>
            get
            {
                NW_AN2K_FING_SEGMENT RetVal = new NW_AN2K_FING_SEGMENT(
			        this.FingerID,
			        (Int16)(this.SegBoundingBoxL),
			        (Int16)(this.SegBoundingBoxR),
			        (Int16)(this.SegBoundingBoxT),
			        (Int16)(this.SegBoundingBoxB)
			    );
                
                return RetVal;
            }
        }

	};

	// VER 2.2.0.3
	public class NW_GBFINIMG_LicenseUSBScannerCheck
	{
		public NW_GBFINIMG_LicenseUSBScannerCheck()
		{
			this.Result = 0;
			this.UsbDriverErrorCodeDetail = 0;
		}

		public Int32 Result;
		public UInt32 UsbDriverErrorCodeDetail;
	}
	// end VER 2.2.0.3

	/**********************************************************************/
	/**********************************************************************/
	//**********			SECTION 3: METHODS		**********************//
	/**********************************************************************/
	/**********************************************************************/

	/// <summary> Input Data:
	/// This structure collects data that can be used as an input for
	/// purchasing finger segmentation
	/// </summary>
	/// <remarks>
	/// Members of this class correspond to the input parameters of the "GBFINIMG_ProcessImage" function.
	/// </remarks>
	public class NW_GBFINIMG_InputData
	{
		protected Byte[] Frame;
		protected UInt32		SizeX;
		protected UInt32		SizeY;
		protected UInt32		Type;
		protected UInt32		Option;
		protected UInt32		SegSizeX;
		protected UInt32		SegSizeY;
		protected List<NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO > ExtInfList;

		/// <summary> Constructor overload:
		/// <param name ="Img" > Input image</param>
		/// <param name ="SX" > Input image Size X</param>
		/// <param name ="SY" > Input image Size Y</param>
		/// <param name ="T" > Input image type: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_IMAGE_TYPES"/></param>
		/// <param name ="EIList" > Input External Info list: if not requested set to null</param>
		/// <param name ="SO" > Segmentation Option</param>
		/// <param name ="SSX" > Segmented Image Size X</param>
		/// <param name ="SSY" > Segmented Image Size Y</param>
		/// </summary>
        public NW_GBFINIMG_InputData(Byte[] Img,
            UInt32 SX,
            UInt32 SY,
            UInt32 T,
            List<NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO> EIList,
            UInt32 SO,
            UInt32 SSX,
            UInt32 SSY
            )
        {
            this.Frame = Img;
            this.SizeX = SX;
            this.SizeY = SY;
            this.Type = T;
            this.ExtInfList = EIList;
            this.Option = SO;
            this.SegSizeX = SSX;
            this.SegSizeY = SSY;
        }

		/// <summary> set and get:
		/// <para> <paramref name ="Img" /> Input image</para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "InputImage" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public Byte [] ImageFrame {
            set
            {
                this.Frame = value;
            }
            get
            {
                return this.Frame;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="SX" /> Input image Size X</para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "InputSizeX" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public UInt32	ImageSizeX {
            set
            {
                this.SizeX = value;
            }
            get
            {
                return this.SizeX;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="SY" /> Input image Size Y</para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "InputSizeY" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public UInt32	ImageSizeY {
            set
            {
                this.SizeY = value;
            }
            get
            {
                return this.SizeY;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="T" /> Input image type: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_IMAGE_TYPES"/></para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "InputImageType" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public UInt32	ImageType {
            set
            {
                this.Type = value;
            }
            get
            {
                return this.Type;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="EIList" /> Input External Info list: if not requested set to null</para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "ExternalInfoList" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public List<NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO >  ExternalInfoList {
            set
            {
                this.ExtInfList = value;
            }
            get
            {
                return this.ExtInfList;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="SO" /> Segmentation Option: 
		/// <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENTATION_OPTIONS"/>
		/// </para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "ImageProcessingOption" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public UInt32	SegmentationOption {
            set
            {
                this.Option = value;
            }
            get
            {
                return this.Option;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="SSX" /> Segmented Images Size X:
		/// Should be less or equal to <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData.ImageSizeX"/>
		/// </para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "SegmentImageSizeX" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
        public UInt32 SegmentImageSizeX
        {
            set
            {
                this.SegSizeX = value;
            }
            get
            {
                return this.SegSizeX;
            }
		}
		/// <summary> set and get:
		/// <para> <paramref name ="SSY" /> Segmented Images Size Y:
		/// Should be less or equal to <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData.ImageSizeY"/>
		/// </para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "SegmentImageSizeY" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
        public UInt32 SegmentImageSizeY
        {
            set
            {
                this.SegSizeY = value;
            }
            get
            {
                return this.SegSizeY;
            }
		}
	};

	/// <summary> Output Data:
	/// This structure collects data that returned after a finger segmentation
	/// </summary>
	/// <remarks>
	/// Members of this class correspond to the output parameters of the "GBFINIMG_ProcessImage" function.
	/// </remarks>
	public class NW_GBFINIMG_OutputData
	{
		// VER 2003: added segment number
		/// <summary> Segment_Number:
		/// Number of NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR items
		/// </summary>
		/// <remarks>
		/// Corresponds to the "DetectedSegmentImageNumber" parameter of the "GBFINIMG_ProcessImage" function.
		/// The <see cref = "List.Count"/> of <see cref = "Segment_List"/> corresponds to the "DetectedSegmentImageNumber" parameter
		/// of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public UInt32	Segment_Number;
		/// <summary> Segment_List:
		/// List of NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR items, where
		/// segmentation results are given for each finger
		/// </summary>
		/// <remarks>
		/// Corresponds to the "SegmentImageList" parameter of the "GBFINIMG_ProcessImage" function.
		/// The <see cref = "List.Count"/> of <see cref = "Segment_List"/> corresponds to the "DetectedSegmentImageNumber" parameter
		/// of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public List<NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR > Segment_List;
		/// <summary> Diag:
		/// <para> Diagnostic messages returned after the finger segmentation </para>
		/// <para> <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SegmentationFailureDiagnostic"/></para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the "DetectionFailureDiagnostic" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public NW_GBFINIMG_SegmentationFailureDiagnostic Diag;

		// VER 2000
		/// <summary> OutputFrame:
		/// <para> image frame returned after the finger segmentation </para>
		/// <para> <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SegmentationFailureDiagnostic"/></para>
		/// </summary>
		/// <remarks>
		/// Corresponds to the I/O "InputImage" parameter of the "GBFINIMG_ProcessImage" function.
		/// </remarks>
		public Byte [] OutputFrame;
		// end VER 2000
	};
	/// <summary> Segment Maker:
	/// This is the class that performs the segmentation
	/// </summary>
	public class NW_GBFINIMG_ProcessImage
	{
	    protected NW_GBFINIMG_InputData FinImg_Input;
	
		public NW_GBFINIMG_ProcessImage()
		{
            return;
		}

		// VER 2006
		/// <summary> GBFINIMG_Slap4_MissingFingers wrapper		
		/// </summary>
		/// <param name ="MissingFingerMask">can be a "|" combination of the values from NW_GBFINIMG_MISSING_FINGER_SLAP4 class</param>
		/// <returns> : </returns>
        public void NW_GBFINIMG_Slap4_MissingFingers(Byte MissingFingerMask)
        {
            GBFINIMGdll_NetWrapper.GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
            return;
        }

		// VER 2005
		/// <summary> GBFINIMG_GetFingerprintCounter wrapper		
		/// </summary>
		/// <returns> ExpectedFingerprintNumber: </returns>
        public int NW_GBFINIMG_GetFingerprintCounter()
        {
            return GBFINIMGdll_NetWrapper.GBFINIMG_GetFingerprintCounter();
        }

        // VER 2200
        /// <summary>
        /// 
        /// </summary>
        /// <param name="Probability"></param>
        /// <param name="PatternDiagnostic"></param>
        public void NW_GBFINIMG_CheckFingerRotation180Degree(out int Probability, out byte PatternDiagnostic)
        {
            GBFINIMGdll_NetWrapper.GBFINIMG_CheckFingerRotation180Degree(out Probability, out PatternDiagnostic);
        }
		/// <summary> Constructor overload:		
		/// </summary>
		/// <param name ="Inp"> Input data: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData"/></param>
		/// <exception cref="System.Exception">Thrown when <paramref name ="Inp" /> is null</exception>
        public NW_GBFINIMG_ProcessImage(NW_GBFINIMG_InputData Inp)
        {
		    if (Inp == null)
		    {
			    // throw exception
			    Exception ex = new Exception("NW_GBFINIMG_ProcessImage consturctor exception: Inp parameter cannot be null");
			    throw(ex);
		    }
		    else
		    {
			    this.FinImg_Input = Inp;
		    }
        }
		/// <summary> Output data:
		/// </summary>
		/// <returns>Output data: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_OutputData"/></returns>
		/// <exception cref="System.Exception">Thrown when an error occurs. See exception message</exception>
		/// <remarks>
		/// Getting this is equivalent to calling the "GBFINIMG_ProcessImage" function of the C version of the library.
		/// </remarks>
		public NW_GBFINIMG_OutputData OutputAfterProcess {
            get
            {
                if (this.FinImg_Input == null)
                {
                    // throw exception
                    Exception ex = new Exception(
                        "OutputAfterProcess exception: Input parameters are set to null, try setting them again");
                    throw(ex);
                }
                
                //////////////////
                // image input
                //////////////////
                IntPtr InpImg = Marshal.AllocHGlobal(this.FinImg_Input.ImageFrame.Length);
                if (InpImg == IntPtr.Zero)
                {
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }
                
                Marshal.Copy(this.FinImg_Input.ImageFrame,0,InpImg,this.FinImg_Input.ImageFrame.Length);

                /////////////////////
                // Temporary buffer
                /////////////////////
                IntPtr TempBuffer = Marshal.AllocHGlobal((int)GBTempBuffConstants.GBFINIMG_MEMORY_BUFFER_SIZE);
                if (TempBuffer == null)
                {
                    // free memory
                    Marshal.FreeHGlobal(InpImg);
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }

                ///////////////////////////
                // segment descriptor list
                ///////////////////////////
                GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR  [] SegmentImageList = new GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR [4];
                
                if (SegmentImageList == null)
                {
                    Marshal.FreeHGlobal(InpImg);
                    Marshal.FreeHGlobal(TempBuffer);
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }

                // segment descriptors allocation
                for (int i=0;i<4;i++)
                {
                    SegmentImageList[i] = new GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR();
                    if ((this.FinImg_Input.SegmentationOption & 
                        NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                    )
                    {
                        SegmentImageList[i].SegmentImagePtr = Marshal.AllocHGlobal(
                            (int)(
                                (this.FinImg_Input.SegmentImageSizeX) *
                                (this.FinImg_Input.SegmentImageSizeY)
                            )
                        );
                        if (SegmentImageList[i].SegmentImagePtr == null)
				        {
					        // free memory
					        for (int j=0;j<i;j++)
					        {
						        Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
					        }
					        Marshal.FreeHGlobal(InpImg);
                            Marshal.FreeHGlobal(TempBuffer);
					        // throw exception
					        Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
					        throw(ex);
				        }
                    }
                }

                /////////////////////////
                // External info list
                /////////////////////////                
                GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  [] ExternalInfoList = null;
                UInt32 ExternalInfoListSize = 0;
                if (this.FinImg_Input.ExternalInfoList != null)
                {
                    ExternalInfoListSize = (UInt32)this.FinImg_Input.ExternalInfoList.Count;
                    ExternalInfoList = new GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO[ExternalInfoListSize];
                    if (ExternalInfoList == null)
                    {
                        // free memory
                        if ((this.FinImg_Input.SegmentationOption &
                            NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                        )
                        {
                            for (int j = 0; j < 4; j++)
                            {
                                Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                            }
                        }
				        Marshal.FreeHGlobal(InpImg);
                        Marshal.FreeHGlobal(TempBuffer);
				        // throw exception
				        Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
				        throw(ex);
                    }
                    for (int i=0; i<(int)ExternalInfoListSize; i++)
                    {
                        ExternalInfoList[i].FingerCenterX = this.FinImg_Input.ExternalInfoList[i].FingerCenterX;
                        ExternalInfoList[i].FingerCenterY = this.FinImg_Input.ExternalInfoList[i].FingerCenterY;
                        ExternalInfoList[i].Finger = this.FinImg_Input.ExternalInfoList[i].Finger;
                        ExternalInfoList[i].Hand = this.FinImg_Input.ExternalInfoList[i].Hand;
                    }
                }
                
                ////////////////////////////////////
                // Call process image function
                ////////////////////////////////////
                UInt32 DetectedSegmentImageNumber,DetectionFailureDiagnostic;

                int RetVal = GBFINIMGdll_NetWrapper.GBFINIMG_ProcessImage(
                    InpImg,	//unsigned char	*
                    this.FinImg_Input.ImageSizeX,
                    this.FinImg_Input.ImageSizeY,
                    this.FinImg_Input.ImageType,
                    this.FinImg_Input.SegmentationOption,

                    SegmentImageList, // GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR * to a list

                    this.FinImg_Input.SegmentImageSizeX, 
                    this.FinImg_Input.SegmentImageSizeY, 
                    out DetectedSegmentImageNumber,
                    out DetectionFailureDiagnostic,

                    ExternalInfoList, // GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  *to a list
                    ExternalInfoListSize,

                    TempBuffer,
                    GBTempBuffConstants.GBFINIMG_MEMORY_BUFFER_SIZE
                );
                
                // check errors
                if (RetVal != NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_NO_ERROR)
                {
                    // free memory
                    if ((this.FinImg_Input.SegmentationOption &
                            NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                        )
                    {
                        for (int j = 0; j < 4; j++)
                        {
                            Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                        }
                    }
			        Marshal.FreeHGlobal(InpImg);
                    Marshal.FreeHGlobal(TempBuffer);
                    // throw exception
                    Exception ex = new Exception(
                        "OutputAfterProcess exception: Low level GBFINIMG_ProcessImage() function returned " + RetVal);
                    throw(ex);
                }
                
                // copy data in output
                NW_GBFINIMG_OutputData ObjToReturn = new NW_GBFINIMG_OutputData();
                ObjToReturn.Diag = new NW_GBFINIMG_SegmentationFailureDiagnostic(DetectionFailureDiagnostic);
                ObjToReturn.Segment_Number = DetectedSegmentImageNumber;
                if (DetectedSegmentImageNumber != 0)
                {
                    ObjToReturn.Segment_List = new List <NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR >();
                    if (ObjToReturn.Segment_List == null)
                    {
                        // free memory
                        if ((this.FinImg_Input.SegmentationOption &
                            NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                        )
                        {
                            for (int j = 0; j < 4; j++)
                            {
                                Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                            }
                        }
			            Marshal.FreeHGlobal(InpImg);
                        Marshal.FreeHGlobal(TempBuffer);
                        // throw exception
                        Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                        throw(ex);
                    }

                    for (int i = 0;i<(int)DetectedSegmentImageNumber;i++)
                    {
                        NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR Item = new NW_GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR();
                        Item.FCX			= ((SegmentImageList[i])).FingerCenterX;
                        Item.FCY			= ((SegmentImageList[i])).FingerCenterY;
                        Item.SegBoundingBoxB	= ((SegmentImageList[i])).BoundingBoxB;
                        Item.SegBoundingBoxL	= ((SegmentImageList[i])).BoundingBoxL;
                        Item.SegBoundingBoxR	= ((SegmentImageList[i])).BoundingBoxR;
                        Item.SegBoundingBoxT	= ((SegmentImageList[i])).BoundingBoxT;
                        Item.Finger		= ((SegmentImageList[i])).Finger;
                        Item.Hand					= ((SegmentImageList[i])).Hand;
                        Item.SegInclination		= ((SegmentImageList[i])).Inclination;
                        Item.SegPatternCompletness	= ((SegmentImageList[i])).PatternCompleteness;
                        Item.SegPatternValidity	= ((SegmentImageList[i])).PatternValidity;
                        Item.SegQuality			= ((SegmentImageList[i])).Quality;
                        Item.mm2Size				= ((SegmentImageList[i])).Size;
                        // size
                        if ((this.FinImg_Input.SegmentationOption &
                            NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                        )
                        {
                            Item.Frame = new Byte[ ((this.FinImg_Input.SegmentImageSizeX)
                                *(this.FinImg_Input.SegmentImageSizeY))];
                            Marshal.Copy(SegmentImageList[i].SegmentImagePtr, Item.Frame, 0, 
                                (int)((this.FinImg_Input.SegmentImageSizeX)
                                *(this.FinImg_Input.SegmentImageSizeY)));
                        }
                        else
                        {
                            Item.Frame = null;
                        }
                        ObjToReturn.Segment_List.Add(Item);
                    }
                }
                
                // VER2000
                ObjToReturn.OutputFrame = new Byte[(this.FinImg_Input.ImageFrame.Length)];
                if (ObjToReturn.OutputFrame == null)
                {
                    // free memory
                    if ((this.FinImg_Input.SegmentationOption &
                            NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                        )
                    {
                        for (int j = 0; j < 4; j++)
                        {
                            Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                        }
                    }
                    Marshal.FreeHGlobal(InpImg);
                    Marshal.FreeHGlobal(TempBuffer);

                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: memory allocation failed");
                    throw(ex);
                }

                Marshal.Copy(InpImg,ObjToReturn.OutputFrame,0,this.FinImg_Input.ImageFrame.Length);
                // end VER2000
                
                // free memory
                if ((this.FinImg_Input.SegmentationOption &
                                    NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA) == 0
                                )
                {
                    for (int j = 0; j < 4; j++)
                    {
                        Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                    }
                }
                Marshal.FreeHGlobal(InpImg);
                Marshal.FreeHGlobal(TempBuffer);

                // return normally
                return ObjToReturn;
            }
		}
		/// <summary> Output data:
		/// </summary>
		/// <returns>Output data: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_OutputData"/></returns>
		/// <exception cref="System.Exception">Thrown when an error occurs. See exception message</exception>
		/// <remarks>
		/// Getting this is equivalent to calling the "GBFINIMG_ProcessImage" with
		/// the "GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY" option of the C version of the library.
		/// </remarks>
		public NW_GBFINIMG_OutputData OutputAfterFastProcess {
            get
            {
                if (this.FinImg_Input == null)
                {
                    // throw exception
                    Exception ex = new Exception(
                        "OutputAfterProcess exception: Input parameters are set to null, try setting them again");
                    throw(ex);
                }
                
                //////////////////
                // image input
                //////////////////
                IntPtr InpImg = Marshal.AllocHGlobal(this.FinImg_Input.ImageFrame.Length);
                if (InpImg == IntPtr.Zero)
                {
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }
                
                Marshal.Copy(this.FinImg_Input.ImageFrame,0,InpImg,this.FinImg_Input.ImageFrame.Length);
                
                UInt32 DetectedSegmentImageNumber,DetectionFailureDiagnostic;

                /////////////////////
                // Temporary buffer
                /////////////////////
                IntPtr TempBuffer = Marshal.AllocHGlobal((int)GBTempBuffConstants.GBFINIMG_MEMORY_BUFFER_SIZE);
                if (TempBuffer == null)
                {
                    // free memory
                    Marshal.FreeHGlobal(InpImg);
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }

                ///////////////////////////
                // segment descriptor list
                ///////////////////////////
                GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR  [] SegmentImageList = new GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR [4];
                
                if (SegmentImageList == null)
                {
                    Marshal.FreeHGlobal(InpImg);
                    Marshal.FreeHGlobal(TempBuffer);
                    // throw exception
                    Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
                    throw(ex);
                }

                // segment descriptors allocation
                for (int i=0;i<4;i++)
                {
                    SegmentImageList[i] = new GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR();
                    {
                        SegmentImageList[i].SegmentImagePtr = Marshal.AllocHGlobal(
                            (int)(
                                (this.FinImg_Input.SegmentImageSizeX) *
                                (this.FinImg_Input.SegmentImageSizeY)
                            )
                        );
                        if (SegmentImageList[i].SegmentImagePtr == null)
				        {
					        // free memory
					        for (int j=0;j<i;j++)
					        {
						        Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
					        }
					        Marshal.FreeHGlobal(InpImg);
                            Marshal.FreeHGlobal(TempBuffer);
					        // throw exception
					        Exception ex = new Exception("OutputAfterProcess exception: Not enough memory");
					        throw(ex);
				        }
                    }
                }

                int RetVal = GBFINIMGdll_NetWrapper.GBFINIMG_ProcessImage(
                    InpImg,	//unsigned char	*
                    this.FinImg_Input.ImageSizeX,
                    this.FinImg_Input.ImageSizeY,
                    this.FinImg_Input.ImageType,
                    this.FinImg_Input.SegmentationOption,

                    SegmentImageList, // GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR * to a list

                    this.FinImg_Input.SegmentImageSizeX, 
                    this.FinImg_Input.SegmentImageSizeY, 
                    out DetectedSegmentImageNumber,
                    out DetectionFailureDiagnostic,

                    null, // GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  *to a list
                    0,

                    TempBuffer,
                    GBTempBuffConstants.GBFINIMG_MEMORY_BUFFER_SIZE
                );
                
                // check errors
                if (RetVal != NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_NO_ERROR)
                {
                    // free memory
                    {
                        for (int j = 0; j < 4; j++)
                        {
                            Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                        }
                    }
			        Marshal.FreeHGlobal(InpImg);
                    Marshal.FreeHGlobal(TempBuffer);
                    // throw exception
                    Exception ex = new Exception(
                        "OutputAfterProcess exception: Low level GBFINIMG_ProcessImage() function returned " + RetVal);
                    throw(ex);
                }
                
                // copy data in output
                NW_GBFINIMG_OutputData ObjToReturn = new NW_GBFINIMG_OutputData();
                ObjToReturn.Diag = new NW_GBFINIMG_SegmentationFailureDiagnostic( DetectionFailureDiagnostic);
                ObjToReturn.Segment_Number = DetectedSegmentImageNumber;
                
                // end VER2000
                // free memory
                {
                    for (int j = 0; j < 4; j++)
                    {
                        Marshal.FreeHGlobal(SegmentImageList[j].SegmentImagePtr);
                    }
                }
                Marshal.FreeHGlobal(InpImg);
                Marshal.FreeHGlobal(TempBuffer);
                // return normally
                return ObjToReturn;
            }
		}
		/// <summary> InputToSet:
		/// </summary>
		/// <param name ="Inp" > Input data: <see cref = "GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData"/></param>
		/// <exception cref="System.Exception">Thrown when <paramref name ="Inp" /> is null</exception>
		public NW_GBFINIMG_InputData InputToSet {
            set
            {
                if (value == null)
                {
                    // throw exception
                    Exception ex = new Exception("InputToSet property set exception: value to set cannot be null");
                    throw (ex);
                }
                else
                {
                    this.FinImg_Input = value;
                }
            }
		}

		// VER 2000: set RotationBuffer1000DPI
		public Int32 NW_GBFINIMG_SetRotationBuffer1000DPI(
			UInt32	RotationBufferSize
        )
        {
            try
            {
                if (GBFINIMG_Globals.RotationBuffer1000Dpi != IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(GBFINIMG_Globals.RotationBuffer1000Dpi);
                }
                GBFINIMG_Globals.RotationBuffer1000Dpi = Marshal.AllocHGlobal((int)RotationBufferSize);
                if (GBFINIMG_Globals.RotationBuffer1000Dpi == IntPtr.Zero)
                {
                    return NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_ERROR_NO_MEMORY;
                }
                GBFINIMG_Globals.RotationBufferSize = RotationBufferSize;
                GBFINIMGdll_NetWrapper.GBFINIMG_SetRotationBuffer1000DPI(
                    GBFINIMG_Globals.RotationBuffer1000Dpi,
                    GBFINIMG_Globals.RotationBufferSize
                );

                return NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_NO_ERROR;
            }
            catch (Exception ex)
            {
                throw new Exception("NW_GBFINIMG_SetRotationBuffer1000DPI exception: " + ex.Message);
            }
        }

		// VER 2000: release RotationBuffer1000DPI
        public void NW_GBFINIMG_ReleaseRotationBuffer1000DPI()
        {
            // ...
            return;
        }

		// VER 2.2.0.2
		public void NW_GBFINIMG_SetInternalParameterValue(int ParameterType, int ParameterValue)
		{
			try
			{
				GBFINIMGdll_NetWrapper.GBFINIMG_SetInternalParameterValue(ParameterType, ParameterValue);
			}
			catch (Exception ex)
			{
				throw new Exception("GBFINIMG_SetInternalParameterValue exception: " + ex.Message);
			}
		}

		// VER 2.2.0.3
		public static Int32 NW_GBFINIMG_GetLicenseInfo(
			out NW_GBFINIMG_LicenseUSBScannerCheck ScannerBasedLicense
		)
		{
			try
			{
				GBFINIMGdll_NetWrapper.GBFINIMG_LICENSE_USB_SCANNER_CHECK WrapScanBasedLic =
					new GBFINIMGdll_NetWrapper.GBFINIMG_LICENSE_USB_SCANNER_CHECK();

				GBFINIMGdll_NetWrapper.GBFINIMG_GetLicenseInfo(ref WrapScanBasedLic);

				ScannerBasedLicense = new NW_GBFINIMG_LicenseUSBScannerCheck();

				ScannerBasedLicense.Result = WrapScanBasedLic.Result;
				ScannerBasedLicense.UsbDriverErrorCodeDetail = WrapScanBasedLic.UsbDriverErrorCodeDetail;

				return NW_GBFINIMG_ErrorCodes.NW_GBFINIMG_NO_ERROR;
			}
			catch (Exception ex)
			{
				ScannerBasedLicense = null;
				throw new Exception("GBFINIMG_GetLicenseInfo exception: " + ex.Message);
			}
		}
		// end VER 2.2.0.3
	};
}