﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;
using System.Drawing;

using JPEG_NET_WRAPPER;
using Jpeg_Net_Wrapper_Utilities;

namespace Jpeg_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Constants definition
     * 
     ************************************************************
    ************************************************************/
    public class Jpeg_Constants
    {
        public const int GBJPEG_OK = 0;
        public const int GBJPEG_ERROR_INVALID_PARAMETERS = 1;
        public const int GBJPEG_ERROR_CREATE_JPEG_STREAM = 2;
        public const int GBJPEG_ERROR_UNSUPPORTED_BITSPERPIXEL = 3;
        public const int GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE = 4;
        public const int GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE_COMPONENT = 5;
        public const int GBJPEG_ERROR_MATRIX_MEMORY_ALLOCATION = 6;
        public const int GBJPEG_ERROR_UNABLE_TO_ENCODE_IMAGE = 7;
        public const int GBJPEG_ERROR_INITIALIZATION = 8;
        public const int GBJPEG_ERROR_UNABLE_TO_DECODE_IMAGE = 9;
        public const int GBJPEG_ERROR_MISSING_IMAGE_CHANNEL = 10;
        public const int GBJPEG_ERROR_COLOR_SPACE_MODEL_UNSUPPORTED = 11;
        public const int GBJPEG_ERROR_IRREGULAR_CHANNEL_GEOMETRY = 12;
        public const int GBJPEG_ERROR_MEMORY_ALLOCATION = 13;
        public const int GBJPEG_ERROR_UNKNOWN_FORMAT = 14;


        public const int GBJPEG_8_BITS_PER_PIXEL = 8;
        public const int GBJPEG_24_BITS_PER_PIXEL = 24;

        public const int GBJPEG_RATE_LOSSLESS = 1;
    }
    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class Jpegdll_NetWrapper
    {
        [DllImport("GBJPEG.dll", EntryPoint = "GBJPEG_JP2_Encode")]
        public static extern int GBJPEG_JP2_Encode(
            IntPtr ImgBuffer, // unsigned char **
            out UInt32 BuffLen, // unsigned long *
            IntPtr RawImage, // unsigned char []
            UInt32 ImageWidth,
            UInt32 ImageHeight,
            Byte BitsPerPixel,
            UInt16 Rate
        );

        [DllImport("GBJPEG.dll", EntryPoint = "GBJPEG_JP2_Decode")]
        public static extern int GBJPEG_JP2_Decode(
            IntPtr ImgBuffer, // unsigned char []
            UInt32 BuffLen,
            IntPtr RawImage, // unsigned char **
            out UInt32 ImageWidth,
            out UInt32 ImageHeight,
            out Byte BitsPerPixel
        );

        [DllImport("GBJPEG.dll", EntryPoint = "GBJPEG_JPG_Encode")]
        public static extern int GBJPEG_JPG_Encode(
            IntPtr ImgBuffer, // unsigned char **
            out UInt32 BuffLen, // unsigned long *
            IntPtr RawImage, // unsigned char []
            UInt32 ImageWidth,
			UInt32 ImageHeight,
			Byte BitsPerPixel,
            UInt16 DPI,
            UInt16 Q
        );

        [DllImport("GBJPEG.dll", EntryPoint = "GBJPEG_JPG_Decode")]
        public static extern int GBJPEG_JPG_Decode(
            IntPtr ImgBuffer, // unsigned char []
            UInt32 BuffLen,
            IntPtr RawImage, // unsigned char **
            out UInt32 ImageWidth,
            out UInt32 ImageHeight,
            out Byte BitsPerPixel
        );

        [DllImport("GBJPEG.dll", EntryPoint = "GBJPEG_Free")]
        public static extern void GBJPEG_Free(IntPtr PtrToFree);

    }
}

namespace JPEG_NET_WRAPPER
{
    /**********************************************************************/
    /**********************************************************************/
    //**********			SECTION 1: CONSTANTS		******************//
    /**********************************************************************/
    /**********************************************************************/

    /// <summary> Error codes </summary>
    public class NW_GBJPEG_ERRORS
    {
        public static Int32 NW_GBJPEG_OK = Jpeg_Constants.GBJPEG_OK;
        public static Int32 NW_GBJPEG_ERROR_INVALID_PARAMETERS = Jpeg_Constants.GBJPEG_ERROR_INVALID_PARAMETERS;
        public static Int32 NW_GBJPEG_ERROR_CREATE_JPEG_STREAM = Jpeg_Constants.GBJPEG_ERROR_CREATE_JPEG_STREAM;
        public static Int32 NW_GBJPEG_ERROR_UNSUPPORTED_BITSPERPIXEL = Jpeg_Constants.GBJPEG_ERROR_UNSUPPORTED_BITSPERPIXEL;
        public static Int32 NW_GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE = Jpeg_Constants.GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE;
        public static Int32 NW_GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE_COMPONENT =
            Jpeg_Constants.GBJPEG_ERROR_UNABLE_TO_CREATE_IMAGE_COMPONENT;
        public static Int32 NW_GBJPEG_ERROR_MATRIX_MEMORY_ALLOCATION = Jpeg_Constants.GBJPEG_ERROR_MATRIX_MEMORY_ALLOCATION;
        public static Int32 NW_GBJPEG_ERROR_UNABLE_TO_ENCODE_IMAGE = Jpeg_Constants.GBJPEG_ERROR_UNABLE_TO_ENCODE_IMAGE;
        public static Int32 NW_GBJPEG_ERROR_INITIALIZATION = Jpeg_Constants.GBJPEG_ERROR_INITIALIZATION;
        public static Int32 NW_GBJPEG_ERROR_UNABLE_TO_DECODE_IMAGE = Jpeg_Constants.GBJPEG_ERROR_UNABLE_TO_DECODE_IMAGE;
        public static Int32 NW_GBJPEG_ERROR_MISSING_IMAGE_CHANNEL = Jpeg_Constants.GBJPEG_ERROR_MISSING_IMAGE_CHANNEL;
        public static Int32 NW_GBJPEG_ERROR_COLOR_SPACE_MODEL_UNSUPPORTED = Jpeg_Constants.GBJPEG_ERROR_COLOR_SPACE_MODEL_UNSUPPORTED;
        public static Int32 NW_GBJPEG_ERROR_IRREGULAR_CHANNEL_GEOMETRY = Jpeg_Constants.GBJPEG_ERROR_IRREGULAR_CHANNEL_GEOMETRY;
        public static Int32 NW_GBJPEG_ERROR_MEMORY_ALLOCATION = Jpeg_Constants.GBJPEG_ERROR_MEMORY_ALLOCATION;
        public static Int32 NW_GBJPEG_ERROR_UNKNOWN_FORMAT = Jpeg_Constants.GBJPEG_ERROR_UNKNOWN_FORMAT;
        /// <summary> This field has no equivalents in the underlying C version of the library.
        /// This error is returned when an exception is encountered while processing. 
        /// The exception message is stored into the 
        /// <see cref = "NW_GBJPEG_ERROR_EXCEPTION_STRING"/> string.</summary>
        public static Int32 NW_GBJPEG_ERROR_EXCEPTION = 255;
        /// <summary> When an exception is encountered while processing, its message is stored here.</summary>
        public static String NW_GBJPEG_ERROR_EXCEPTION_STRING;
    };

    /// <summary> Bit per pixel constants </summary>
    public class NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS
    {
        public static Byte NW_GBJPEG_8_BITS_PER_PIXEL = Jpeg_Constants.GBJPEG_8_BITS_PER_PIXEL;
        public static Byte NW_GBJPEG_24_BITS_PER_PIXEL = Jpeg_Constants.GBJPEG_24_BITS_PER_PIXEL;
    };


    /// <summary> Compression rate constants </summary>
    /// <remarks> To be used for JPEG 2000 compression algorithm only </remarks>
    public class NW_GBJPEG_RATE_DEFINITIONS
    {
        /// <summary> Indicates a compression rate = 1 </summary>
        public static UInt16 NW_GBJPEG_RATE_LOSSLESS = Jpeg_Constants.GBJPEG_RATE_LOSSLESS; // compression rate = 1
        /// <summary> Indicates a compression rate = 1/100 </summary>
        /// <remarks> This constant has no equivalents in the C version of the library. </remarks>
        public static UInt16 NW_GBJPEG_MAX_RATE = 100; // compression rate = 1/100
    };

    /// <summary> Compression quality constants </summary>
    /// <remarks> To be used for JPEG compression algorithm only </remarks>
    public class NW_GBJPEG_QUALITY_DEFINITIONS
    {
        /// <summary> Indicates a compression quality = 0 </summary>
        /// <remarks> This constant has no equivalents in the C version of the library. </remarks>
        public static UInt16 NW_GBJPEG_MIN_QUAL = 0;
        /// <summary> Indicates a compression quality = 100 </summary>
        /// <remarks> This constant has no equivalents in the C version of the library. </remarks>
        public static UInt16 NW_GBJPEG_MAX_QUAL = 100;
    };

    /// <summary> Compression algorithm identifier values </summary>
    public class NW_GBJPEG_ALGORITHMS_DEFINITIONS
    {
        /// <summary> Indicates the JPEG 2000 algorithm </summary>
        /// <remarks> This constant has no equivalents in the C version of the library. </remarks>
        public static Int32 NW_GBJPEG_COMP_ALG_JPEG2000 = 1;
        /// <summary> Indicates the JPEG algorithm </summary>
        /// <remarks> This constant has no equivalents in the C version of the library. </remarks>
        public static Int32 NW_GBJPEG_COMP_ALG_JPG = 2;
    };




    /**********************************************************************/
    /**********************************************************************/
    //**********			SECTION 2: METHODS		**********************//
    /**********************************************************************/
    /**********************************************************************/

    /// <summary>
    /// This structure is used to collect information generated by the NW_GBJPEG_BmpReader class.
    /// </summary>
    /// <remarks> This structure has no equivalents in the C version of the library. </remarks>
    public class NW_GBJPEG_BmpReader_OutputData
    {
        /// <summary>
        /// Raw frame generated as a result of a bmp file reading.
        /// </summary>
        public Byte[] Frame;
        /// <summary>
        /// <see cref = "Frame"/> horizontal dimension size.
        /// </summary>
        public int SizeX;
        /// <summary>
        /// <see cref = "Frame"/> vertical dimension size.
        /// </summary>
        public int SizeY;
    };

    /// <summary>
    /// This class allows user to get in a fast way the raw frame from a bmp file
    /// </summary>
    /// <remarks>
    /// <para> This class can be used ONLY for 8bit bitmaps.</para>
    /// <para> This structure has no equivalents in the C version of the library.</para>
    /// </remarks>
    public class NW_GBJPEG_BmpReader
    {
        private String BMPFname;
        /// <summary>
        /// constructor overload
        /// </summary>
        /// <param name = "FName">
        /// Name of the Bmp file to be read
        /// </param>
        /// <exception cref = "System.NullReferenceException">
        /// Thrown when <paramref name = "FName"/> is null
        /// </exception>
        public NW_GBJPEG_BmpReader(String FName)
        {
            if (FName == null)
            {
                throw new NullReferenceException("NW_GBJPEG_BmpReader Constructor: null parameter");
            }
            this.BMPFname = FName;
        }
        /// <summary>
        /// This property returns a raw image from the bmp file set by the
        /// <see cref = "NW_GBJPEG_BmpReader.NW_GBJPEG_BmpReader(String FName)"/> constructor
        /// </summary>
        /// <exception cref = "System.Exception">
        /// Thrown when some error occurs:
        /// <see cref = "NW_GBJPEG_ERRORS.NW_GBJPEG_ERROR_EXCEPTION_STRING"/>
        /// </exception>
        public NW_GBJPEG_BmpReader_OutputData Buffer
        {
            get
            {
                if (this.BMPFname == null)
                {
                    throw new System.NullReferenceException("NW_GBJPEG_BmpReader Buffer get(): null parameter");
                }

                Bitmap bmp = new Bitmap(BMPFname);
                if (bmp == null)
                {
                    throw new System.NullReferenceException("NW_GBJPEG_BmpReader Buffer get(): file load error");
                }
                if (bmp.PixelFormat != PixelFormat.Format8bppIndexed)
                {
                    throw new System.NullReferenceException("NW_GBJPEG_BmpReader Buffer get(): 8bpp only is accepted");
                }

                // copy results
                JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader_OutputData RetVal = new JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader_OutputData();

                RetVal.Frame = new Byte[bmp.Width * bmp.Height];

                // get Scan0
                Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                System.Drawing.Imaging.BitmapData bmpData =
                    bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                    bmp.PixelFormat);

                // Get the address of the first line.
                IntPtr ptr = bmpData.Scan0;

                Marshal.Copy((IntPtr)(ptr), RetVal.Frame, 0, RetVal.Frame.Length);

                bmp.UnlockBits(bmpData);

                RetVal.SizeX = bmp.Width;
                RetVal.SizeY = bmp.Height;

                //free memory
                return RetVal;
            }
        }
    };



    /// <summary>
    /// This class allows user to get images encoded by using the JPEG or JPEG 2000 algorithm
    /// with the requested quality (or compression rate).
    /// </summary>
    /// <remarks>
    /// <para>the "Encoded" property returns the image encoded with the currently set properties.
    /// Therefore it's recommended to set them just before encoding.</para>
    /// <para>This class, depending on the <see cref = "CompressionAlgorithm"/> chosen, is the equivalent of the
    /// "GBJPEG_JP2_Encode" or "GBJPEG_JPG_Encode" functions in the C version of the library. In this case, since
    /// the wrapped function is not one only, a "X" appears in the name of the class, that can be interpreted as a "2" or
    /// a "G" depending on the <see cref = "CompressionAlgorithm"/> chosen. Input parameters for encoding are passed
    /// as <see cref = "NW_GBJPEG_JPX_Encode"/> properties, while the output frame is given by getting the
    /// <see cref = "Encoded"/> property.</para>
    /// </remarks>
    public class NW_GBJPEG_JPX_Encode
    {
        /// <summary>
        /// constructor overload
        /// </summary>
        /// <param name = "RawImage">
        /// Image to be encoded
        /// </param>
        /// <param name = "w">
        /// Image width in pixel
        /// </param>
        /// <param name = "h">
        /// Image height in pixel
        /// </param>
        /// <remarks>
        /// the encoding settings specified by the other properties in this class
        /// are set to their default values
        /// </remarks>
        public NW_GBJPEG_JPX_Encode(Byte[] RawImage, UInt32 w, UInt32 h)
        {
            this.Original = RawImage;
            this.ImageW = w;
            this.ImageH = h;
            this.Alg = JPEG_NET_WRAPPER.NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000;
            this.Rate = JPEG_NET_WRAPPER.NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_RATE_LOSSLESS;
            this.BPP = JPEG_NET_WRAPPER.NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_8_BITS_PER_PIXEL;
            this.Qual = 100;
            this.DPI = 500;
        }

        /// <summary>
        /// This property returns the encoded array
        /// </summary>
        /// <remarks>
        /// <para>If a null value is returned see the
        /// <see cref = "NW_GBJPEG_ERRORS.NW_GBJPEG_ERROR_EXCEPTION_STRING"/>
        /// error message.</para>
        /// <para>This property is the equivalent of the "ImgBuffer" output parameter of the "GBJPEG_JP2_Encode" or
        /// "GBJPEG_JPG_Encode" (depends on the <see cref = "CompressionAlgorithm"/> chosen) in the C version of the library.</para>
        /// </remarks>
        public Byte[] Encoded
        {
            get
            {
                if (this.Original == null)
                {
                    throw new Exception("Cannot Encode: Class not initialized");
                }

                // encode
                IntPtr RawImage = Marshal.AllocHGlobal(this.Original.Length);
                if (RawImage == IntPtr.Zero)
                {
                    throw new Exception("Cannot Encode: Cannot alloc RawImage");
                }

                Marshal.Copy(this.Original, 0, RawImage, this.Original.Length);

                UInt32 BufLen;
                int RetVal;
                IntPtr pImgBuffer = Marshal.AllocHGlobal(IntPtr.Size);

                if (this.CompressionAlgorithm == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000)
                {
                    RetVal = Jpegdll_NetWrapper.GBJPEG_JP2_Encode(
                        pImgBuffer,
                        out BufLen,
                        RawImage,
                        this.ImageW, this.ImageH,
                        this.BitPerPixel,
                        this.CompressionRate
                    );
                }
                else
                {
                    RetVal = Jpegdll_NetWrapper.GBJPEG_JPG_Encode(
                        pImgBuffer,
                        out BufLen,
                        RawImage,
                        this.ImageW, this.ImageH,
						this.BitPerPixel,
                        this.DPI,
                        this.CompressionQuality
                    );
                }

                // free memory for RawImage
                Marshal.FreeHGlobal(RawImage);

                if (RetVal != Jpeg_Constants.GBJPEG_OK)
                {
                    throw new Exception("Cannot Encode: Low level encoder returned " + RetVal);
                }

                IntPtr ImgBuffer = Marshal.ReadIntPtr(pImgBuffer);
                // copy image to output
                Byte[] Output = new Byte[BufLen];
                Marshal.Copy((IntPtr)ImgBuffer, Output, 0, (int)BufLen);

                // free memory
                Jpegdll_NetWrapper.GBJPEG_Free(ImgBuffer);
                // return normally
                return Output;
            }
        }


        /// <summary>
        /// This property gets or sets the compression rate for the JPEG 2000 algorithm.
        /// Default value is
        /// <see cref = "NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_RATE_LOSSLESS"/>
        /// </summary>
        /// <remarks>
        /// <para> when setting, value must be 
        /// bigger or equal than <see cref = "NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_RATE_LOSSLESS"/>
        /// and smaller or equal than <see cref = "NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_MAX_RATE"/>
        /// otherwise operation will take no effect
        /// </para>
        /// <para> This value is considered only if the
        /// <see cref = "NW_GBJPEG_JPX_Encode.CompressionAlgorithm"/> is set to
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000"/>
        /// </para>
        /// <para> This property is the equivalent of the "Rate" parameter of the "GBJPEG_JP2_Encode" function in the C 
        /// version of the library. </para>
        /// </remarks>
        public UInt16 CompressionRate
        {
            get
            {
                return this.Rate;
            }
            set
            {
                if (value >= NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_RATE_LOSSLESS &&
                    value <= NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_MAX_RATE)
                {
                    this.Rate = value;
                }
                else
                {
                    throw new Exception("CompressionRate: value to set is out of range");
                }
            }
        }

        /// <summary>
        /// This property gets or sets the compression quality for the JPEG algorithm.
        /// Default value is
        /// <see cref = "NW_GBJPEG_QUALITY_DEFINITIONS.NW_GBJPEG_MAX_QUAL"/>
        /// </summary>
        /// <remarks>
        /// <para> when setting, value must be 
        /// bigger or equal than <see cref = "NW_GBJPEG_QUALITY_DEFINITIONS.NW_GBJPEG_MIN_QUAL"/>
        /// and smaller or equal than <see cref = "NW_GBJPEG_QUALITY_DEFINITIONS.NW_GBJPEG_MAX_QUAL"/>
        /// otherwise operation will take no effect
        /// </para>
        /// <para> This value is considered only if the
        /// <see cref = "NW_GBJPEG_JPX_Encode.CompressionAlgorithm"/> is set to
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPG"/>
        /// </para>
        /// <para> This property is the equivalent of the "Q" parameter of the "GBJPEG_JPG_Encode" function in the C 
        /// version of the library. </para>
        /// </remarks>
        public UInt16 CompressionQuality
        {
            get
            {
                return this.Qual;
            }
            set
            {
                if (value >= NW_GBJPEG_QUALITY_DEFINITIONS.NW_GBJPEG_MIN_QUAL &&
                    value <= NW_GBJPEG_QUALITY_DEFINITIONS.NW_GBJPEG_MAX_QUAL)
                {
                    this.Qual = value;
                }
                else
                {
                    throw new Exception("CompressionQuality: value to set is out of range");
                }
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets the number of bits per pixel used in the raw image.
        /// Can be one of the values defined into the
        /// <see cref = "NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS"/> class.
        /// </para>
        /// <para> 
        /// Default value is
        /// <see cref = "NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_8_BITS_PER_PIXEL"/>
        /// </para>
        /// </summary>
        /// <remarks>
        /// <para> When setting, value must be within the values defined into the
        /// <see cref = "NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS"/> class,
        /// otherwise operation will take no effect. </para>
        /// <para> This property is the equivalent of the "BitsPerPixel" parameter of the "GBJPEG_JP2_Encode" or
        /// "GBJPEG_JPG_Encode" (depends on the <see cref = "CompressionAlgorithm"/> chosen) in the C version of the library. </para>
        /// </remarks>
        public Byte BitPerPixel
        {
            get
            {
                return this.BPP;
            }
            set
            {
                if (value == NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_24_BITS_PER_PIXEL ||
                    value == NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_8_BITS_PER_PIXEL
                )
                {
                    this.BPP = value;
                }
                else
                {
                    throw new Exception("BitPerPixel: value to set is out of range");
                }
                return;
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets the algorithm used for compression.
        /// Can be one of the values defined into the
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS"/> class.
        /// </para>
        /// <para> 
        /// Default value is
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000"/>
        /// </para>
        /// </summary>
        /// <remarks>
        /// <para> When setting, value must be within the values defined into the
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS"/> class,
        /// otherwise operation will take no effect. </para>		
        /// <para> This property has no equivalents in the C version of the library. </para>
        /// </remarks>
        public Int32 CompressionAlgorithm
        {
            get
            {
                return this.Alg;
            }
            set
            {
                if (value == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000 ||
                    value == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPG
                )
                {
                    this.Alg = value;
                }
                else
                {
                    throw new Exception("CompressionAlgorithm: value to set is out of range");
                }
                return;
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets resolution of the raw image in Dots per Inch units.
        /// </para>
        /// <para> 
        /// Default value is 500 dpi.
        /// </para>
        /// <para> This property is the equivalent of the "DPI" parameter of the "GBJPEG_JPG_Encode" function in the C 
        /// version of the library. </para>
        /// </summary>
        public UInt16 ImageResolution
        {
            get
            {
                return this.DPI;
            }
            set
            {
                this.DPI = value;
            }
        }

        protected Byte[] Original;
        protected UInt32 ImageW, ImageH;
        protected UInt16 Rate;
        protected UInt16 Qual;
        protected UInt16 DPI;
        protected Int32 Alg;
        protected Byte BPP;
    };

    /// <summary>
    /// This structure is used to collect information generated by the NW_GBJPEG_JPX_Decode class
    /// </summary>
    /// <remarks>
    /// This class represents, depending on the compression algorithm used, the output of the "GBJPEG_JP2_Decode" or of the
    /// "GBJPEG_JPG_Decode" functions of the C version of the library
    /// </remarks>
    public class NW_GBJPEG_DecoderInfo
    {
        /// <summary>
        /// Represents the "RawImage" output parameter of the "GBJPEG_JP2_Decode" or of the "GBJPEG_JPG_Decode" functions,
        /// depending on the compression algorithm.
        /// </summary>
        public Byte[] RawImage;
        /// <summary>
        /// Represents the "ImageWidth" output parameter of the "GBJPEG_JP2_Decode" or of the "GBJPEG_JPG_Decode" functions,
        /// depending on the compression algorithm.
        /// </summary>
        public Int32 ImageWidth;
        /// <summary>
        /// Represents the "ImageHeight" output parameter of the "GBJPEG_JP2_Decode" or of the "GBJPEG_JPG_Decode" functions,
        /// depending on the compression algorithm.
        /// </summary>
        public Int32 ImageHeight;
        /// <summary>
        /// Represents the "BitsPerPixel" output parameter of the "GBJPEG_JP2_Decode" or of the "GBJPEG_JPG_Decode" functions,
        /// depending on the compression algorithm.
        /// </summary>
        public Byte BitsPerPixel;
    };

    /// <summary>
    /// This class allows user to get the original images previously encoded by using the JPEG or JPEG 2000 algorithm.
    /// </summary>
    /// <remarks>
    /// <para>This class, depending on the <see cref = "CompressionAlgorithm"/> chosen, is the equivalent of the
    /// "GBJPEG_JP2_Decode" or "GBJPEG_JPG_Decode" functions in the C version of the library. In this case, since
    /// the wrapped function is not one only, a "X" appears in the name of the class, that can be interpreted as a "2" or
    /// a "G" depending on the <see cref = "CompressionAlgorithm"/> chosen. The output frame is given by getting the
    /// <see cref = "ImageDecoded"/> property.</para>
    /// </remarks>
    public class NW_GBJPEG_JPX_Decode
    {
        /// <summary>
        /// constructor overload
        /// </summary>
        /// <param name = "EncodedImage">
        /// Image to be decoded
        /// </param>
        /// <remarks>
        /// the encoding settings specified by the other properties in this class
        /// are set to their default values
        /// </remarks>
        public NW_GBJPEG_JPX_Decode(Byte[] EncodedImage)
        {
            this.Original = EncodedImage;
        }

        /// <summary>
        /// This property returns the decoded array
        /// </summary>
        /// <remarks>
        /// <para>If a null value is returned see the
        /// <see cref = "NW_GBJPEG_ERRORS.NW_GBJPEG_ERROR_EXCEPTION_STRING"/>
        /// error message</para>
        /// <para> This property represents the output of the "GBJPEG_JP2_Decode" or of the
        /// "GBJPEG_JPG_Decode" functions of the C version of the library</para>
        /// </remarks>
        public NW_GBJPEG_DecoderInfo ImageDecoded
        {
            get
            {
                if (this.Original == null)
                {
                    throw new Exception("Cannot Decode: Class not initialized");
                }
                if (this.CompressionAlgorithm != NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000 &&
                    this.CompressionAlgorithm != NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPG)
                {
                    throw new Exception("Cannot Decode: Class not initialized");
                }

                NW_GBJPEG_DecoderInfo Info = new NW_GBJPEG_DecoderInfo();

                // encode
                IntPtr ImgBuffer = Marshal.AllocHGlobal(this.Original.Length);
                if (ImgBuffer == IntPtr.Zero)
                {
                    throw new Exception("Cannot Decode: Cannot alloc ImgBuffer");
                }

                Marshal.Copy(this.Original, 0, ImgBuffer, this.Original.Length);
                IntPtr pRawImage = Marshal.AllocHGlobal(IntPtr.Size); 
                UInt32 ImageWidth, ImageHeight;
                Byte BitsPerPixel;
                int RetVal;

                if (this.CompressionAlgorithm == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000)
                {
                    RetVal = Jpegdll_NetWrapper.GBJPEG_JP2_Decode(
                        ImgBuffer,
                        (uint)this.Original.Length,
                        pRawImage,
                        out ImageWidth,
                        out ImageHeight,
                        out BitsPerPixel
                    );
                }
                else
                {
                    RetVal = Jpegdll_NetWrapper.GBJPEG_JPG_Decode(
                        ImgBuffer,
                        (uint)this.Original.Length,
                        pRawImage,
                        out ImageWidth,
                        out ImageHeight,
                        out BitsPerPixel
                    );
                }

                // free memory for ImgBuffer
                Marshal.FreeHGlobal(ImgBuffer);

                if (RetVal != Jpeg_Constants.GBJPEG_OK)
                {
                    throw new Exception("Cannot Decode: low level decoder returned " + RetVal);
                }

                // copy image to output
                Info.RawImage = new Byte[ImageWidth * ImageHeight];
                IntPtr RawImage = Marshal.ReadIntPtr(pRawImage);
                Marshal.Copy((IntPtr)RawImage, Info.RawImage, 0, (int)(ImageWidth * ImageHeight));

                Info.ImageWidth = (int)ImageWidth;
                Info.ImageHeight = (int)ImageHeight;
                Info.BitsPerPixel = BitsPerPixel;

                // free memory
                Jpegdll_NetWrapper.GBJPEG_Free(RawImage);
                return Info;
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets the algorithm used for compression.
        /// Can be one of the values defined into the
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS"/> class.
        /// </para>
        /// <para> 
        /// Default value is
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000"/>
        /// </para>
        /// </summary>
        /// <remarks>
        /// <para> When setting, value must be within the values defined into the
        /// <see cref = "NW_GBJPEG_ALGORITHMS_DEFINITIONS"/> class,
        /// otherwise operation will take no effect. </para>		
        /// <para> This property has no equivalents in the C version of the library. </para>
        /// </remarks>
        public Int32 CompressionAlgorithm
        {
            get
            {
                return this.Alg;
            }
            set
            {
                if (value == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000 ||
                    value == NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPG
                )
                {
                    this.Alg = value;
                }
                else
                {
                    throw new Exception("CompressionAlgorithm: value to set is out of range");
                }
                return;
            }
        }

        protected Byte[] Original;
        protected Int32 Alg;
    };
}
